/*++

  Tandem Confidential: Need to know only

  Copyright (C) 1997, Tandem Computers

  Protected as an unpublished work.
  All rights reserved.

  The computer program listings, specifications and documentation
  herin are the property of Tandem Computers Incorporated or a third
  party supplier and shall not be reproduced, copied, disclosed, or
  used in whole or in part for any reason without the prior express
  written permission of Tandem Computers.

  Project Name :

	VIA - Virtual Interface Architecture Emulator

  File Name:

	VipKA.h

  Abstract :

	This file defines the IOCTL based interface provided by the 
	VIA Kernel Agent.

  Revision History:

	12/29/1997	jrh		Changed the definition of the VIPKA_NET_ADDRESS so that
						it wouldn't have to be type case into something else in
						order to extract its contents in an easy manner.  Changed
						the definition of the VIPKA_NIC_ATTRIBUTES data structure
						so that only the Servernet ID of the nic is returned;
						the previous definition was trying to return the entire
						VIPKA_NET_ADDRESS, which can't be known because the 
						port number isn't known.  Changed some NT types to their
						VIP_* equivalents to conform to what the rest of this
						file is doing.  Changed C++ style "//" comments to be
						C-style comments to aid portability.
	
	12/30/1997  ssc     Removed the nicAttribs field from the VIPKA_NIC_OPEN_OUT 
						structure. VIPL will do a Nic Query to retrieve Nic attributes.
	
	12/30/1997  ssc     In VIPKA_CREATE_CQ_OUT, changed the header output to be a pointer.
	
	12/31/1997  ssc     Removed VIPKA_XFER_DONE_IN, IOCTL_VIPKA_SEND_DONE, and 
						IOCTL_VIPKA_RECV_DONE. VIPL does not interact with kernel agent 
						when polling for xfer completion in VipSendDone and VipRecvDone.
						Added the hNic field to XFER_WAIT_IN struct. 
						Misc changes on variable names to conform with VIE coding convention. 
						Removed pDescr field from XFER_COMPLETION_OUT; the kernel agent does 
						not need to return the descriptor to VIPL because VIPL always 
						remove THE descriptor at the head of the queue.
						Changed the VIPRA_RESIZE_CQ_IN.hCQ field to type VIPKA_CQ_HANDLE.
	01/05/98	jrh		Added new ioctl code to support (future) user mode tracing.
						Modified structure member names to conform to the coding
						standard.
	01/29/98	dfc		Added count of descriptors VIPL has completed to XFER_WAIT_IN. This
						provides a mechanism by which the KA can detect a descriptor being
						completed between the time VIPL checked the queue and the time
						the KA gets control.
	02/08/98	ssc		Removed CQ_DONE ioctl, in/out declarations. There is no kernel agent
	                    interaction for CQ_DONE. Modified CQ data structures to use the VIP 
						data types (e.g. VIP_UINT16 instead of unsigned int) and renamed some 
						CQ field names to better reflect its usage. There are a couple 
						extra fields that will be removed by Athena.
	02/26/98	dfc		Modified VIPKA_NOTIFY_IN structure, used by notify ioctls
	03/05/98    ssc     Modified VIPKA_ERROR_DESCRIPTOR's CQ handle field to use 
						VIPKA_CQ_HANDLE instead of VIP_CQ_HANDLE.
	03/16/98	dfc		Increased size of VIPKA_CQ_NOTIFY_IN.retrievedCnt to 32 bits 
	03/27/98	dfc		Added support for using ServerNet system services.
	04/09/98	ssc		Added a filler field to VIPKA_NET_ADDRESS. Reserve for future use.
	04/17/98	jrh		Added doorBellCount field to the Send/Recv post ioctl to help
						fix a bug that was causing descriptor completions to be done
						twice.
	06/10/98	sck		Handles discriminator up to 56 bytes although only
						the first 4 bytes are significant.
	08/05/98	jrh		Added ioctl to allow clients to set the  path to be used
						to reach a Servernet endnode.

--*/

#ifndef  _VIPKA_H_
#define  _VIPKA_H_  1

#include "viadesc.h"

/* max size of NIC names */
#define	VIPKA_MAX_NIC_NAME_SIZE	64

/* 
 * Some of the application visible types are defined by the U.A. and are not available to the K.A.
 * The K.A interface has to define its own types for these.
 *
 * A value of zero is considered invalid (or NULL) handle for the following index type handles 
 * in the kernel agent.
 */
typedef unsigned long		VIPKA_NIC_HANDLE;	/* Handle to a NIC returned by the K.A.        */
typedef unsigned long		VIPKA_VI_HANDLE;	/* Handle to a VI returned by the K.A.         */
typedef unsigned long		VIPKA_CONN_HANDLE;	/* Handle to a connection returned by K.A.     */
typedef unsigned long		VIPKA_CQ_HANDLE;    /* Handle to a CQ table entry returned by K.A. */

/* Net address as defined by the Snet K.A. */
typedef struct _VIPKA_NET_ADDRESS {
	VIP_UINT16	hostAddressLen;		/* length of the host address part */
	VIP_UINT16	discriminatorLen;	/* length of the endpoint discriminator part */
	VIP_ULONG	snid;				/* Servernet address of node. */
	union
	{
		VIP_ULONG	port;				/* Port on node. */
		VIP_UINT8	discriminator[56];	/* Discriminator */
#ifdef WINDOWS_NT
	};
#else
	} u;
#endif
} VIPKA_NET_ADDRESS, *PVIPKA_NET_ADDRESS;

/* The K.A. defines an extension to the NIC attributes structure defined by the standard. */
/* It will ignore the LocalNicAddress field of VIP_NIC_ATTRIBUTES. */
/* Instead it fills the sLocalNicAddress field of VIPKA_NIC_ATTRIBUTES */
typedef struct _VIPKA_NIC_ATTRIBUTES {
	VIP_NIC_ATTRIBUTES		stdAttribs;			/* as defined by the std */
	VIP_ULONG				snid;				/* the local address whose format is VIP dependent */
} VIPKA_NIC_ATTRIBUTES, *PVIPKA_NIC_ATTRIBUTES ;

/* K.A. definition of error descriptor using the VIPKA types */
typedef struct _VIPKA_ERROR_DESCRIPTOR {
	VIPKA_NIC_HANDLE	hNic;		/* The Nic on which the error occurred */
	VIPKA_VI_HANDLE		hVi;		/* If non-NULL, the VI instance related to the error */
	VIPKA_CQ_HANDLE		hCQ;		/* If non-NULL, the CQ related to the error */
	VIP_DESCRIPTOR*		pDescriptor;/* If non-NULL the Descriptor related to the error */
	VIP_ULONG			opCode;		/* The operation being performed */
	VIP_RESOURCE_CODE	resourceCode;/* The resource which caused the problem */	
	VIP_ERROR_CODE		errorCode;	/* The specific error */
}VIPKA_ERROR_DESCRIPTOR, *PVIPKA_ERROR_DESCRIPTOR ;

/*
 * The VIPKA_SERVICE structure is used to record the service numbers
 * assigned to the registered service handlers 
 */
typedef struct _VIPKA_SERVICE_NUMBER {
	ULONG		postSend;			/* VipPostSend		*/
	ULONG		postRecv;			/* VipPostRecv		*/
} VIPKA_SERVICE_NUMBER, *PVIPKA_SERVICE_NUMBER;

/* 
 * The constants for the Ioctls and the struct definitions capturing the 
 * input/output parameters to these ioctls are defined below.
 */

/*
 * Define the various device type values.  Note that values used by Microsoft
 * Corporation are in the range 0-32767, and 32768-65535 are reserved for use
 * by customers.
 */
#define FILE_DEVICE_VIP	0x00008000

/*
 * Macro definition for defining IOCTL function control codes.  Note
 * that function codes 0-2047 are reserved for Microsoft Corporation, and
 * 2048-4095 are reserved for customers.
 */
#define VIPKA_IOCTL_INDEX  0x800

#ifdef WINDOWS_NT
/* 
 * Macro to define a VIP ioctl.
 */
#define  VIPKA_IOCTL(x)  CTL_CODE(FILE_DEVICE_VIP,	\
				VIPKA_IOCTL_INDEX + (x),			\
				METHOD_BUFFERED,					\
				FILE_ANY_ACCESS)
#else
typedef struct _DEVICE_IO_CONTROL {
	PVOID	pInBuffer;
	ULONG	InBufferSize;
	PVOID	pOutBuffer;
	ULONG	OutBufferSize;
	PLONG	pBytesReturned;
} DEVICE_IO_CONTROL;

#define VIPKA_IOCTL(x)  ('V'<< 24 | 'I' << 16 | 'P' << 8 | (x))
#endif


/* 
 * All the Ioctls have two structs associated with them to capture the input
 * and output parameters.
 * To avoid bogus pointers coming in from user mode we should avoid passing a
 * pointer as an input parameter. (If we have to pass pointers as an input 
 * parameter, the KA will have to validate the pointer.)
 */

/*
 * Some common structs which are used to package the input/output parameters
 * for more than one ioctl.
 */

/*
 * VIPKA_IOCTL_STATUS_OUT is the struct returned for ioctls where only status is
 * returned as the ouptut parameter.
 */
typedef struct _VIPKA_IOCTL_STATUS_OUT {
	VIP_RETURN		status;		/* status/error code */
} VIPKA_IOCTL_STATUS_OUT, *PVIPKA_IOCTL_STATUS_OUT;

/* 
 * VIPKA_POST_IN is used for formatting input parameters to the 
 * VIPKA_POST_SEND and VIPKA_POST_RECV ioctls.
 */
typedef struct  _VIPKA_POST_IN  {
	VIPKA_NIC_HANDLE    hNic;		/* the nic instance                         */
	VIPKA_VI_HANDLE		hVi;		/* handle to Vi on which data is to be sent */
	VIP_DESCRIPTOR*		pDescr;		/* Descriptor specifying the transfer		*/
	VIP_MEM_HANDLE		hMemRegion;	/* Memory region of the descriptor			*/
	VIP_ULONG			doorBellCount;	/* Doorbell # for this operation        */
} VIPKA_POST_IN, *PVIPKA_POST_IN;

/* 
 * VIPKA_XFER_WAIT_IN is used for formatting input parameters to the 
 * VIPKA_SEND_WAIT and VIPKA_RECV_WAIT ioctls.
 * The handle of the VI to be polled and a timeout value are passed.
 */
typedef struct  _VIPKA_XFER_WAIT_IN  {
	VIPKA_NIC_HANDLE    hNic;		/* the nic instance                         */
	VIPKA_VI_HANDLE		hVi;		/* handle to Vi on which data is to be sent */
	VIP_ULONG			timeout;	/* Timeout period in milliseconds           */
	VIP_ULONG			doneCnt;	/* Count of descriptors VIPL has completed	*/
} VIPKA_XFER_WAIT_IN, *PVIPKA_XFER_WAIT_IN;


/*
 * VIPKA_XFER_COMPLETION_OUT captures the output parameters to the
 * VIPKA_SEND_WAIT and VIPKA_RECV_WAIT ioctls. Kernel agent will only
 * complete the ioctl IRP if the Descriptor at the Head of the
 * VI work queue is marked DONE. KA does not need to return the 
 * Descriptor pointer to VIPL.
 * 
 * VIPL is responsible to removing the Descriptor from the Head Queue,
 * this ensures that Descriptors are completed in the order they were 
 * queued.
 */

typedef struct  _VIPKA_XFER_COMPLETION_OUT  {
	VIP_RETURN			status;				/* status/error code */
} VIPKA_XFER_COMPLETION_OUT, *PVIPKA_XFER_COMPLETION_OUT;


/*
 ********************************************************************************
 *************************H A R D W A R E  C O N N E C T I O N*******************
 ********************************************************************************
 */

/*
 * Ioctl for the VipOpenNic API. 
 */
#define  IOCTL_VIPKA_OPEN_NIC	VIPKA_IOCTL( 1)

/*
 * VIPKA_OPEN_NIC_IN captures the input parameter to the VIPKA_OPEN_NIC ioctl.
 * The multiple NICs controlled by the (single instance of) VIP Kernel Agent 
 * are distinguished by names.
 */
typedef struct  _VIPKA_OPEN_NIC_IN  {
	char	nicName[VIPKA_MAX_NIC_NAME_SIZE]; /* Name of the NIC to be opened */
} VIPKA_OPEN_NIC_IN, *PVIPKA_OPEN_NIC_IN;

/*
 * VIPKA_OPEN_NIC_OUT captures the output parameters to the VIPKA_OPEN_NIC ioctl.
 * The handle to the opened NIC and an error status is returned. bViCompliant
 * is set to TRUE if the underlined NIC card supports VIA, otherwise it's 
 * set to FALSE (which indicates we are running on the VIA Emulator.
 *
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : K.A. could not recognize the given name.
 *		VIP_ERROR_RESOURCE    : Ran out of resources.
 */
typedef struct  _VIPKA_OPEN_NIC_OUT  {
	VIP_RETURN				status;			/* status/error code */
	VIPKA_NIC_HANDLE		hNic;			/* handle to the opened NIC */
	VIP_BOOLEAN				bViCompliant;	/* set to True if Nic is VIA H/W */
	VIPKA_SERVICE_NUMBER	serviceNumber;	/* service numbers allocated by KA */
} VIPKA_OPEN_NIC_OUT, *PVIPKA_OPEN_NIC_OUT;


/*
 * Ioctl for the VipCloseNic API. 
 */
#define  IOCTL_VIPKA_CLOSE_NIC	VIPKA_IOCTL( 2)

/*
 * VIPKA_CLOSE_NIC_IN captures the input parameter to the VIPKA_CLOSE_NIC ioctl.
 * The handle to the NIC to be closed is passed.
 */
typedef struct  _VIPKA_CLOSE_NIC_IN  {
	VIPKA_NIC_HANDLE			hNic;		/* handle to the NIC to be closed */	
} VIPKA_CLOSE_NIC_IN, *PVIPKA_CLOSE_NIC_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC handle.
 */

/*
 ********************************************************************************
 *************************V I  C R E A T I O N/D E S T R U C T I O N*************
 ********************************************************************************
 */

/*
 * Ioctl for the VipCreateVi API. 
 */
#define  IOCTL_VIPKA_CREATE_VI	VIPKA_IOCTL( 3)

/*
 * VIPKA_CREATE_VI captures the input parameters to the VIPKA_CREATE_VI ioctl.
 * The NIC handle, VI attributes and the CQ handles (for the send/recv Q) are
 * passed.
 */
typedef struct  _VIPKA_CREATE_VI_IN  {
	VIPKA_NIC_HANDLE			hNic;		/* handle to the NIC instance*/	
	VIP_VI_ATTRIBUTES			viAttribs;	/* Requested Attributes */
	VIPKA_CQ_HANDLE				hSendCQ;	/* CQ to be associated with the send Work Queue */
	VIPKA_CQ_HANDLE				hRecvCQ;	/* CQ to be associated with the receive Work Queue */
} VIPKA_CREATE_VI_IN, *PVIPKA_CREATE_VI_IN;

/*
 * VIPKA_CREATE_VI_OUT captures the output parameters to the VIPKA_CREATE_VI ioctl.
 * The handle to the new VI and an error status is returned.
 * Possible error codes :
 *		VIP_ERROR_RESOURCE	  : Insufficient resources.
 *		VIP_INVALID_PARAMETER : Invalid NIC/CQ handle.
 *		VIP_INVALID_RELIABLILITY_LEVEL 
 *		VIP_INVALID_MTU
 *		VIP_INVALID_QOS
 *		VIP_INVALID_PTAG
 */

typedef struct  _VIPKA_CREATE_VI_OUT  {
	VIP_RETURN					status;	/* status/error code */
	VIPKA_VI_HANDLE				hVi;	/* the handle to the new VI */
} VIPKA_CREATE_VI_OUT, *PVIPKA_CREATE_VI_OUT;

/*
 * Ioctl for the VipDestroyVi API. 
 */
#define  IOCTL_VIPKA_DESTROY_VI	VIPKA_IOCTL( 4)

/*
 * VIPKA_DESTROY_VI_IN captures the input parameter to the VIPKA_DESTROY_VI ioctl.
 * The handle to the VI to be torn down is passed.
 */
typedef struct  _VIPKA_DESTROY_VI_IN  {
	VIPKA_NIC_HANDLE		hNic;		/* handle to the NIC instance */	
	VIPKA_VI_HANDLE			hVi;		/* handle to the VI to be destroyed */	
} VIPKA_DESTROY_VI_IN, *PVIPKA_DESTROY_VI_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle.
 */

/*
 ********************************************************************************
 *************************C O N N.  M G M T.*************************************
 ********************************************************************************
 */

/*
 * Ioctl for the VipConnectWait API. 
 */
#define  IOCTL_VIPKA_CONNECT_WAIT	VIPKA_IOCTL( 5)

/*
 * VIPKA_CONNECT_WAIT_IN captures the input parameters to the VIPKA_CONNECT_WAIT ioctl.
 * The handle to the NIC and the local address on which the conns are to be waited, 
 * and the timeout period of the wait are passed as parameters.
 */
typedef struct  _VIPKA_CONNECT_WAIT_IN  {
	VIPKA_NIC_HANDLE			hNic;		/* handle to the NIC on which to look for incoming conn reqs */	
	VIPKA_NET_ADDRESS			localAddr;	/* local n/w addr. on which to wait */
	VIP_ULONG					timeout;	/* timeout count in milliseconds */
} VIPKA_CONNECT_WAIT_IN, *PVIPKA_CONNECT_WAIT_IN;

/*
 * VIPKA_CONNECT_WAIT_OUT captures the output parameters to the VIPKA_CONNECT_WAIT ioctl.
 * The remote address from which the connection has come in, a connection handle and an error 
 * status is returned.
 * Possible error codes :
 *		VIP_TIMEOUT
 *		VIP_ERROR_RESOURCE
 *		VIP_INVALID_PARAMETER : Invalid NIC handle/ invalid local address.
 */

typedef struct  _VIPKA_CONNECT_WAIT_OUT  {
	VIP_RETURN					status;		/* status/error code */
	VIPKA_NET_ADDRESS			remoteAddr; /* remote address that is requesting a connection */
	VIP_VI_ATTRIBUTES			remoteViAttribs; /* remote Vi Attributes */
	VIPKA_CONN_HANDLE			hConn;		/* handle to be used in subsequent conn. operations */
} VIPKA_CONNECT_WAIT_OUT, *PVIPKA_CONNECT_WAIT_OUT;

/*
 * Ioctl for the VipConnectAccept API. 
 */
#define  IOCTL_VIPKA_CONNECT_ACCEPT	 VIPKA_IOCTL( 6)

/*
 * VIPKA_CONNECT_ACCEPT_IN captures the input parameters to the VIPKA_CONNECT_ACCEPT ioctl.
 * The handle of the connection to be accepted and the VI to be associated with it are
 * passed.
 */
typedef struct  _VIPKA_CONNECT_ACCEPT_IN  {
	VIPKA_NIC_HANDLE			hNic;	/* handle to the local NIC instance */
	VIPKA_CONN_HANDLE			hConn;	/* conn. handle to be accepted */
	VIPKA_VI_HANDLE				hVi;	/* VI handle to be associated with it */
} VIPKA_CONNECT_ACCEPT_IN, *PVIPKA_CONNECT_ACCEPT_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid Conn/VI handle
 *		VIP_INVALID_RELIABILITY_LEVEL,
 *		VIP_INVALID_MTU,
 *		VIP_INVALID_QOS		  : The attibutes of the local endpoint conflicted with the remote endpoint.
 */

/*
 * Ioctl for the VipConnectReject API. 
 */
#define  IOCTL_VIPKA_CONNECT_REJECT	VIPKA_IOCTL( 7)

/*
 * VIPKA_CONNECT_REJECT_IN captures the input parameters to the VIPKA_CONNECT_REJECT ioctl.
 * The handle of the connection to be rejected is passed.
 */
typedef struct  _VIPKA_CONNECT_REJECT_IN  {
	VIPKA_NIC_HANDLE		hNic;	/* handle to the local NIC instance */
	VIPKA_CONN_HANDLE		hConn;	/* conn. handle to be rejected */
} VIPKA_CONNECT_REJECT_IN, *PVIPKA_CONNECT_REJECT_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid Conn handle
 */

/*
 * Ioctl for the VipConnectRequest API. 
 */
#define  IOCTL_VIPKA_CONNECT_REQUEST	VIPKA_IOCTL( 8)

/*
 * VIPKA_CONNECT_REQUEST_IN captures the input parameters to the VIPKA_CONNECT_REQUEST ioctl.
 * The handle of the local VI, local and remote n/w addresses and the timeout for the request
 * are passed as input parameters.
 */
typedef struct  _VIPKA_CONNECT_REQUEST_IN  {
	VIPKA_NIC_HANDLE			hNic;		/* handle to the local NIC instance */
	VIPKA_VI_HANDLE				hVi;		/* local VI endpoint */
	VIPKA_NET_ADDRESS			localAddr;	/* local n/w address */
	VIPKA_NET_ADDRESS			remoteAddr;	/* remote n/w address */
	VIP_ULONG					timeout;		/* timeout value */
} VIPKA_CONNECT_REQUEST_IN, *PVIPKA_CONNECT_REQUEST_IN;

/*
 * VIPKA_CONNECT_REQUEST_OUT captures the output parameters to the VIPKA_CONNECT_REQUEST ioctl.
 * The remote VI attributes and an error status are returned.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle/local address.
 *		VIP_TIMEOUT
 *		VIP_ERROR_RESOURCE
 *		VIP_REJECT
 */

typedef struct  _VIPKA_CONNECT_REQUEST_OUT  {
	VIP_RETURN					status;				/* status/error code */
	VIP_VI_ATTRIBUTES			remoteViAttribs;	/* remote vi attribs, if successful */
} VIPKA_CONNECT_REQUEST_OUT, *PVIPKA_CONNECT_REQUEST_OUT;


/*
 * Ioctl for the VipDisconnect API. 
 */
#define  IOCTL_VIPKA_DISCONNECT	VIPKA_IOCTL( 9)

/*
 * VIPKA_DISCONNECT_IN captures the input parameters to the VIPKA_DISCONNECT ioctl.
 * The handle of the VI to be disconnected is passed.
 */
typedef struct  _VIPKA_DISCONNECT_IN  {
	VIPKA_NIC_HANDLE	hNic;	/* handle to the local NIC instance */
	VIPKA_VI_HANDLE		hVi;	/* Vi handle to be disconnected */
} VIPKA_DISCONNECT_IN, *PVIPKA_DISCONNECT_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid Vi handle
 */

/*
 ********************************************************************************
 *************************M E M O R Y  M G M T.**********************************
 ********************************************************************************
 */

/*
 * Ioctl for the VipCreatePtag API. 
 */
#define  IOCTL_VIPKA_CREATE_PTAG	VIPKA_IOCTL( 10)

/*
 * VIPKA_CREATE_PTAG_IN captures the input parameters to the VIPKA_CREATE_PTAG ioctl.
 * The NIC handle on which the tag is to be created is passed.
 */
typedef struct  _VIPKA_CREATE_PTAG_IN  {
	VIPKA_NIC_HANDLE		hNic;	/* Nic handle for Ptag */
} VIPKA_CREATE_PTAG_IN, *PVIPKA_CREATE_PTAG_IN;

/*
 * VIPKA_CREATE_PTAG_OUT captures the output parameters to the VIPKA_CREATE_PTAG ioctl.
 * The Ptag and an error status are returned.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC handle
 *		VIP_ERROR_RESOURCE
 */

typedef struct  _VIPKA_CREATE_PTAG_OUT  {
	VIP_RETURN					status;		/* status/error code */
	VIP_PROTECTION_HANDLE		hPtag;		/* the created ptag */
} VIPKA_CREATE_PTAG_OUT, *PVIPKA_CREATE_PTAG_OUT;


/*
 * Ioctl for the VipDestroyPtag API. 
 */
#define  IOCTL_VIPKA_DESTROY_PTAG	VIPKA_IOCTL( 11)

/*
 * VIPKA_DESTROY_PTAG_IN captures the input parameters to the VIPKA_DESTROY_PTAG ioctl.
 * The NIC handle and the tag to be destroyed are passed.
 */
typedef struct  _VIPKA_DESTROY_PTAG_IN  {
	VIPKA_NIC_HANDLE			hNic;		/* Nic handle for Ptag */
	VIP_PROTECTION_HANDLE		hPtag;		/* tag to be destroyed */
} VIPKA_DESTROY_PTAG_IN, *PVIPKA_DESTROY_PTAG_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC/Ptag handle
 *		VIP_ERROR_RESOURCE	  : A VI instance and/or a registered memory region is still associated
 *								with the specified protection tag.
 */

/*
 * Ioctl for the VipRegisterMem API. 
 */
#define  IOCTL_VIPKA_REGISTER_MEM	VIPKA_IOCTL( 12)

/*
 * VIPKA_REGISTER_MEM_IN captures the input parameters to the VIPKA_REGISTER_MEM ioctl.
 * The virtual address, length and the memory attributes of the region and the NIC handle
 * on which it is to be registered are passed.
 */
typedef struct  _VIPKA_REGISTER_MEM_IN  {
	VIPKA_NIC_HANDLE			hNic;			/* Nic handle for memory region */
	VIP_PVOID					virtualAddress;	/* starting address of the region */
	VIP_ULONG					length;			/* length of the region */
	VIP_MEM_ATTRIBUTES			memAttribs;		/* requested Attributes for the region */
} VIPKA_REGISTER_MEM_IN, *PVIPKA_REGISTER_MEM_IN;

/*
 * VIPKA_REGISTER_MEM_OUT captures the output parameters to the VIPKA_REGISTER_MEM ioctl.
 * The handle to the registered memory region and an error status are returned.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC handle, virtual address/length.
 *		VIP_INVALID_PTAG	  : The protection tag attribute was invalid.
 *		VIP_ERROR_RESOURCE
 */

typedef struct  _VIPKA_REGISTER_MEM_OUT  {
	VIP_RETURN					status;		/* status/error code */
	VIP_MEM_HANDLE				hMemRegion;	/* handle to the registered  memory region */
} VIPKA_REGISTER_MEM_OUT, *PVIPKA_REGISTER_MEM_OUT;


/*
 * Ioctl for the VipDeregisterMem API. 
 */
#define  IOCTL_VIPKA_DEREGISTER_MEM	VIPKA_IOCTL( 13)

/*
 * VIPKA_DEREGISTER_MEM_IN captures the input parameters to the VIPKA_DEREGISTER_MEM ioctl.
 * The NIC handle, memory region handle and the virtual address of the region to be 
 * deregistered are passed.
 */
typedef struct  _VIPKA_DEREGISTER_MEM_IN  {
	VIPKA_NIC_HANDLE			hNic;			/* Nic handle for memory region */
	VIP_PVOID					pVirtualAddress;	/* start addres of the region */
	VIP_MEM_HANDLE				hMemRegion;		/* handle to the mem region */
} VIPKA_DEREGISTER_MEM_IN, *PVIPKA_DEREGISTER_MEM_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC handle, memory handle or virtual address.
 */

/*
 ********************************************************************************
 *************************D A T A  T R A N S F E R / C O M P L E T I O N*********
 ********************************************************************************
 */

/*
 * Ioctl for the VipPostSend API. 
 */
#define  IOCTL_VIPKA_POST_SEND	VIPKA_IOCTL( 14)

/*
 * VIPKA_POST_IN captures the input parameters to the VIPKA_POST_SEND ioctl.
 * The VI handle, descriptor pointer and the memory handle (of the descriptor
 * memory region) are passed.
 */
/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle
 */

/*
 * Ioctl for the VipSendWait API. 
 */
#define  IOCTL_VIPKA_SEND_WAIT		VIPKA_IOCTL( 16)

/*
 * VIPKA_XFER_WAIT_IN captures the input parameters to the VIPKA_SEND_WAIT ioctl.
 * The VI handle and the timeout period are passed.
 */

/*
 * VIPKA_XFER_COMPLETION_OUT captures the output parameters to the VIPKA_SEND_WAIT ioctl.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle
 *		VIP_ERROR_RESOURCE	  : The send queue is associated with a completion queue.
 *		VIP_TIMEOUT			  : Timeout expired without a descriptor completing.
 */

/*
 * Ioctl for the VipPostRecv API. 
 */
#define  IOCTL_VIPKA_POST_RECV		VIPKA_IOCTL( 17)

/*
 * VIPKA_POST_IN captures the input parameters to the VIPKA_POST_RECV ioctl.
 * The VI handle is passed.
 */

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle
 */

/*
 * Ioctl for the VipRecvWait API. 
 */
#define  IOCTL_VIPKA_RECV_WAIT		VIPKA_IOCTL( 19)

/*
 * VIPKA_XFER_WAIT_IN captures the input parameters to the VIPKA_RECV_WAIT ioctl.
 * The VI handle and the timeout period are passed.
 */

/*
 * VIPKA_XFER_COMPLETION_OUT captures the output parameters to the VIPKA_RECV_WAIT ioctl.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle
 *		VIP_ERROR_RESOURCE	  : The send queue is associated with a completion queue.
 *		VIP_TIMEOUT			  : Timeout expired without a descriptor completing.
 */


/*
 * Ioctl for the VipCQWait API. 
 */
#define  IOCTL_VIPKA_CQ_WAIT	VIPKA_IOCTL( 21)

/*
 * VIPKA_CQ_WAIT_IN captures the input parameters to the VIPKA_CQ_WAIT ioctl.
 * The CQ handle to be polled/waited upon is passed in.
 */
typedef struct  _VIPKA_CQ_WAIT_IN  {
	VIPKA_NIC_HANDLE	hNic;			/* handle to the local NIC instance */
	VIPKA_CQ_HANDLE		hCQ;			/* CQ to be polled */
	VIP_ULONG			timeout;		/* timeout in milliseconds */			
	VIP_ULONG	  		retrievedCnt;	/* number of entries retrieved by VIPL */
} VIPKA_CQ_WAIT_IN, *PVIPKA_CQ_WAIT_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle
 */


/*
 * Ioctl for the VipSendNotify API. 
 */
#define  IOCTL_VIPKA_SEND_NOTIFY	VIPKA_IOCTL( 22)

/*
 * VIPKA_NOTIFY_IN captures the input parameters to the VIPKA_SEND_NOTIFY ioctl.
 * The VI handle, Context and the routine to be notified are passed in.
 * Context and the routine are passed back uninterpreted by the K.A.
 */
typedef struct  _VIPKA_NOTIFY_IN  {
	VIPKA_NIC_HANDLE	hNic;				/* handle to the local NIC instance */
	VIPKA_VI_HANDLE		hVi;				/* Handle of the VI the user is interested in */
	VIP_PVOID			pContext;			/* Context to be passed back to the user */
	void				(*pfnHandler)(void *,void *,void *); /* Routine to be called back */
	HANDLE				hNotifyThread;		/* Thread handle for notify thread */
	VIP_ULONG			doneCnt;			/* Count of descriptors VIPL has completed	*/
} VIPKA_NOTIFY_IN, *PVIPKA_NOTIFY_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle
 *		VIP_ERROR_RESOURCES	  : Send Queue of the VI is associated with a CQ.
 */


/*
 * Ioctl for the VipRecvNotify API. 
 */
#define  IOCTL_VIPKA_RECV_NOTIFY	VIPKA_IOCTL( 23)

/*
 * VIPKA_NOTIFY_IN captures the input parameters to the VIPKA_RECV_NOTIFY ioctl.
 * The VI handle, Context and the routine to be notified are passed in.
 */
/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle
 *		VIP_ERROR_RESOURCES	  : Send Queue of the VI is associated with a CQ.
 */


/*
 * Ioctl for the VipCQNotify API. 
 */
#define  IOCTL_VIPKA_CQ_NOTIFY	VIPKA_IOCTL( 24)

/*
 * VIPKA_CQ_NOTIFY_IN captures the input parameters to the VIPKA_CQ_NOTIFY ioctl.
 * The CQ handle, Context and the routine to be notified are passed in.
 * Context and the routine are passed back uninterpreted by the K.A.
 */
typedef struct  _VIPKA_CQ_NOTIFY_IN  {
	VIPKA_NIC_HANDLE	hNic;			/* handle to the local NIC instance */
	VIPKA_CQ_HANDLE		hCQ;				/* Handle of the CQ the user is interested in */
	VIP_PVOID			pContext;			/* Context to be passed back to the user */
	void				(*pfnHandler)(void *,void *,void *); /* Routine to be called back */
	HANDLE				hNotifyThread;		/* Thread handle for notify thread */
	VIP_ULONG			retrievedCnt;		/* Count of CQ entries VIPL has retrieved	*/
} VIPKA_CQ_NOTIFY_IN, *PVIPKA_CQ_NOTIFY_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid CQ handle
 */

 
/*
 ********************************************************************************
 *************************C Q  M G M T. *****************************************
 ********************************************************************************
 */

/*
 ****************************
 * CQ data structures 
 *****************************
 */

/* VIPKA_CQ_HEAD defines the completion queue Head structure */

typedef union _VIPKA_CQ_HEAD {
	VIP_UINT16	qOffset;
	VIP_UINT64  wrapCount;
} VIPKA_CQ_HEAD, *PVIPKA_CQ_HEAD;
 
/* VIPKA_CQ_USER_HDR contains the completion queue header information needed by VIPL */

typedef struct _VIPKA_CQ_USER_HDR {
    VIP_PULONG      qBase;			/* pointer to queue base address (in user space) */
    VIPKA_CQ_HEAD   qHead;			/* queue Head (64 bit structure) */
	VIP_UINT16		mask;
    VIP_UINT32      entryCount;		/* number of entries in the queue (including the marker */
} VIPKA_CQ_USER_HDR, *PVIPKA_CQ_USER_HDR;

/* VIP_CQ_ENTRY defines the CQ entry used by VIPL and KA */

typedef struct _VIPKA_CQ_ENTRY {
	VIP_UINT16	viNum;       /* VI number (zero is not used) */
    VIP_UINT16	recvQueue;   /* = 1 if completion is for the VI receive Queue */
} VIPKA_CQ_ENTRY, *PVIPKA_CQ_ENTRY;

/* Ioctl for the VipCreateCQ API.  */

#define  IOCTL_VIPKA_CREATE_CQ	VIPKA_IOCTL( 25)

/*
 * VIPKA_CREATE_CQ_IN captures the input parameters to the VIPKA_CREATE_CQ ioctl.
 * The NIC handle and the number of entries in the CQ are passed in.
 */
typedef struct  _VIPKA_CREATE_CQ_IN  {
	VIPKA_NIC_HANDLE	hNic;		/* Nic handle for Completion Queue */
	VIP_ULONG			entryCount;	/* Number of CQ entries requested, excluding marker entry */
#ifndef WINDOWS_NT
	VIP_PVOID			userAddr;	/* User address of CQ */
#endif
} VIPKA_CREATE_CQ_IN, *PVIPKA_CREATE_CQ_IN;

/*
 * VIPKA_CREATE_CQ_OUT captures the output parameters to the VIPKA_CREATE_CQ ioctl.
 * The status of the operation, handle of the newly created completion queue and 
 * a pointer to the completion queue header information is returned. The pCQHdr
 * points to a memory region allocated in KA and reversed mapped to user space.
 *
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC handle or requested size is too high.
 *		VIP_ERROR_RESOURCE
 */

typedef struct  _VIPKA_CREATE_CQ_OUT  {
	VIP_RETURN					status;		/* status/error code */
	VIPKA_CQ_HANDLE				hCQ;		/* handle to the newly created CQ */
    PVIPKA_CQ_USER_HDR          pCQHdr;  /* header information for the CQ */
} VIPKA_CREATE_CQ_OUT, *PVIPKA_CREATE_CQ_OUT;


/*
 * Ioctl for the VipDestroyCQ API. 
 */
#define  IOCTL_VIPKA_DESTROY_CQ	VIPKA_IOCTL( 26)

/*
 * VIPKA_DESTROY_CQ_IN captures the input parameters to the VIPKA_DESTROY_CQ ioctl.
 * The CQ handle is passed in.
 */
typedef struct  _VIPKA_DESTROY_CQ_IN  {
	VIPKA_NIC_HANDLE		hNic;	/* handle to the NIC instance */
	VIPKA_CQ_HANDLE			hCQ;	/* Handle of the Completion Queue */
} VIPKA_DESTROY_CQ_IN, *PVIPKA_DESTROY_CQ_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid CQ handle
 *		VIP_ERROR_RESOURCE	  : One or more VI instances are still associated with the CQ.
 */

/*
 * Ioctl for the VipResizeCQ API. 
 */
#define  IOCTL_VIPKA_RESIZE_CQ	VIPKA_IOCTL( 27)

/*
 * VIPKA_RESIZE_CQ_IN captures the input parameters to the VIPKA_RESIZE_CQ ioctl.
 * The CQ handle and the new length of the CQ are passed in.
 */
typedef struct  _VIPKA_RESIZE_CQ_IN  {
	VIPKA_NIC_HANDLE		hNic;			/* handle to the NIC instance */
	VIPKA_CQ_HANDLE			hCQ;			/* KA Handle of the Completion Queue */
	VIP_ULONG				entryCount;		/* new length for the CQ */
} VIPKA_RESIZE_CQ_IN, *PVIPKA_RESIZE_CQ_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid CQ handle or requested size is too high.
 *		VIP_ERROR_RESOURCE
 */


/*
 ********************************************************************************
 *************************Misc Functions*****************************************
 ********************************************************************************
 */

/*
 * Ioctl for the VipQueryNic API. 
 */
#define  IOCTL_VIPKA_QUERY_NIC		VIPKA_IOCTL( 28)

/*
 * VIPKA_QUERY_NIC_IN captures the input parameters to the VIPKA_QUERY_NIC ioctl.
 * The NIC handle to be queried is passed in.
 */
typedef struct  _VIPKA_QUERY_NIC_IN  {
	VIPKA_NIC_HANDLE			hNic;			/* Nic handle to be queried */
} VIPKA_QUERY_NIC_IN, *PVIPKA_QUERY_NIC_IN;

/*
 * VIPKA_QUERY_NIC_OUT captures the output parameters to the VIPKA_QUERY_NIC ioctl.
 * The attributes of the NIC and an error status are returned.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC handle.
 */

typedef struct  _VIPKA_QUERY_NIC_OUT  {
	VIP_RETURN					status;		/* status/error code */
	VIPKA_NIC_ATTRIBUTES		nicAttribs;	/* NIC attributes */
} VIPKA_QUERY_NIC_OUT, *PVIPKA_QUERY_NIC_OUT;


/*
 * Ioctl for the VipSetViAttributes API. 
 */
#define  IOCTL_VIPKA_SET_VI_ATTRIBUTES		VIPKA_IOCTL( 29)

/*
 * VIPKA_SET_VI_ATTRIBUTES_IN captures the input parameters to the VIPKA_SET_VI_ATTRIBUTES ioctl.
 * The VI handle whose attributes are to be set is passed in.
 */
typedef struct  _VIPKA_SET_VI_ATTRIBUTES_IN  {
	VIPKA_NIC_HANDLE		hNic;			/* handle to the NIC instance */
	VIPKA_VI_HANDLE			hVi;			/* VI whose attributes are to be set */
	VIP_VI_ATTRIBUTES		viAttribs;		/* New attributes for the VI */
} VIPKA_SET_VI_ATTRIBUTES_IN, *PVIPKA_SET_VI_ATTRIBUTES_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle.
 *		VIP_INVALID_RELIABILITY_LEVEL,
 *		VIP_INVALID_MTU,
 *		VIP_INVALID_QOS,
 *		VIP_INVALID_PTAG,
 *		VIP_INVALID_RDMAREAD.
 */


/*
 * Ioctl for the VipQueryVi API. 
 */
#define  IOCTL_VIPKA_QUERY_VI		VIPKA_IOCTL( 30)

/*
 * VIPKA_QUERY_VI_IN captures the input parameters to the VIPKA_QUERY_VI ioctl.
 * The VI handle to be queried is passed in.
 */
typedef struct  _VIPKA_QUERY_VI_IN  {
	VIPKA_NIC_HANDLE		hNic;			/* handle to the NIC instance */
	VIPKA_VI_HANDLE			hVi;			/* Handle of the VI to be queried */
} VIPKA_QUERY_VI_IN, *PVIPKA_QUERY_VI_IN;

/*
 * VIPKA_QUERY_VI_OUT captures the output parameters to the VIPKA_QUERY_VI ioctl.
 * The state, attributes of the VI and an error status are returned.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid VI handle.
 */

typedef struct  _VIPKA_QUERY_VI_OUT  {
	VIP_RETURN					status;			/* status/error code */
	VIP_VI_STATE				state;			/* current state of the VI */
	VIP_VI_ATTRIBUTES			viAttribs;		/* VI's attributes */
} VIPKA_QUERY_VI_OUT, *PVIPKA_QUERY_VI_OUT;


/*
 * Ioctl for the VipSetMemAttributes API. 
 */
#define  IOCTL_VIPKA_SET_MEM_ATTRIBUTES		VIPKA_IOCTL( 31)

/*
 * VIPKA_SET_MEM_ATTRIBUTES_IN captures the input parameters to the VIPKA_SET_MEM_ATTRIBUTES ioctl.
 * The NIC handle, address/memory handle of the region whose attributes are to be set and the 
 * new attributes are passed in.
 */
typedef struct  _VIPKA_SET_MEM_ATTRIBUTES_IN  {
	VIPKA_NIC_HANDLE			hNic;			/* NIC to which the memory region belongs */
	VIP_PVOID					pAddress;		/* base address of the memory region */
	VIP_MEM_HANDLE				hMemRegion;		/* Handle of the memory region */
	VIP_MEM_ATTRIBUTES		    memAttribs;	/* New attributes for the memory region */
} VIPKA_SET_MEM_ATTRIBUTES_IN, *PVIPKA_SET_MEM_ATTRIBUTES_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid Memory handle/address.
 *		VIP_INVALID_PTAG,
 *		VIP_INVALID_RDMAREAD.
 */

/*
 * Ioctl for the VipQueryMem API. 
 */
#define  IOCTL_VIPKA_QUERY_MEM		VIPKA_IOCTL( 32)

/*
 * VIPKA_QUERY_MEM_IN captures the input parameters to the VIPKA_QUERY_MEM ioctl.
 * The NIC handle, memory handle and the address of the region to be queried are passed in.
 */
typedef struct  _VIPKA_QUERY_MEM_IN  {
	VIPKA_NIC_HANDLE			hNic;			/* NIC Handle of the memory region */
	VIP_PVOID					pAddress;		/* Address of the memory region */
	VIP_MEM_HANDLE				hMemRegion;		/* Handle of the memory region */
} VIPKA_QUERY_MEM_IN, *PVIPKA_QUERY_MEM_IN;

/*
 * VIPKA_QUERY_MEM_OUT captures the output parameters to the VIPKA_QUERY_MEM ioctl.
 * The attributes of the region and an error status are returned.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid Address/memory handle.
 */

typedef struct  _VIPKA_QUERY_MEM_OUT  {
	VIP_RETURN					status;			/* status/error code */
	VIP_MEM_ATTRIBUTES			memAttribs;		/* Region's attributes */
} VIPKA_QUERY_MEM_OUT, *PVIPKA_QUERY_MEM_OUT;


/*
 * Ioctl for the VipQuerySystemManagementInfo API. 
 */
#define  IOCTL_VIPKA_QUERY_SYS_MGMT_INFO		VIPKA_IOCTL( 33)
/* TBD */

/*
 * Ioctl for the VipErrorCallback API. 
 */
#define  IOCTL_VIPKA_SET_ERROR_CALLBACK		VIPKA_IOCTL( 34)

/*
 * VIPKA_SET_ERROR_CALLBACK_IN captures the input parameters to the VIPKA_SET_ERROR_CALLBACK ioctl.
 * The NIC handle, user context and the error callback routine are passed in.
 */
typedef struct  _VIPKA_SET_ERROR_CALLBACK_IN  {
	VIPKA_NIC_HANDLE	hNic;				/* NIC to which the memory region belongs */
	VIP_PVOID			pContext;			/* user context to be passed back to the callback */
	void				(*pfnCallback)(IN VIP_PVOID, IN VIP_ERROR_DESCRIPTOR*);
} VIPKA_SET_ERROR_CALLBACK_IN, *PVIPKA_SET_ERROR_CALLBACK_IN;

/*
 * An error status is returned through VIPKA_IOCTL_STATUS_OUT.
 * Possible error codes :
 *		VIP_INVALID_PARAMETER : Invalid NIC handle.
 */


/*
 ********************************************************************************
 *************************N o n  A P I  I O C T L s *****************************
 ********************************************************************************
 */

/*
 * Ioctl to block for notify/callback routines.
 * When an asynchronous event happens and there is a notify/callback routine to be invoked,
 * this Ioctl will return with the kind of notify/callback,the routine to be called back and
 * the parameters to the called back routine.
 * Otherwise, the ioctl will block until a notify/callback routine is ready to be invoked.
 */
#define  IOCTL_VIPKA_BLOCK_FOR_CALLBACK		VIPKA_IOCTL( 35)

typedef struct _VIPKA_ERROR_CALLBACK_OUT {
	VIP_RETURN				status;			/* status/error code */
	VIP_PVOID				pContext;		/* user context to be passed back to the callback */
	void					(*pfnCallback)(IN VIP_PVOID, IN VIP_ERROR_DESCRIPTOR*);
	VIPKA_ERROR_DESCRIPTOR	errorDescr;
}VIPKA_ERROR_CALLBACK_OUT, *PVIPKA_ERROR_CALLBACK_OUT;

/*
 * VIPKA_CALLBACK_IN captures the input parameters to the VIPKA_CALLBACK ioctl.
 * The NIC handle (of the NIC which the User Agent is interested in) is passed in.
 */
typedef struct  _VIPKA_CALLBACK_IN  {
	VIPKA_NIC_HANDLE			hNic;		/* Handle of the NIC from which events are to be captured */
} VIPKA_CALLBACK_IN, *PVIPKA_CALLBACK_IN;



/*
 * Ioctl to allow the user to attach to the kernel trace buffer.
 */
#define	IOCTL_VIPKA_REGISTER_TRACE			VIPKA_IOCTL(36)

typedef void* VIPKA_EVENT_HANDLE;

/*
 * Input and output parameter blocks for this ioctl.
 */
typedef struct  _VIPKA_REGISTER_TRACE_IN  {
  VIPKA_EVENT_HANDLE hEvent;
} VIPKA_REGISTER_TRACE_IN, *PVIPKA_REGISTER_TRACE_IN;

typedef struct _VIPKA_REGISTER_TRACE_OUT {
  struct VIT_TRACE_INFO* pTraceInfo;
}VIPKA_REGISTER_TRACE_OUT, *PVIPKA_REGISTER_TRACE_OUT;

#ifdef ALLOW_SET_PATH
#define IOCTL_VIPKA_SET_PATH				VIPKA_IOCTL(37)

/*
 * Input block for this ioctl.  The purpose of the ioctl is to specify the initial path
 * to try to use to reach a given Servernet endpoint over a particular NIC.
 */
typedef struct _VIPKA_SET_PATH_IN {
	VIPKA_NIC_HANDLE			hNic;		/* The NIC that the node will be access through */
	VIP_ULONG					snid;		/* The endpoint to access. */
	VIP_ULONG					path;		/* The initial path to use.  0 = X, 1 = Y */

} VIPKA_SET_PATH_IN, *PVIPKA_SET_PATH_IN;
#endif	/* ALLOW_SET_PATH */

#ifndef WINDOWS_NT

/* 
 * Changes we need to support notify APIs.  Unix doesn't support
 * Asynchronous Procedure Calls (APCs), so the notify thread will
 * sleep in an ioctl, waiting for a completion event.
 */

#define  IOCTL_VIPKA_NOTIFY_WAIT		VIPKA_IOCTL( 38)

/*
 * VIPKA_NOTIFY_WAIT_IN captures the input parameters to the 
 * IOCTL_VIPKA_NOTIFY_WAIT ioctl.
 */
typedef struct  _VIPKA_NOTIFY_WAIT_IN  {
	VIPKA_NIC_HANDLE		hNic;
} VIPKA_NOTIFY_WAIT_IN, *PVIPKA_NOTIFY_WAIT_IN;

typedef struct  _VIPKA_NOTIFY_WAIT_OUT  {
	VIP_RETURN			status;		/* status/error code */
	VIP_PVOID			pContext;	/* context to run the notify handler */
} VIPKA_NOTIFY_WAIT_OUT, *PVIPKA_NOTIFY_WAIT_OUT;


#endif /* WINDOWS_NT */

#endif /* _VIP_KA_H */
