/*++

	Tandem Confidential: Need to Know Only

	Copyright (C) 1997, Tandem Computers Incorporated
	Protected as an unpublished work.
	All rights reserved.

	The computer program listings, specifications and documentation
	herein are the property of Tandem Computers Incorporated or a third
	party supplier and shall not be reproduced, copied, disclosed, or
	used in whole or in part for any reason without the prior express
	written permission of Tandem Computers Incorporated.

Project Name:

    VIA Emulator

File Name:

    viatype.h

Abstract:

    This file contains structure definitions defined in the VIA
    Spec that are NOT hardware aware. The clients, the User Agent, the Kernel 
    Agent all need access to these structures. This file is sourced in by 
    viadesc.h.

Author:

	VIA Emulator Team
    
Revision History:

	12/19/1997	ssc		initial creation
	12/29/1997 	ssc		changed the VIP_NET_ADDRESS to conform to VIA 1.0 spec
	12/29/1997	jrh		changed the VIP_NET_ADDRESS to have a convenient internal
						structure, while remaining compatible with the VIA 1.0
						spec.
	01/06/1997	jrh		Changed the types for the VIP_MEM_HANDLE and
						VIP_PROTECTION_HANDLE to correspond to the way that the
						kernel agent memory management code is actually using them,
						to make some compilation warnings go away.
  	02/08/1998 	ssc		Added VIP_PULONG typedef and a couple new error codes
						for VIP_RETURN.
	03/03/1998	sck		Added VIP_ADDR_ANY for auto-assigned port number and other
						port number default ranges.
	03/05/1998  ssc     Added new VIP_RETURN error codes from SanMan and 
						VIP_MAX_SERVICE_NAMELEN
	03/17/1998	jrh		Added VIP_REJECT alias for VIP_REJECTED error return code
						to work around ambiguity in the VIA 1.0 spec.
	04/03/1998	jrh		Converted to new descriptor format.
	04/09/98	ssc		Added a filler field to VIP_NET_ADDRESS. Reserve for future use.
	06/08/1998	sck		Handles discriminator up to 56 bytes although only
						the first 4 bytes are significant.
	08/13/1998	jrh		Made changes to conform to the 0.97 version of the Intel
						IG and to an agreement that we made with Giganet, so that
						IBM can have a common code module to interface to both
						vendor's VIA implementations.

--*/

#ifndef _VIATYPE_H
#define _VIATYPE_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef IN
#define IN
#endif

#ifndef OUT
#define OUT
#endif

/**********************************************************************
 * Data Types
 **********************************************************************/

/*
 * Obfuscations of basic ANSI C types.
 */
typedef unsigned short		VIP_UINT16;
typedef unsigned short		VIP_USHORT;
typedef unsigned long		VIP_UINT32;
typedef unsigned long		VIP_ULONG;
typedef void				VIP_VOID;
typedef void*				VIP_PVOID;
typedef char				VIP_CHAR;
typedef unsigned char		VIP_UINT8;
typedef unsigned long*		VIP_PULONG;

/*
 * Microsoft compiler-specific 64-bit value type.
 */
#ifdef WINDOWS_NT
typedef unsigned __int64	VIP_UINT64;
#else
typedef unsigned long long		VIP_UINT64;
#endif

/*
 * A VIP_BOOLEAN is a boolean value used in several places is the spec.
 */
typedef int					VIP_BOOLEAN;
#ifndef WINDOWS_NT /* NT probably needs these too */
#define VIP_FALSE			0
#define VIP_TRUE			1
#endif /* WINDOWS_NT */

/*
 * The reliability level is used in conveying NIC attributes.
 */
typedef VIP_USHORT			VIP_RELIABILITY_LEVEL;

/*
 * A workaround to try to make it easier for users whose compilers
 * only support 32-bit addresses to use this header file.
 */
typedef union {
	VIP_UINT64				AddressBits;
	VIP_PVOID				Address;
} VIP_PVOID64;

/*
 * The VIP_NIC_HANDLE is used in many of the interfaces to refer to
 * the kernel agent. The user obtains this handle from VipOpenNic().
 *
 * NOTE: this is an opaque data structure in VIPL. Kernel Agent please 
 *       do not use this.
 */
typedef void*				VIP_NIC_HANDLE;

/*
 * The VIP_VI_HANDLE is used in many of the interfaces to refer to a
 * particular VI.  The user obtains this handle from VipCreateVi().
 *
 * NOTE: this is an opaque data structure in VIPL. Kernel Agent please 
 *       do not use this.
 */
typedef void*				VIP_VI_HANDLE;

/*
 * The VIP_CQ_HANDLE is used by several of the interfaces to refer to a
 * particular completion queue.  The user obtains this handle from 
 * VipCreateCQ(). 
 *
 * NOTE: this is an opaque data structure in VIPL. Kernel Agent please 
 *       do not use this.
 */
typedef void*				VIP_CQ_HANDLE;

/*
 * The VIP_PROTECTION_HANDLE is an attribute associated both with
 * registered memory and with a VI.  VIPL obtains this handle from 
 * the kernel agent and return to user process untouched.
 */
typedef unsigned long		VIP_PROTECTION_HANDLE;

/*
 * The VIP_CONN_HANDLE is used to refer to pending connections. The 
 * user obtains this handle from VipConnectWait() and uses this 
 * handle to call VipConnectAccept().
 * 
 * NOTE: this is an opaque data structure in VIPL. Kernel Agent please 
 *       do not use this.
 */
typedef void*				VIP_CONN_HANDLE;

/* 
 * The VIP_QOS is one of the attributes of the VI.  Since it has no
 * definition in the spec at the moment, we just stub it out below.
 */
typedef void*				VIP_QOS;

/*
 * A VIP_MEM_HANDLE refers to a registered memory region (buffer).
 * The user obtains this handle from VipRegisterMem(). VIPL returns 
 * this handle to user as VIP_MEM_HANDLE.
 */
typedef unsigned long		VIP_MEM_HANDLE;

/*
 * VIP_RESOURCE_CODE describes the ResourceCode field of the 
 * VIP_ERROR_DESCRIPTOR structure.
 */
typedef enum _VIP_RESOURCE_CODE {
	VIP_RESOURCE_NIC,
	VIP_RESOURCE_VI,
	VIP_RESOURCE_CQ,
	VIP_RESOURCE_DESCRIPTOR
} VIP_RESOURCE_CODE;

/*
 * VIP_ERROR_CODE describes the ErrorCode field of the VIP_ERROR_DESCRIPTOR
 * structure.
 */
typedef enum _VIP_ERROR_CODE {
	VIP_ERROR_POST_DESC,
	VIP_ERROR_CONN_LOST,
	VIP_ERROR_RECVQ_EMPTY,
	VIP_ERROR_VI_OVERRUN,
	VIP_ERROR_RDMAW_PROT,
	VIP_ERROR_RDMAW_DATA,
	VIP_ERROR_RDMAW_ABORT,
	VIP_ERROR_RDMAR_PROT,
	VIP_ERROR_COMP_PROT
} VIP_ERROR_CODE;

/*
 * VIP_ERROR_DESCRIPTOR is a data structure passed by reference 
 * to the error handler that is registered via a call to VipErrorCallback().
 */
typedef struct {	
	VIP_NIC_HANDLE			NicHandle;
	VIP_VI_HANDLE			ViHandle;
	VIP_CQ_HANDLE			CqHandle;
	struct _VIP_DESCRIPTOR*	DescriptorPtr;
	VIP_ULONG				OpCode;
	VIP_RESOURCE_CODE		ResourceCode;
	VIP_ERROR_CODE			ErrorCode;
} VIP_ERROR_DESCRIPTOR, *PVIP_ERROR_DESCRIPTOR;

/*
 * VIP_NIC_ATTRIBUTES describes the attributes of the NIC.
 */
typedef struct {
	VIP_CHAR				Name[64];
	VIP_ULONG				HardwareVersion;
	VIP_ULONG				ProviderVersion;
	VIP_UINT16				NicAddressLen;
	const VIP_UINT8*		LocalNicAddress;
	VIP_BOOLEAN				ThreadSafe;
	VIP_UINT16				MaxDiscriminatorLen;
	VIP_ULONG				MaxRegisterBytes;
	VIP_ULONG				MaxRegisterRegions;
	VIP_ULONG				MaxRegisterBlockBytes;
	VIP_ULONG				MaxVI;
	VIP_ULONG				MaxDescriptorsPerQueue;
	VIP_ULONG				MaxSegmentsPerDesc;
	VIP_ULONG				MaxCQ;
	VIP_ULONG				MaxCQEntries;
	VIP_ULONG				MaxTransferSize;
	VIP_ULONG				NativeMTU;
	VIP_ULONG				MaxPtags;
	VIP_RELIABILITY_LEVEL	ReliabilityLevelSupport;
	VIP_RELIABILITY_LEVEL	RDMAReadSupport;
} VIP_NIC_ATTRIBUTES, *PVIP_NIC_ATTRIBUTES;

/*
 * VIP_VI_ATTRIBUTES is used to create and query connections.
 */
typedef struct {
    VIP_RELIABILITY_LEVEL	ReliabilityLevel;
	VIP_ULONG		MaxTransferSize;
	VIP_QOS			QoS;
	VIP_PROTECTION_HANDLE	Ptag;
	VIP_BOOLEAN		EnableRdmaWrite;
	VIP_BOOLEAN		EnableRdmaRead;
} VIP_VI_ATTRIBUTES, *PVIP_VI_ATTRIBUTES;

/*
 * VIP_MEM_ATTRIBUTES is used to register, query and alter
 * protections on memory regions.
 */
typedef struct {
	VIP_PROTECTION_HANDLE	Ptag;
	VIP_BOOLEAN				EnableRdmaWrite;
	VIP_BOOLEAN				EnableRdmaRead;
} VIP_MEM_ATTRIBUTES, *PVIP_MEM_ATTRIBUTES;

/*
 * VIP_VI_STATE describes the VI state. The user obtains this from 
 * VipQueryVi().
 */
typedef enum {
	VIP_STATE_IDLE, 
	VIP_STATE_CONNECTED,
	VIP_STATE_CONNECT_PENDING,
	VIP_STATE_ERROR
} VIP_VI_STATE;

/*
 * VIP_NET_ADDRESS is used to hold the network-specific address for
 * a VI endpoint.  The VI Provider specific address format is opaque 
 * to the user. The user is required to allocate storage for 
 * VIP_NET_ADDRESS according to the NicAddressLen value returned 
 * in VIP_NIC_ATTRIBUTES.
 *
 * When using VIP_NET_ADDRESS as input parameter to VIPL functions,
 * the user specifies, in the HostAddressLen field, the actual storage length
 * allocated for the VIP_NET_ADDRESS structure. This value is validated by
 * VIPL to make sure the VI Provider dependent address will fit into the 
 * structure provided by the user.
 */
typedef struct {
	VIP_UINT16	HostAddressLen;
	VIP_UINT16	DiscriminatorLen;
	VIP_UINT8	HostAddress[4];
	VIP_UINT8	HostDiscriminator[4];
	VIP_UINT8	filler[52];				/* Reserved */
} VIP_NET_ADDRESS, *PVIP_NET_ADDRESS;

/* 
 * VIP_RETURN describes return codes for Vip<xxx> functions as specified
 * in the VIA spec.
 *
 * There are error codes that are not specified in the VIA specification.
 * The VIA emulator added these return code to help clarify the error
 * conditions.
 *
 * Temporarily added VIP_NOT_IMPLEMENTED to be returned from functions 
 * that are not implemented (yet). This error code will be removed prior
 * to RQA date.
 */
typedef enum {
    VIP_SUCCESS,
	VIP_NOT_DONE,
	VIP_INVALID_PARAMETER,
    VIP_ERROR_RESOURCE,
	VIP_TIMEOUT,
	VIP_REJECTED,
	VIP_REJECT = VIP_REJECTED,
	VIP_INVALID_RELIABILITY_LEVEL,
	VIP_INVALID_MTU,
	VIP_INVALID_QOS,
	VIP_INVALID_PTAG,
	VIP_INVALID_RDMAREAD,
	VIP_DESCRIPTOR_ERROR,
	VIP_INVALID_STATE,
	VIP_ERROR_NAMESERVICE,
	VIP_NO_MATCH,
	VIP_NOT_REACHABLE,				   /* connection endpoint not reachable */
	VIP_INVALID_VI_HANDLE	= 1000,	   /* start VIE extended error code at 1000 */
	VIP_INVALID_VI_STATE,			   /* invalid VI state  */
	VIP_INVALID_CONNECTION_HANDLE,	   /* bad connection handle */
	VIP_VI_DISCONNECTED,			   /* the VI is disconnected */
	VIP_HOST_UNREACHABLE,			   /* remote host can't be reached */
	VIP_KA_INTERNAL_ERROR,			   /* Kernel Agent internal error */
	VIP_VIPL_INTERNAL_ERROR,		   /* VIPL internal error  */
    VIP_DEVICE_COMM_ERROR,			   /* IOCTL to Kernel Agent failed */
	VIP_ERROR_SANMAN_VERSION,		   /* SANMAN version mismatch */
	VIP_ERROR_SANMAN,				   /* SANMAN error */
	VIP_NOT_IMPLEMENTED				   /* function not implemented */
} VIP_RETURN;


/**********************************************************************
 * Definitions
 **********************************************************************/

/*
 * VIP_INFINITE is the value used to described a timer value. When used 
 * in the "Timeout" parameter to several of the VIPL routines, VIP_INFINITE
 * means to wait forever.
 */
#define VIP_INFINITE					0xffffffff
/*
 * VIP_ADDR_ANY is the value used to indicate the port of an address is to
 * be chosen by the system. Kernel agent assigns an unused port within the
 * range of 1024 through 5000.
 */
#define VIP_ADDR_ANY					0xffffffff
/*
 * Maximum service name used by Vipx functions for name space registration
 */
#define VIP_MAX_SERVICE_NAMELEN			32

/*
 * The various levels of reliability, used in conjunction with the 
 * VIP_RELIABILITY_LEVEL type.
 */
#define VIP_SERVICE_UNRELIABLE			0x0001
#define VIP_SERVICE_RELIABLE_DELIVERY	0x0002
#define VIP_SERVICE_RELIABLE_RECEPTION	0x0004

#ifdef __cplusplus
} /* End of C wrapper */
#endif

#endif	/* _VIATYPE_H */
