/*++

	Tandem Confidential: Need to Know Only

	Copyright (C) 1997, Tandem Computers Incorporated
	Protected as an unpublished work.
	All rights reserved.

	The computer program listings, specifications and documentation
	herein are the property of Tandem Computers Incorporated or a third
	party supplier and shall not be reproduced, copied, disclosed, or
	used in whole or in part for any reason without the prior express
	written permission of Tandem Computers Incorporated.

Project Name:

    VIA Emulator 

File Name:

    vipl.h

Abstract:

    This file contains structure definitions defined in the VIA
    Spec. Clients, the VIPL, and the Kernel Agent all need access
    to these structures.

Author:

	VIA Emulator Team	

Revision History:

	12/18/1997	ssc		Split old vipl.h into vipl.h and viadesc.h
	12/29/1997  ssc		added exported declaration for all functions
	01/01/1998  ssc		added Vipx functions
	03/20/1998	pmk		Added VipRegisterTrace
	06/08/1998  ssc		Added VipNSInit, VipNSShutdown, VipNSGetHostByName
	08/05/98	jrh		Added ioctl to allow clients to set the  path to be used
						to reach a Servernet endnode.
--*/

#ifndef _VIPL_H
#define _VIPL_H

#include "viadesc.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WINDOWS_NT
#define DLLEXPORT __declspec(dllexport)
#else
#define DLLEXPORT
#endif

DLLEXPORT
VIP_RETURN
VipOpenNic(
	IN const VIP_CHAR*			DeviceName,
	OUT VIP_NIC_HANDLE*			NicHandle);

DLLEXPORT
VIP_RETURN
VipCloseNic(
	IN VIP_NIC_HANDLE			NicHandle);

DLLEXPORT
VIP_RETURN
VipCreateVi(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_VI_ATTRIBUTES*		ViAttribs,
	IN VIP_CQ_HANDLE			SendCQHandle,
	IN VIP_CQ_HANDLE			RecvCQHandle,
    OUT VIP_VI_HANDLE*			ViHandle);

DLLEXPORT
VIP_RETURN
VipDestroyVi(
	IN VIP_VI_HANDLE			ViHandle);

DLLEXPORT
VIP_RETURN
VipConnectWait(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_NET_ADDRESS*			LocalAddr,
	IN VIP_ULONG				Timeout,
	OUT VIP_NET_ADDRESS*		RemoteAddr,
	OUT VIP_VI_ATTRIBUTES*		RemoteViAttribs,
	OUT VIP_CONN_HANDLE*		ConnHandle);

DLLEXPORT
VIP_RETURN
VipConnectAccept(
	IN VIP_CONN_HANDLE			ConnHandle,
	IN VIP_VI_HANDLE			ViHandle);

DLLEXPORT
VIP_RETURN
VipConnectReject(
	IN VIP_CONN_HANDLE			ConnHandle);

DLLEXPORT
VIP_RETURN
VipConnectRequest(
	IN VIP_VI_HANDLE			ViHandle,
	IN VIP_NET_ADDRESS*			LocalAddr,
	IN VIP_NET_ADDRESS*			RemoteAddr,
	IN VIP_ULONG				Timeout,
	OUT VIP_VI_ATTRIBUTES*		RemoteViAttribs);

DLLEXPORT
VIP_RETURN
VipDisconnect(
	IN VIP_VI_HANDLE			ViHandle);

DLLEXPORT
VIP_RETURN
VipCreatePtag(
	IN VIP_NIC_HANDLE			NicHandle,
	OUT VIP_PROTECTION_HANDLE*	ProtectionTag);

DLLEXPORT
VIP_RETURN
VipDestroyPtag(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_PROTECTION_HANDLE	ProtectionTag);

DLLEXPORT
VIP_RETURN
VipRegisterMem(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_PVOID				VirtualAddress, 
	IN VIP_ULONG				Length,
	IN VIP_MEM_ATTRIBUTES*		MemAttributes,
	OUT VIP_MEM_HANDLE*			MemoryHandle);

DLLEXPORT
VIP_RETURN
VipDeregisterMem(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_PVOID				VirtualAddress,
	IN VIP_MEM_HANDLE			MemoryHandle);

DLLEXPORT
VIP_RETURN
VipPostSend(
	IN VIP_VI_HANDLE			ViHandle,
	IN VIP_DESCRIPTOR*			DescriptorPtr,
	IN VIP_MEM_HANDLE			MemoryHandle);

DLLEXPORT
VIP_RETURN
VipSendDone(
	IN VIP_VI_HANDLE			ViHandle, 
	OUT VIP_DESCRIPTOR**		DescriptorPtr);

DLLEXPORT
VIP_RETURN
VipSendWait(
	IN VIP_VI_HANDLE			ViHandle,
	IN VIP_ULONG				TimeOut,
	OUT VIP_DESCRIPTOR**		DescriptorPtr);

DLLEXPORT
VIP_RETURN
VipPostRecv(
	IN VIP_VI_HANDLE			ViHandle, 
	IN VIP_DESCRIPTOR*			DescriptorPtr, 
	IN VIP_MEM_HANDLE			MemoryHandle);

DLLEXPORT
VIP_RETURN
VipRecvDone(
	IN VIP_VI_HANDLE			ViHandle, 
	OUT VIP_DESCRIPTOR**		DescriptorPtr);

DLLEXPORT
VIP_RETURN
VipRecvWait(
	IN VIP_VI_HANDLE			ViHandle,
	IN VIP_ULONG				TimeOut,
	OUT VIP_DESCRIPTOR**		DescriptorPtr);

DLLEXPORT
VIP_RETURN
VipCQDone(
	IN VIP_CQ_HANDLE			CQHandle,
	OUT VIP_VI_HANDLE*			ViHandle,
	OUT VIP_BOOLEAN*			RecvQueue);

DLLEXPORT
VIP_RETURN
VipCQWait(
	IN VIP_CQ_HANDLE			CQHandle,
	IN VIP_ULONG				TimeOut,
	OUT VIP_VI_HANDLE*			ViHandle,
	OUT VIP_BOOLEAN*			RecvQueue);

DLLEXPORT
VIP_RETURN
VipSendNotify(
	IN VIP_VI_HANDLE			ViHandle,
	IN VIP_PVOID				Context,
	IN void						(*Handler)(
		IN VIP_PVOID				Context,
		IN VIP_NIC_HANDLE			NicHandle,
		IN VIP_VI_HANDLE			ViHandle,
		IN VIP_DESCRIPTOR*			DescriptorPtr));

DLLEXPORT
VIP_RETURN
VipRecvNotify(
	IN VIP_VI_HANDLE			ViHandle,
	IN VIP_PVOID				Context,
	IN void						(*Handler)(
		IN VIP_PVOID				Context,
		IN VIP_NIC_HANDLE			NicHandle,
		IN VIP_VI_HANDLE			ViHandle,
		IN VIP_DESCRIPTOR*			DescriptorPtr));

DLLEXPORT
VIP_RETURN
VipCQNotify(
	IN VIP_CQ_HANDLE			CQHandle,
	IN VIP_PVOID				Context,
	IN void						(*Handler)(
		IN VIP_PVOID				Context,
		IN VIP_NIC_HANDLE			NicHandle,
		IN VIP_VI_HANDLE			ViHandle,
		IN VIP_BOOLEAN				RecvQueue));

DLLEXPORT
VIP_RETURN
VipCreateCQ(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_ULONG				EntryCount,
	OUT VIP_CQ_HANDLE*			CQHandle);

DLLEXPORT
VIP_RETURN
VipDestroyCQ(
	IN VIP_CQ_HANDLE			CQHandle);

DLLEXPORT
VIP_RETURN
VipResizeCQ(
	IN VIP_CQ_HANDLE			CQHandle,
	IN VIP_ULONG				EntryCount);

DLLEXPORT
VIP_RETURN
VipQueryNic(
	IN VIP_NIC_HANDLE			NicHandle,
	OUT VIP_NIC_ATTRIBUTES*		Attributes);

DLLEXPORT
VIP_RETURN
VipSetViAttributes(
	IN VIP_VI_HANDLE			ViHandle,
	IN VIP_VI_ATTRIBUTES*		Attributes);

DLLEXPORT
VIP_RETURN
VipQueryVi(
	IN VIP_VI_HANDLE			ViHandle, 
	OUT VIP_VI_STATE*			State, 
	OUT VIP_VI_ATTRIBUTES*		Attributes); 

DLLEXPORT
VIP_RETURN
VipSetMemAttributes(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_PVOID				Address,
	IN VIP_MEM_HANDLE			MemHandle,
	IN VIP_MEM_ATTRIBUTES*		MemAttribs);

DLLEXPORT
VIP_RETURN
VipQueryMem(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_PVOID				Address,
	IN VIP_MEM_HANDLE			MemHandle,
	OUT VIP_MEM_ATTRIBUTES*		MemAttribs);

DLLEXPORT
VIP_RETURN
VipQuerySystemManagementInfo(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_ULONG				InfoType,
	OUT VIP_PVOID*				SysManInfo);

DLLEXPORT
VIP_RETURN
VipErrorCallback(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_PVOID				Context,
	IN void						(*Handler)(
		IN VIP_PVOID				Context,
		IN VIP_ERROR_DESCRIPTOR*	ErrorDesc));

/* 
 * The following functions are extension to the VIA spec. These functions
 * are provided to help the users set up their Network addresses without 
 * internal knowledge of ServerNet addresses. Vipx functions provide a 
 * simple way to manage connection collision among different applications
 * on the same system.
 */

DLLEXPORT
VIP_RETURN
VipxAddService(
	IN	VIP_CHAR*	service,
	IN	VIP_ULONG	port);

DLLEXPORT
VIP_RETURN
VipxDeleteService(
	IN	VIP_CHAR*	service);

DLLEXPORT
VIP_RETURN
VipxGetServByName(
	IN  VIP_CHAR*	service,
	OUT VIP_ULONG*  port);

DLLEXPORT
VIP_RETURN
VipxCreateNetAddress(
	IN	VIP_CHAR*			nodeName,
	IN	VIP_CHAR*			service,
	OUT	VIP_NET_ADDRESS*	netAddr,
	IN OUT VIP_ULONG*		netAddrLen);

DLLEXPORT
VIP_RETURN
VipxCreateLocalAddress(
	IN		VIP_ULONG			port,
	OUT		VIP_NET_ADDRESS*	localAddr,
	IN OUT	VIP_ULONG*			localAddrLen);

DLLEXPORT
VIP_RETURN
VipxGetServByPort(IN		VIP_ULONG	port,
				  OUT		VIP_CHAR*	service,
				  IN OUT	VIP_ULONG*  serviceLen);

DLLEXPORT
VIP_RETURN
VipNSInit(IN	VIP_NIC_HANDLE	NicHandle,
		  IN	VIP_PVOID		NSInitInfo);

DLLEXPORT
VIP_RETURN
VipNSShutdown(IN	VIP_NIC_HANDLE NicHandle);

DLLEXPORT
VIP_RETURN
VipNSGetHostByName(IN	VIP_NIC_HANDLE	NicHandle,
				   IN	VIP_CHAR		*Name,
				   OUT	VIP_NET_ADDRESS *Address,
				   IN	VIP_ULONG		NameIndex);

/*
 * The following function is used to register a trace
 * buffer. This feature is not supported externally.
 */
#ifndef WINDOWS_NT
struct _VIP_NIC;
#endif
DLLEXPORT
VIP_RETURN
VipRegisterTrace(
	IN struct _VIP_NIC*			hNicHandle,
	IN VIP_UINT8				fNotifyProcess);

#ifdef ALLOW_SET_PATH
/*
 * The following function is used to set the default path to be used
 * to access a node with Servernet.
 */
DLLEXPORT
VIP_RETURN
VipxSetPath(
	IN VIP_NIC_HANDLE			NicHandle,
	IN VIP_ULONG				snid,
	IN VIP_ULONG				path);
#endif	/* ALLOW_SET_PATH */

#ifdef __cplusplus
} /* End of C wrapper */
#endif

#endif	/* _VIPL_H */
