/*
 * windows.h -- Unix version of windows.h
 */

#ifndef _WINDOWS_H
#define _WINDOWS_H

#include <errno.h>
#include <synch.h>
#include <sys/time.h>


#ifndef TRUE
#define TRUE	1
#endif

#ifndef FALSE
#define FALSE	0
#endif

#ifndef IN
#define IN
#endif

#ifndef OUT
#define OUT
#endif

#ifndef OPTIONAL
#define OPTIONAL
#endif

#define _cdecl
#define __cdecl
#define __stdcall
#define __fastcall

#define NTAPI
#define WINAPI

#define MAX_PATH		1024

#define DLL_PROCESS_ATTACH		(1)
#define DLL_PROCESS_DETACH		(2)
#define DLL_THREAD_ATTACH		(3)
#define DLL_THREAD_DETACH		(4)

#define GENERIC_READ			(0x1)
#define GENERIC_WRITE			(0x2)

#define CREATE_NEW				(1)
#define CREATE_ALWAYS			(2)
#define OPEN_EXISTING			(3)
#define OPEN_ALWAYS				(4)
#define TRUNCATE_EXISTING		(5)

/* CreationFlag values */
#define CREATE_SUSPENDED		(0x1)

#define FILE_BEGIN				(0)
#define FILE_CURRENT			(1)
#define FILE_END				(2)

/* CreatFile ShareMode flags */
#define FILE_SHARE_READ			(0x1)

#define FILE_ATTRIBUTE_NORMAL	(0x1)
#define FILE_FLAG_OVERLAPPED	(0x2)

#define OPEN_ALWAYS				(4)

/* VirtualAlloc Protect flag values */
#define PAGE_READONLY			(0x2)
#define PAGE_READWRITE			(0x4)

/* VirtualAlloc AllocationType values */
#define MEM_COMMIT				(0x1000)
#define MEM_RESERVE				(0x2000)

/* VirtualFree FreeType values */
#define MEM_DECOMMIT			(0x4000)

#define INFINITE				0xFFFFFFFF

#define MAX_COMPUTERNAME_LENGTH	(15)

/*
 * 64-bit types.
 */
typedef struct {
	unsigned int	lop;
	int				hop;
} __int64;

typedef struct {
	unsigned int	lop;
	unsigned int	hop;
} __uint64;

/*
 * Windows basic types
 */
typedef int				BOOL, boolean;
typedef unsigned char	BOOLEAN;
typedef char			CHAR, *PCHAR, *LPCTSTR, *LPTSTR;
typedef mutex_t			CRITICAL_SECTION;
typedef long			DWORD, *PDWORD, *LPDWORD;
typedef int				*HANDLE, *HINSTANCE;
typedef long			LONG, *PLONG, *LPLONG;
#if 0
typedef long			LONGLONG;
#else
typedef long long		LONGLONG;
#endif
typedef void			*OVERLAPPED, *LPOVERLAPPED;
typedef short			SHORT;
typedef unsigned char	UCHAR;
typedef unsigned int	UINT;
typedef unsigned int	ULONG, *PULONG;
typedef unsigned short	USHORT;
typedef void			VOID, *PVOID, *LPVOID, *LPSECURITY_ATTRIBUTES;
typedef void *			(*LPTHREAD_START_ROUTINE)(LPVOID lpThreadParameter);
typedef void *			(*PAPCFUNC)(DWORD dwParam);
typedef struct timeval	SYSTEMTIME, *PSYSTEMTIME;
#define wMilliseconds	tv_usec

/*
 * NT LARGE INTEGER
 */
typedef struct _LARGE_INTEGER {
	union {
		long	qp;
		struct {
			ulong lp;
			long up;
		} st;
	} un;
} LARGE_INTEGER, *PLARGE_INTEGER;
#define QuadPart un.qp
#define LowPart un.st.lp
#define HighPart un.st.hp

typedef struct _LIST_ENTRY {
	struct _LIST_ENTRY *	Flink;
	struct _LIST_ENTRY *	Blink;
} LIST_ENTRY, *PLIST_ENTRY;

#define INVALID_HANDLE_VALUE	((HANDLE)(-1))

/*
 * winerror.h stuff
 */
#define GetLastError()		(errno)
#define ERROR_IO_PENDING	EAGAIN

/* WaitForSingleObject error codes */
#define WAIT_OBJECT_0		(0)
#define WAIT_FAILED			(-1)

#define CONTAINING_RECORD(address, type, field) \
	((type *)((PCHAR)(address) - (PCHAR)(&((type *)0)->field)))

#define HeapAlloc(hHeap, flags, bytes) \
	(calloc((size_t)(bytes), (size_t)sizeof(char)))

#define HeapFree(hHeap, flags, pMem) \
	(free(pMem), TRUE)

/*
 * File primitives
 */
extern HANDLE
CreateFile(LPCTSTR, DWORD, DWORD, LPSECURITY_ATTRIBUTES, DWORD, DWORD, HANDLE);
/*
 * Thread primitives
 */
#define GetCurrentThreadId() \
	((DWORD)thr_self())

extern HANDLE
CreateThread(LPSECURITY_ATTRIBUTES, DWORD, LPTHREAD_START_ROUTINE, LPVOID, DWORD, LPDWORD);

/*
 * CriticalSection primitives
 */
#define InitializeCriticalSection(cs) \
	((void)mutex_init((cs), USYNC_THREAD, NULL))

#define DeleteCriticalSection(cs) \
	((void)mutex_destroy(cs))

#define EnterCriticalSection(cs) \
	((void)mutex_lock(cs))

#define LeaveCriticalSection(cs) \
	((void)mutex_unlock(cs))

/*
 * Miscellaneous primitives
 */
#define _ultoa(ul, str, base) \
	(sprintf((str), "%u", (ul)))

#define GetCurrentTime() (time((time_t *)NULL))

#endif /* _WINDOWS_H */
