#!/bin/sh
#				Filename: /etc/dbms_guard/raw_support.sh
 
###################################################################
#								  #
# Code for implementing Raw Device Failover for dbms_guard	  #
#								  #
#	/etc/cluster/nodedown/node.X will be links to this script,	  #
#	where X is each node that shares the shared SCSI disk.	  #
#								  #
###################################################################

ERRLOG="/etc/dbms_guard/error_log"
PARTEMP="/c.b.t.d.s."
RLINK_DIR="/dev/rlnk"
RAW_DIR="/dev/rdsk"
CONSOLE="/dev/console"
RLOCK=".raw_dbms_ok"

# Is there even a directory of rawfile links?
    if [ -d $RLINK_DIR ] && [ `ls -a $RLINK_DIR|wc -w` -gt 2 ]
    # i.e. [ it exists ] and [ is not empty ]
    then
	# Get File Attributes
	ATTRIB=`ls -l $RLINK_DIR|tail -1|sed "s/  */:/g"|cut -d: -f3,4`
	if [ $ATTRIB = root:sys ]
	then
	    echo $RLINK_DIR entries still owned by $ATTRIB|\
	    tee -a $ERRLOG $CONSOLE
	    exit 1
	fi

	# Perform redirection for all files in directory
	for p in $RLINK_DIR/*
	do
	    # Obtain name of raw file linked to existing $p file
	    lnk=`ls -o $p|cut -f2 -d">"`
	    if [ `echo $lnk|grep -c $RAW_DIR/` -lt 1 ]
	    then
		echo "$RLINK_DIR/$p is not a raw device link"|\
		tee -a $ERRLOG $CONSOLE
		exit 1
	    fi

	    # Verify naming convention
	    if [ `echo $lnk|grep -c $PARTEMP` -lt 1 ]
	    then
		echo "raw device filename $lnk is unconventional"|\
		tee -a $ERRLOG $CONSOLE
		exit 1
	    fi

	    # Restore file attributes to appropriate raw device slice
	    chown $ATTRIB $lnk
	    chmod g+rw $lnk
	done

	# Give the go-ahead to start up database server
	touch /tmp/$RLOCK
    fi
