#ident "@(#)$Header: /src/nsc_uw7/usr/src/nsc/uts/net/nsc_dnet/nsc_dnet.cf/Space.c,v 1.13 2000/03/06 06:08:30 zabby Exp $"

/*
 * Copyright (c) 1995 Locus Computing Corporation
 *        All Rights Reserved
 */

#include <config.h>
#include <sys/types.h>
#include <sys/nsc.h>
#include <sys/nsc_nodelist.h>
#include <net/nerd_tuple.h>
#include <net/if.h>
#include <netinet/in_f.h>

/*
 * Boolean indicating whether parallel servers are allowed by
 * default (subject to policy) or whether the SO_PARALLELSVR
 * flag must be specified.
 */
int dnet_psvr_default = DNET_PSVR_DEFAULT;

/*
 * Number of async daemon threads to be spawned to service
 * Distributed Networking async events.
 */
int dnet_async_daemons = DNET_ASYNC_DAEMONS;

/*
 * Tuneables for individual protocol hash sizes.
 */
int dnet_tcp_lhashsz = DNET_TCP_LHASHSZ;
int dnet_tcp_fhashsz = DNET_TCP_FHASHSZ;
int dnet_udp_lhashsz = DNET_UDP_LHASHSZ;
int dnet_udp_fhashsz = DNET_UDP_FHASHSZ;
int dnet_icmp_lhashsz = DNET_ICMP_LHASHSZ;
int dnet_icmp_fhashsz = DNET_ICMP_FHASHSZ;
int dnet_rawip_lhashsz = DNET_RAWIP_LHASHSZ;
int dnet_rawip_fhashsz = DNET_RAWIP_FHASHSZ;
int dnet_fsid_lhashsz = DNET_FSID_LHASHSZ;
int dnet_fsid_fhashsz = DNET_FSID_FHASHSZ;


/*
 * dnet_psvr_default_policy()
 *	This routine provides policy for the dnet_psvr_default
 *	tuneable option.  If this option is enabled, this routine
 *	is called to determine whether a bind on a particular
 *	port should be defaulted to SO_PARALLELSVR.
 */
int
dnet_psvr_default_policy(unsigned short port)
{
	/*
	 * Always allow web servers to run in parallel.
	 */
	if (port == 80)
		return 1;

	return 0;
}

/*
 * dnet_parallelsvr_policy()
 *	The distributed networking support provides a new SO_PARALLELSVR
 *	flag to allow multiple servers to bind to the same local address
 *	and port.  This routine provides policy decisions for choosing
 *	a parallel server to service an incomming connection request.
 */
clusternode_t
dnet_parallelsvr_policy(
	nsc_nodelist_t *nodelist,
	nsc_nlcookie_t *cookiep,
	clusternode_t if_node,
	clusternode_t from_node,
	nerd_tuple_t *tp)
{
	clusternode_t node;

	/*
	 * If if_node == CLUSTERNODE_INVAL, then we're dealing with a
	 * connection that was sent to a CVIP address.  In that
	 * case we fallthrough and select amongst any parallel
	 * servers.
	 *
	 * Otherwise, if if_node is a valid node and is not the
	 * same node as the interface the connection originally
	 * came in on:
	 *
	 * (1)	The interface node doesn't have a listener for
	 *	this connection, otherwise the connection
	 *	request would have been serviced there and
	 *	not forwarded to the NeRD.
	 *
	 * (2)  If a listener exists on the node the connection
	 *	originally came in on, we should favor that
	 *	listener.
	 */
	if ((if_node != CLUSTERNODE_INVAL) &&
	    (from_node != if_node) &&
	    NSC_NODELIST_TEST1(nodelist, from_node))
		return from_node;

	/*
	 * Select the destination node via round-robin.
	 */
	node = NSC_NODELIST_GET_NEXT(cookiep, nodelist);
	if (node == CLUSTERNODE_INVAL) {
		NSC_NLCOOKIE_INIT(cookiep);
		node = NSC_NODELIST_GET_NEXT(cookiep, nodelist);
	}

	return node;
}
