/*
 * Copyrighted as an unpublished work.
 * (c) Copyright 1987-1995 Computer Associates International, Inc.
 * All rights reserved.
 *
 * RESTRICTED RIGHTS
 *
 * These programs are supplied under a license.  They may be used,
 * disclosed, and/or copied only as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this restricted rights notice.  Use, copying, and/or disclosure
 * of the programs is strictly prohibited unless otherwise provided
 * in the license agreement.
 */
/*
 * Copyright (c) 1982-1995
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ident "@(#)unixsrc:usr/src/common/uts/net/inet/if.h /main/uw7_uk/4"

#ifndef __net_if_h
#define __net_if_h

#ifdef _KERNEL_HEADERS
#include <net/socket.h>		/* REQUIRED */
#include <util/ksynch.h>
#include <net/inet/if_types.h>
#include <net/inet/in_f.h>
#include <net/inet/in6_f.h>
#else /* !_KERNEL_HEADERS */
#include <sys/socket.h>		/* REQUIRED */
#include <sys/ksynch.h>
#include <net/if_types.h>
#include <netinet/in_f.h>
#include <netinet/in6_f.h>
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

#define LOOPM_ID	100	/* Module ID for Loopback stream */
#define ENETM_ID	101	/* Module ID for Ethernet stream */
#define SLIPM_ID	102	/* Module ID for SLIP stream */
#define IFX25M_ID	103	/* Module ID for IP/X.25 stream */
#ifdef NSC_DNET
#define CVIPM_ID	104	/* Module ID for NSC CVIP stream */
#define NSCGLUEM_ID	105	/* Module ID for NSC GLUE stream */
#define NSCRIOCTLM_ID	106	/* Module ID for Remote IOCTL stream */
#endif /* NSC_DNET */

/* flags for interface status (should be common among protos */

#define	IFF_UP		0x1	/* interface is up */
#define	IFF_BROADCAST	0x2	/* broadcast address valid */
#define	IFF_DEBUG	0x4	/* turn on debugging */
#define	IFF_LOOPBACK	0x8	/* is a loopback net */
#define	IFF_POINTOPOINT	0x10	/* interface is point-to-point link */
#define	IFF_POINTTOPOINT	IFF_POINTOPOINT
#define	IFF_WANTIOCTLS	0x20	/* interface wants to hear IP ioctls */
#define	IFF_RUNNING	0x40	/* resources allocated */
#define	IFF_NOARP	0x80	/* no address resolution protocol */
#define	IFF_LINK0	0x100	/* link-specific flag */
/* next one not supported now, but reserved: */
#define	IFF_ALLMULTI	0x200	/* receive all multicast packets */
#define	IFF_LINK1	0x400	/* link-specific flag */
#define IFF_ONEPACKET	0x800	/* one-packet mode */
#define	IFF_LINK2	0x1000	/* link-specific flag */
#define	IFF_SIMPLEX	0x2000	/* can't hear own transmissions */
#define	IFF_MULTICAST	0x4000	/* supports IP multicasting */
#define	IFF_ALLHOSTS	0x8000	/* interface is all hosts on network */
#define IFF_NO1323OPT	0x10000	/* don't offer/accept RFC 1323 options */
#if defined(NSC_DNET) || defined(NSC_ICS)
#define IFF_ICS		0x20000	/* interface used for ICS traffic */
#define IFF_REMOTE	0x40000	/* interface remote from marked node */
#define IFF_CVIP	0x80000	/* CVIP Driver */

/* flags set internally only: */
#define	IFF_CANTCHANGE	(IFF_BROADCAST | IFF_POINTOPOINT | IFF_RUNNING | \
	 IFF_SIMPLEX | IFF_MULTICAST | IFF_WANTIOCTLS | IFF_ALLHOSTS | \
	 IFF_ICS | IFF_REMOTE | IFF_CVIP)
#else  /* !NSC_DNET && !NSC_ICS */
/* flags set internally only: */
#define	IFF_CANTCHANGE	(IFF_BROADCAST | IFF_POINTOPOINT | IFF_RUNNING | \
	 IFF_SIMPLEX | IFF_MULTICAST | IFF_WANTIOCTLS | IFF_ALLHOSTS)
#endif /* !NSC_DNET && !NSC_ICS */

/*
 * Struct used for one-packet mode params in if ioctls
 */
struct onepacket {
	u_int	spsize;		/* short packet size */
	u_int	spthresh;	/* short packet threshold */
};

/*
 * Interface specific tuning information that TCP can use to its
 * advantage.
 */
struct ifperf {
	u_int		ip_recvspace;	/* Receive window to use */
	u_int		ip_sendspace;	/* Send window to use */
	u_int		ip_fullsize;	/* use full-size frames */
};

#define	IFA_ROUTE	RTF_UP	/* route installed */

/*
 * Interface request structure used for socket ioctl's.  All interface
 * ioctl's must have parameter definitions which begin with ifr_name.  The
 * remainder may be interface specific. 
 */

struct ifreq {
#define	IFNAMSIZ	16
	char            ifr_name[IFNAMSIZ];	/* if name, e.g. "en0" */
	union {
		struct sockaddr ifru_addr;
		struct sockaddr ifru_dstaddr;
		struct sockaddr ifru_broadaddr;
		int		ifru_int[2];
		caddr_t         ifru_data;
		char            ifru_enaddr[6];
		struct onepacket ifru_onepacket;
		struct ifperf   ifru_perf;
	}               ifr_ifru;
#define	ifr_addr	ifr_ifru.ifru_addr	/* address */
#define	ifr_dstaddr	ifr_ifru.ifru_dstaddr	/* other end of p-to-p link */
#define	ifr_broadaddr	ifr_ifru.ifru_broadaddr	/* broadcast address */
#define	ifr_flags	ifr_ifru.ifru_int[0]	/* flags */
#define	ifr_metric	ifr_ifru.ifru_int[1]	/* metric */
#define ifr_mtu		ifr_ifru.ifru_int[1]	/* mtu */
#define ifr_ifindex	ifr_ifru.ifru_int[1]	/* ifindex */
#define ifr_nif		ifr_ifru.ifru_int[1]	/* # of interfaces */
#define ifr_naddr	ifr_ifru.ifru_int[1]	/* # of addresses */
#define ifr_type	ifr_ifru.ifru_int[1]	/* type of interface */
#define ifr_debug	ifr_ifru.ifru_int[1]	/* debug level */
#define ifr_muxid	ifr_ifru.ifru_int[1]	/* multiplexor id */
#define	ifr_data	ifr_ifru.ifru_data	/* for use by interface */
#define ifr_enaddr	ifr_ifru.ifru_enaddr	/* ethernet address */
#define ifr_onepacket	ifr_ifru.ifru_onepacket	/* one-packet mode params */
#define ifr_perf	ifr_ifru.ifru_perf	/* performance tuning params */
};

struct ifaliasreq {
        char    ifra_name[IFNAMSIZ];            /* if name, e.g. "en0" */
        struct  sockaddr ifra_addr;
        struct  sockaddr ifra_broadaddr;
        struct  sockaddr ifra_mask;
};

/*
 * Structure used in SIOCGIFCONF request. Used to retrieve interface
 * configuration for machine (useful for programs which must know all
 * networks accessible). 
 */
struct ifconf {
	int             ifc_len;/* size of associated buffer */
	union {
		caddr_t         ifcu_buf;
		struct ifreq   *ifcu_req;
	} ifc_ifcu;
#define	ifc_buf	ifc_ifcu.ifcu_buf	/* buffer address */
#define	ifc_req	ifc_ifcu.ifcu_req	/* array of structures returned */
};

/*
 * The ifaddr structure contains information about one address of an
 * interface.  They are maintained by the different address families, are
 * allocated and attached when an address is set, and are linked together so
 * all addresses for an interface can be located. 
 */
typedef struct ifaddr {
	struct sockaddr	*ifa_addr;	/* address of interface */
	struct sockaddr *ifa_dstaddr;	/* destination address */
#define	ifa_broadaddr	ifa_dstaddr	
	struct sockaddr *ifa_netmask;	/* network mask */
	struct ifnet   *ifa_ifp;	/* back-pointer to interface */
	struct ifstats *ifa_ifs;	/* back-pointer to interface stats */
	struct ifaddr  *ifa_next;	/* next address for interface */
	int	(*ifa_rtrequest)();	/* check or clean routes (+ or -)'d */
	struct rtentry *ifa_rt;		/* for ROUTETOIF */
	u_int		ifa_flags;	/* mostly rt_flags for cloning */
	u_short		ifa_llinfolen;	/* extra to malloc for link info */
	u_short		ifa_size;	/* size of the full ifaddr (ie. this
					 * structure + following data */
#ifdef _KERNEL
	atomic_int_t   *ifa_refcnt;	/* reference count */
#else
        void   		*if_refcnt;	/* queue reference count (I_UNLINK) */
#endif
} ifaddr_t;

#if defined(_KERNEL) || defined(INKERNEL) || defined(_INKERNEL)
#define IFA_HOLD(ifa) ATOMIC_INT_INCR((ifa)->ifa_refcnt)
#define IFA_FREE(ifa) { \
	ATOMIC_INT_DECR((ifa)->ifa_refcnt); \
	if (ATOMIC_INT_READ((ifa)->ifa_refcnt) == 0) { \
		if_ifafree((ifa)); \
	} \
}
#endif

#ifndef M_DATA
#include <sys/stream.h>
#endif

/*
 * The interface structure, formerly known as ip_provider.
 */

typedef struct ifnet {
	char		if_name[IFNAMSIZ]; /* provider name (e.g., en0) */
	struct ifnet	*if_next;	/* next in the chain */
#ifdef _KERNEL
	queue_t		*if_qbot;	/* lower write queue */
#else
	void		*if_qbot;	/* lower write queue */
#endif
	int             if_lindex;	/* unique ID of lower stream */
	u_int		if_flags;	/* up/down, broadcast, etc. */
	u_int		if_metric;	/* routing metric (external only) */
	u_int		if_maxtu;	/* maximum transmission unit */
	u_int		if_mintu;	/* minimum transmission unit */
	struct ifaddr	*if_addrlist;	/* address chain maintained by if */
	/*
	 * The following stuff is used for interfaces that can't handle
	 * back-to-back packets combined with systems that don't send
	 * packets as big as the MSS.  These values are actually used
	 * by TCP, but are kept here since they are set by ifconfig.
	 */
	u_int		if_spsize;	/* short packet size to watch for */
	u_int		if_spthresh;	/* short packet threshold */
	/* should be a real timeval, but too many programs won't compile */
	struct if_timeval {
		u_long tv_sec;
		u_long tv_usec;
	} if_lastchange; 		/* time of status change for SNMP */
	struct ifstats	*if_ifs;	/* pointer to ifstats structure */
	/*
	 * ARP devices are cross-linked with IP via the SIOCSIFNAME ioctl.
	 * It is the responsibility of the ARP device to find the right
	 * ifnet and update if_arp and if_arpresolve.
	 */
	struct arp_pcb	*if_arp;	/* pointer to cross-linked arp dev */
	/*
	 * if_arpresolve is called to resolve a MAC level address if
	 * necessary.  If passed a message, it will return 1 if the message
	 * can be sent now, or 0 if it will be sent when the ARP reply
	 * comes in.
	 */
#ifdef _KERNEL
	int		(*if_arpresolve)
	     (struct ifnet *, mblk_t *, struct rtentry *);
#else
	void		(*if_arpresolve)();
#endif
	int		if_addrlen;	/* address length for this medium */
	int		if_type;	/* type of interface */
	int		(*if_ioctl)	/* process ioctls */
	     (struct ifnet *, u_long, caddr_t, int);
	u_short		if_noq;		/* queue on input or not */
	u_long		if_index;	/* SNMP index */
	u_short		if_dl_version;	/* DLPI version */
	u_short		if_debuglevel;	/* Debug level */
	struct ifperf	if_perf;	/* Performance information */
	ulong           if_unlink_clnt;	/* IP minor the I_UNLINK came from */
#ifdef _KERNEL
	atomic_int_t	*if_refcnt;	/* reference count */
	atomic_int_t	*if_unack_iocs; /* number of unacked ioctls pending */
	mblk_t		*if_unlink_ioc; /* pointer to deferred I_UNLINK */
	atomic_int_t	*if_qref;	/* queue reference count (I_UNLINK) */
#else /* !_KERNEL */
	void		*if_refcnt;	/* reference count */
	void		*if_unack_iocs; /* number of unacked ioctls pending */
	void		*if_unlink_ioc; /* pointer to deferred I_UNLINK */
	void		*if_qref;	/* queue reference count (I_UNLINK) */
#endif /* _KERNEL */
	int		if_igmp_type;   /* Mrouter version seen by
					 * this interface */
	int		if_igmp_time;   /* Timer used to determine
					 * if_ifmp_type */
} ifnet_t;

#ifdef NSC_DNET
/*
 * DNET needs an additoinal field to point at remote interface state.
 * The ifnet_t structure is visible to the user via ioctls.  To avoid
 * changing the user's view of ifnet_t, all ifnet_t's are allocated in
 * the kernel embedded in an rifnet_t structure.  Only DNET code uses
 * the additional data contained in the rifnet_t.
 */
typedef struct rifnet {
	ifnet_t		rif_ifnet;	 /* embedded ifnet_t */
	void		*rif_remotedata; /* ptr to data for remote interfaces */
} rifnet_t;
#endif /* NSC_DNET */

struct in_multi;

#if defined(_KERNEL) || defined(INKERNEL) || defined(_INKERNEL)

/* queue referencing */
#define IFP_QHOLD(ifp) ATOMIC_INT_INCR((ifp)->if_qref)
#define IFP_QFREE(ifp) ATOMIC_INT_DECR((ifp)->if_qref)

/* ifnet referencing */
#define IFP_HOLD(ifp) ATOMIC_INT_INCR((ifp)->if_refcnt)
#define IFP_FREE(ifp) { \
	ATOMIC_INT_DECR((ifp)->if_refcnt); \
	if (ATOMIC_INT_READ((ifp)->if_refcnt) == 0) { \
		if_ifpfree((ifp)); \
	} \
}

#define	IFP_THRESHOLD	1000000
#define IFP_QSPIN(ifp) { \
	u_long spinct = 0;	\
	while (ATOMIC_INT_READ((ifp)->if_qref) > 0) { \
		if ((++spinct % IFP_THRESHOLD) == 0) { \
			cmn_err(CE_WARN, "ip: spinning on iface %lx", (ifp)); \
		} \
	} \
}
#endif

/*
 * The ifstats structures are linked by each unit into a chain.  Note that
 * interfaces have to know their own name to play this game. 
 * ::TODO:: This should be part of ifnet, but for drivers that want
 * to use it but not drag in tons of IP-stuff, this is better.
 */

typedef struct ifstats {
	struct ifstats *ifs_next;	/* next if on chain */
	char           *ifs_name;	/* interface name */
	short           ifs_unit;	/* unit number */
	short           ifs_active;	/* non-zero if this if is running */
	struct ifaddr  *ifs_addrs;	/* list of addresses */
	short		ifs_mtu;/* Maximum transmission unit */

	/* generic interface statistics */
	int		ifs_ipackets;	/* packets received on interface */
	int		ifs_ierrors;	/* input errors on interface */
	int		ifs_opackets;	/* packets sent on interface */
	int             ifs_oerrors;	/* output errors on interface */
	int		ifs_collisions;	/* collisions on csma interfaces */
	/* end statistics */
	/* MIB-II */
	struct ifs_mib {
		int		if_type;
		int		if_speed;
		struct ifs_ins {
			unsigned long	if_inoctets;
			unsigned long	if_inucastpkts;
			unsigned long	if_innucastpkts;
			unsigned long	if_indiscards;
			unsigned long	if_inunkprotos;
		} ifs_ins;
		struct ifs_outs {
			unsigned long	if_outoctets;
			unsigned long	if_outucastpkts;
			unsigned long	if_outnucastpkts;
			unsigned long	if_outdiscards;
		} ifs_outs;
	} ifs_mib;

#	define iftype		ifs_mib.if_type
#	define ifspeed		ifs_mib.if_speed
#	define ifinoctets	ifs_mib.ifs_ins.if_inoctets
#	define ifinucastpkts	ifs_mib.ifs_ins.if_inucastpkts
#	define ifinnucastpkts	ifs_mib.ifs_ins.if_innucastpkts
#	define ifindiscards	ifs_mib.ifs_ins.if_indiscards
#	define ifinunkprotos	ifs_mib.ifs_ins.if_inunkprotos
#	define ifoutoctets	ifs_mib.ifs_outs.if_outoctets
#	define ifoutucastpkts	ifs_mib.ifs_outs.if_outucastpkts
#	define ifoutnucastpkts	ifs_mib.ifs_outs.if_outnucastpkts
#	define ifoutdiscards	ifs_mib.ifs_outs.if_outdiscards
	/* MIB-II */
} ifstats_t;

#if defined(_KERNEL) || defined(INKERNEL) || defined(_INKERNEL)
extern struct ifstats	*ifstats;		/* COMPATABILITY */
extern struct ifnet	*ifnet;

#if defined(__STDC__)
struct sockaddr_in;
struct in_ifaddr;
#endif

ifaddr_t *ifa_ifwithaddr(struct sockaddr_in *);
ifaddr_t *ifa_ifwithdstaddr(struct sockaddr_in *);
#ifdef NSC_DNET
ifaddr_t *ifa_ifwithaddrx(struct sockaddr_in *, int);
ifaddr_t *ifa_ifwithdstaddrx(struct sockaddr_in *, int);
#endif
ifaddr_t *ifa_ifwithnet(struct sockaddr_in *);
ifaddr_t *ifaof_ifpforaddr(struct sockaddr_in *, struct ifnet *);
#ifdef NSC_ICS
ifaddr_t *ifa_ificsaddr(void);
#endif /* NSC_ICS */
#ifdef NSC_DNET
int if_attach(ifnet_t **, int);
#else
int if_attach(ifnet_t **);
#endif
void if_detach(ifnet_t *);
void if_freeaddr(struct ifaddr *);
int ifconf(queue_t *, mblk_t *);
void if_get_data(queue_t *, mblk_t *);
ifaddr_t *if_primary_address(ifnet_t *, int);
struct in_ifaddr *ifptoia(ifnet_t *);
ifnet_t *if_find_if_by_index(u_long);
void if_ifafree(ifaddr_t *);
void if_ifpfree(ifnet_t *);
void if_process_multi_reqs(void);
#endif	/* KERNEL */

/* mgmt. ioctls and related structures */

#define IF_MAXADDRS	10

struct all_addrs {
	struct sockaddr	addr;
	struct sockaddr	dstaddr;
	struct sockaddr	netmask;
	u_short		flags;
	u_long		multi[IF_MAXADDRS];/* ::TODO:: should be in_addr */
};

struct ifreq_all {
#define if_number if_entry.if_index
	ifnet_t	if_entry;	/* if_index = -1 => get # of interfaces */
				/*  = 0 => get interface from if_lindex */
				/*  = other  => get req. interface data */
	int	if_statsavail;	/* indicates if stats are avail. */
	struct	ifstats	if_stats;
	int	if_outqlen;	/* # of packets in the out queue */
	int	if_naddr;	/* indicates # of addrs returned */
	struct	all_addrs addrs[IF_MAXADDRS];
};

/*
 * Message format for use in obtaining information about interfaces
 * from the routing stream
 */
struct if_msghdr {
	u_short	ifm_msglen;	/* to skip over non-understood messages */
	u_char	ifm_version;	/* future binary compatability */
	u_char	ifm_type;	/* message type */
	int	ifm_addrs;	/* like rtm_addrs */
	u_int	ifm_flags;	/* value of if_flags */
	u_long	ifm_index;	/* index for associated ifp */
	struct	ifm_data {	/* statistics and other data about if */
		int	ifi_mtu;	/* Maximum transmission unit */
		int	ifi_metric;	/* routing metric (external) */
		int	ifi_ipackets;	/* packets received on interface */
		int	ifi_ierrors;	/* input errors on interface */
		int	ifi_opackets;	/* packets sent on interface */
		int	ifi_oerrors;	/* output errors on interface */
		int	ifi_collisions;	/* collisions on csma interfaces */
	}	ifm_data;
};

/*
 * Message format for use in obtaining information about interface addresses
 * from the routing stream
 */
struct ifa_msghdr {
	u_short	ifam_msglen;	/* to skip over non-understood messages */
	u_char	ifam_version;	/* future binary compatability */
	u_char	ifam_type;	/* message type */
	int	ifam_addrs;	/* like rtm_addrs */
	u_int	ifam_flags;	/* value of ifa_flags */
	u_long	ifam_index;	/* index for associated ifp */
	int	ifam_metric;	/* value of ifa_metric */
};

/*
 * For conversion to and from interface indexes to names.
 */
struct if_nameindex {
	unsigned int	if_index;	/* unique identifier > 0 */
	char *		if_name;	/* null terminated name (eg. "le0") */
};

/*
 * Structure used to select the interface 
 */
struct if_select {
	struct	in_addr notused;/* for compatibility with old IP_BROADCAST_IF */
	int	flags;		/* type of matching performed */
	union { 
		char name[IFNAMSIZ];
		struct in_addr	ip;
		struct in6_addr	ipv6;
	} matching;
};

#define IF_SELECTED_BY_NOTHING	0
#define IF_SELECTED_BY_NAME	1
#define IF_SELECTED_BY_IP	11
/* #define IF_SELECTED_BY_IPV6	12 */

#ifdef __STDC__

extern unsigned int		if_nametoindex(const char *);
extern char *			if_indextoname(unsigned int, char *);
extern struct if_nameindex *	if_nameindex(void);
extern void			if_freenameindex(struct if_nameindex *);

#else /* __STDC__ */

extern unsigned int		if_nametoindex();
extern char *			if_indextoname();
extern struct if_nameindex *	if_nameindex();
extern void			if_freenameindex();

#endif /* __STDC__ */

#endif	/* __net_if_h */
