/*
 * lcuxprt.h
 *
 *	This file contains LCU Transport related data structures.
 */

/*
 *      Copyright (c) Tandem Computers Incorporated 1995.
 *
 *      The computer program listings and specification herein are the
 *      property of Tandem Computers Incorporated and are not to be
 *      reproduced or copied in whole or in part for any reason without
 *      written permission of Tandem.
 */

#ifndef	_LCUXPRT_H_
#define	_LCUXPRT_H_

#ident "@(#)$Header: /src/nsc_uw7/usr/src/nsc/uts/net/nsc_lts/lcuxprt.h,v 1.4 1999/02/10 20:18:15 jbyrne Exp $"

/*
 * Miscellaneous typedefs
 */

typedef short sysnum_t;
typedef short nodenum_t;
typedef unsigned short port_t;
typedef struct lcumsg lcumsg_t;

#if	!(defined(_KERNEL) ||  defined(_SYS_TYPES_H))
#ifndef _UINT
#define _UINT
typedef unsigned int uint;
#endif
#ifndef _ULONG
#define _ULONG
typedef unsigned long ulong;
#endif
#ifndef _USHORT
#define _USHORT
typedef unsigned short ushort;
#endif
#endif	/* !(defined(_KERNEL) ||  defined(_SYS_TYPES_H))*/

#ifndef TRUE
#define	TRUE	1
#endif

#ifndef FALSE
#define	FALSE	0
#endif


/*
 * Various resource limits
 */

#define	LCU_UNACKSZ_MAX		56	/* max size of unack datagram */
#define	LCU_CTRLSZ_MAX		880	/* max size of control buffer */
#define	LCU_DATASZ_MAX		0x3FFFF	/* max size of data buffer */
#define LCU_INLINE_MAX		1024	/* max size of inline data */
#define	LCU_SGCNT_MAX		16	/* max #scatter/gather entries */
#define	LCU_PORTS_MAX		65536 	/* max number of ports within a node */
#define	LCU_NODES_MAX		255	/* max node number within a system */
#define	LCU_SYS_MAX		256	/* max number of systems */
#define	LCU_NODEUP_TIMEOUT	30	/* timeout (secs) to mark a node UP */

#define	LCU_NULL_SYS		(sysnum_t)-1
#define	LCU_NULL_NODE		(nodenum_t)-1

/*
 * The following error codes can be returned by the LCU transport.
 */

#define	ELCU_OK			0
#define	ELCU_AGAIN		1
#define	ELCU_INVAL		2
#define	ELCU_PORT_DOWN		3
#define	ELCU_NODE_DOWN		4
#define	ELCU_BAD_VADDR		5
#define	ELCU_CALLBACK		6
#define	ELCU_PATH_DOWN		7
#define	ELCU_BUSY		8
#define ELCU_SERVER_REJECT	9
#define ELCU_CANCELED		10


/*
 * sgent_t data structure
 *
 *	The sgent_t data structure defines each entry of the scatter/gather
 *	array.
 */

typedef struct {
	char	*sg_ioaddr;		/* reserved for LCU transport */
	char	*sg_buf;		/* pointer to a buffer */
	uint	sg_buflen;		/* buffer length  */
} sgent_t;


/*
 * lcumbuf_t data structure
 *
 *	The lcumbuf_t data structure is used to define a request
 *	and reply message.  A message consists of a control section
 *	(up to LCU_CTRLSZ_MAX) and an optional data secton with scatter
 *	/gather capability.
 */


typedef struct {
	ushort	lcu_sgcnt;	/* #scatter/gather entries */
	ushort	lcu_ctrllen;	/* length of the control portion of a msg */
	char	*lcu_ctrlbuf;	/* ptr to the control portion of a message */
	uint	lcu_datalen;	/* length of the data portion of a message */
	union {
	  char	*databuf;	/* lcu_sgcnt = 0 --> message data buffer addr */
	  sgent_t *sgaddr;	/* lcu_sgcnt != 0 --> SG array addr */
	} _buf;
} lcumbuf_t;

#define	lcu_databuf	_buf.databuf
#define	lcu_sgaddr	_buf.sgaddr


/*
 * lcumsg data structure
 *
 *	Each message, client or server, is represented by a lcumsg 
 *	data structure.  The lcu_sysnum and lcu_node fields are used
 *	to describe the recepient or the originator of this message.
 *	The lcu_port field contains the port to which this message
 *	is delivered.  The lcu_reqmbuf field contains the size and 
 *	location of the control and data portions of the message.
 *
 *	Note that this structure is allocated and deallcoated by the
 *	LCU transport via lcuxprt_msg_alloc and lcuxprt_msg_free 
 *	routines respectively.
 */

struct lcumsg {
	void	*lcu_reserved;		/* reserved space for LCU transport */
	void	*lcu_tag;		/* reserved space for client/server */
	sysnum_t lcu_sysnum;		/* remote system number */
	nodenum_t lcu_node;		/* remote node number */
	port_t	 lcu_port;		/* target port */
	char	lcu_msgtype;		/* message type: LCU_UNACKMSG or 
					 *  LCU_ACKMSG */
	char	lcu_flags;		/* message delivery flags */
	lcumbuf_t lcu_reqmbuf;		/* message request buffer */
};

/*
 * The following message types are supported (lcu_msgtype):
 */

#define	LCU_UNACKMSG	1		/* unacknowledged message */
#define	LCU_ACKMSG	2		/* acknowledged message */


/*
 * Message delivery flags (lcu_flags):
 */

#define	LCUMSG_CRITICAL		0x01	/* critical message */
#define	LCUMSG_ORD_DELIVERY	0x02	/* ordered delivery */
#define	LCUMSG_REQREP		0x04	/* request/reply message */

/*
 * lcuopt_t data structure (used when registering a port)
 */

#define	LCUOPT_MAX	8		/* max number of options */
typedef struct {
	int	lcu_optcnt;		/* # LCU options */
	struct {
		int	opt_cmd;		/* option command */
		void	*opt_arg;		/* option argument */
	} lcu_options[LCUOPT_MAX];
} lcuopt_t;

/*
 * Server listen options (opt_cmd):
 */

#define	LCUOPT_INLINEDATA	0x01
#define LCUOPT_ORD_DELIVERY	0x02
#define	LCUOPT_FLOW_CONTROL	0x04


/* 
 *	The following commands & data structures are used by the 
 *	lcuxprt_config routine.
 */

#define	LCU_SET_NODE_STATE	1
#define	LCU_GET_NODE_STATE	2
#define	LCU_SET_NODEINFO	3
#define	LCU_DEL_NODEINFO	4
#define	LCU_GET_NODEINFO	5
#define	LCU_GET_MYNODENUM	6
#define	LCU_GET_MYSYSNUM	7
#define	LCU_SET_SYSINFO		8
#define	LCU_GET_SYSINFO		9
#define	LCU_DEL_SYSINFO		10

/* 
 * LCU node state structure declaration (lcunodestate_t)
 */
typedef struct {
	sysnum_t lcu_sysnum;		/* system number */
	nodenum_t lcu_node;		/* node number */
	long	lcu_nodestate;		/*node state: 0 --> down, 1--> UP */
} lcunodestate_t;

/*
 * LCU node address structure declaration (lcuaddr_t)
 */
typedef struct {
	short	lcu_addr_family;	/* address family (defined below) */
	ushort	lcu_addr_length;	/* length of address */
	char	lcu_addr_data[28];	/* up to 28 bytes of direct address */
} lcuaddr_t;

/*
 * LCU address family:
 */
#define LCU_AF_SERVERNET	1	/* Servernet address (20-bits) */
#define LCU_AF_TNET		LCU_AF_SERVERNET	/* Old name support. */
#define	LCU_AF_INET		2	/* Internetwork address (TCP/IP, ...) */


/* 
 * LCU node information structure declaration (lcunodeinfo_t)
 */
#define	LCUNODENAMESZ	16
typedef struct {
	sysnum_t lcu_sysnum;	/* system number */
	nodenum_t lcu_node;	/* node number */
	unsigned reserved:31,	/* Other flags:  reserved for future use */
		 smp_node:1;	/* remote node is an SMP node */
	ushort	lcu_maxaddr;	/* size (max #entries) in lcu_addr array */
	ushort	lcu_naddr;	/* number of valid entries in lcu_addr array */
	lcuaddr_t *lcu_addr;	/* pointer to an array of LCU node addresses */
	char	lcu_nodename[LCUNODENAMESZ];
				/* node name (NULL terminated ASCII string) */
} lcunodeinfo_t;


/* 
 * LCU system information structure declaration (lcusysinfo_t)
 */
#define	LCUSYSNAMESZ	16
typedef struct {
	sysnum_t lcu_sysnum;	  /* system number */
	nodenum_t lcu_maxnodenum; /* max node number within this system */
	char	lcu_sysname[LCUSYSNAMESZ];
				  /* system name (NULL terminated string) */
} lcusysinfo_t;


/* 
 *	The following commands and data structures are used by the 
 *	lcuxprt_event() interface.
 */
#define	LCU_CATCH_EVENTS	1
#define	LCU_IGNORE_EVENTS	2

/* 
 *	The following events are currently supported:
 */
#define	LCUEV_SET_SYSINFO	0x01
#define	LCUEV_DEL_SYSINFO	0x02
#define	LCUEV_SET_NODEINFO	0x04
#define	LCUEV_DEL_NODEINFO	0x08
#define	LCUEV_NODE_UP		0x10
#define	LCUEV_NODE_DOWN		0x20
#define	LCUEV_NODE_REACHABLE	0x40
#define	LCUEV_NODE_UNREACHABLE	0x80
#define LCUEV_ALL 			0xff

/*
 *	The following defines the number of words in the bitmasks
 *	for the configuration arrays returned by the events.
 */
#define	LCU_WORDS_NODE		((LCU_NODES_MAX+1)/NBITPW)	/* words in node mask */
#define	LCU_WORDS_SYS		(LCU_SYS_MAX/NBITPW)		/* words in system mask */

/*
 * Message allocation flags:
 */

#define	LCUF_NOSLEEP		0x01	/* don't block */
#define	LCUF_SENDMSG		0x02	/* used for sending a message */
#define	LCUF_RECVMSG		0x04	/* used for receiving a message */

/* 
 * Exported routines which are called by the clients of the LCU transport.
 */

lcumsg_t *
lcuxprt_msg_alloc(	/* allocate a message structure */
    int		msgtype,		/* message type */
    int		flags			/* message allocate flags */
);


int
lcuxprt_msg_free(	/* deallocate a message structure */
    lcumsg_t	*lcumsgp		/* pointer to a message structure */
);


int 
lcuxprt_msg_send(	/* send a message */
    lcumsg_t	*lcumsgp,		/* pointer to a message */
    lcumbuf_t	*repmbufp,		/* pointer to reply buffers */
    void	(*msgsend_done)(	/* addr of a callback routine */
		    lcumsg_t	*lcumsgp,
		    int		status),
    int		inline_datasz		/* amount of data to be delivered */
);


int 
lcuxprt_listen(		/* listen for incoming messages */
    port_t	port,			/* port */
    int		(*serv_listen)( 	/* points to a callback routine */
		    void	*listen_callarg,
		    lcumsg_t	*lcumsgp,
		    uint	moredata),
    void	*listen_callarg,	/* callback argument */
    lcuopt_t	*options		/* options */
);


int 
lcuxprt_msg_recv(	/* save an incoming message for deferred processing */
    lcumsg_t	*lcumsgp,		/* incoming message pointer */
    lcumsg_t	*recvmsgp		/* buffer to receive incoming message */
);

int 
lcuxprt_unlisten(	/* deregister a port */
    port_t	port			/* port being disconnected */
);


int 
lcuxprt_msg_getdata(	/* get message data */
    lcumsg_t	*lcumsgp,		/* LCU message pointer */
    ulong_t	dataoff,		/* starting data off within the msg */
    uint	datalen,		/* #bytes to be picked up */
    lcumbuf_t	*databuf,		/* ptr to a scatter/gather data buf */
    void	(*getdata_done)(	/* address of a callback routine */
		    lcumsg_t	*lcumsgp,
		    int		status)
);

int 
lcuxprt_msg_done(	/* indicate message completion */
    lcumsg_t	*lcumsgp,		/* LCU message pointer */
    lcumbuf_t	*repmbuf,		/* pointer to a reply buffer */
    void	(*msgdone_callback)(	/* address of a callback routine */
		    lcumsg_t	*lcumsgp,
		    int		status)
);

int 
lcuxprt_config(		/* Set/change transport configuration */
    int		cmd,			/* configuration command */
    void	*config_arg		/* config command specific argument */
);


int 
lcuxprt_events(		/* catch/ignore transport events */
    int		cmd,			/* events monitoring command */
    ulong	events,			/* bit-mask, one bit for each event */
    sysnum_t	sysnum,			/* system being monitored */
    void	(*event_callback)(	/* address of a callback routine */
		    ulong	event,
		    sysnum_t	sysnum,
		    nodenum_t	node,
		    uint_t*	configp)
);

void
lcuxprt_init(		/* transport initialization routine */
    lcusysinfo_t	*lcusysinfop,	/* my system specific information */
    lcunodeinfo_t	*lcunodeinfop	/* my node specific information */
);

int
lcuxprt_connect(	/* utility routine for connecting to remote node */
	sysnum_t	sys,		/* system to connect to */
	nodenum_t	node,		/* node to connect to */
	int		timeout		/* how long to try for in seconds */
);

int
lcuxprt_accept(		/* utility routine to indicate willingness to connect */
	sysnum_t	sys,		/* system to connect with */
	nodenum_t	node		/* node to connect with */
);

int
lcuxprt_msg_cancel(	/* cancel an outstanding acknowledged message */
	lcumsg_t	*msg,		/* message to cancel */
	void		(*canceldone_callback)(	/* addr of callback routine */
			    lcumsg_t	*msg,
			    int		status,
			    void	*param),
	void		*param		/* Parameter for callback routine */
);

#endif	/* _LCUXPRT_H_ */
