#ident "@(#)$Header: /src/nsc_uw7/usr/src/nsc/uts/net/nsc_dnet/nerd/nerd_tuple.h,v 1.10 2000/03/06 06:08:25 zabby Exp $"

/*
 * Copyright (c) 1995 Locus Computing Corporation
 *        All Rights Reserved
 */

#ifndef NSC_NERD_NERD_TUPLE_H
#define NSC_NERD_NERD_TUPLE_H

#ifdef _KERNEL_HEADERS
#include <nsc/nsc_synch.h>
#else
#include <sys/nsc_synch.h>
#endif

#define NERD_TUPLE_MAXADDRLEN	4	/* Maximum bytes for an address */

typedef struct {
	char			na_addr[NERD_TUPLE_MAXADDRLEN];
	int			na_addrlen;
} nerd_addr_t;

#define NERD_ADDR_COPYTO(_naddrp, _paddrp, _paddrlen)			\
{									\
	ASSERT(_paddrlen <= NERD_TUPLE_MAXADDRLEN);			\
	bcopy((caddr_t)_paddrp, (caddr_t)(_naddrp)->na_addr, _paddrlen);\
	(_naddrp)->na_addrlen = _paddrlen;				\
}

#define NERD_ADDR_COPYFROM(_paddrp, _naddrp)				\
	bcopy((caddr_t)(_naddrp)->na_addr, (caddr_t)_paddrp,		\
	      (_naddrp)->na_addrlen)

#define NERD_ADDR_COMP(_addr1, _addr2)					\
	bcmp((caddr_t)(_addr1)->na_addr, (caddr_t)(_addr2)->na_addr,	\
	     (_addr1)->na_addrlen)

#define NERD_ADDR_PTR(_addr)						\
	(caddr_t)(_addr)->na_addr


typedef unsigned int nerd_port_t;

typedef struct nerd_tuple {
	nerd_addr_t		nt_laddr;
	nerd_port_t		nt_lport;
	nerd_addr_t		nt_faddr;
	nerd_port_t		nt_fport;
	clusternode_t		nt_node;
	struct nerd_tuple	*nt_prev;
	struct nerd_tuple	*nt_next;
} nerd_tuple_t;

typedef struct nerd_tuple_bucket {
	int			ntb_count;
	int			ntb_fhashsz;
	nerd_tuple_t		**ntb_fhash;
	RW_SPIN_LOCK_T		ntb_mutex;
} nerd_tuple_bucket_t;

typedef struct nerd_tuple_set {
	int			nts_family;
	int			nts_protocol;
	int			nts_lhashsz;
	nerd_tuple_bucket_t	*nts_lhash;
} nerd_tuple_set_t;


#define NERD_TUPLE_LHASH(_set, _tp) \
	(&(_set)->nts_lhash[(_tp)->nt_lport % (_set)->nts_lhashsz])

#define NERD_TUPLE_FHASH(_lhash, _tp) \
	(&(_lhash)->ntb_fhash[(_tp)->nt_fport % (_lhash)->ntb_fhashsz])

#define NERD_TUPLE_SET_FAMILY(_set)		(_set)->nts_family
#define NERD_TUPLE_SET_PROTOCOL(_set)		(_set)->nts_protocol


#define NERD_TUPLE_SET_LOCK_SHR(_set, _tp) \
	LOCK_SHR_RW_SPIN_LOCK(&NERD_TUPLE_LHASH(_set, _tp)->ntb_mutex)

#define NERD_TUPLE_SET_UNLOCK_SHR(_set, _tp, _ipl) \
	UNLOCK_SHR_RW_SPIN_LOCK(&NERD_TUPLE_LHASH(_set, _tp)->ntb_mutex, _ipl)

#define NERD_TUPLE_SET_LOCK_EXCL(_set, _tp) \
	LOCK_EXCL_RW_SPIN_LOCK(&NERD_TUPLE_LHASH(_set, _tp)->ntb_mutex)

#define NERD_TUPLE_SET_UNLOCK_EXCL(_set, _tp, _ipl) \
	UNLOCK_EXCL_RW_SPIN_LOCK(&NERD_TUPLE_LHASH(_set, _tp)->ntb_mutex, _ipl)


/*
 * void nerd_tuple_copy(nerd_tuple_t *dst, nerd_tuple_t *src)
 *	Copy the 'src' tuple to 'dst'.
 */
#define nerd_tuple_copy(_dst, _src) \
	*(_dst) = *(_src)

/*
 * int nerd_tuple_comp(nerd_tuple_t *t1p, nerd_tuple_t *t2p)
 *	Compare the contents of two nerd_tuple_t structures.
 */
#define nerd_tuple_comp(_t1p, _t2p) \
	( ((_t1p)->nt_lport != (_t2p)->nt_lport) || \
	  ((_t1p)->nt_fport != (_t2p)->nt_fport) || \
	  NERD_ADDR_COMP(&(_t1p)->nt_laddr, &(_t2p)->nt_laddr) || \
	  NERD_ADDR_COMP(&(_t1p)->nt_faddr, &(_t2p)->nt_faddr) )


typedef long nerd_tuple_cookie_t;

/*
 * nerd_tuple_t *
 * NERD_TUPLE_SET_GETFIRST(nerd_tuple_set_t *set, nerd_tuple_t *tp,
 *			   nerd_tuple_cookie_t *)
 *	Get the first possible match in the set for the passed
 *	in tuple.
 */
#define NERD_TUPLE_SET_GETFIRST(_set, _tp, _cookie) \
	nerd_tuple_set_getfirst(_set, _tp, _cookie)

/*
 * nerd_tuple_t *
 * NERD_TUPLE_SET_GETNEXT(nerd_tuple_set_t *set, nerd_tuple_t *tp,
 *			  nerd_tuple_cookie_t *cookie)
 *	Get the next possible match in the set for the passed
 *	in tuple.
 */
#define NERD_TUPLE_SET_GETNEXT(_set, _tp, _cookie) \
	((_tp)->nt_next ? (_tp)->nt_next : \
			  nerd_tuple_set_getnext(_set, _tp, _cookie))


extern void nerd_tuple_set_init(int, int, nerd_tuple_set_t *, int, int);
extern void nerd_tuple_set_add(nerd_tuple_set_t *, nerd_tuple_t *);
extern void nerd_tuple_set_rmv(nerd_tuple_set_t *, nerd_tuple_t *);
extern nerd_tuple_t *nerd_tuple_set_find(nerd_tuple_set_t *, nerd_tuple_t *);
extern nerd_tuple_t *nerd_tuple_set_getfirst(nerd_tuple_set_t *, nerd_tuple_t *,
					     nerd_tuple_cookie_t *);
extern nerd_tuple_t *nerd_tuple_set_getnext(nerd_tuple_set_t *, nerd_tuple_t *,
					    nerd_tuple_cookie_t *);
extern int nerd_tuple_set_apply(nerd_tuple_set_t *,
				int (*)(void *, nerd_tuple_t *), void *, int);

#endif /* NSC_NERD_NERD_TUPLE_H */
