/*
 * Copyrighted as an unpublished work.
 * (c) Copyright 1987-1995 Computer Associates International, Inc.
 * All rights reserved.
 *
 * RESTRICTED RIGHTS
 *
 * These programs are supplied under a license.  They may be used,
 * disclosed, and/or copied only as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this restricted rights notice.  Use, copying, and/or disclosure
 * of the programs is strictly prohibited unless otherwise provided
 * in the license agreement.
 */
/*
 * Copyright (c) 1982-1995
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ident "@(#)unixsrc:usr/src/common/uts/net/inet/in.h /main/uw7_uk/4"

#ifndef __netinet_in_h
#define __netinet_in_h

#ifdef _KERNEL_HEADERS

#include <io/stream.h>			/* REQUIRED */
#include <net/inet/byteorder.h>		/* REQUIRED */
#include <net/inet/in_f.h>		/* PORTABILITY */
#include <net/inet/in6.h>		/* REQUIRED */
#include <net/inet/insrem.h>
#include <net/socket.h>			/* REQUIRED */
#include <util/types.h>			/* REQUIRED */
#include <net/bitypes.h>		/* REQUIRED */
#include <net/convsa.h>			/* REQUIRED */

#elif defined(_KERNEL)

#include <netinet/in_f.h>		/* PORTABILITY */
#include <netinet/in6.h>		/* REQUIRED */
#include <sys/byteorder.h>		/* REQUIRED */
#include <sys/insrem.h>
#include <sys/socket.h>			/* REQUIRED */
#include <sys/stream.h>			/* REQUIRED */
#include <sys/types.h>			/* REQUIRED */
#include <sys/bitypes.h>		/* REQUIRED */
#include <sys/convsa.h>			/* REQUIRED */

#else

#include <netinet/in_f.h>               /* PORTABILITY */

/* Protect UNIX95 namespace */
#if !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1)
#include <netinet/in6.h>
#include <sys/byteorder.h>		/* SVR4.0COMPAT */
#include <sys/stream.h>			/* SVR4.0COMPAT */
#endif /* !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1) */

#include <sys/types.h>		
#include <sys/bitypes.h>
#include <sys/convsa.h>

#endif /* _KERNEL_HEADERS */

/* XPG4 Version 2 typedefs */

#ifndef _IN_PORT_T
#define _IN_PORT_T
typedef unsigned short in_port_t;
typedef unsigned long in_addr_t;
#endif

/* Protect UNIX95 name space */
#if !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1)
#define IPM_ID		200	/* Module ID for IP stream */
#define ICMPM_ID	201	/* Module ID for ICMP stream */
#define TCPM_ID		202	/* Module ID for TCP stream */
#define UDPM_ID		203	/* Module ID for UDP stream */
#define ARPM_ID		204	/* Module ID for ARP stream */
#define APPM_ID		205	/* Module ID for ProcARP stream */
#define RIPM_ID         206     /* Module ID for RIP stream */
#define PPPM_ID         207     /* Module ID for PPP stream */
#define AHDLCM_ID       208     /* Module ID for ASYHDLC stream */
#define MHDLCRIPM_ID    209     /* Module ID for MDMHDLC stream */
#define HDLCM_ID        210     /* Module ID for HDLC stream */
#define PPCID_ID        211     /* Module ID for PPP control stream */
#define IGMPM_ID	212	/* Module ID for IGMP stream */
#define IPIPM_ID	213	/* Module ID for IPIP stream */
#endif /* !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1) */

/*
 * Constants and structures defined by the internet system, Per RFC 790,
 * September 1981. 
 */

/*
 * Protocols 
 */
#define	IPPROTO_IP		0	/* dummy for IP */
#define IPPROTO_HOPOPTS		0	/* IPv6 hop-by-hop options */
#define	IPPROTO_ICMP		1	/* control message protocol */
#define	IPPROTO_IGMP		2	/* group control protocol */
#define	IPPROTO_GGP		3	/* gateway^2 (deprecated) */
#define	IPPROTO_IPIP		4	/* ip in ip encapsulation */
#define	IPPROTO_TCP		6	/* tcp */
#define	IPPROTO_EGP		8	/* exterior gateway protocol */
#define	IPPROTO_PUP		12	/* pup */
#define	IPPROTO_UDP		17	/* user datagram protocol */
#define	IPPROTO_IDP		22	/* xns idp */
#define	IPPROTO_TP		29	/* TP-4 w/ class negotiation */
#define IPPROTO_IPV6		41	/* IPv6 in IPv4 or sockopt value */
#define IPPROTO_ROUTING		43	/* IPv6 routing header */
#define IPPROTO_FRAGMENT	44	/* IPv6 fragmentation header */
#define IPPROTO_ESP		50	/* encapsulation security payload */
#define IPPROTO_AH		51	/* authentication header */
#define IPPROTO_ICMPV6		58	/* ICMPv6 */
#define IPPROTO_NONE		59	/* IPv6 no next header */
#define IPPROTO_DSTOPTS		60	/* IPv6 destination options */
#define IPPROTO_HELLO		63	/* hello routing protocol */
#define IPPROTO_ND		77	/* net disk proto, UNOFFICIAL */
#define	IPPROTO_EON		80	/* ISO CLNP */

#define	IPPROTO_RAW		255	/* raw IP packet */
#define	IPPROTO_MAX		256

/*
 * Port/socket numbers: network standard functions
 */
#define IPPORT_ECHO             7
#define IPPORT_DISCARD          9
#define IPPORT_SYSTAT           11
#define IPPORT_DAYTIME          13
#define IPPORT_NETSTAT          15
#define IPPORT_FTP              21
#define IPPORT_TELNET           23
#define IPPORT_SMTP             25
#define IPPORT_TIMESERVER       37
#define IPPORT_NAMESERVER       42
#define IPPORT_WHOIS            43
#define IPPORT_MTP              57

/*
 * Port/socket numbers: host specific functions
 */
#define IPPORT_TFTP             69
#define IPPORT_RJE              77
#define IPPORT_FINGER           79
#define IPPORT_TTYLINK          87
#define IPPORT_SUPDUP           95

/*
 * UNIX TCP sockets
 */
#define IPPORT_EXECSERVER       512
#define IPPORT_LOGINSERVER      513
#define IPPORT_CMDSERVER        514
#define IPPORT_EFSSERVER        520

/*
 * UNIX UDP sockets
 */
#define IPPORT_BIFFUDP          512
#define IPPORT_WHOSERVER        513
#define IPPORT_ROUTESERVER      520     /* 520+1 also used */

/*
 * Local port number conventions:
 * Ports < IPPORT_RESERVED are reserved for privileged processes (e.g. root).
 * Ports > IPPORT_USERRESERVED are reserved for servers, not necessarily
 * privileged.
 *
 * These values are somewhat superceded by inconfig(1Mtcp) tunables:
 * IPPORT_RESERVED -> ipport_reserved_high (see also ipport_reserved_low)
 * IPPORT_USERRESERVED -> ipport_userreserved_high (see also
 * 					ipport_userreserved_low)
 */
#define	IPPORT_RESERVED		1024
#define	IPPORT_USERRESERVED	65535

/*
 * Default values for some port related inconfig(1Mtcp) tunables.
 */
#define IPPORT_RESERVED_LOW		512
#define IPPORT_RESERVED_HIGH		1023
#define IPPORT_USERRESERVED_LOW		32768
#define IPPORT_USERRESERVED_HIGH	65535

#if !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1)
/*
 * The maximum length needed for representing a struct in_addr in printable
 * address format.  Useful for inet_ntop().
 * Computed as (sizeof("255.255.255.255")-1).
 */
#define INET_ADDRSTRLEN		16
#endif /* !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1) */

/*
 * Socket address, internet style. 
 */
struct sockaddr_in {
#ifdef __NEW_SOCKADDR__
	sa_len_t	sin_len;	/* 8-bit structure length */
#endif
	sa_family_t     sin_family;	/* 8/16-bit address family (AF_INET) */
	in_port_t       sin_port;	/* 16-bit port number */
	struct in_addr  sin_addr;	/* 32-bit internet address */
	unsigned char   sin_zero[8];	/* optional with TLI */
};

/*
 * The transport providers allow any address length between
 * IN_MINADDRLEN and IN_MAXADDRLEN.  The minimum length corresponds to
 * a sockaddr_in without the sin_zero field.  The maximum length is
 * the size of the sockaddr_in structure.
 *
 * in_chkaddrlen returns true if the given length is valid.
 */

#define IN_MINADDRLEN	(sizeof(struct sockaddr_in) - 8)
#define IN_MAXADDRLEN	(sizeof(struct sockaddr_in))
#define in_chkaddrlen(x) ((x) >= IN_MINADDRLEN && (x) <= IN_MAXADDRLEN)

/*
 * Options for use with [gs]etsockopt at the IP level. 
 * These use the same values as the equivalent IPV6 level options if they
 * have samilar semantics.
 */
#ifndef IP_OPTIONS
#define	IP_OPTIONS	1	/* set/get IP per-packet options */
#endif

#ifndef IP_TOS
#define	IP_TOS		2	/* set/get IP per-packet tos */
#endif

#ifndef IP_TTL
#define	IP_TTL		3	/* set/get IP per-packet ttl */
#endif

#define	IP_HDRINCL	4	/* int; header is included with data (raw) */
#define	IP_RECVOPTS	5	/* bool; receive all IP options w/ datagram */
#define	IP_RECVRETOPTS	6	/* bool; receive all IP options for response */
#define	IP_RECVDSTADDR	7	/* bool; receive IP dst addr w/ datagram */
#define	IP_RETOPTS	8	/* ip_opts; set/get IP per-packet options */

#define	IP_MULTICAST_IF	9	/* set/get IP multicast interface  */
#define	IP_MULTICAST_LOOP	10	/* set/get IP multicast loopback   */
#define	IP_ADD_MEMBERSHIP	11	/* add  an IP group membership     */
#define	IP_DROP_MEMBERSHIP	12	/* drop an IP group membership     */

/*			13 */	/* unassigned */
#define IP_BROADCAST_IF	14	/* set/get interface for limited broadcast */
#define IP_RECVIFINDEX	15	/* bool; receive if_index w/datagram */

#define	IP_MULTICAST_TTL	16	/* set/get IP multicast ttl */

/* Protect UNIX95 name space.*/
#if !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1)
/*
 * Multicast Routing set/getsockopt commands.
 */
#define MRT_INIT	17	/* initialize forwarder */
#define MRT_DONE	18	/* shut down forwarder */
#define MRT_ADD_VIF	19	/* create virtual interface */
#define MRT_DEL_VIF	20	/* delete virtual interface */
#define MRT_ADD_MFC	21	/* insert forwarding cache entry */
#define MRT_DEL_MFC	22	/* delete forwarding cache entry */
#define MRT_VERSION	23	/* get kernel version number */

/* #define IPV6_ADDRFORM	24 */
/* #define IPV6_MULTICAST_HOPS	25 */
/* #define IPV6_MULTICAST_IF	26 */
/* #define IPV6_MULTICAST_LOOP	27 */
/* #define IPV6_ADD_MEMBERSHIP	28 */
/* #define IPV6_DROP_MEMBERSHIP	29 */

/*
 * Other definitions
 */
#define	IP_DEFAULT_MULTICAST_TTL   1	/* normally limit m'casts to 1 hop  */
#define	IP_DEFAULT_MULTICAST_LOOP  1	/* normally hear sends if a member  */
#define	IP_MAX_MEMBERSHIPS         20	/* per socket; must fit in one mblock */

#define INADDR_UNSPEC_GROUP	(u_long)0xe0000000	/* 224.0.0.0   */
#define INADDR_ALLHOSTS_GROUP	(u_long)0xe0000001	/* 224.0.0.1   */
#define INADDR_ALLRTRS_GROUP    (u_long)0xe0000002      /* 224.0.0.2   */
#define INADDR_MAX_LOCAL_GROUP	(u_long)0xe00000ff	/* 224.0.0.255 */

/*
 * Argument structure for IP_ADD_MEMBERSHIP and IP_DROP_MEMBERSHIP.
 */
struct ip_mreq {
	struct in_addr	imr_multiaddr;	/* IP multicast address of group */
	struct in_addr	imr_interface;	/* local IP address of interface */
};

/*
 * Structure used to describe IP options.
 * Used to store options internally, to pass them to a process, or to restore
 * options retrieved earlier.
 * The ip_dst is used for the first-hop gateway when using a source route.
 * (this gets put into the header proper).
 */

struct ip_opts {
	struct	in_addr	ip_dst;		/* first hop, 0 w/o src route */
	char	ip_opts[40];		/* actually variable in size */
};

#if defined(_KERNEL) || defined(_KMEMUSER)
/*
 * Maximum size of all supported IPPROTO_IP level options (non-XTI optmgmt). 
 * Should be changed when new IPPROTO_IP options added/deleted.
 */
#define IPPROTO_IP_OPT_MAXSZ \
/* IP_OPTIONS */ \
 ( (sizeof(struct opthdr) + IP_OPTIONS_OPT_MAXSZ) \
/* IP_TOS, IP_TTL, IP_HDRINCL, IP_MULTICAST_LOOP, IP_MULTICAST_TTL */ \
 + (sizeof(struct opthdr) + sizeof(uchar_t))*5 \
/* IP_RECVOPTS, IP_RECVRETOPTS, IP_RECVDSTADDR, IP_RECVIFINDEX */ \
 + (sizeof(struct opthdr) + sizeof(uint_t))*4 \
/* IP_MULTICAST_IF, IP_BROADCAST_IF */ \
 + (sizeof(struct opthdr) + sizeof(struct in_addr))*2 \
/* IP_ADD_MEMBERSHIP, IP_DROP_MEMBERSHIP */ \
 + (sizeof(struct opthdr) + sizeof(struct ip_mreq))*2 \
/* MRT_INIT */ \
 + (sizeof(struct opthdr) + sizeof(int *)) \
/* MRT_ADD_VIF, MRT_DEL_VIF */ \
 + (sizeof(struct opthdr) + sizeof(struct vifctl *))*2 \
/* MRT_ADD_MFC, MRT_DEL_MFC */ \
 + (sizeof(struct opthdr) + sizeof(struct mfcctl *))*2 \
/* MRT_VERSION */ \
 + (sizeof(struct opthdr) + sizeof(int)) )
#endif /* defined(_KERNEL) || defined(_KMEMUSER) */
		
#endif /* !(defined(_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1) */

#if defined(INKERNEL) || defined(_KERNEL)

#ifdef _KERNEL_HEADERS
#include <net/inet/in_mp.h>
#else /* !_KERNEL_HEADERS */
#include <netinet/in_mp.h>
#endif /* _KERNEL_HEADERS */

struct iocblk_in {
	struct iocblk	iocblk;			/* basic STREAMS iocblk */
	dev_t		ioc_transport_client;	/* the transport client */
	dev_t		ioc_network_client;	/* the network client (IP) */
	u_int		ioc_ifflags;		/* flags for interfaces */
};

/* useful defines.  should be in stream.h ?
 */
#define msgbavail(bp)	((bp)->b_datap->db_lim - (bp)->b_wptr)
#define msgblen(bp)	((bp)->b_wptr - (bp)->b_rptr)

#define satosin(sa)	((struct sockaddr_in *) (sa))
#ifdef NSC_BUG_FIX
/* Suppress a warning from idbuild. */
#ifndef INET
#define INET
#endif /* !INET */
#else  /* !NSC_BUG_FIX */
#define INET
#endif /* !NSC_BUG_FIX */

#ifdef DEBUG
#define INET_DEBUG 1
#endif /* DEBUG */

int inet_netmatch(struct in_addr in1, struct in_addr in2);
struct in_addr in_makeaddr(u_long net, u_long host);
u_long in_netof(struct in_addr in);
int in_sockmaskof(struct in_addr in, struct sockaddr_in *sockmask);
u_long in_lnaof(struct in_addr in);
int in_localaddr(struct in_addr in);
int in_canforward(struct in_addr in);
int in_ouraddr(struct in_addr addr);
struct in_ifaddr *in_iaonnetof(u_long net);
int in_broadcast(struct in_addr in);
int in_nl_broadcast(struct in_addr in);
struct ifnet *in_localbroadcast(struct in_addr in);
int in_cksum(mblk_t *, int);
u_char *inet_ntoa(u_long, u_char *);

#endif	/* KERNEL */
#endif	/* __netinet_in_h */
