/*
 * Copyrighted as an unpublished work.
 * (c) Copyright 1987-1995 Computer Associates International, Inc.
 * All rights reserved.
 *
 * RESTRICTED RIGHTS
 *
 * These programs are supplied under a license.  They may be used,
 * disclosed, and/or copied only as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this restricted rights notice.  Use, copying, and/or disclosure
 * of the programs is strictly prohibited unless otherwise provided
 * in the license agreement.
 */
/*
 * Copyright (c) 1982-1995
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ident "@(#)unixsrc:usr/src/common/uts/net/inet/in_pcb.h /main/uw7_uk/uw710ptf_uk/1"
#ident "$Header: /src/nsc_uw7/usr/src/common_nsc/uts/net/inet/in_pcb.h,v 1.9 2000/04/12 16:29:16 zabby Exp $"

#ifndef __netinet_in_pcb_h
#define __netinet_in_pcb_h

#ifdef _KERNEL_HEADERS
#include <net/tihdr.h>
#include <net/xti.h>
#else
#include <sys/tihdr.h>
#include <sys/xti.h>
#endif

/*
 * Common structure pcb for internet protocol implementation. Here are stored
 * pointers to local and foreign host table entries, local and foreign socket
 * numbers, and pointers up (to a socket structure) and down (to a
 * protocol-specific) control block. 
 */

struct inpcbhead {
	struct inpcb	*lh_first;
};

struct inpcbporthead {
	struct inpcbport *lh_first;
};

struct inp_hash {
	struct inpcb *le_next;
	struct inpcb **le_prev;
};

struct inp_portlist {
	struct inpcb *le_next;
	struct inpcb **le_prev;
};

typedef struct inpcb {
	struct inpcb   *inp_next, *inp_prev;
					/* list for all PCB's of this proto */
	struct inpcbinfo *inp_pcbinfo;	/* PCB list info */ 
	uint_t		inp_state;	/* old so_state from sockets */
	short           inp_tstate;	/* TLI state for this endpoint */
	short           inp_error;	/* error on this pcb */
	dev_t           inp_minor;	/* minor device number allocated */
	queue_t        *inp_q;		/* queue for this minor dev */
	struct in_addr  inp_faddr;	/* foreign host table entry */
	struct in_addr  inp_laddr;	/* local host table entry */
	u_short         inp_fport;	/* foreign port */
	u_short         inp_lport;	/* local port */
#define inp_proto	inp_lport       /* overload port field for protocol */
	caddr_t         inp_ppcb;	/* pointer to per-protocol pcb */
	struct route    inp_route;	/* placeholder for routing entry */
	mblk_t         *inp_options;	/* IP options */
	mblk_t         *inp_inpopts;	/* recvd IP options */
#ifdef NSC_DNET
	uint		inp_protoopt;	/* old so_options from sockets */
#else
	ushort          inp_protoopt;	/* old so_options from sockets */
#endif
	ushort          inp_linger;	/* time to linger while closing */
	ushort          inp_protodef;	/* old pr_flags from sockets */
	int		inp_addrlen;	/* address length client likes */
	short		inp_family;	/* address family client likes */
	unchar		inp_iptos;	/* TOS for IP header */
	unchar		inp_ipttl;	/* TTL for IP header */
	u_long		inp_ipsum;	/* Sum of IP header so far */
	int		inp_flags;	/* generic IP/datagram flags */
	int		inp_rflags;	/* raw endpoint flags */
	struct inpcb	*inp_mgmtidx;	/* current inp we're retrieving */
	ushort 		inp_mgmtcnt;	/* reference count */
	struct gi_arg	gi_arg;		/* current copyin status */
	struct ip_moptions	*inp_moptions;	/* multicast options */
	u_long		inp_rbufsize;	/* size user set with SO_RCVBUF */
	u_long		inp_sbufsize;	/* size user set with SO_SNDBUF */
	struct inp_hash inp_hash;       /* hash list */
	struct inp_portlist inp_portlist; /* list for this PCB's local port */
	struct inpcbport *inp_phd;      /* head of this list */
	rwlock_t *inp_lock;		/* shared r/w lock */
	lkinfo_t inp_lkinfo;		/* lock info */
	u_long 		inp_ctr;	/* lock counter */
	atomic_int_t	*inp_refcnt;	/* inpcb reference counter */
	atomic_int_t	*inp_qref;	/* queue reference counter */
	unsigned long   inp_xdebug;     /* Debugging directives form XTI_DEBUG*/
	struct t_linger inp_xlinger;    /* XTI optmgmt linger */
	unsigned long   inp_xiplvlopt;  /* IP level options whose possible
                                         * values are binary */
        mblk_t          *inp_xretopts;  /* results of option negotiations that
                                         * will be returned at a later time */
        unsigned char   inp_xiptos;     /* XTI optmgmt IP type of service */
        unsigned char   inp_xipttl;     /* IP datagram time to live */
} inpcb_t;

struct phd_hash {
	struct inpcbport *le_next;
	struct inpcbport **le_prev;
};

struct inpcbport {
	struct phd_hash phd_hash;
	struct inpcbhead phd_pcblist;
	u_short phd_port;
};

/*
 * inpcbinfo contains information about the PCB linked
 * list, port hash table and a pointer to per-protocol
 * info structute.
 */
struct inpcbinfo {
	inpcb_t *listhead;                      /* PCB linked list */
	struct inpcbporthead *porthashbase;     /* port PCB hash table */
	u_long porthashsize;
	u_long porthashmask;
	caddr_t	*ppcbinfo;			/* per-protocol info */
	int     flags;
};

/*
 * inppcbinfo contains information about a connection
 * hash table which can be used by protocols which hash
 * their PCBs but do not implement their own optimised 
 * connection hash tables.
 */
struct inppcbinfo {
	struct inpcbhead *hashbase;             /* PCB hash table */
	u_long hashsize;
	u_long hashmask;
};

#define IP_NPCBHASH	512

#define INP_PCBHASH(faddr, lport, fport, mask) \
	(((faddr) ^ ((faddr) >> 16) ^ ntohs((lport) ^ (fport))) & (mask))
#define INP_PCBPORTHASH(lport, mask) \
	(ntohs((lport)) & (mask))

/* inpcb_t reference macros */
#define	INP_HOLD(inp)	ATOMIC_INT_INCR((inp)->inp_refcnt)
#define	INP_FREE(inp)	(ASSERT(ATOMIC_INT_READ((inp)->inp_refcnt) > 0), ATOMIC_INT_DECR((inp)->inp_refcnt))

/* queue reference macros */
#define	INP_QHOLD(inp)	ATOMIC_INT_INCR((inp)->inp_qref)
#define	INP_QFREE(inp)	(ASSERT(ATOMIC_INT_READ((inp)->inp_qref) > 0), ATOMIC_INT_DECR((inp)->inp_qref))

#define	INP_THRESHOLD	1000000
#define INP_SPIN(inp) { \
	u_long spinct = 0;	\
	while (ATOMIC_INT_READ((inp)->inp_refcnt) > 0) { \
		drv_usecwait(1); \
		if ((++spinct % INP_THRESHOLD) == 0) { \
			cmn_err(CE_WARN, "ip: spinning on PCB %lx", (inp)); \
		} \
	} \
}

#define INP_QSPIN(inp) { \
	u_long spinct = 0;	\
	while (ATOMIC_INT_READ((inp)->inp_qref) > 0) { \
		drv_usecwait(1); \
		if ((++spinct % INP_THRESHOLD) == 0) { \
			cmn_err(CE_WARN, "ip: spinning on queue for PCB %lx", \
				(inp)); \
		} \
	} \
}

/* flags in inp_flags: */
#define INP_RECVOPTS            0x01    /* receive incoming IP options */
#define INP_RECVRETOPTS         0x02    /* receive IP options for reply */
#define INP_RECVDSTADDR         0x04    /* receive IP dst address */
#define INP_RECVIFINDEX		0x08	/* receive if_index */
#define INP_CONTROLOPTS         (INP_RECVOPTS|INP_RECVRETOPTS|INP_RECVDSTADDR \
				|INP_RECVIFINDEX)
#define INP_IPV6MAPPED		0x10	/* actually IPv4 mapped IPv6 address */
#define INP_IPV4OPENED		0x20	/* initially opened as IPv4 endpoint */
#define INP_IPV6OPENED		0x40	/* initially opened as IPv6 endpoint */
#define INP_XTIOPTMGMT		0x80	/* process OPTMGMT XTI-style */
#define	INP_TLI			0x100	/* process old TLI T_BIND request */
#define INP_PORT_HASHED		0x200	/* inp is hashed in port table */
#define INP_EXTTPI		0x400	/* use extended TPI */
#define INP_CONN_HASHED		0x800	/* inp is hashed in connection table */
#define	INP_TMP_LISTEN		0x1000	/* marks inp as only a temporary
					 * resident in the LISTEN cache. */
#define INP_ALLRAW		0x2000	/* inp requires all raw input */
#ifdef NSC_DNET
#define INP_GHOST		0x4000	/* Ghost pcb */
#define INP_DONOTFREE		0x8000	/* Do not free the inpcb */
#endif /* NSC_DNET */
#ifdef NSC_ICS
#define INP_ICS			0x10000 /* ICS socket */
#endif

/* flags in inpcbinfo */
#define IN_PCB_HASH     1               /* Do a hash on binds and connects */

#define	INPLOOKUP_WILDCARD	1
#define	INPLOOKUP_SETLOCAL	2

#define IN_PCBLOOKUP_LOCAL(pcbinfo, faddr, fport, laddr, lport, wild, inp) \
if ((pcbinfo)->flags & IN_PCB_HASH) \
	inp = in_pcblookup_local((pcbinfo), (laddr), (lport), (wild)); \
else \
	inp = in_pcblookup((pcbinfo)->listhead, (faddr), (fport), (laddr), \
		(lport), (wild));

#define IN_PCBLOOKUP(pcbinfo, faddr, fport, laddr, lport, wild, inp) \
if ((pcbinfo)->flags & IN_PCB_HASH) \
	inp = in_pcblookup_hash((pcbinfo), (faddr), (fport), (laddr), \
		(lport), (wild)); \
else \
	inp = in_pcblookup((pcbinfo)->listhead, (faddr), (fport), (laddr), \
		(lport), (wild));

#define IN_PCBINSHASH(inp, err) \
if ((inp)->inp_pcbinfo->flags & IN_PCB_HASH) { \
	if (!(err = in_pcbinsporthash((inp)))) { \
		in_pcbinsconnhash((inp)); \
	} \
}

#define IN_PCBREHASH(inp) \
if ((inp)->inp_pcbinfo->flags & IN_PCB_HASH) { \
	in_pcbrehash((inp)); \
}

#define IN_PCBREMHASH(inp) \
if ((inp)->inp_pcbinfo->flags & IN_PCB_HASH) { \
	in_pcbremhash((inp)); \
}

/* Raw flags */
#define RINPF_HDRINCL	0x1		/* user is sending the whole header */

/*
 * retrieval ioctls
 */
#ifndef __MGMT
#define __MGMT 'M'
#endif

#define	STIOCGTCB       ((__MGMT << 8) | 7)	/* fetch TCP control blocks */
#define	STIOCGUDB       ((__MGMT << 8) | 9)	/* fetch UDP control blocks */
#define	SIOCSMGMT       ((__MGMT << 8) | 10)	/* enable mgmt func. */
#define	STIOCGRCB	((__MGMT << 8) | 12)  	/* fetch raw IP control blks */

#if defined(INKERNEL) || defined(_KERNEL)

#define qtoinp(q) ((inpcb_t *) (q)->q_ptr)

#if defined(__STDC__)
struct T_optmgmt_req;
#endif

inpcb_t *in_pcballoc(struct inpcbinfo *);
inpcb_t *in_pcballoc_only(struct inpcbinfo *);
void in_pcblink(inpcb_t *);
void in_pcbfree(inpcb_t *);
int in_pcbbind(inpcb_t *, u_char *, int);
int in_pcbconnect(inpcb_t *, u_char *, int, inpcb_t *);
int in_pcbdisconnect(inpcb_t *);
int in_pcbdetach(inpcb_t *);
int in_setsockaddr(inpcb_t *, mblk_t *);
int in_setpeeraddr(inpcb_t *, mblk_t *);
int in_pcbnotify(inpcb_t *, struct sockaddr *, u_short, struct in_addr, u_short, int, int, int (*)(inpcb_t *, int));
int in_losing(inpcb_t *);
int in_rtchange(inpcb_t *, int);
inpcb_t *in_pcblookup(inpcb_t *, struct in_addr, u_short, struct in_addr, u_short, int);
int in_pcboptmgmt(queue_t *, struct T_optmgmt_req *, struct opthdr *, mblk_t *);
int in_pcbchk_srcaddr(u_long);
void in_proto_optmgmt(queue_t *);
void in_chek_mgmt_que(inpcb_t *);
void in_pcb_ioc(queue_t *, mblk_t *, inpcb_t *, int, int);
void in_copy_pcb_to_buffer(mblk_t *, inpcb_t *, inpcb_t *, int *, int, int);
void T_conn_con(inpcb_t *);
void in_abortendpoints(inpcb_t *, queue_t *);
struct inpcb *in_pcblookup_local(struct inpcbinfo *, struct in_addr, in_port_t, int);
struct inpcb *in_pcblookup_hash(struct inpcbinfo *, struct in_addr, in_port_t, struct in_addr, in_port_t, int);
int in_pcbinsporthash(inpcb_t *);
void in_pcbinsconnhash(inpcb_t *);
void in_pcbrehash(inpcb_t *);
void in_pcbremhash(inpcb_t *);

int inxtigen_options(queue_t *, union T_primitives *,
		     struct t_opthdr *, mblk_t **, int);

#endif
#endif	/* __netinet_in_pcb_h */
