#ifndef _IO_AIOSYS_H
#define _IO_AIOSYS_H

#ident	"@(#)unixsrc:usr/src/i386at/uts/io/async/aiosys.h /main/uw7_nj/3"

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef  _KERNEL_HEADERS

#include <io/async/aio_hier.h>
#include <util/types.h> 			/* REQUIRED */
#include <util/param.h> 			/* REQUIRED */
#include <proc/proc.h> 				/* REQUIRED */
#include <fs/buf.h> 				/* REQUIRED */
#include <fs/file.h> 				/* REQUIRED */
#include <mem/as.h>				/* REQUIRED */
#include <mem/seg.h>				/* REQUIRED */

#else  /*_KERNEL_HEADERS */

#include <sys/aio_hier.h>
#include <sys/types.h> 				/* REQUIRED */
#include <sys/param.h> 				/* REQUIRED */
#include <sys/proc.h> 				/* REQUIRED */
#include <sys/buf.h> 				/* REQUIRED */
#include <sys/file.h> 				/* REQUIRED */
#include <vm/as.h>				/* REQUIRED */
#include <vm/seg.h>				/* REQUIRED */

#endif  /* _KERNEL_HEADERS */


/* Definitions of the various ioctls */

#define 	AIOMEMLOCK      1
#define 	AIORW		2
#define 	AIOREAD         3
#define 	AIOWRITE        4
#define 	AIOPOLL         5
#define		AIOLISTIO	6
#define		AIOGETTUNE	7
#define		AIOWAIT		8
#define		AIOSUSPEND	9
#define		AIOSTATCTL	10


/* sub commands for AIOSTATCTL */
#define		AIOSTATSON	1
#define		AIOSTATSOFF	2
#define		AIOCLRSTATS	3
#define		AIOGETSTATS	4

/* Max number of elements in a poll request. */
#define		NSTATUS		32

/*
 * struct aioresult is returned when the library poll's for completed
 * jobs, and the errno and count are copied back to the user aiocb.  
 * apflag tells if this is a 32bit or 64bit user aiocb.
 */
typedef struct aiostatus {
	void		*ast_cbp;	/* User provided cbp */
	int		ast_errno;	/* Error returned by the I/O */
	ssize_t		ast_count;	/* Size of I/O completed */
	int		ast_apflag;	/* Flag indicate the cbp type */
} aiostatus_t;

typedef struct aioresult {
	int		ar_total;
	int		ar_timeout;
	aiostatus_t	ar_stat[NSTATUS];
} aioresult_t; 

/*
 * struct asyncmlock is the argument to the aio_memlock call,
 * and is used to lock down a range of user memory for asyncio.
 * It is also used to tell the kernel the address of a word
 * in the library that the kernel sets to 1 when a job completes,
 * so the library will not need to call aio_memlock unless there
 * are completed jobs to return to the user.
 */
typedef struct asyncmlock {
	vaddr_t	am_vaddr;		/* Addr of user provided buffer */
	size_t	am_size;		/* Size of the user buffers */
	caddr_t	am_polladdr;		/* Addr of user poll status */
} asyncmlock_t;

/*
 * The library builds a struct aiojob from the user's aiocb, and 
 * passes the aiojob into the kernel for aio_read/aio_write/lio_listio.
 */
typedef struct aiojob {
	int		aj_fd;		/* File descriptor from user */
	vaddr_t		aj_buf;		/* User buffer for I/O */
	int		aj_cnt;		/* Number of bytes for I/O */
	off64_t		aj_offset;	/* Offset into the raw slice */
	uint_t		aj_flag;	/* Flag variable for general use*/
	uint_t		aj_cmd;		/* Read or write */
	void		*aj_cbp;	/* Ptr to the libctl structure */
	int		aj_errno;	/* Errno for list I/O */
	int		aj_apflag;	/* Flag variable for aj_cbp type */
} aiojob_t;

/*
 * For an lio_listo, the library builds a struct aiolistio to pass
 * in all the aiojob structs, and to tell us if we are supposed to
 * wait or not, and if we are supposed to send a signal when all
 * the IOs are complete.
 */
typedef struct aiolistio {
	uint_t		al_mode;	/* LIOWAIT versus LIONOWAIT */
	uint_t		al_nent;	/* Number of entries in the list */
	uint_t		al_flag;	/* Async notifcation ? */
	aiojob_t	al_jobs[1];	/* List of jobs */
} aiolistio_t;

#define AIOLIST_HDRSIZE	(sizeof(aiolistio_t) - sizeof(aiojob_t))

/* These must match the user vaules in aio.h */
#define LIONOWAIT 	0
#define LIOWAIT		1

/*
 * These flags communicate setup and result information between the lib and
 * kernel.  The notify flag is set by the lib in the job aj_flag, and used
 * in the kernel in aio_kflags.
 */
#define	A_ASYNCNOTIFY	0x1	/* async notification needed on completion */
#define A_LIOWAIT	0x2	/* wakep lwp on LIST I/O completion */
#define A_LIONOWAIT	0x4	/* send SIGEMT on LIST I/O completion */
#define A_USERFLAGS	A_ASYNCNOTIFY



/*
 * The library uses struct aio_tune to find out the size of kernel tunables.
 */
typedef struct aio_tune {
	uint_t 	at_listio_max;	/* Max size of any LIST I/O */
	uint_t	at_max;		/* Max outstanding I/O per process */
	uint_t	at_num_ctlblks;	/* Total no of configured aio_t structs */
} aio_tune_t;

/*
 * We use struct aio_stats to get the various statistics we keep
 * on async IO ops.
 */
typedef struct aio_stats {
	int aio_memlock;
	int aio_memlock00;
	int aio_memlock_total;
	int aio_memlock_overlaps;
	int aio_memlock_aio_able;
	int aio_memlock_npolladdr;
	int aio_memlock_polladdr;
	int aio_rw_alloc;
	int aio_read;
	int aio_readblks;
	int aio_write;
	int aio_writeblks;
	int aio_read00;
	int aio_read00blks;
	int aio_write00;
	int aio_write00blks;
	int aio_lio_wait;
	int aio_lio_wait_ents;
	int aio_lio_nowait;
	int aio_lio_nowait_ents;
	int aio_lio_read;
	int aio_lio_readblks;
	int aio_lio_write;
	int aio_lio_writeblks;
	int aio_lio_read00;
	int aio_lio_read00blks;
	int aio_lio_write00;
	int aio_lio_write00blks;
	int aio_lio_read_errors;
	int aio_lio_write_errors;
	int aio_lio_nent;
	int aio_lio_faults;
	int aio_lio_allocs;
	int aio_lio_eagain;
	int aio_as_free;
	int aio_waitforio;
	int aio_waitunlock;
	int aio_cleanall;
	int aio_memunlockall;
	int aio_intersect;
	int aio_intersected;
	int aio_lockrange_locked;
	int aio_lockrange;
	int aio_lockrange_fault;
	int aio_lockrange_aio_able;
	int aio_unlockrange;
	int aio_badcnt;
	int aio_badalign;
	int aio_badgetf;
	int aio_badtype;
	int aio_badrange;
	int aio_badsize;
	int aio_poll;
	int aio_wait_einval;
	int aio_wait_etime;
	int aio_wait_eagain;
	int aio_wait;
	int aio_wait_forever;
	int aio_wait_eintr;
	int aio_wait0einval;
	int aio_wait_tmdout;
	int aio_waitstats[2];
	int aio_pollstats[NSTATUS+1];
} aio_stats_t;


#ifdef _KERNEL

typedef struct aio_queue {
	struct aio_queue	*aq_fowd;
	struct aio_queue	*aq_back;
} aio_queue_t;

#ifdef	NSC_ASYNCIO

typedef struct {
	clusternode_t	ahdl_node;
	long		ahdl_unique;
} aio_handle_t;

#endif	/* NSC_ASYNCIO */

/*
 * Each I/O request submitted to the underlying device by the aio driver is
 * described by an aio structure.  An aio structure will always be on either
 * the freelist, the wait queue, the completed queue, and for listio jobs,
 * it may be on a private list for short periods.   Every job that is part
 * of a listio (LIOWAIT, or LIONOWAIT with signal notification) has a pointer
 * to a lioctl structure in aio_lioctl.  Any job where we must lock the user
 * memory around each IO, will have a list of the pages we locked in aio_plist.
 * Any job that references DSHM memory, and thus must be locked around each IO,
 * will have an aio_saved_segp.
 */
typedef struct aio {
	aio_queue_t	aio_queue;	/* Thread for current list */
	uint_t		aio_kflags;	/* Flag variable for local use */
	file_t		*aio_filep;	/* fp corresponding to user fd */
	int		aio_apflag;	/* Flag variable for cbp type */
	void 		*aio_cbp;	/* Ptr to ctlblk provided by user */
	int		aio_nbytes;	/* Bytes actually read or written */
	int		aio_errno;	/* Error number value */
	struct lioctl *	aio_lioctl;	/* Ptr to info for lio_listio */
	struct page **	aio_plist;	/* pagelist for user buffer */
	struct seg *	aio_saved_segp;	/* for AIOs which need aio_prep() */
	struct buf *	aio_buf;	/* Buffer header for disk request */
#ifdef	NSC_ASYNCIO
	aio_handle_t	aio_handle;	/* Unique ID for remote I/O. */
	char		*aio_resp_buf;	/* Response buffer for remote I/O. */
#endif	/* NSC_ASYNCIO */
} aio_t;

/*
 * The free list of per-request data structures, its counter and lock.
 */
typedef struct aio_free {
	aio_queue_t	queue;
	uint_t 		count;
	lock_t		lock;
} aio_free_t;

/*
 * We currently think that having a process with multiple LWPs doing AIO
 * is uncommon, but we need to handle it.  If this becomes more common 
 * in the future, we will have to revisit some of the design choices below.
 */


/*
 * To keep from having to kmem_alloc and kmem_free an lioctl struct
 * on every lio_listio, we embed one in the aio_proc struct.
 * For now we have a flag to tell if the embedded struct is busy, and
 * we just make subsequent LWPs kmem_alloc and free their own struct.
 */
typedef struct lioctl {
	sv_t		lio_sv;		/* Sync on I/O completion, protected
					 * by ap_lock in aio_proc. */
	uint_t		lio_lcount;	/* No of elements in the sub list */
} lioctl_t;


/*
 * A process doing aio thru the routines in libthread must call
 * aio_memlock at least once.  A process can make multiple calls
 * to aio_memlock to lock down virtual address ranges for IO.
 * These locked ranges are tracked via a linked list of aio_range_t.
 * In addition to such calls, or instead of them, a process can 
 * call aio_memlock(0,0) to tell us that it wants the kernel to do the
 * locking of the user buffer if it is not in an aio_memlocked range.
 * This is recorded as AP_NEEDLOCK in ap_flags.
 *
 * If an address space operation intersects with an aio_memlocked range,
 * the as routines will call aio_intersect to wait for the AIO to finish.
 * Then any aio_memlocked ranges are unlocked, and the address space
 * operation is allowed to complete. Subsequent aio operations will return
 * EINVAL, unless the process has also done an aio_memlock(0,0).
 *
 * ar_needs_prep will be true for DSHM ranges as they require locking 
 * and unlocking around each IO.
 */
typedef struct aio_range {
	vaddr_t		ar_saddr;	/* start of locked user addr space */
	vaddr_t		ar_eaddr;	/* end   of locked user addr space */
	struct aio_range *ar_next;	/* next range in list */
	struct aio_range *ar_prev;	/* previous range in list */
	boolean_t	ar_needs_prep;	/* TRUE if must call as_aio_prep() */
} aio_range_t;

/*
 * The aio_proc structure is the per-process aio data structure.
 * It is reachable from the p_aioprocp proc structure member, which is
 * protected by the process's address space lock.  
 * We allocate an aio_proc structure when the process does the [first] call
 * to aio_memlock.  It remains in existance until the process exits.
 *
 * To keep from having to kmem_alloc and kmem_free an lioargs struct
 * on every lio_listio, we keep one cached off the aio_proc struct.
 * For now we have a flag to tell if the cached struct is busy, and
 * we just make subsequent LWP kmem_alloc and free their own struct.
 */
typedef struct aio_proc {
	aio_range_t	*ap_rangelist;	/* list of ranges for locked area */
	aio_range_t	*ap_rangelast;	/* hint for aio_find_range */
	vaddr_t		ap_polladdr;	/* Ptr to locked user addr for poll */
	caddr_t		ap_kpolladdr;	/* Ptr to kernel addr for poll */
	aio_queue_t	ap_waitq;	/* Queue of jobs to be completed */
	aio_queue_t	ap_compq;	/* Queue of completed jobs */
	uint_t		ap_flags;
	int		ap_needlockcnt; /* Jobs we did softlock for */
	lock_t		ap_lock;	/* Spin lock at pldisk. */
	sv_t		ap_cleansv;	/* Sync on end of all I/O, protected
					 * by ap_lock. */
	aiolistio_t *	ap_lioargs;	/* Ptr to list I/O args buffer */
	lioctl_t 	ap_lioctl;	/* list I/O control buffer */
	uint_t		ap_waiters;	/* # of LWPs in aiowait */
	event_t		ap_pollevent;	/* sync for aiopoll with timeout */
} aio_proc_t;

/* ap_flags values. */
#define AP_STOP		0x1	/* Process is sleeping for I/O to stop. */
#define AP_NEEDLOCK	0x2	/* We may need to lock memory during I/O */
#define AP_LIOCTL_BUSY	0x4	/* the embeded lioctl struct is busy */
#define AP_LIOARGS_BUSY	0x8	/* the cached lioargs struct is busy */



/* Convert a pointer to aio_queue to a pointer to the enclosing aio structure. */
#define QPTOAIOP(aioqp) \
	((aio_t *)((char *)(aioqp) - offsetof(aio_t, aio_queue)))

/* Queue operations require application of QPTOAIOP to get the aio structure. */
#define INIT_QUEUE(headp)	((headp)->aq_fowd = (headp)->aq_back = (headp))
#define ISNULL_QUEUE(headp)	((headp) == (headp)->aq_fowd)
#define FIRST_ELEM(headp)	((headp)->aq_fowd)
#define LAST_ELEM(headp)	((headp)->aq_back)
#define NEXT_ELEM(elem)		((elem)->aq_fowd)
#define END_QUEUE(headp)	(headp)
#define ADD_QUEUE(headp, elem)  ((void)( \
	(elem)->aq_fowd = (headp)->aq_fowd, \
	(elem)->aq_back = (headp)->aq_fowd->aq_back, \
	(headp)->aq_fowd->aq_back = (elem), \
	(headp)->aq_fowd = (elem) \
))
#define DEL_QUEUE(elem) ((void)( \
	(elem)->aq_fowd->aq_back = (elem)->aq_back, \
	(elem)->aq_back->aq_fowd =  (elem)->aq_fowd, \
	(elem)->aq_fowd = NULL, \
	(elem)->aq_back = NULL \
))

/*
 * Tuneable objects.
 */
extern uint_t numaio;		/* Number of kernel aio_t structs. */
extern uint_t aio_listio_max;	/* Max number of jobs in a listio. */
extern uint_t listio_size;	/* Max size of a list i/o request. */

/* Hooks for address space operations. */
extern void	aio_intersect(struct as *as, vaddr_t base, size_t size);
extern void	aio_as_free(struct as *as);

/* Exported entry points. */
extern int	aio_init(void);
extern int	aio_open(dev_t *dev_p, int flags, int otype,
			 struct cred *cred_p);
extern int	aio_close(dev_t dev, int flag, int otyp, struct cred *cred_p);
extern int	aio_ioctl(dev_t dev, int cmd, int arg, int mode,
			  struct cred *cred_p, int *rval_p);

#endif /* _KERNEL */

#if defined(__cplusplus)
        }
#endif

#endif /* _IO_AIOSYS_H */
