#ifndef _CLUSTER_H
#define _CLUSTER_H

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef _KERNEL_HEADERS
#include <svc/time.h>
#else
#include <sys/time.h>
#endif /* _KERNEL_HEADERS */

/* clusternode state valid values */
#define	CLUSTERNODE_NEVERUP	0x1
#define	CLUSTERNODE_COMINGUP	0x2
#define	CLUSTERNODE_UP		0x3
#define	CLUSTERNODE_SHUTDOWN	0x4
#define	CLUSTERNODE_GOINGDOWN	0x5
#define	CLUSTERNODE_KCLEANUP	0x6
#define	CLUSTERNODE_UCLEANUP	0x7
#define	CLUSTERNODE_DOWN	0x8

/* valid action flags for setnodeinfo() */
#define	CLUSTERNODE_SETSTATE	1

/* valid values for why field in cluster_transinfo */
#define	CLUSTER_TRANSWHY_API		0x1
#define	CLUSTER_TRANSWHY_SYSTEM		0x2
#define	CLUSTER_TRANSWHY_UNEXPECTED	0x3

/* valid values for which field in cluster_detailedtransition() */
#define CLUSTER_TRANSWHICH_FIRST	0x1  /* Get first transition. */
#define CLUSTER_TRANSWHICH_NEXT		0x2  /* Get transition after transid */
#define CLUSTER_TRANSWHICH_THIS		0x3  /* Get this transid's transition */
#define CLUSTER_TRANSWHICH_LAST	0x4  /* Get latest transition */
#define CLUSTER_TRANSWHICH_PREVIOUS	0x5  /* Get transition before transid */

typedef u_long transid_t;

struct cluster_transinfo {
	transid_t	transid;
	struct timeval	transtime;
	clusternode_t	transnode;
	ulong		fromstate;
	ulong		tostate;
	int		why;
};

typedef struct cluster_transinfo cluster_transinfo_t;

struct clusternode_info { 
	clusternode_t		node_num;
	ulong			node_state;
	ulong			node_previous_state;
	int			node_transwhy;
	transid_t		node_lasttransid;
	struct timeval		node_lasttranstime;
	transid_t		node_firsttransid;
	struct timeval		node_firsttranstime;
	int 			node_totalcpus;
	int			node_onlinecpus;
	int 			node_cpupower;
	int 			node_totalmemory;
};

typedef struct clusternode_info clusternode_info_t;

#ifndef _KERNEL

#define CLUSTERNAME	"/etc/clustername"


#ifdef __STDC__

/* Prototypes for libcluster API's */
extern clusternode_t clusternode_num(void); 
extern char * cluster_name(void);
extern int cluster_membership(ulong *, int, clusternode_t *);
extern int cluster_transition(ulong *, int, cluster_transinfo_t *);
extern int cluster_detailedtransition(int, ulong *, int, cluster_transinfo_t *);
extern int clusternode_info(clusternode_t, int, clusternode_info_t *);
extern int clusternode_setinfo(clusternode_t, int, int, clusternode_info_t *);
extern int clusternode_avail(clusternode_t);
extern int cluster_maxnodes(void);

/* System calls */
extern int rexecl(const char *file, ...);
extern int rexecle(const char *file, ...);
extern int rexeclp(const char *file, ...);
extern int rexecv(const char *file, char *const *argv, clusternode_t node);
extern int rexecvp(const char *name, char *const *argv, clusternode_t node);
extern int rfork(clusternode_t node);
extern int rfork1(clusternode_t node);
extern int rforkall(clusternode_t node);
extern int kill3(pid_t pid, int sig, int arg);
extern int migrate(clusternode_t node);

#else /*!__STDC__*/

/* Prototypes for libcluster API's */
extern clusternode_t clusternode_num(); 
extern char * cluster_name();
extern int cluster_membership();
extern int cluster_transition();
extern int cluster_detailedtransition();
extern int clusternode_info();
extern int clusternode_setinfo();
extern int clusternode_avail();
extern int cluster_maxnodes();

/* System calls */
extern int rexecl();
extern int rexecle();
extern int rexeclp();
extern int rexecv();
extern int rexecvp();
extern int rfork();
extern int rfork1();
extern int rforkall();
extern int kill3();
extern int migrate();

#endif /*__STDC__*/

#endif /* !_KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif /* _CLUSTER_H */
