#ifndef _UTIL_CMN_ERR_H	/* wrapper symbol for kernel use */
#define _UTIL_CMN_ERR_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/i386/uts/util/cmn_err.h /main/uw7_nj/1"
#ident	"$Header: /src/nsc_uw7/usr/src/i386_nsc/uts/util/cmn_err.h,v 1.5 2000/03/06 05:58:26 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef _KERNEL_HEADERS

#include <util/ksynch.h>	/* REQUIRED */

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/ksynch.h>		/* REQUIRED */

#endif /* _KERNEL_HEADERS */

#if defined(_KERNEL)

/*
 * Common error handling severity levels
 */

#define CE_CONT  0	/* continuation	*/
#define CE_NOTE  1	/* notice	*/
#define CE_WARN  2	/* warning	*/
#define CE_PANIC 3	/* panic	*/

#define VA_LIST void *
#define VA_START(list, name) list = \
  (void*)((char*)&name+((sizeof(name)+(sizeof(int)-1))&~(sizeof(int)-1)))
#define VA_ARG(list, mode) ((mode *) \
  (list=(void*)((char*)list+sizeof(mode))))[-1]

/*
 * Machine-dependent macros for s_assfail.
 *  (1)	S_ASSFAIL_ARGDECL declares the general registers in the
 *	order in which they are pushed on the stack
 *
 *  (2)	S_ASSFAIL_FMT is the format used by the cmn_err call which
 *	actually prints out the general registers for the assertion failure
 *
 *  (3)	S_ASSFAIL_ARGVAL list the general registers as they are passed to
 *	cmn_err to print the assertion failure.
 */
#define	S_ASSFAIL_ARGDECL	uint_t edi, uint_t esi, uint_t ebp, \
				uint_t esp, uint_t ebx, uint_t edx, \
				uint_t ecx, uint_t eax

#define	S_ASSFAIL_FMT		"\teax:%8X ebx:%8X ecx:%8X edx:%8X\n\tesi:%8X edi:%8X ebp:%8X esp:%8X"

#define	S_ASSFAIL_ARGVAL	eax, ebx, ecx, edx, esi, edi, ebp, esp

#ifdef __STDC__
extern void cmn_err_init(void);
/*PRINTFLIKE2*/
extern void cmn_err(int, const char *, ...);
/*PRINTFLIKE1*/
extern void printf(const char *, ...);
/*PRINTFLIKE1*/
extern void debug_printf(const char *, ...);
extern boolean_t debug_output_aborted(void);
extern int conslog_set(int);

#ifdef NSC
/*PRINTFLIKE1*/
extern void panic(const char *, ...);
#endif

#else /* !__STDC__ */
extern void cmn_err();
#endif

extern int putbufrpos;
extern int putbufwpos;
extern lock_t putbuf_lock;

/*
 * console logging attribute (see conslog_set())
 */
#define	CONSLOG_DIS	0
#define	CONSLOG_ENA	1
#define	CONSLOG_STAT	2

/*
 * Global data structures to handle cmn_err() calls from NMI handlers
 * to prevent deadlock on cmn_err_lock and putbuf_lock. See the notes at
 * the top of cmn_err.c.
 */

/*
 * Number of special buffers set aside for NMI handlers.
 */
#define NMI_MAX_BUFFERS		8
/*
 * Size of each buffer. Although the implementation allows a single message
 * to span more than 1 buffer, the buffer size has been chosen to accommodate
 * any message as of the date of this implementation.
 */
#define NMI_BUFFER_SIZE		128

/*
 * Per CPU data structures.
 */

/*
 * The Intel Architecture Software Developer's Manual - Vol 3 says that
 * when an NMI occurs, the x86 processor disables further interrupts including
 * another NMI until the NMI handler returns (via iret). So, on Intel, we
 * cannot have a stack of NMIs. But on some other architecture, this may be
 * possible. The implementation allows NMIs to be stacked. By modifying the
 * following define, we could control the number of NMIs that we allow to
 * stack.
 */
#define NMI_MAX_DEPTH	1

/*
 * Data structures used to maintain state information. Since state needs
 * to be maintained on every CPU, these are per-CPU.
 *
 * nmi_index
 *	contains the number of NMIs stacked up on a cpu. This is
 *	incremented (and decremented) by nmi() for every NMI.
 * nmi_guard
 *	incremented before accessing putbuf, decremented afterwards.
 *	This informs an NMI whether or not it has interrupted a
 *	critical section. If this is set, then the NMI handler is
 *	compelled to put its cmn_err() message in a separate buffer
 *	as noted above.
 * nmi_cmn_guard
 *	incremented before acquiring cmn_err_lock, decremented after releasing
 *	cmn_err_lock. This informs the NMI whether or not it is safe for it
 *	to acquire this lock. This lock serializes putchar(). If it is safe
 *	for the NMI handler to acquire this lock, it should acquire it.
 *	Otherwise, its cmn_err() message could get interleaved with another
 *	one from a different CPU.
 * nmi_buf_index
 *	index of the buffer (in the array of special buffers set aside for NMI
 *	handlers). Since the implementation allows NMIs to be stacked
 *	we need one per NMI.
 */
typedef struct {
	int	nmi_index;
	int	nmi_guard;
	int	nmi_cmn_guard;
	int	nmi_buf_index[NMI_MAX_DEPTH+1];
} nmi_variables_t;

/*
 * Enter NMI. Called from nmi().
 */
#define nmi_enter()	\
	atomic_int_incr(&nmi_variables[myengnum].nmi_index)

/*
 * Exit from NMI. Called from nmi().
 */
#define nmi_exit()	\
	atomic_int_decr(&nmi_variables[myengnum].nmi_index)

/*
 * Called by cmn_err() to check if it is being called by an NMI handler.
 */
#define nmi_entered()	\
	(nmi_variables[myengnum].nmi_index != 0)

/*
 * Used to guard putbuf access in the OSM driver, the log driver and
 * cmn_err() from NMI handlers.
 */
#define nmi_guard()						\
{								\
	DISABLE_PRMPT();					\
	atomic_int_incr(&nmi_variables[myengnum].nmi_guard);	\
}

/*
 * See the comment above.
 */
#define nmi_unguard()						\
{								\
	atomic_int_decr(&nmi_variables[myengnum].nmi_guard);	\
	ENABLE_PRMPT();						\
}

/*
 * Called by cmn_err() to check if it has been called by an NMI handler
 * which has interrupted a critical section.
 */
#define nmi_guarded()	\
	(nmi_variables[myengnum].nmi_guard > 1)

/*
 * Called by output() to check if it needs to do output to a special buffer.
 */
#define nmi_output_required()	\
	((nmi_variables[myengnum].nmi_index > NMI_MAX_DEPTH) ? B_TRUE :	\
			(nmi_variables[myengnum].nmi_guard > 1))

/*
 * Used to guard the cmn_err_lock from an NMI handler.
 */
#define nmi_cmn_guard()							\
{									\
	DISABLE_PRMPT();						\
	atomic_int_incr(&nmi_variables[myengnum].nmi_cmn_guard);	\
}

/*
 * See the comment above.
 */
#define nmi_cmn_unguard()						\
{									\
	atomic_int_decr(&nmi_variables[myengnum].nmi_cmn_guard);	\
	ENABLE_PRMPT();							\
}

/*
 * Called by cmn_err() to check if it has been called by an NMI handler
 * which has interrupted a critical section in which cmn_err_lock is
 * acquired.
 */
#define nmi_cmn_guarded()	\
	((nmi_variables[myengnum].nmi_cmn_guard > 1) || (nmi_variables[myengnum].nmi_guard > 0))

/*
 * Called by xprintf() to initialize nmi_buf_index. This makes sure that
 * every NMI handler message that needs to go to a special buffer goes to
 * a new buffer.
 */
#define nmi_init_buf()	\
	((void) ((nmi_variables[myengnum].nmi_index <= NMI_MAX_DEPTH) ?	\
		nmi_variables[myengnum].nmi_buf_index[nmi_variables[myengnum].nmi_index] = -1 : 0))

extern nmi_variables_t	nmi_variables[];

extern int	nmi_alloc(void);
extern void	nmi_output(char);
extern void	nmi_sync(void);
extern pl_t	putbuf_lock_acquire(void);
extern void	putbuf_lock_release(pl_t);
extern pl_t	cmn_err_lock_acquire(void);
extern void	cmn_err_lock_release(pl_t);

/*
 * _DEBUG_PRINTF hook, for calling a kernel debugger printing function.
 * See debug_printf() for details.  Should only be invoked from debug_printf().
 *
 * _DEBUG_OUTPUT_ABORTED hook to check if the debugger has aborted the
 * output for the current command.
 */
#ifndef NODEBUGGER
#ifdef _KERNEL_HEADERS
#include <util/kdb/kdebugger.h>	/* PORTABILITY */
#endif
#define _DEBUG_PRINTF(fmt, va)	kdb_printf(fmt, va)
#define _DEBUG_OUTPUT_ABORTED()	kdb_check_aborted()
#endif

#endif /* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif /* _UTIL_CMN_ERR_H */
