/*
 *	Copyright (c) Tandem Computers Incorporated 1998.
 *
 *	The computer program listings and specification herein are the
 *	property of Tandem Computers Incorporated and are not to be
 *	reproduced or copied in whole or in part for any reason without
 *	written permission of Tandem.
 */

/* max number of metric pairs that can multiplexed per register */
/* (16  is a nice round number) */
#define MAX_MULTI_METRICS         16

/* The events and the enable bits will be or'ed together */
/* to form the contents of the control register */
/* The events bits must be in the correct position (bits 0 - 7) */
/* The events bits must also include the unit mask (bits 8 - 15) */
/* The enable bits must contain the OS and/or USR bits (bits 16 - 17) */
/* The enable bit (bit 22) will be set in the kernel */
struct cpuregs_events
{
    unsigned int event0;
    unsigned int event1;
    unsigned int enable0;
    unsigned int enable1;
};

struct cpuregs_set_params
{
    unsigned int cpus;  /* number of CPUs used to allocate counter array */
    unsigned int ticks; /* ticks between register reads */
                        /* all the events */
    struct cpuregs_events events[MAX_MULTI_METRICS];
};

/* all the counters are 64-bit */
struct cpuregs_counters
{
    unsigned int counter0[2];
    unsigned int counter1[2];
    unsigned int timer[2];
};

/* counter array for one CPU */
struct cpuregs_get_params
{
    struct cpuregs_counters counters[MAX_MULTI_METRICS];
};

