/*
 * cts.h
 *
 *	This file contains Cluster Time Synchronization
 *	data repository definitions and I/O interface support.
 *
 *	This file is used by both the kernel and user apps.
 */

/*
 *      Copyright (c) Tandem Computers Incorporated 1995.
 *
 *      The computer program listings and specification herein are the
 *      property of Tandem Computers Incorporated and are not to be
 *      reproduced or copied in whole or in part for any reason without
 *      written permission of Tandem.
 */

#ident "@(#)$Header: /src/nsc_uw7/usr/src/nsc/uts/net/nsc_cts/cts.h,v 1.10 2000/03/06 06:07:39 zabby Exp $"

#ifndef _NSC_CTS_CTS_H
#define _NSC_CTS_CTS_H

#ifdef _KERNEL_HEADERS

#include <util/types.h>
#include <util/ksynch.h>
#include <svc/time.h>
#include <nsc/nsc_config.h>

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/types.h>
#include <sys/ksynch.h>
#include <sys/time.h>
#include <sys/nsc_config.h>

#else

#include <sys/time.h>   /* SVR4.0COMPAT */

#endif /* _KERNEL_HEADERS */

#ifdef _KERNEL

#define	IMMEDIATE_STIME			/* in order to pass UniSoft.. */
#if 0
#define DEAL_WITH_MISSED_TICKS		/* while we are still missing ticks */
#endif
#define	FILTER_DRIFT_RATES		/* include a filter for drift rate
					   estimates */
#define	RESET_BAD_XTAL_FLAGS		/* until we get really stable clocks */
#define	CTS_INTEL			/* until I know the correct #define */
#define CTS_DEBUG
#define volatile_externs_are_broken


/*
 *  Two types of debug output is available: printfs (for when things are
 *  badly hosed, the system hangs, or strlog isn't reliable), and STRLOG
 *  (for when things are essentially running but not quite right).  printfs
 *  severely impact the accuracy and operation of CTS (they take relatively
 *  forever to print output on the console) and so STRLOG must be used to
 *  fine tune CTS.
 *
 *  The level of debug output can be controlled from ctsm -g.  The five
 *  STRLOG levels are always enabled.  The more serious debugging
 *  is only available when CTS_DEBUG is compiled in.
 *
 *  CTS_DBG0 and CTS_DBGP0 should always show up no matter what.
 */

#define CTS_DBG0(xpres)		xpres
#define CTS_DBG1(xpres)		if ((cts_debug & 0xff) >= 1) xpres
#define CTS_DBG2(xpres)		if ((cts_debug & 0xff) >= 2) xpres
#define CTS_DBG3(xpres)		if ((cts_debug & 0xff) >= 3) xpres
#define CTS_DBG4(xpres)		if ((cts_debug & 0xff) >= 4) xpres
#define CTS_DBG5(xpres)		if ((cts_debug & 0xff) >= 5) xpres

#ifdef CTS_DEBUG
#define CTS_DBGP0(xpres)	xpres
#define CTS_DBGP1(xpres)	if ((cts_debug & 0xff00) >= 0x100) xpres
#define CTS_DBGP2(xpres)	if ((cts_debug & 0xff00) >= 0x200) xpres
#else
#define CTS_DBGP0(x)
#define CTS_DBGP1(x)
#define CTS_DBGP2(x)
#endif

#define US_PER_TICK	(MICROSEC/HZ)	/* the duration of the system clock
					   tick in microseconds. */
#define NS_PER_TICK	(NANOSEC/HZ)	/* and in nanoseconds. */

#define NS_PER_US	1000		/* nanoseconds per microsecond */
#define NS_PER_MS	1000000		/* nanoseconds per millisecond */

#define PLCTS		PLSTR		/* block {disk&stream} ints */

#define abs(x) ((x)>=0 ? (x) : -(x))	/* <macros.h> is causing problems */

/*
 * ADJUSTABLE defines   (change these if crystal tolerances change,
 *                      accuracy requirements change, or if the comm
 *                      transport improves significantly) */

#ifdef CTS_INTEL
#define MAX_XTAL	(150 * 1000)	/* maximum expected crystal tolerance
					   in parts per billion */
#endif
#ifdef CTS_PUMA
#define MAX_XTAL	(25 * 1000)	/* maximum expected crystal tolerance
					   in parts per billion */
#endif

#define MAX_DRIFT	(MAX_XTAL * 2)	/* maximum expected drift rate between
					   any two machines.  (should be 2
					   times the crystal tolerance) */

#define REQ_ACCURACY	5000000		/* master-slave accuracy requirements
					   (in nanoseconds) - should be half
					   of the Stated Requirement so that
					   any arbitrary PAIR of slaves will
					   be within spec */

#define EXP_MAX_DELAY	2400000		/* maximum allowed round trip delay
					   for sync messages (in nanoseconds).
					   needs to at least be less than half
					   of REQ_ACCURACY.  used to reject
					   very late sync messages.  must allow
					   well over 25% of msgs to be accepted
					   even with heavy traffic
					   (see REQ_ATTEMPTS) */

#define ABSURD_MIN_DELAY  10		/* an absurdly small minimum possible
					   round trip delay (in nanoseconds)
					   (essentially to eliminate zero
					   values from showing up in the
					   transport stats. */

#ifdef CTS_PUMA
#define REQ_ATTEMPTS	4		/* minimum number of attempts that
					   should be made to sync with slaves
					   within the TOO_LONG time (below) */
#endif
#ifdef CTS_INTEL
#define REQ_ATTEMPTS	1		/* Intel crytals are so bad, we gotta
					   have room for long adjtime().
					   Id like to make this at least 2 but
					   MAX_XTAL has to be 100ppm first */
#endif

#define ADJ_TIME	1		/* time (in seconds) reserved within
					   the sync cycle for adjtime() */

#define TICKADJ	(US_PER_TICK/64)	/* The CTS slave node adjustment rate
					   (in nanoseconds per tick).  This
					   needs to be big enough to make at
					   least one adjsutment of REQ_ACCURACY
					   size well within ADJ_TIME.
					   1/64th correspeonds to about 15ms
					   per second or a 1.5% skew rate */

#define FIRST_NODE	1		/* smallest node number we will ever
					   see for a node */

#define LAST_NODE  NSC_MAX_NODE_VALUE	/* biggest node we will ever get */

#if (LAST_NODE < FIRST_NODE)
oops hey_developer_LAST_NODE_is_less_than_FIRST_NODE!
#endif
#define MAX_NODES	(LAST_NODE+1)

#define NULL_NODE	(FIRST_NODE-1)	/* a node number that can't happen.
					   note: code below assumes NULL_NODE
					   is 1 less than FIRST_NODE.. */

#define CYCLE_TIME	4		/* amount of time (in seconds) required
					   for a complete sync cycle.  this
					   value could probably be computed but
					   I don't know how to "round up" with
					   the C preproc.  The equation is
					   actually:
	CYCLE_TIME =
		US_PER_TICK * 256	   : time to sync 256 nodes
		/ 1,000,000		   : scaled from microsecs to seconds
		+ ADJ_TIME		   : plus time reserved for adjtime()
		(round up)		   : rounded up to a power of two */

					/* 256 is used instead of LAST_NODE
					   so that operation of CTS will not
					   change when more than 16 nodes
					   (or whatever the current limit) is
					   supported in a future OS release.
					   this is a debatable decision. */

#define CORRECTION_RATE 8		/* if this many corrections per 8 min.
					   (see CTS_ATTEMPT_CEILING in cts_fsm)
					   are being made, then there is
					   probably an external time sync
					   daemon running */

/*
 *  Weighted average constants (see cts_do_slave_stats() below)
 */
#define FRACBITS	8
#define NORMALIZE(i)	((i) << FRACBITS)
#define DAMPBITS	7		/* set damping to 1/128th */
#define RESMASK		((1 << DAMPBITS) - 1)
/* We don't need to limit the rolling average input value */
/* so long as EXP_MAX_DELAY is less than MAX_AVG_INPUT. */
/* FRACBITS has been adjusted to that EXP_MAX_DELAY can be */
/* as much as 2500000ns (2.5ms) without overflowing the avg math. */
/* #define MAX_AVG_INPUT	((1 << (32 - FRACBITS)) - 1) */

/******************************/
/*
 * COMPUTED defines     (will change appropriately when above are changed)
 */

#if ((REQ_ACCURACY / 2) < EXP_MAX_DELAY)
oops hey_developer_REQ_ACCURACY_cannot_be_met_with_existing_EXP_MAX_DELAY!
#endif

#define CRITICAL_DRIFT  (REQ_ACCURACY / CYCLE_TIME)
					/* the absloute worst case drift
					   that can be allowed before CTS
					   will most certainly fail */

#define TOO_LONG	((REQ_ACCURACY - EXP_MAX_DELAY) / MAX_DRIFT)
					/* number of seconds after which some
					   slave could be outside of the spec.
					   if we've gone this long without an
					   update, we may have a a "fatal"
					   CTS error */

#define MAX_ADJ_TIME	(TOO_LONG - (REQ_ATTEMPTS * CYCLE_TIME))
					/* maximum time (in seconds) that the
					   system can suspend updates and fool
					   around doing an adjtime(2) */


/*
 *  determine the number of ulongs we need to pack
 *  1 flag per node into a bit array
 */
#define BITS_PER_ULONG	(sizeof(ulong) * 8)
#define NLGS		(((MAX_NODES - FIRST_NODE) + BITS_PER_ULONG - 1) \
				/ BITS_PER_ULONG)

/******************************/

/*
 *  CTS msg data types
 */

/*
 * Info messages are sent out by masters asynchronously.
 */
typedef struct {			/* MATCHES cts_chng_msg */
	timestruc_t	mark;		/* time when change is to occur */
	timestruc_t	stime;		/* if stime change is needed */
	adjstruc_t	adj;		/* if a system adjtime is needed */
	long		drift;		/* pass on estimated drift for node */
	ulong		bad_xtals[NLGS];/* bit array indexed by node number */
} cts_info_msg_t;

/*
 * Change messages are sent out by masters asynchronously.
 */
typedef struct {			/* MATCHES cts_info_msg */
	timestruc_t	when;		/* when stime was to have occured */
	timestruc_t	stime;		/* if stime change is needed */
	adjstruc_t	adj;		/* if a system adjtime is needed */
} cts_chng_msg_t;

/*
 * Sync messages are sent in two cases:
 *	- Time request messages can happen between any two nodes any time.
 *	  No node can ask for more than one time req from another node at
 *	  a time.
 *	- Time change messages are function-shipped stime(2), and adjtime(2)
 *	  calls.  Any slave is allowed to attempt only one time-change at a
 *	  time.
 */
typedef struct {
	timestruc_t	stamp1;		/* 1st master hrestime stamp */
	timestruc_t	stamp2;		/* 1st slave hrestime stamp */
	timestruc_t	stamp3;		/* 2nd master hrestime stamp */
} cts_sync_msg_t;

/*
 * Response message contains return information for the initiating request.
 */
typedef struct {
	long	rval;			/* return value */
} cts_rval_msg_t;

/*
 * Msg flags
 */
#define MSGFLG_LASTADJ	0x01


/*
 *  Per node storage is allocated at boot time and is never released.
 */
typedef struct {
	cts_sync_msg_t	tmsg;		/* gettime() return results */
	int		tstate;		/* state of tmsg */
	event_t		tevent;		/* completion event for tmsg */
	rwsleep_t	trwslp;		/* r/w sleep lock for time req */
} cts_per_node_buffers_t;

#define T_REQ		1		/* time request is in progress */
#define T_DONE		2		/* time request data is available */
#define T_TIMEOUT	3		/* time request timed out */

typedef struct cts_extra_node_data {
	int	corrections;		/* corrections from this node */
	int	ign_sync;		/* ignored syncs from this node */
} cts_extra_node_data_t;

#ifdef FILTER_DRIFT_RATES

#define FILTER_MAX_SAMPLES	6	/* needs to be an even number */

typedef struct {
	long	sample[FILTER_MAX_SAMPLES];	/* filter sample set */
	int	nsamples;		/* number of valid samples */
} cts_filter_data_t;

#endif /* FILTER_DRIFT_RATES */

#endif  /* _KERNEL */


/*
 *  CTS data repository
 *
 *  Each node keeps an array of all known nodes.  Fields commented with
 *  (M) are only valid when the node returning the data is a master.
 *  This structure should be a power of two in size to optimize
 *  kernel indexes into the repository array.  It is currently 32 bytes.
 */
typedef struct {
	uint	node_up : 1;		/* this node: is up and reachable */
	uint	bad_crystal : 1;	/* ... has a bad crystal */
	uint	transport_err : 1;	/* ... has transport problems (M) */
	uint	sync_in_prog : 1;	/* ... expected to return sync (M) */
	uint	adjusting : 1;		/* ... is being adjusted (M) */
	uint	est_drift_stab : 3;	/* relative stability of drift est (M)*/
	uchar_t	attempted;		/* number of attempted syncs (M) */
	uchar_t	accepted;		/* number of accepted syncs (M) */
	uchar_t	avg_count;		/* counter for weighting average (M) */
	clock_t	last_contact;		/* lbolt time of last contact (M) */
	clock_t	last_correction;	/* lbolt time of last correction (M) */
	long	est_drift_rate;		/* slave drift relative to master (M) */
	int	min_delay;		/* smallest rnd trip delay (ns) (M) */
	int	max_delay;		/* largest rnd trip delay (ns) (M) */
	int	avg_delay;		/* average round trip delay (ns) (M) */
#ifdef _KERNEL
	cts_per_node_buffers_t *x;		/* message buffers */
#else
	caddr_t	x;				/* message buffers */
#endif
} cts_repository_entry_t;

#define	CTS_ATTEMPT_CEILING	0xff	/* ceiling value for 'attempted' */


/*
 *  Extra variables returned with the repository data that are useful to know
 */

typedef struct {
	short	version;		/* ctsio_getdata version number */
	short	max_nodes;		/* size of repository array */
	int	current_master;		/* master's node number */
	int	last_master;		/* last master's node number */
	clock_t	lbolt;			/* copy of lbolt */
	timestruc_t	time;		/* hrestime w/subtick interpolation */
	adjstruc_t	adj_pending;	/* adjustment to be scheduled */
	adjstruc_t	adj_current;	/* fraction of adj being scheduled */
	int		adj_delay;	/* delay start of adj for N cycles */
	int	skipped_updates;	/* updates skipped due to adj */
} cts_xdata_t;

typedef struct {			/* the caddr_t for CTS_GETDATA */
	cts_xdata_t	x;		/* extra data */
	cts_repository_entry_t	n[1];	/* array of repository entries */
} ctsio_getdata_t;

typedef struct {			/* the caddr_t for CTS_GETTIME */
	int		node;		/* input arg: node select */
	timestruc_t	stamp1;		/* return value: local time1 */
	timestruc_t	stamp2;		/* return value: remote time */
	timestruc_t	stamp3;		/* return value: local time2 */
} ctsio_gettime_t;

/*
 * Be sure to bump this version number if the cts_xdata_t
 * OR cts_repository_entry_t structures change.
 */
#define	CTSIO_VERSION	1


#ifdef _KERNEL
extern cts_repository_entry_t cts_node_data[];
extern cts_extra_node_data_t cts_xnode_data[];
#ifdef FILTER_DRIFT_RATE
extern cts_filter_data_t cts_drift_filters[];
#endif
#endif /* _KERNEL */


/*
 *  ioctl cmds
 */
#define	CTS_IOCTL	('c' << 8)

#define	CTS_DATASIZE	(CTS_IOCTL | 0)
#define	CTS_GETDATA	(CTS_IOCTL | 1)

#define	CTS_GETTIME	(CTS_IOCTL | 2)

#define	CTS_SETMSGLEVEL	(CTS_IOCTL | 7)
#define	CTS_RESET_STATS	(CTS_IOCTL | 8)
#define	CTS_RESET_XTALS	(CTS_IOCTL | 9)
#define	CTS_SETBADXTAL	(CTS_IOCTL | 10)
#define	CTS_SETGOODXTAL	(CTS_IOCTL | 11)
#define	CTS_KILLFSM	(CTS_IOCTL | 12)
#define	CTS_USPROF	(CTS_IOCTL | 13)
#define	CTS_USPROF_BUFSIZE	1024
#define	CTS_GETFREQ	(CTS_IOCTL | 14)
#define	CTS_SETFREQ	(CTS_IOCTL | 15)


/*
 *  STRLOG call levels used for CTS:
 *  (a single define is used in place of two strlog args)
 */
			/* kernel error (major) */
#define CTSSL_KRNLERR0  LOG_ALERT, (SL_FATAL | SL_NOTIFY | SL_CONSOLE |SL_ERROR)

			/* kernel error (minor) */
#define CTSSL_KRNLERR1  LOG_CRIT, (SL_WARN | SL_NOTIFY | SL_CONSOLE | SL_ERROR)

			/* kernel resource error */
#define CTSSL_KRNLRES   LOG_ERR, (SL_WARN | SL_NOTIFY | SL_CONSOLE | SL_ERROR)

			/* lts resource error (operation impossible) */
#define CTSSL_ICSRES    LOG_ERR, (SL_WARN | SL_TRACE | SL_CONSOLE | SL_ERROR)

			/* cts resource error (operation impossible) */
#define CTSSL_CTSRES    LOG_ERR, (SL_WARN | SL_TRACE | SL_CONSOLE | SL_ERROR)

			/* cts operation or configuration error */
#define CTSSL_ERROR     LOG_ERR, (SL_WARN | SL_TRACE | SL_CONSOLE | SL_ERROR)

			/* warning (operation failure possible) */
#define CTSSL_WARN      LOG_WARNING, (SL_WARN | SL_TRACE | SL_CONSOLE)

			/* lts error (but cts operation continues) */
#define CTSSL_ICSERR    LOG_WARNING, (SL_WARN | SL_TRACE | SL_CONSOLE)

			/* cts concerns (cts operation continues) */
#define CTSSL_CONCERN   LOG_INFO, (SL_WARN | SL_TRACE | SL_CONSOLE)

		       /* cts information */
#define CTSSL_INFO      LOG_INFO, (SL_NOTE | SL_TRACE | SL_CONSOLE)

		       /* cts debugging */
#define CTSSL_DEBUG	LOG_DEBUG, (SL_NOTE | SL_TRACE | SL_CONSOLE)


#define	STRLOG		strlog
#define	CTSM_ID		2099		/* strlog module id number */

#endif  /* _NSC_CTS_CTS_H */
