#ifndef _FS_FCNTL_H	/* wrapper symbol for kernel use */
#define _FS_FCNTL_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/fs/fcntl.h /main/uw7_nj/2"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/fs/fcntl.h,v 1.7 2000/03/06 05:51:06 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *	Copyright (c) 1982, 1986, 1988
 *	The Regents of the University of California
 *	All Rights Reserved.
 *	Portions of this document are derived from
 *	software developed by the University of
 *	California, Berkeley, and its contributors.
 */

#ifdef _KERNEL_HEADERS

#include <util/types.h>	/* REQUIRED */

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/types.h>	/* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#else

#include <sys/types.h> /* SVR4.0COMPAT */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

/*
 * Flag values accessible to open(2) and fcntl(2)
 * (the first three can only be set by open).
 */
#define	O_RDONLY	0
#define	O_WRONLY	1
#define	O_RDWR		2
#define	O_NDELAY	0x04	/* non-blocking I/O */
#define	O_APPEND	0x08	/* append (writes guaranteed at the end) */
#define	O_SYNC		0x10	/* synchronous write option */
#define	O_NONBLOCK	0x80	/* non-blocking I/O (POSIX) */

#define	O_LARGEFILE	0x80000	/* open or fcntl with large mode with off64_t */
/*
 * Flag values accessible only to open(2).
 */
#define	O_CREAT		0x100	/* open with file create (uses third open arg) */
#define	O_TRUNC		0x200	/* open with truncation */
#define	O_EXCL		0x400	/* exclusive open */
#define	O_NOCTTY	0x800	/* don't allocate controlling tty (POSIX) */



/* fcntl(2) requests */
#define	F_DUPFD		0	/* Duplicate fildes */
#define	F_GETFD		1	/* Get fildes flags */
#define	F_SETFD		2	/* Set fildes flags */
#define	F_GETFL		3	/* Get file flags */
#define	F_SETFL		4	/* Set file flags */

/*
 * Applications that read /dev/mem must be built like the kernel.  A
 * new symbol "_KMEMUSER" is defined for this purpose.
 */
#if defined(_KERNEL) || defined(_KMEMUSER)
#define	F_GETLK		14	/* Get file lock */
#define	F_O_GETLK	5	/* SVR3 Get file lock */

#else	/* user definition */

#if defined(_STYPES)	/* SVR3 definition */
#define	F_GETLK		5	/* Get file lock */
#else
#define	F_GETLK		14	/* Get file lock */
#endif	/* defined(_STYPES) */

#endif	/* defined(_KERNEL) */

#define	F_SETLK		6	/* Set file lock */
#define	F_SETLKW	7	/* Set file lock and wait */

#if !defined(_POSIX_SOURCE) 
#define	F_CHKFL		8	/* Unused */

#define	F_ALLOCSP	10	/* Reserved */
#define	F_FREESP	11	/* Free file space */

#define F_RSETLK	20	/* Remote SETLK for NFS */
#define F_RGETLK	21	/* Remote GETLK for NFS */
#define F_RSETLKW	22	/* Remote SETLKW for NFS */

#define	F_GETOWN	23	/* Get owner (socket emulation) */
#define	F_SETOWN	24	/* Set owner (socket emulation) */
#define	F_DUP2		25	/* Duplicate fildes onto a given fildes */

#define	F_GETLK64	100	/* get file lock flock64 */
#define	F_SETLKW64	101	/* Set a record lock and wait using flock64 */
#define	F_SETLK64	102	/* Set a record lock using flock64 */
#define	F_RSETLK64	103	/* Remote SETLK64 for NFS */
#define	F_RGETLK64	104	/* Remote GETLK64 for NFS */
#define	F_RSETLKW64	105	/* Remote SETLKW64 for NFS */
#define	F_FREESP64	106	/* Free file space for LFS */
#endif /* !defined(_POSIX_SOURCE) */ 

#define F_RDCHK         0x6001  /* XENIX rdchk() system call */

/*
 * File segment locking set data type - information passed to system by user.
 */
#if defined(_KERNEL) || defined(_KMEMUSER)

typedef struct flock64 {
	short	l_type;
	short	l_whence;
	off64_t	l_start;
	off64_t	l_len;		/* len == 0 means until end of file */
        long	l_sysid;
        pid_t	l_pid;
	long	l_pad[4];	/* reserve area */
} flock64_t;

	/* EFT definition */
typedef struct flock {
	short	l_type;
	short	l_whence;
	off32_t	l_start;
	off32_t	l_len;		/* len == 0 means until end of file */
        long	l_sysid;
        pid_t	l_pid;
	long	l_pad[4];	/* reserve area */
} flock32_t;

typedef struct o_flock {
	short	l_type;
	short	l_whence;
	long	l_start;
	long	l_len;		/* len == 0 means until end of file */
        short   l_sysid;
        o_pid_t l_pid;
} o_flock_t;

#if defined(_FSKI) && (_FSKI == 1)
typedef	flock32_t	flock_t;
#else
typedef	flock64_t	flock_t;
#undef	flock
#define	flock		flock64
#endif

#else		/* user level definition */

typedef struct
#if defined(_LARGEFILE64_SOURCE) || defined(_FILE_OFFSET_BITS) \
	|| !defined(_XOPEN_SOURCE) && !defined(_POSIX_SOURCE) \
	&& !defined(_POSIX_C_SOURCE)
		flock64
#endif
{
	short	l_type;
	short	l_whence;
	off64_t	l_start;
	off64_t	l_len;		/* len == 0 means until end of file */
	long	l_sysid;
        pid_t	l_pid;
	long	l_pad[4];	/* reserve area */
} flock64_t;

#if defined(_STYPES)
	/* SVR3 definition */
typedef struct flock {
	short	l_type;
	short	l_whence;
	off32_t	l_start;
	off32_t	l_len;		/* len == 0 means until end of file */
	short	l_sysid;
        o_pid_t	l_pid;
} flock32_t;


#else
typedef struct flock {
	short	l_type;
	short	l_whence;
	off32_t	l_start;
	off32_t	l_len;		/* len == 0 means until end of file */
	long	l_sysid;
        pid_t	l_pid;
	long	l_pad[4];	/* reserve area */
} flock32_t;

#endif	/* defined(_STYPES) */

#if !defined(_FILE_OFFSET_BITS) || (_FILE_OFFSET_BITS - 0 == 32)
typedef	flock32_t	flock_t;
#elif _FILE_OFFSET_BITS - 0 == 64
typedef flock64_t	flock_t;
#define flock	flock64
#undef F_GETLK
#define F_GETLK		F_GETLK64
#undef F_SETLKW
#define F_SETLKW	F_SETLKW64
#undef F_SETLK
#define F_SETLK		F_SETLK64
#undef F_RSETLK
#define F_RSETLK	F_RSETLK64
#undef F_RGETLK
#define F_RGETLK	F_RGETLK64
#undef F_RSETLKW
#define F_RSETLKW	F_RSETLKW64
#undef F_FREESP
#define F_FREESP	F_FREESP64
#else
#error "_FILE_OFFSET_BITS, if defined, must be 32 or 64"
#endif

#endif	/* defined(_KERNEL) */


/*
 * File segment locking types.
 */
#define	F_RDLCK	01	/* Read lock */
#define	F_WRLCK	02	/* Write lock */
#define	F_UNLCK	03	/* Remove lock(s) */
#ifdef NSC_EXNFS	/* This is part of Lockd and should be in the base */
#define F_UNLKSYS 04	/* Unlock all locks held by sysid */
#endif

/*
 * POSIX constants 
 */

#define	O_ACCMODE	3	/* Mask for file access modes */
#define	FD_CLOEXEC	1	/* close on exec flag */

#if defined(__cplusplus)
        }
#endif
#endif	/* _FS_FCNTL_H */
