#ifndef _FS_NAMEFS_NAMENODE_H	/* wrapper symbol for kernel use */
#define _FS_NAMEFS_NAMENODE_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/fs/namefs/namenode.h /main/uw7_nj/1"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/fs/namefs/namenode.h,v 1.7 2000/03/06 05:51:40 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef _KERNEL_HEADERS

#include <util/types.h>	/* REQUIRED */
#include <util/ksynch.h>	/* REQUIRED */
#include <fs/vnode.h>	/* REQUIRED */
#include <acc/dac/acl.h>	/* REQUIRED */
#include <util/sysmacros_f.h>

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/types.h>	/* REQUIRED */
#include <sys/ksynch.h>	/* REQUIRED */
#include <sys/vnode.h>	/* REQUIRED */
#include <sys/acl.h>	/* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

/*
 * This structure is used to pass a file descriptor from user
 * level to the kernel. It is first used by fattach() and then
 * be NAMEFS.
 */
struct namefd {
	int fd;
};

#if defined(_KERNEL) || (_KMEMUSER)
#ifdef NSC_NAMEFS
struct nodelist {
	clusternode_t nl_node;
	struct nodelist *nl_next;
};
#endif /* NSC_NAMEFS */

/*
 * Each NAMEFS object is identified by a struct namenode/vnode pair.
 */
struct namenode {
	struct vnode    nm_vnode;	/* represents mounted file desc.*/
	ushort		nm_flag;	/* flags defined below */
	struct vattr    nm_vattr;	/* attributes of mounted file desc.*/
	struct vnode	*nm_filevp;	/* file desc. prior to mounting */
	struct file	*nm_filep;	/* file pointer of nm_filevp */
	struct vnode	*nm_mountpt;	/* mount point prior to mounting */
	struct namenode *nm_nextp;	/* next link in the linked list */
	struct namenode *nm_backp;	/* back link in linked list */
	struct acl	*nm_aclp;	/* ACL entries */
	rwsleep_t	nm_lock;	/* protects namenode */
#ifdef NSC_NAMEFS
	clusternode_t	nm_server;		/* Server node */
	struct nodelist *nm_remlist;	/* List of interested remote nodes */
#endif /* NSC_NAMEFS */
};

#endif /* _KERNEL || _KMEMUSER */

#ifdef _KERNEL

/*
 * Valid flags for namenodes.
 */
#define NMUNMOUNT	 01	/* the namenode is unmounted */
#define NMREMOVED	 02	/* the namenode is removed from namenode list */
#define NMREAD     	 04	/* the mount file has FREAD set */
#define NMWRITE     	010	/* the mount file has FWRITE set */

/*
 * Constants.
 */
#define	NMBSIZE		1024	/* NAMEFS block size */
#define	NMFSIZE		1024	/* NAMEFS fundamental block size */

/*
 * Macros to convert a vnode to a namenode, and vice versa.
 */
#define VTONM(vp) ((struct namenode *)((vp)->v_data))
#define NMTOV(nm) (&(nm)->nm_vnode)

#define	STREAM_LOCK(stp) LOCK((stp)->sd_mutex, PLSTR);
#define	STREAM_UNLOCK(stp, pl) UNLOCK((stp)->sd_mutex, pl);

extern int nm_tflush;		/* the frequency of flush namefs */

#ifdef NSC_NAMEFS
extern struct namenode *namefind(), *namefindbynodeid();

/*
 * This overlays the fid structure (see vfs.h)
 */
struct nmfid {
	u_short nmfid_len;
	u_long nmfid_nodeid;
};

/*
 * Making NMINODEBITS 13 results in 8192 ids per node just as the base
 * sets for NMMAXID.  This value could be larger, but would
 * result in larger bit masks being allocated.
 * We compute the NMNODEBITS so that the nodeid fits in the minor
 * of a dev_t.  This allows us to give CMS a device with major of zero
 * so that it is unique in the cluster.  If there aren't enough
 * bits in the minor we could put the node encoding using setnode().
 * This may be unavailable in the future when we have a clusterwide device
 * model without device node encoding.
 */
#define NMINODEBITS	13
#define	NMNODEBITS	(L_BITSMINOR - NMINODEBITS)
#define	NMMAXNODE	((1 << NMNODEBITS) - 1)

#if NSC_MAX_NODE_VALUE > NMMAXNODE
	#error "NSC_MAX_NODE_VALUE is too large to fit in ino_t"
#endif

/* Nodeid to unique id */
#define NITOI(i)	((i) & ((1 << NMINODEBITS) - 1))
/* Nodeid to site number */
#define NITOS(i)	((i) >> NMINODEBITS)

#endif /* NSC_NAMEFS */

extern int namefs_fstype;	/* index into vfssw returned by vfs_attach */

#endif	/* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif	/* _FS_NAMEFS_NAMENODE_H */
