#ifndef _MEM_KMEM_H	/* wrapper symbol for kernel use */
#define _MEM_KMEM_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/mem/kmem.h /main/uw7_nj/2"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/mem/kmem.h,v 1.6 2000/03/06 05:53:26 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *	public definitions for kernel memory allocator
 */

#ifdef _KERNEL_HEADERS

#include <util/types.h>		/* REQUIRED */

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/types.h>		/* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

#if defined(_KERNEL) || defined(_KMEMUSER)

/*
 * Physical Alignment Requirements structure.
 *
 * Used by routines which need to indicate alignment constraints on
 * physical addresses.
 *
 * Always allocate these structures by calling physreq_alloc(D3).  Never
 * create them statically.
 *
 * Before a physreq_t can be passed to any other routine, besides
 * physreq_free(D3), it must be passed to physreq_prep(D3).  After
 * calling physreq_prep, none of the fields should be changed.
 */
typedef struct physreq {
	/*
	 * Fields exported to the client.
	 */
	paddr_t		phys_align;	/* physical address must be a
					 * multiple of phys_align */
	paddr_t		phys_boundary;	/* if non-zero, physical addresses
					 * may not cross a multiple of
					 * phys_boundary */
	uchar_t		phys_dmasize;	/* DMA-ability requirement; 0 if none,
					 * else number of address bits used */
	uchar_t		phys_max_scgth;	/* maximum length of scatter/gather
					 * list; used only by buf_breakup
					 * in the BA_SCGTH case */

	/*
	 * This field has flags that are used only internally as well
	 * as flags exported to the client.
	 */
	uchar_t		phys_flags;	/* misc flags */

	/*
	 * The remaining fields are used only internally, and should not
	 * be accessed by physreq_alloc callers.
	 */
	void		*phys_brkup_poolp;	/* used by buf_breakup */
	uchar_t		phys_effective_dmasize;	/* effective DMA size */
	paddr64_t	phys_max_dmaaddr;	/* max addressable DMA addr. */
} physreq_t;

/*
 * phys_flags values:
 */

/* internal only flags */
#define PREQ_PREPPED	(1 << 0)	/* physreq_prep has been called;
					 * used under DEBUG only */

/* exported flags */
#define PREQ_PHYSCONTIG	(1 << 1)	/* contiguous memory required */

#endif /* _KERNEL || _KMEMUSER */

/*
 * Flag argument values for routines in the kmem_alloc() family:
 *
 * Notes:
 *	1) KM_SLEEP and KM_NOSLEEP must match SLEEP and NOSLEEP as defined
 *	   in util/param.h.
 *
 *	2) It is erroneous for a caller to specify two or more of KM_DMA,
 *	   KM_REQ_DMA, and KM_REQ_EDMA. KM_DMA is obsolete, a NOP.
 *
 *	3) The KM_DMA, KM_REQ_DMA, KM_PHYSCONTIG, and KM_REQ_EDMA flags
 *	   are not defined in the DDI, and are therefore not for use in
 *	   drivers.
 *
 *	4) The KM_FAIL_OK flag is a DDI argument to kmem_alloc_phys(),
 *	   but it is not passed to kmem_alloc() et al.
 */
#define KM_SLEEP	0	/* Can sleep to get memory */
#define KM_NOSLEEP	1	/* Cannot sleep to get memory */
#define KM_DMA		2	/* Caller prefers DMAable memory */
#define KM_REQ_DMA	4	/* Caller requires DMAable memory */
#define KM_PHYSCONTIG	8	/* Caller requires physically contiguous
				 *  memory
				 */
#define KM_REQ_EDMA	16	/* Caller requires EDMAable memory */
#define KM_FAIL_OK	32	/* Can return failure when memory is not
				 *  available.
				 */
#define KM_REQ_CG	64	/* Memory required only on specifed CG */
#if NSC_GEN
#define KM_NONCACHEABLE 128	/* caller wants non-cacheable mapping (used
				 * for mapping device I/O registers into
				 * kernel address space non-cacheable).
				 */
#endif /* NSC_GEN */
#define KM_FLAGMASK	31

/*
 * Valid kmem_alloc() family flags.
 */
#define KM_VALID_FLAGS	(KM_NOSLEEP|KM_DMA|KM_REQ_DMA|KM_PHYSCONTIG|KM_REQ_EDMA)

/*
 * _KMEM_C is defined only when compiling kma.c
 */
#if defined(_KERNEL) && !defined(_KMEM_C)

#if defined(_KMEM_STATS) || defined(_KMEM_HIST)

/*
 * instrumented kmem functions for statistics gathering
 */

#define kmem_alloc(size, flag)	\
	kmem_instr_alloc(size, (flag), __LINE__, __FILE__)
#define kmem_zalloc(size, flag) \
	kmem_instr_zalloc(size, (flag), __LINE__, __FILE__)
#define kmem_alloc_physcontig(size, physreq, flags) \
	kmem_i_alloc_physcont(size, physreq, flags, __LINE__, __FILE__)
#define kmem_alloc_physreq(size, physreq, flags) \
	kmem_i_alloc_physreq(size, physreq, flags, __LINE__, __FILE__)
#define kmem_zalloc_physreq(size, physreq, flags) \
	kmem_i_zalloc_physreq(size, physreq, flags, __LINE__, __FILE__)
#define kmem_alloc_on_cg(size, flags, cg) \
	kmem_i_alloc_on_cg(size, flags, cg, __LINE__, __FILE__)
#define kmem_free(addr, size) \
	kmem_instr_free(addr, size, __LINE__, __FILE__)
#define kmem_free_physcontig(addr, size) \
	kmem_instr_free(addr, size, __LINE__, __FILE__)

extern void *kmem_instr_alloc(size_t, int, int, char *);
extern void *kmem_instr_zalloc(size_t, int, int, char *);
extern void *kmem_i_alloc_physcont(size_t, const physreq_t *, int, int, char *);
extern void *kmem_i_alloc_physreq(size_t, const physreq_t *, int, int, char *);
extern void *kmem_i_zalloc_physreq(size_t, const physreq_t *, int, int, char *);
extern void *kmem_i_alloc_on_cg(size_t, int, cgnum_t, int, char *);
extern void kmem_instr_free(void *, size_t, int, char *);

#else /* !KMEM_STATS && !_KMEM_HIST */

extern void *kmem_alloc(size_t, int);
extern void *kmem_zalloc(size_t, int);
extern void *kmem_alloc_physcontig(size_t, const physreq_t *, int);
extern void *kmem_alloc_physreq(size_t, const physreq_t *, int);
extern void *kmem_zalloc_physreq(size_t, const physreq_t *, int);
extern void *kmem_alloc_on_cg(size_t, int, cgnum_t);
extern void kmem_free(void *, size_t);

#endif /* !KMEM_STATS && !_KMEM_HIST */

extern void kmem_advise(size_t);
extern int kmem_avail(size_t, int);
extern void *kmem_alloc_phys(size_t, const physreq_t *, paddr32_t *, int);
extern physreq_t *physreq_alloc(int);
extern void physreq_free(physreq_t *);
extern boolean_t physreq_prep(physreq_t *, int);
extern boolean_t physreq_met(const void *, size_t, const physreq_t *);

#endif /* defined(_KERNEL) && !defined(_KMEM_C) */

#if defined(__cplusplus)
	}
#endif

#endif /* _MEM_KMEM_H */
