#ifndef _FS_MOUNT_H	/* wrapper symbol for kernel use */
#define _FS_MOUNT_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/fs/mount.h /main/uw7_nj/2"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/fs/mount.h,v 1.8 2000/03/06 05:51:10 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

#ifndef NOT_NSC
#ifndef _KERNEL_HEADERS
#include <sys/nsc_defines.h>
#endif /* !_KERNEL_HEADERS */
#endif /* !NOT_NSC */

/*
 * Flags bits passed to mount(2).
 */
#define	MS_RDONLY	0x01	/* Read-only */
#define	MS_FSS		0x02	/* Old (4-argument) mount (compatibility) */
#define	MS_DATA		0x04	/* 6-argument mount */
#define MS_HADBAD	0x08	/* File system incurred a bad block */
                                /* so set s_state to FsBADBLK on umount */
#define	MS_NOSUID	0x10	/* Setuid programs disallowed */
#define MS_REMOUNT	0x20	/* Remount */
#define MS_NOTRUNC	0x40	/* Return ENAMETOOLONG for long filenames */
#define MS_SOFTMNT	0x80	/* Writes should not be sync'd to disk */
#define MS_SYSSPACE	0x800	/* mount initiated from kernel space */
/*
 * SSI clustering
 */
#define MS_CFS          0x01000 /* Stack CFS vfs on top of mounted fs */
#define MS_CFSSOFT      0x02000 /* CFS softmount */
#define MS_CFSHARD      0x04000 /* CFS hardmount */
#ifdef NSC_BUG_FIX
/*
 * 0x8000 conflicts with MS_SCO_NOEXEC defined in vfs.c.
 */
#define MS_CFSHARDDUP   0x80000 /* CFS hardmount + all writes to server */
#else /* !NSC_BUG_FIX */
#define MS_CFSHARDDUP   0x08000 /* CFS hardmount + all writes to server */
#endif /* !NSC_BUG_FIX */
#define MS_CFSINTR      0x10000 /* Hardmount is interruptible */
#define MS_CFSFORCE     0x20000 /* Force an override of the fsid */
#define MS_CFSBKUP      0x40000 /* CFS backup servers were specified */

#ifdef NSC_CFS

/* 
 * NSC - Different/extra than in UW7 (above flags in base) 
 */

#define MS_CTOPFS       0x100000    /* No Close-to-open consistency check */
#define MS_HARDMOUNT    0x200000    /* PFS hardmount semantics, keep trying */
#define MS_NOATTRCACHE  0x400000    /* No attribute caching */
#define MS_TOPFS        0x800000    /* file system caches time out */
#define MS_LOCKDPFS     0x1000000   /* Lockd daemon in place */
#define MS_CFSSYNC      0x2000000   /* Perform synchronous writes to disk. */
#define MS_INTR         MS_CFSINTR  /* Hardmount is interruptable */
#define MS_NOCNODECACHE	0x4000000   /* No cnode caching */
#define	MS_CFSSYNCONCLOSE 0x8000000   /* Sync on close semantic. */	

/* Any CFS options mask. */

#define	MS_CFSANY	(MS_CFS | MS_CFSHARD | MS_CFSHARDDUP | MS_CFSSOFT \
			 | MS_CFSFORCE | MS_CFSBKUP | MS_INTR \
			 | MS_CFSSYNCONCLOSE)

/* Unsupport options flags. */

#define UNSUP_NFS_OPTS  (MS_CFSHARD | MS_CFSHARDDUP | MS_CFSFORCE \
			 | MS_CFSBKUP | MS_CFS)

#define UNSUP_VXFS_OPTS (MS_CFS)

#endif /* NSC_CFS */

/*
 * Exit codes for file system specific mount commands.
 * The codes from 32-55 are mainly common errors.
 * The rest are specific to the file systems.
 */
#define	RET_OK		0	/* success */
#define	RET_USAGE	32	/* usage */

#define	RET_EBUSY	33	/* mount() - is already mounted */
#define	RET_EPERM	34	/* mount() - permission denied */
#define	RET_ENXIO	35	/* mount() - no such device */
#define	RET_ENOTDIR	36	/* mount() - not a directory */
#define	RET_ENOENT	37	/* mount() - no such file or directory */
#define	RET_EINVAL	38	/* mount() - is not a <fs> file system */
#define	RET_ENOTBLK	39	/* mount() - not a block device */
#define	RET_EROFS	40	/* mount() - write-protected */
#define	RET_ENOSPC	41	/* mount() - is corrupted. needs checking */
#define	RET_ENOLOAD	42	/* mount() - file system module cannot be loaded */
#define	RET_ENODEV	43	/* mount() - no such device  or write-protected */
#define	RET_MISC	44	/* mount() - misc errors */

#define	RET_TMP_OPEN	45	/* cannot open temp file */
#define	RET_TMP_WRITE	46	/* cannot write to temp file */

#define	RET_MNT_OPEN	47	/* cannot open mnttab */
#define	RET_MNT_LOCK	48	/* cannot lock mnttab */
#define	RET_MNT_TOOLONG	49	/* line in mnttab exceeds <max> characters */
#define	RET_MNT_TOOFEW	50	/* line in mnttab has too few entries */
#define	RET_MNT_TOOMANY	51	/* line in mnttab has too many entries */

#define	RET_VFS_OPEN	52	/* cannot open vfstab */
#define	RET_VFS_STAT	53	/* cannot stat vfstab */
#define	RET_VFS_NOENT	54	/* entry not in vfstab */

#define	RET_MALLOC	55	/* malloc() error - no more memory */

/* sfs and ufs specific */
#define	RET_REALPATH	61	/* realpath() error */
#define	RET_ABS_PATH	62	/* directory path must begin with "/" */

/* ufs specific */
#define	RET_O_OPTION	65	/* cannot specify both -o f and -o n */

/* nfs specific */
#define	RET_RETRY	70	
#define	RET_GIVE_UP	71	/* gave up retrying */
#define	RET_HP		72	/* specify host:path */
#define	RET_INVOP	73	/* invalid option */
#define	RET_SERV	74	/* server not responding */
#define	RET_ADDR	75	/* could not get nfs service addr */
#define	RET_SEC		76	/* could not negotiate secure protocol */
#define	RET_ACCESS	77	/* access denied */
#define	RET_NOENT	78	/* no such directory */


#if defined(__STDC__) && !defined(_KERNEL)
int mount(const char *, const char *, int, ...);
int umount(const char *);
#endif

#if defined(__cplusplus)
	}
#endif

#endif	/* _FS_MOUNT_H */
