#ident "@(#)nsc.h	Revision: 3.9	93/12/22 11:14:00"

/*
 * Copyright (c) 1993, Locus Computing Corporation
 * All rights reserved
 */

#ifndef	_NSC_GENERIC_NSC_H
#define	_NSC_GENERIC_NSC_H

#ifdef NSC_SVR42
#ifdef _KERNEL_HEADERS
#include <util/types.h>
#include <util/param.h>
#include <proc/pid.h>
#include <proc/cred.h>
#include <mem/kmem.h>
#include <fs/fcntl.h>
#include <nsc/nsc_timer.h>
#else  /* !_KERNEL_HEADERS */
#include <sys/types.h>
#include <sys/param.h>
#include <sys/pid.h>
#include <sys/cred.h>
#include <sys/kmem.h>
#include <sys/fcntl.h>
#include <sys/nsc_timer.h>
#endif /* !_KERNEL_HEADERS */

#else  /* !NSC_SVR42 */

#include <sys/types.h>
#include <sys/kmem.h>
#endif /* !NSC_SVR42 */

extern clusternode_t this_node;
extern void *nsc_generic_async_queue;

#ifdef NSC_UW7
typedef off64_t l_off_t;
#define MAXEND MAXEND64

typedef flock64_t l_flock_t;
#else  /* !NSC_UW7 */
typedef off_t l_off_t;
typedef off_t off32_t;

typedef flock_t l_flock_t;

#define TICKS()			lbolt
#define NANOSEC_PER_TICK	TICK
#endif /* !NSC_UW7 */

#define PID_DATA	(this_node << NODESHIFT)
#ifdef NSC_SVR42
#define INVALPID	NOPID
#endif

#define	NSC_YIELD()		yieldx()

#ifdef NSC_SVR42
#define NSC_COMM(_procp)	(_procp)->p_execinfo->ei_comm
#define NSC_PSARGS(_procp)	(_procp)->p_execinfo->ei_psargs
#define NSC_UID(_procp)		(_procp)->p_cred->cr_ruid
#define NSC_PID(_procp)		(_procp)->p_pidp->pid_id
#define	NSC_PFLAG(_procp)	(_procp)->p_flag
#define	NSC_PSYS		P_SYS
#define	NSC_MY_PID		u.u_procp->p_pidp->pid_id
#define	NSC_MY_LWPID		u.u_lwpp->l_lwpid
#define NSC_MY_EPID		u.u_procp->p_epid
#define NSC_MY_ELWPID		u.u_lwpp->l_elwpid
#define	NSC_CRED()		CRED()
#define NSC_SYSCRED()		sys_cred
#define	NSC_UNIQUE_ID		((long)u.u_lwpp)
#else
#define NSC_COMM(_procp)	(_procp)->p_comm
#define NSC_PSARGS(_procp)	(_procp)->p_psargs
#define NSC_UID(_procp)		(_procp)->p_uid
#define NSC_PID(_procp)		(_procp)->p_pid
#define	NSC_PFLAG(_procp)	(_procp)->p_flag
#define	NSC_PSYS		SSYS
#define	NSC_MY_PID		u.u_procp->p_pid
#define	NSC_CRED()		u.u_cred
#define NSC_SYSCRED()		XXXX		/* NSC_XXX */
#define	NSC_UNIQUE_ID		((long)u.u_procp)
#endif

#ifdef NSC_SVR4
/*
 * Signal set manipulation macros.
 */
#define SIGINITSET(_sigset)		sigemptyset(&_sigset)
#define SIGADDSET(_sigset, _sig)	sigaddset(&_sigset, _sig)
#define SIGORSET(_sigset1, _sigset2)	sigorset(&_sigset1, &_sigset2)
#define SIGISMEMBER(_sigset, _sig)	sigismember(&_sigset, _sig)


/*
 * NOTE: Code calling the sleep allocate interfaces should be coded
 *	 to check for allocation failures.  Some platforms may not
 *	 return errors in the sleep case, but NSC code should not
 *	 depend on this behavior.
 */
#define KMEM_SLEEP_ALLOC(_size)		kmem_alloc(_size, KM_SLEEP)
#define KMEM_SLEEP_ZALLOC(_size)	kmem_zalloc(_size, KM_SLEEP)
#define KMEM_SLEEP_FREE(_ptr, _size)	kmem_free((void *)_ptr, _size)

#define KMEM_NOSLEEP_ALLOC(_size)	kmem_alloc(_size, KM_NOSLEEP)
#define KMEM_NOSLEEP_ZALLOC(_size)	kmem_zalloc(_size, KM_NOSLEEP)
#define KMEM_NOSLEEP_FREE(_ptr, _size)	kmem_free((void *)_ptr, _size)
#endif /* NSC_SVR4 */

/*
 * Generic queuing/dequeuing macros for circular, doubly-linked lists
 *	- add "ep" to TAIL of list pointed at by "epp"
 *	- "nextlabel" and "prevlabel" allow any structures with
 *	  arbitrarily named next/previous fields to use this macro.
 * NOTE: The "nextlabel" and "prevlabel" arguments are NOT enclosed
 *	 within parentheses due to incompatibility with "->" notation.
 *	 The caller should only specify the exact name of the next/previous
 *	 fields for these arguments.
 */
#define NSC_CDL_ENQUEUE(epp, ep, nextlabel, prevlabel) do {		\
	if (*(epp) == NULL) {						\
		(ep)->nextlabel = (ep)->prevlabel = (ep);		\
		*(epp) = (ep);						\
	} else {							\
		(ep)->nextlabel = *(epp);				\
		(ep)->prevlabel = (*(epp))->prevlabel;			\
		(*(epp))->prevlabel->nextlabel = (ep);			\
		(*(epp))->prevlabel = (ep);				\
	}								\
} while (0)

#define NSC_CDL_DEQUEUE(epp, ep, nextlabel, prevlabel) do {		\
	ASSERT(*(epp) != NULL);						\
	ASSERT((ep)->nextlabel && (ep)->prevlabel);			\
	if (*(epp) == (ep))						\
		*(epp) = (ep)->nextlabel;				\
									\
	if (*(epp) == (ep))						\
		*(epp) = NULL;						\
	else {								\
		(ep)->prevlabel->nextlabel = (ep)->nextlabel;		\
		(ep)->nextlabel->prevlabel = (ep)->prevlabel;		\
	}								\
} while (0)

/*
 * NSC_XXX: Do these belong here? If not, where would be better?
 */
#define MODNAME_ARP			"arp"
#define MODNAME_ICMP			"icmp"
#define MODNAME_IGMP			"igmp"
#define MODNAME_IP			"ip"
#define MODNAME_LOOP			"llc_loop"
#define MODNAME_NSC_CVIP		"nsc_cvip"
#define MODNAME_NSC_GENTTY		"nsc_gentty"
#define MODNAME_NSC_GLUE		"nsc_glue"
#define MODNAME_NSC_MUX			"nsc_mux"
#define MODNAME_NSC_RIOCTL		"nsc_rioc"
#define MODNAME_NSC_SBA			"nsc_sba"
#define MODNAME_NSC_RGP			"nsc_rgp"
#define MODNAME_NSC_SHIP		"nsc_ship"
#define MODNAME_NSC_TNETSIM		"nsc_tsim"
#define MODNAME_PTY			"pts"
#define MODNAME_RAWIP			"rip"
#define MODNAME_ROUTE			"rte"
#define MODNAME_SERIAL			"iasy"
#define MODNAME_TCP			"tcp"
#define MODNAME_UDP			"udp"

#define FSNAME_FDFS			"fdfs"
#define FSNAME_MEMFS			"memfs"
#define FSNAME_NAMEFS			"namefs"
#define FSNAME_NFS			"nfs"
#define FSNAME_NSC_CFS			"nsc_cfs"
#define FSNAME_NSC_CSPECFS		"specfs"
#define FSNAME_NSC_CPROCFS		"proc"
#define FSNAME_PROCFS			"procfs"
#define FSNAME_PROFS			"processorfs"
#define FSNAME_SPECFS			"old_specfs"

extern int spawn_daemon_proc(char *, void (*)(void *), void *);
extern void exit_daemon_proc();
extern int spawn_daemon_thread(char *, void (*)(void *), void *);
extern void exit_daemon_thread();
extern struct vnode;
extern int spawn_daemon_user_proc(const char *, struct vnode *, const char *);

extern dev_t nsc_getdev(const char *, boolean_t, boolean_t);
extern struct file *nsc_dopen(dev_t, int, struct cred *, int *);

#endif /* !_NSC_GENERIC_NSC_H */
