#ident "%Z%$RCSfile: nsc_ld.h,v $	Revision: $Revision: 3.15 $	$Date: 2000/03/06 06:11:32 $ "

/*
 * Copyright (c) 1993 Locus Computing Corporation
 *        All Rights Reserved
 */

#ifndef _NSC_DP_SYS_NSCLD_H
#define _NSC_DP_SYS_NSCLD_H

#ifdef _KERNEL_HEADERS
#include <nsc/nsc.h>
#else /* !_KERNEL_HEADERS */
#include <sys/nsc.h>
#endif /* !_KERNEL_HEADERS */

#ifdef VPROC

/*****************************************************************************/

/*
 * NSC Load Leveler USER Support
 */

/*
 * Number of NSC load average values.
 */
#define NSC_AVENRUNSZ		5

/*
 * Defines for NSC load average calculations.
 */
#define AVENRUN_FSCALE	1024	/* scaling factor for the 1 minute
				 * load average */
#define AVENRUN_FSHIFT	10	/* log (FSCALE) */

/* moving window mean */
#define AVENRUN_AVE(x,newval,n)	(x) = ((((n) - 1) * (x) + (newval)) / (n))

/*
 * Enable modes for NSC_LD_SETEXEC
 */
#define	NSCLD_REXEC_ENBL	0x1	/* enable load level rexec's */
#define	NSCLD_REXEC_UID		0x2	/* debug rexec by uid */
#define	NSCLD_REXEC_INCL	0x4	/* inclusive rexec list */

#define NSCLD_NOTIMEOUT		UINT_MAX
#define NSCLD_MAXTIMEOUT	(UINT_MAX - 1)

/*
 * Max. message length for SSISYS_LD_{GET,PUT}LOAD calls.
 */
#define NSCLD_MAXMSGLEN	8192	

#define NSCLD_VQMAX		200	/* maximum queue length */
#define NSCLD_OVERRUNS_INTERVAL	100	/* interval b/t overrun notifications */

typedef struct tncld_rninfo {
	clusternode_t	node;		
	int		nrexec;		/* # rexec's available */
} tncld_rninfo_t;


/*****************************************************************************/

#ifdef _KERNEL

/*
 * NSC Load Leveler KERNEL Support
 */

/*
 * function prototypes for load-leveler EXTERNAL routines
 */
int tnc_ldlvl_init(pid_t, int);
int tnc_ldlvl_getvec(int, caddr_t, caddr_t);
int tnc_ldlvl_putvec(clusternode_t, int, caddr_t);
int tnc_ldlvl_putrcmds(int, int, caddr_t);
int tnc_ldlvl_setrexec(int, caddr_t);

/*
 * NSC average load vector 
 */
extern u_int tnc_avenrun[];

/*
 * Per node load vector
 */
typedef struct tncld_vec {
	caddr_t 		data;
	int			len;
	struct tncld_vec 	*next;
} tncld_vec_t;

/*
 * load vector queue structure 
 */
typedef struct tncld_vq {
	COND_LOCK_T	vq_lock;	/* lock on vector queue structure */
	INTR_CONDITION_T
			vq_cond;	/* cond variable for vector queue */
	tncld_vec_t	*vq_queue;	/* vector queue of load vectors */
	tncld_vec_t	*vq_tail;	/* last entry in queue */
	int		vq_cnt;		/* # entries in queue */
	int		vq_maxcnt;	/* max # of entries to store */
	int		vq_overruns;	/* # msgs rx when at capacity */
	u_int		vq_imsgcnt;	/* # msgs rx */
	u_int		vq_omsgcnt;	/* # msgs tx */
	u_int		vq_pmsgcnt;	/* # msg processed */
	u_int		vq_ptime;	/* time (ticks) of last processing */
} tncld_vq_t;
	
/*
 * rexec resource by node
 */
typedef struct tncld_rexec_node {
	clusternode_t	node;		 /* available rexec node */
	int		nrexec;		 /* # of rexecs available at node */
	struct 	tncld_rexec_node  *next; /* next node in list */
} tncld_rnode_t;

/*
 * rexec config parms
 */
typedef struct tncld_rexec_info {
	caddr_t		rcmds;		/* rexec commands list (rcmds[][]) */
	int		nrcmds;		/* # of entries in rcmds */
	tncld_rnode_t	*rnode;		/* list of nodes available for rexec */
	int		nrnode;		/* # of entries in rnode list */
	u_int		timeout;	/* when rnode entries expire */
	int		enable;		/* true if node is to rexec */
	int		inclusive;	/* inclusive cmd list */
} tncld_rinfo_t;

/*
 * load leveler information structure
 /*/
typedef struct tnc_ld {
	LOCK_T		ld_user_lock;	/* lock for user calls */
	SPIN_LOCK_T	ld_exec_lock;	/* lock for data */
	pid_t		ld_procpid;	/* pid of load-leveler process */
	int		*ld_nlistp;	/* list of nodes in cluster */
	int		ld_ncnt;	/* # nodes in node list */
	int		ld_nxt_rexec;	/* ## hm next entry for rexec */
	tncld_rinfo_t	ld_rinfo;	/* rexec info */
	int		ld_service_run;	/* load-leveler ON/OFF flag */
} tncld_t;

#endif /* _KERNEL */

#endif /* VPROC */

#endif /* !_NSC_DP_SYS_NSCLD_H */
