/*
 * Copyright (c) 1993, 1994 Locus Computing Corporation
 *        All Rights Reserved
 */

/*
 * NSC_XXX: Need to #ifdef this code, e.g. for VPROC, NSC_ICS, NSC_CLMS, etc.
 */

#ifndef _NSC_DP_SYS_TABLE_H
#define _NSC_DP_SYS_TABLE_H

#ifdef _KERNEL_HEADERS
#include <vproc/nscsys.h>
#include <nsc/nsc_config.h>
#else
#include <sys/nscsys.h>
#include <sys/nsc_config.h>
#endif /* _KERNEL_HEADERS */

/*****************************************************************************/
/* 
 * Top-level: stuff for table interface
 */

/*
 * Miscellaneous defines
 */
#define MAXLEL			(sizeof(long))	/* max element length */

/*
 * VERSION SUPPPORT
 *	When changes are necessary to a structure, e.g. additional fields
 *	are needed, a new version must be created. Each version of a structure
 *	must be given a version number. This version number is supplied
 *	by the user to indicate the version of a structure it is using.
 *	
 * Details:
 *	The table routine uses a version number that specifies its user
 *	interface. This version number corresponds to the version of the
 *	structures used by the top-level table routine.
 *
 *	Each table suboption also uses a version number that specifies its 
 *	user interface. This version number corresponds to the version of
 *	the structures used by the bottom-level table subroutines.
 *
 *	Each structure is de-version-alized by being typedef'ed to its latest 
 *	version. Both the kernel and the user process will use the 
 *	non-version-specific typedef to declare its structures. The kernel
 *	(except the table code) will always use the non-version-specific table 
 *	structures. Only the table code will have access to the different
 *	versions and coordinate between the user's version (identified by
 *	the user-supplied version number) and the kernel version (the current
 *	version defined below).
 */
#define TBL_VER1	1		/* initial version */
#define TBL_CURVER	TBL_VER1	/* current version */

/*
 * IN argument structure used for ssisys_node_table()
 */
struct tnctbl_inargs_v1 {
	identifier_t	tblin_id;
	int		tblin_index;
	clusternode_t	tblin_node;
};
typedef struct tnctbl_inargs_v1 tnctbl_inargs_t;
 
/*
 * OUT argument structure used for ssisys_node_table()
 */
struct tnctbl_outargs_v1 {
	caddr_t		tblout_addr;
	u_int		tblout_nel;
	u_int		tblout_lel;
};
typedef struct tnctbl_outargs_v1 tnctbl_outargs_t;


/*****************************************************************************/
/* 
 * Bottom-level: stuff for table suboptions
 */

/*
 *  TBL_LOADAVG
 */
#define TBLLA_CURVER	TBL_VER1
#define TBLLA_INTERVALS	3
struct tbl_loadavg_v1 {
	union {
		long	l[TBLLA_INTERVALS];
		double	d[TBLLA_INTERVALS];
	} tl_avenrun;
	int	tl_lscale;		/* 0 scale when floating point */
	long	tl_mach_factor[TBLLA_INTERVALS];
};
typedef struct tbl_loadavg_v1	tbl_loadavg_t;

/*
 * TBL_PROCINFO
 */
#define TBLPI_CURVER	TBL_VER1
#define	TBLPI_EMPTY	0	/* no process */
#define	TBLPI_ZOMB	1	/* zombie process */
#define	TBLPI_ACT	2	/* active process */
#define	TBLPI_SLEEP	3	/* awaiting an event */
#define	TBLPI_STOP	4	/* process is stopped */
struct tbl_procinfo_v1 {
	pid_t	tblpi_slot;		/* process table slot # */
	pid_t	tblpi_pid;		/* process id */
	pid_t	tblpi_ppid;		/* parent process id */
	uid_t	tblpi_uid;		/* user id */
	gid_t	tblpi_pgid;		/* process group */
	dev_t	tblpi_cttyd;		/* controlling tty number */
	pid_t	tblpi_session;		/* session id */
	int	tblpi_status;		/* process status (defined below) */
	long	tblpi_sys_cpu_sec;	/* system cpu usage (seconds) */
	long	tblpi_sys_cpu_nsec;	/* system cpu usage (nanoseconds) */
	long	tblpi_usr_cpu_sec;	/* user cpu usage (seconds) */
	long	tblpi_usr_cpu_nsec;	/* user cpu usage (nanoseconds) */
	char	tblpi_comm[MAXNAMELEN];	/* basename of the process */
	long	tblpi_node_cpu_sec;	/* cpu usage on node (seconds) */
	long	tblpi_node_cpu_nsec;	/* cpu usage on node (nanoseconds) */
	long	tblpi_mig_pending;	/* SIGMIGRATE pending on process */
};
typedef struct tbl_procinfo_v1	tbl_procinfo_t;

/*
 * TBL_VPROCINFO
 */
#define	TBLVI_CURVER	TBL_VER1
#define TBLVI_EMPTY	0		/* no process */
#define TBLVI_LOCAL	1		/* local process */
#define TBLVI_REMOTE	2		/* remote process */
struct tbl_vprocinfo_v1 {
	int	tblvi_status;
	int	tblvi_pid;
	long	tblvi_ref_cnt;
	int	tblvi_flag;
	int	tblvi_pproc_slot;
	int	tblvi_head_childl;
	int	tblvi_childl;
	int	tblvi_head_pgrpl;
	int	tblvi_pgrpl;
	int	tblvi_sessionl;
	int	tblvi_ppid;
	int	tblvi_pgid;
	int	tblvi_sid;
	int	tblvi_cttyd;
	long	tblvi_jobc;
	unsigned int	tblvi_mach_receives;
	unsigned int	tblvi_mach_sends;
	int	tblvi_foster_ppid;
	int	tblvi_head_foster_childl;
	int	tblvi_foster_childl;
};
typedef struct tbl_vprocinfo_v1	tbl_vprocinfo_t;
#ifdef VPROC_DEBUG
#define VPROC_INDEX(vp)		((vp)==NULL ? -1 : ((vp)->vp_index))
#else /* !VPROC_DEBUG */
#define VPROC_INDEX(vp)		(-1)
#endif /* !VPROC_DEBUG */

/*
 * Values used for TBL_ICS*
 */
#define TBL_ICS_NAMELEN		MAXNAMELEN

/*
 * TBL_ICSSYSINFO
 */
#define TBLIS_CURVER	TBL_VER1
struct tbl_icssysinfo_v1 {
	int     tis_num_chans;
	int     tis_num_prios;
	int	tis_max_subsvcs;
	/* client statistics */
	long    tis_clihand;
	long    tis_clirpc;
	long    tis_climsg;
	long    tis_clisigforward;
	long    tis_clisigforward_retry;
	long    tis_clithrottled;
	long    tis_clithrottled_event;
	/* server statistics */
	long    tis_svrhand_create;
	long    tis_svrhand;
	long    tis_svrrpc;
	long    tis_svrmsg;
	long    tis_svrsigforward;
	long    tis_svrsigforward_retry;
	long    tis_svrhand_nomem;
	long    tis_svrhand_nomem_norecv;
	long    tis_svrdaemon_create;
	long    tis_svrdaemon;
	long    tis_svrdaemon_active;
};
typedef struct tbl_icssysinfo_v1	tbl_icssysinfo_t;

/*
 * TBL_ICSCHANINFO
 */
#define TBLIC_CURVER	TBL_VER1
struct tbl_icschaninfo_v1 {
	int	tic_chan;
	int	tic_num_subsvcs;
	/* client */
	long	tic_clisend;
	long	tic_llclithrottled;
	/* server */
	long	tic_svrrecv;
	long	tic_svrhand_recv;
	long	tic_svrhand_norecv;
	long	tic_svrthrottled;
};
typedef struct tbl_icschaninfo_v1	tbl_icschaninfo_t;

/*
 * TBL_ICSSVCINFO
 */
#define TBLIV_CURVER	TBL_VER1
struct tbl_icssvcinfo_v1 {
	int     tisv_chan;
	int     tisv_subsvc;
	char    tisv_svc_name[TBL_ICS_NAMELEN];
	int     tisv_num_procs;
	int     tisv_clisend;
	int     tisv_svrrecv;
};
typedef struct tbl_icssvcinfo_v1	tbl_icssvcinfo_t;

/*
 * TBL_ICSPROCINFO
 */
#define TBLIP_CURVER	TBL_VER1
struct tbl_icsprocinfo_v1 {
	int     tip_chan;
	int     tip_subsvc;
	int     tip_proc_id;
	char    tip_proc_name[TBL_ICS_NAMELEN];
	int     tip_clisend;
	int     tip_svrrecv;
};
typedef struct tbl_icsprocinfo_v1	tbl_icsprocinfo_t;

/*
 * TBL_CMSINFO
 */
#define TBLCI_CURVER	TBL_VER1
struct tbl_cmsinfo_v1 {
	int		tci_errno;
	ulong		tci_mod_stamp;
	int		tci_total_entries;
};
typedef struct tbl_cmsinfo_v1	tbl_cmsinfo_t;

/*
 * TBL_CMSENTRYINFO
 */
#define TBLCE_CURVER	TBL_VER1
#define TBLCE_MAXPATH	80
#define TBLCE_MININDEX	0
struct tbl_cmsentry_v1 {
	ulong	tce_mod_stamp;
	fsid_t	tce_fsid;
	char	tce_fsname[FSTYPSZ];
	fsid_t	tce_covered_fsid;
	u_int	tce_server;
	u_int	tce_cms_server;
	u_int	tce_flags;
	int	tce_vfs_flags;
	u_long	tce_bsize;
	dev_t	tce_dev;
	char	tce_mount_path[TBLCE_MAXPATH];
	char	tce_mnton_path[TBLCE_MAXPATH];
};
typedef struct tbl_cmsentry_v1	tbl_cmsentry_t;

/*
 * TBL_SNETINFO
 */
#define TBLSN_CURVER		TBL_VER1
struct tbl_pernodeinfo_v1 {
	int		node_is_present;
	u_long	tnetOpen;
	u_long	tnetClose;
	u_long	tnetAbort;
	u_long	tnetWrite;
	u_long	tnet_amount_Write;
	u_long	tnetRead;
	u_long	tnet_amount_Read;
	u_long	lcuxprt_msg_send;
	u_long	lcuxprt_msg_callback;
	u_long	inline_data;
	u_long	outofline_data;
	u_long	rpc_sent;
	u_long	ipc_sent;
	u_long	rpc_rcvd;
	u_long	ipc_rcvd;
};
typedef struct tbl_pernodeinfo_v1     tbl_pernodeinfo_t;

struct tbl_snetinfo_v1 {
	u_long	tsn_timestamp;
	u_long	tsn_nodenumber;
	tbl_pernodeinfo_t tsn_pni[NSC_MAX_NODE_VALUE];
};
typedef struct tbl_snetinfo_v1     tbl_snetinfo_t;

extern tbl_snetinfo_t snetinfo;
extern int do_snetinfo;

#define SN_INCR(node, element) if (do_snetinfo) snetinfo.tsn_pni[node-1].element++
#define SN_ADD(node, element, amount) if (do_snetinfo) snetinfo.tsn_pni[node-1].element += amount


/*
 * TBL_ARGUMENTS
 */
#define TBLAR_CURVER	TBL_VER1

/*
 * Macro to create a version-embedded index for table suboptions
 */
#define TBL_VERBITS			8
#define TBL_MKINDEX(op, version)	(((op) << TBL_VERBITS) + (version))
#define TBL_GETOP(index)		((index) >> TBL_VERBITS)
#define TBL_GETVER(index)		((index) & ((1 << TBL_VERBITS) - 1))

/*
 * table suboptions
 * NOTE: this value will be left-shifted and then OR'ed with version number
 */
#define	TBL_LOADAVG_OPVAL		10
#define TBL_ARGUMENTS_OPVAL		20
#define	TBL_PROCINFO_OPVAL		30
#define	TBL_VPROCINFO_OPVAL		40
#define	TBL_NODEINFO_OPVAL		50
#define TBL_ICSSYSINFO_OPVAL		60
#define TBL_ICSCHANINFO_OPVAL		61
#define TBL_ICSSVCINFO_OPVAL		62
#define TBL_ICSPROCINFO_OPVAL		63
#define TBL_CMSINFO_OPVAL		70
#define TBL_CMSENTRYINFO_OPVAL		71
#define TBL_SNETINFO_OPVAL		72

/*
 * index of NSC table suboptions
 *	- version of suboption is embedded in index value
 */
#define	TBL_LOADAVG		TBL_MKINDEX(TBL_LOADAVG_OPVAL, TBLLA_CURVER)
#define TBL_ARGUMENTS		TBL_MKINDEX(TBL_ARGUMENTS_OPVAL, TBLAR_CURVER)
#define	TBL_PROCINFO		TBL_MKINDEX(TBL_PROCINFO_OPVAL, TBLPI_CURVER)
#define	TBL_VPROCINFO		TBL_MKINDEX(TBL_VPROCINFO_OPVAL, TBLVI_CURVER)
/* The following suboption is deprecated; use the Cluster Membership API */
#define	TBL_NODEINFO		TBL_MKINDEX(TBL_NODEINFO_OPVAL, TBLNI_CURVER)
#define TBL_ICSSYSINFO		TBL_MKINDEX(TBL_ICSSYSINFO_OPVAL, TBLIS_CURVER)
#define TBL_ICSCHANINFO		TBL_MKINDEX(TBL_ICSCHANINFO_OPVAL, TBLIC_CURVER)
#define TBL_ICSSVCINFO		TBL_MKINDEX(TBL_ICSSVCINFO_OPVAL, TBLIV_CURVER)
#define TBL_ICSPROCINFO		TBL_MKINDEX(TBL_ICSPROCINFO_OPVAL, TBLIP_CURVER)
#define TBL_CMSINFO		TBL_MKINDEX(TBL_CMSINFO_OPVAL, TBLCI_CURVER)
#define TBL_CMSENTRYINFO	TBL_MKINDEX(TBL_CMSENTRYINFO_OPVAL, \
					    TBLCE_CURVER)
#define TBL_SNETINFO	TBL_MKINDEX(TBL_SNETINFO_OPVAL, TBLSN_CURVER)

#endif /* !_NSC_DP_SYS_TABLE_H */
