#ident "@(#)$Header: /src/nsc_uw7/usr/src/nsc/uts/nsc/nsc_timer.h,v 3.5 2000/03/06 06:09:56 zabby Exp $"
/*
 * Copyright (c) 1993 Locus Computing Corporation
 *	  All Rights Reserved
 */

#ifndef _NSC_GENERIC_NSC_TIMER_H
#define _NSC_GENERIC_NSC_TIMER_H

/*
 * Includes moved outside of #ifdef NSC_TIMER as it has having the
 * includes change could make building break in annoying ways.
 */
 
#ifdef _KERNEL_HEADERS
#include <util/types.h>
#include <util/debug.h>
#include <util/cmn_err.h>
#else /* !_KERNEL_HEADERS */
#include <sys/types.h>
#include <sys/debug.h>
#include <sys/cmn_err.h>
#endif /* !_KERNEL_HEADERS */

#ifdef NSC_TIMER
#ifdef _KERNEL

/*
 * High resolution timer support
 */

/*
 * Patchable flag to turn on timing
 */
extern int tnc_hrt_enable_flags;
#define TNC_HRT_ENABLE_STAMPS	0x0001
#define TNC_HRT_ENABLE_PRINT	0x0002

/*
 * Flags for a specific timer structure
 */
#define	TNC_HRT_INITED		0x0001
#define	TNC_HRT_ON		0x0002

/*
 * Flags controlling init routine
 */
#define	TNC_HRT_INIT_SLEEPOK	0x0001
#define	TNC_HRT_INIT_NOSLEEP	0x0002

#define	TNC_HRT_MAXSNAPS	1000
#define	TNC_HRT_MAXSTAMPS	100
#ifdef NSC_UW7
#define TNC_GET_USEC()		tnc_usec_time()
#else
#define TNC_GET_USEC()		pit_usec_time()
#endif

/*
 * Flags to control printing
 */
#define	TNC_HRT_PRINTSNAP	1
#define	TNC_HRT_PRINTALL	2

typedef struct tnc_hrt_stamp {
	int	ths_depth;
	char	*ths_tagp;
	ulong_t	ths_usec;
} tnc_hrt_stamp_t;

typedef struct tnc_hrt_info {
	int	thi_flags;
	int	thi_cur_depth;
	int	thi_stamp_offset;
	int	thi_snap_offset;
	tnc_hrt_stamp_t	*thi_stamps[TNC_HRT_MAXSNAPS];
}tnc_hrt_info_t;

#define TNC_STAMP_HRTIMER(sp, tagp, depth)				\
	(sp)->ths_tagp = (tagp);					\
	(sp)->ths_depth = (depth);					\
	(sp)->ths_usec = TNC_GET_USEC();

#define TNC_ISENABLED_HRTIMER(ip)					\
	((ip) != NULL && (ip)->thi_flags & TNC_HRT_ON)

#define	TNC_START_HRTIMER(tagp, ip)	 				\
	if (TNC_ISENABLED_HRTIMER(ip)) {				\
		if ((ip)->thi_stamp_offset >= TNC_HRT_MAXSTAMPS - 1) {	\
			cmn_err(CE_NOTE, "HRT: full, wrapping stamps");	\
			(ip)->thi_stamp_offset = 0;			\
		}							\
		TNC_STAMP_HRTIMER(*((ip)->thi_stamps +			\
				    (ip)->thi_snap_offset) +		\
				  (ip)->thi_stamp_offset,		\
				  (tagp),				\
				  (ip)->thi_cur_depth);			\
		(ip)->thi_cur_depth++;					\
		(ip)->thi_stamp_offset++;				\
	}

#define TNC_STOP_HRTIMER(tagp, ip)					\
	if (TNC_ISENABLED_HRTIMER(ip)) {				\
		if ((ip)->thi_stamp_offset >= TNC_HRT_MAXSTAMPS) {	\
			cmn_err(CE_NOTE, "HRT: full, overwriting");	\
			(ip)->thi_stamp_offset = TNC_HRT_MAXSTAMPS - 1;	\
		}							\
		ASSERT((ip)->thi_cur_depth > 1);			\
		(ip)->thi_cur_depth--;					\
		TNC_STAMP_HRTIMER(*((ip)->thi_stamps +			\
				    (ip)->thi_snap_offset) +		\
				  (ip)->thi_stamp_offset,		\
				  (tagp),				\
				  (ip)->thi_cur_depth);			\
		(ip)->thi_stamp_offset++;				\
	}
/*
 * External functions
 */
void tnc_init_hrtimer(tnc_hrt_info_t **, int);
void tnc_deinit_hrtimer(tnc_hrt_info_t **);
void tnc_enable_hrtimer(tnc_hrt_info_t *, int);
void tnc_reenable_hrtimer(tnc_hrt_info_t *);
void tnc_disable_hrtimer(tnc_hrt_info_t *);
void tnc_print_hrtimer(int, tnc_hrt_info_t *);

#endif /* _KERNEL */
#else /* !NSC_TIMER */

#define TNC_STAMP_HRTIMER(a, b, c)
#define TNC_START_HRTIMER(a, b)
#define TNC_STOP_HRTIMER(a, b)
#define TNC_ISENABLED_HRTIMER(a)

#endif /* !NSC_TIMER */

#endif /* !_TNC_GENERIC_TNC_TIMER_H */
