#ifndef _PROC_SESSION_H	/* wrapper symbol for kernel use */
#define _PROC_SESSION_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/proc/session.h /main/uw7_nj/1"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/proc/session.h,v 1.5 2000/03/06 05:55:25 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef _KERNEL_HEADERS

#include <util/ksynch.h>	/* REQUIRED */
#include <util/types.h>		/* REQUIRED */
#include <proc/pid.h>

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/ksynch.h>		/* REQUIRED */
#include <sys/types.h>		/* REQUIRED */
#include <sys/pid.h>
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

#if defined(_KERNEL) || defined(_KMEMUSER)

#ifndef VPROC
typedef struct sess {
        lock_t s_mutex;                 /* session state lock */
        uchar_t s_ctty;                 /* TRUE iff session has or had ctty */
        ulong_t s_cttyref;              /* number of LWPs doing ops via s_vp */
	uint_t s_ref;			/* number of processes in session */
        struct vnode *s_vp;             /* tty's vnode */
        struct pid *s_pgrps;            /* non-zombie process group set */
        struct pid *s_sidp;             /* session ID info */
        struct cred *s_cred;            /* ctty allocation credentials */
} sess_t;

#define s_sid s_sidp->pid_id

#else  /* VPROC */

typedef struct ctty {
	lock_t ctty_mutex;		/* ctty state lock */
	short ctty_ref;			/* reference count */
	dev_t ctty_dev;			/* controlling tty device */
	pid_t ctty_sid;			/* ctty session */
	struct vnode *ctty_vp;		/* ctty's vnode */
	struct cred *ctty_cred;		/* ctty allocation credentials */
} ctty_t;
#endif /* VPROC */

/*
 * Enumeration of the types of access that can be requested for a
 * controlling terminal under job control.
 */

enum jcaccess {
	JCREAD,		/* read data on a ctty */
	JCWRITE,	/* write data to a ctty */
	JCSETP,		/* set ctty parameters */
	JCGETP		/* get ctty parameters */
};

#endif /* _KERNEL || _KMEMUSER */

#ifdef _KERNEL

#ifndef VPROC
/*
 * Hold an execution reference to the controlling terminal of the given session.
 * The session lock (s_mutex) is held upon entry and remains held upon exit.
 */
#define SP_CTTYHOLD(sp) \
	((sp)->s_cttyref++);

/* 
 * Release a previously acquired execution reference on the controlling
 * terminal of the given session.
 * The session lock is released at PLBASE upon return.  
 */
#define SP_CTTYREL(sp, ttyvp) { \
	(void) LOCK(&(sp)->s_mutex, PL_SESS);		\
	if (--(sp)->s_cttyref == 0 && (sp)->s_vp == NULL) { \
		if ((sp)->s_ref != 0) { \
			UNLOCK(&(sp)->s_mutex, PLBASE); \
			relectty((ttyvp), (sp)->s_cred); \
		} else { \
			UNLOCK(&(sp)->s_mutex, PLBASE); \
			relectty((ttyvp), (sp)->s_cred); \
			LOCK_DEINIT(&(sp)->s_mutex); \
			kmem_free((sp), sizeof(sess_t)); \
		} \
	} else \
		UNLOCK(&(sp)->s_mutex, PLBASE); \
}

extern sess_t *sess_create(sess_t *);

#else  /* VPROC */

/*
 * Hold an execution reference to the controlling terminal.
 */
#define CTTYHOLD(cttyp) \
	((cttyp)->ctty_ref++);

/* 
 * Release a previously acquired execution reference on the controlling
 * terminal.
 * The ctty lock is released at PLBASE upon return.  
 */
#define CTTYREL(cttyp, ttyvp) { \
	(void) LOCK(&(cttyp)->ctty_mutex, PL_SESS);	\
	if (--(cttyp)->ctty_ref == 0 && (cttyp)->ctty_vp == NULL) { \
		UNLOCK(&(cttyp)->ctty_mutex, PLBASE); \
		relectty(ttyvp, (cttyp)->ctty_cred); \
		ctty_dealloc(cttyp); \
	} else \
		UNLOCK(&(cttyp)->ctty_mutex, PLBASE); \
}

extern ctty_t *ctty_alloc();
extern void ctty_dealloc(ctty_t *);
#endif /* VPROC */

extern void freectty(void);
extern void relectty(struct vnode *, struct cred *);

#endif /* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif /* _PROC_SESSION_H */
