#ifndef _PROC_IPC_SHM_H	/* wrapper symbol for kernel use */
#define _PROC_IPC_SHM_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/proc/ipc/shm.h /main/uw7_nj/1"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/proc/ipc/shm.h,v 1.6 2000/03/06 05:55:46 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

/*
**	IPC Shared Memory Facility.
*/

#ifdef _KERNEL_HEADERS

#include <util/types.h>		/* REQUIRED */
#include <util/ksynch.h>	/* REQUIRED */
#include <util/param.h>  	/* SVR4COMPAT */
#include <proc/ipc/ipc.h>	/* REQUIRED */

#elif defined(_KERNEL)

#include <sys/types.h>		/* REQUIRED */
#include <sys/ksynch.h>		/* REQUIRED */
#include <sys/param.h>  	/* SVR4COMPAT */
#include <sys/ipc.h>		/* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#else /* user */


#if defined(_XOPEN_SOURCE)
#include <sys/types.h>
#else
#include <sys/param.h>		/* SVR4COMPAT */
#endif /* defined(_XOPEN_SOURCE)*/

#include <sys/ipc.h>
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

/*
 * Implementation constants.
 */

#define SHMLBA	_SHMLBA	/* segment low boundary address multiple */

/*
 * Padding constants used to reserve space for future use.
 */

#define	SHM_PAD		2
#define	SHM_PAD1	4

/*
 * Permission Definitions.
 */

#define	SHM_R	IPC_R	/* read permission */
#define	SHM_W	IPC_W	/* write permission */

/*
 * Message Operation Flags.
 */

#define	SHM_RDONLY	010000	/* attach read-only (else read-write) */
#define	SHM_RND		020000	/* round attach address to SHMLBA */


typedef ulong_t shmatt_t;


/*
 * Structure Definitions.
 */

#if !defined(_STYPES)

/*
 * There is a shared mem id data structure (kshmid_ds and shmid_ds) for each 
 * segment in the system.
 */
struct shmid_ds {
	struct ipc_perm	shm_perm;	/* operation permission struct */
	int		shm_segsz;	/* size of segment in bytes */
	size_t		shm_off;	/* offset for SHM_SETPLACE */
	ushort_t	shm_lkcnt;	/* number of times it is being locked */
	pid_t		shm_lpid;	/* pid of last shmop */
	pid_t		shm_cpid;	/* pid of creator */
	shmatt_t	shm_nattch;	/* used only for shminfo */
	ulong_t		shm_cnattch;	/* used only for shminfo */
	time_t		shm_atime;	/* last shmat time */
	long		shm_pad1;	/* reserved for time_t expansion */
	time_t		shm_dtime;	/* last shmdt time */
	long		shm_pad2;	/* reserved for time_t expansion */
	time_t		shm_ctime;	/* last change time */
#ifdef XXX_temporary_hack
        /*
         * XXX - with the expansion of cgid_t to 64 bits, we no longer
         * have room for everything while retaining the same structure
         * size as previously.  As a very short-term hack, we're just
         * removing shm_pad3 to make up for the extra space.  A better
         * solution is planned for BL10.
         */
        long            shm_pad3;       /* reserved for time_t expansion */
#endif
	size_t		shm_nbytes;	/* len for SHM_SETPLACE */
	int		shm_placepolicy;/* policy for SHM_SETPLACE */
	cgid_t		shm_cg;		/* cg for SHM_CPUGROUP in SHM_SETPLACE */
	uint_t		shm_granularity;/* for SHM_BALANCED in SHM_SETPLACE */
};

#else	/* _STYPES */

/* SVR3 binary compatibility shmid_ds */
struct shmid_ds {
	struct ipc_perm	shm_perm;	/* operation permission struct */
	int		shm_segsz;	/* size of segment in bytes */
	_VOID		*shm_pad0;	/* placeholder for historical shm_reg */
	ushort_t	shm_lkcnt;	/* number of times it is being locked */
	char 		pad[SHM_PAD];
	o_pid_t		shm_lpid;	/* pid of last shmop */
	o_pid_t		shm_cpid;	/* pid of creator */
	ushort_t	shm_nattch;	/* used only for shminfo */
	ushort_t	shm_cnattch;	/* used only for shminfo */
	time_t		shm_atime;	/* last shmat time */
	time_t		shm_dtime;	/* last shmdt time */
	time_t		shm_ctime;	/* last change time */
};

#endif	/* _STYPES */


#if defined(_KERNEL) || defined(_KMEMUSER)

struct fgashm_t;

struct kshmid_ds {
	struct shmid_ds kshm_ds;	/* SVR4 shared memory header */
	lock_t        	kshm_lck;	/* mutex lock for this object.	
					 * This lock potects all fields
					 * in shmid_ds, kshm_asp contents,
					 * kshm_flag, and shm_perm contents.
					 */
	struct vnode	*kshm_mvp;	/* vnode providing backing store */
	int		kshm_mapsize;	/* actual amount of backing store
					 * kshm_size >= shm_segsz (above)
					 */
	int		kshm_align;	/* byte alignment required for
					 * mapping segment
					 */
	uint_t   	kshm_refcnt;	/* reference count of this header */
	struct as 	*kshm_asp;	/* shadow as used to lock the object
					 * in memory and unlock/free all
					 * resources currently locked
					 */
	char    	kshm_flag;	/* set to SHM_BUSY to hold off all
					 * accesses to this object.
					 */
	sv_t         	kshm_sv;	/* synchronization variable for this
					 * object.
					 */
	struct fgashm_t *kshm_fgap;	/* opaque pointer to the fga shm
					 * management data structures.  SHM
					 * code can see this pointer but not
					 * dereference it.  May be able to
					 * combine this with kshm_mvp in a
					 * union (since one of the fields will
					 * always be NULL).
					 */
};

/* kshm_flag values */
#define	SHM_BUSY		0x01
				/* the shared memory segment is currently
                                 * been manipulated. This will block out
				 * shmconv and consequently all shmat, shmctl,
				 * aclipc and lvlipc calls.
				 * The other user shmdt will do explicit
				 * serialization with the above routines.
				 */
#define SHM_SUPPORTS_AFFINITY	0x02
				/* the shared memory segment supports fine
				 * grained affinity (page granular affinity
				 * ranges within the segment).  These types of
				 * shared memory segments are always locked
				 * into memory and require P_LOCK privilege
				 * to create or attach.
				 */
#define SHM_PAGE_BACKED		0x04
				/* the shared memory segment will be accessed
				 * in a page granular manner and the OS should
				 * ensure that the backing for the shared
				 * memory allows page granular manipulation.
				 * Such segments are not allowed to be backed
				 * by PSE pages.
				 */

/* SVR3 structure */
struct o_shmid_ds {
	struct o_ipc_perm shm_perm; /* operation permission struct */
	int               shm_segsz;	/* size of segment in bytes */
	_VOID		 *shm_pad0;	/* segment anon_map pointer */
	ushort_t	 shm_lkcnt;	/* number of times it is being locked */
	char 		 pad[SHM_PAD];		
	o_pid_t		 shm_lpid;	/* pid of last shmop */
	o_pid_t		 shm_cpid;	/* pid of creator */
	ushort_t	 shm_nattch;	/* used only for shminfo */
	ushort_t	 shm_cnattch;	/* used only for shminfo */
	time_t		 shm_atime;	/* last shmat time */
	time_t		 shm_dtime;	/* last shmdt time */
	time_t		 shm_ctime;	/* last change time */
};


struct	shminfo {
	int	shmmax,		/* max shared memory segment size */
		shmmin,		/* min shared memory segment size */
		shmmni,		/* # of shared memory identifiers */
		shmseg;		/* max attached shared memory	  */
				/* segments per process		  */
};

typedef enum { SHM, DSHM } shmacc_t; /* type of accounting record */
				     /* SHM or DSHM		  */

typedef struct segacct {
	struct segacct	*sa_next;	/* the next item in the link list */
	vaddr_t		 sa_addr;	/* address at which a shared memory
					 * object is attched in current address
					 * space
					 */
#ifdef NSC_IPC
	size_t		 sa_length;	/* actual size of the shared memory
					 * object in bytes
					 */
	int		 sa_shmid;	/* Shared memory ID of segment */
#else
	size_t		 sa_len;	/* actual size of the shared memory
					 * object in bytes
					 */
	shmacc_t	sa_type;	/* the type of shared memory object */
					/* SHM or DSHM */
	union {
		struct kshmid_ds  *__sa_kshmp; /* shm object header */
		struct kdshmid_ds *__sa_kdshmp; /* dshm object header */
	} sa_uaccp;
#endif
} segacct_t;
#define sa_kshmp sa_uaccp.__sa_kshmp
#define sa_kdshmp sa_uaccp.__sa_kdshmp

#endif	/* _KERNEL || _KMEMUSER */


/*
 * Shared memory control operations
 */
#define SHM_LOCK	3	/* Lock segment in core */
#define SHM_UNLOCK	4	/* Unlock segment */
#define SHM_SETPLACE	5	/* Change shm segment affinity */

/*
 * Extra options for shmctl(), shmget().  These were chosen so as not
 * to conflict with the options IPC_CREAT etc in ipc.h.
 */
#define IPC_SPECIAL	 0010000	/* create a fga, locked shm segment */
#define SHM_BALANCED	 0020000	/* shm affinity policy */
#define SHM_CPUGROUP	 0040000	/* shm affinity policy - near cg */
#define SHM_FIRSTUSAGE	 0200000	/* shm affinity policy - first
					 * access places page */
#define SHM_PLC_DEFAULT  0400000	/* shm sys default affinity policy */
#define SHM_MIGRATE	01000000	/* migrate shm to match new affinity */
#define SHM_FINEGRAIN	02000000	/* disallow large page backed shm */

#ifdef	_KERNEL

/*
 * Handy macros when using affinity.
 */
#define SHM_AFFINITY_MASK	(SHM_BALANCED | SHM_CPUGROUP | SHM_FIRSTUSAGE |\
				 SHM_PLC_DEFAULT)

/*
 * Macros to lock and unlock a kshmid_ds cell.
 * idp is a pointer to the kshmid_ds cell to be locked/unlocked.
 */
#define SHMID_LOCK(kshmp)	LOCK_PLMIN(&(kshmp)->kshm_lck)
#define SHMID_UNLOCK(kshmp, pl)	UNLOCK_PLMIN(&(kshmp)->kshm_lck, (pl))

struct proc;
#ifdef NSC_IPC
extern int	shmconv(int , struct kshmid_ds **, int);
#else
extern int	shmconv(int , struct kshmid_ds **);
#endif
extern void	shminit(void);
extern void	shmfork(struct proc *, struct proc *);
extern void	shmexit(struct proc *pp);
extern void	shmexec(struct proc *pp);
#ifndef NSC_IPC
extern void	shm_add(struct proc *, vaddr_t, size_t, void *, shmacc_t );
extern void	shm_del(struct proc *, vaddr_t);
#endif /* !NSC_IPC */
extern segacct_t *shm_find(struct proc *, vaddr_t);

#else

#ifdef __STDC__

void	*shmat(int, const void *, int);
int	shmctl(int, int, struct shmid_ds *);
int	shmdt(const void *);
int	shmget(key_t, size_t, int);

#else

int shmctl();
int shmget();
void *shmat();
int shmdt();

#endif /* __STDC__ */

#endif /* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif /* _PROC_IPC_SHM_H */
