/*
 *    This software is licensed by Tandem Computers Incorporated and is
 *    subject to restrictions on use, disclosure and copying. If a user
 *    is an entity of the United States Government, this software is
 *    Commercial Software and only LIMITED RIGHTS or RESTRICTED RIGHTS
 *    are granted.
 *
 *             Tandem and the Tandem logo are trademarks of
 *                   Tandem Computers Incorporated.
 *
 *             Copyright (c) Tandem Computers Incorporated, 1998
 *
 *                   Protected as an Unpublished Work.
 *
 *                         All Rights Reserved
 */
#ident	"@@(#)$Header: /src/nsc_uw7/usr/src/nsc/uts/io/nsc_tserv/ship/spad.h,v 1.5 2000/03/06 06:06:42 zabby Exp $"

/* --- tabstops=4 --- */

#ifdef _KERNEL_HEADERS
#include <io/autoconf/confmgr/cm_i386at.h>
#else
#include <sys/cm_i386at.h>
#endif /* _KERNEL_HEADERS */

/*
 * SPAD (ServerNet PCI Adapter Driver) IOCTL cmds
 */
#define SPAD_IOCTL	('S' << 8)

#define SPAD_GET_LLSC_STATS	(SPAD_IOCTL | 0)
#define SPAD_GET_LE_STATS	(SPAD_IOCTL | 1)
#define SPAD_CLR_STATS		(SPAD_IOCTL | 2)
#define SPAD_LE_ON			(SPAD_IOCTL | 3)
#define SPAD_LE_OFF			(SPAD_IOCTL | 4)
#define SPAD_GET_SHIP_REGS	(SPAD_IOCTL | 5)
#define SPAD_CHECK_PATH		(SPAD_IOCTL | 6)
#define SPAD_GET_CRNT_PATH	(SPAD_IOCTL | 7)
#define SPAD_SET_CRNT_PATH	(SPAD_IOCTL | 8)

/*
 * Link-Level Self-Check Recovery Statistics
 */
typedef struct llsc_stats {
	ulong recovery_time_min;
	ulong recovery_time_max;
	ulong recovery_total_time;
	ulong recovery_poll_mode;
	ulong recovery_intr_mode;
	ulong recovery_sail_llsc_count;
	ulong recovery_sail_xbsc_count;
} llsc_stats_t;


/* 
 * Link Exception Statistics Per Fabric
 */
#define FABRICS	2

typedef struct le_stats {
	ulong link_exception_reporting; /* 0=OFF 1=ON */
	ulong link_exception_second[FABRICS];
	ulong link_exception_crc[FABRICS];
	ulong link_exception_command[FABRICS];
	ulong link_exception_fifo[FABRICS];
	ulong link_exception_keepalive[FABRICS];
	ulong link_exception_efifo[FABRICS];
} le_stats_t;


/*
 * SPAD_CLR_STATS bit mask
 */
#define SPAD_CLR_LLSC_STATS		1
#define SPAD_CLR_LE_X_STATS		2
#define SPAD_CLR_LE_Y_STATS		4

#define SPAD_CLR_STATS_MAX		(SPAD_CLR_LLSC_STATS | \
								 SPAD_CLR_LE_X_STATS | \
								 SPAD_CLR_LE_Y_STATS)

/*
 * ServerNet Path Status Structure
 */
typedef struct path_stat {
	clusternode_t node;
	int x_status; /* 1=up 0=down */
	int y_status; /* 1=up 0=down */
} path_stat_t;

/*
 * ServerNet Current Path Structure
 */
typedef struct crnt_path {
	clusternode_t node;
	ulong path; /* 0=Path X; 1=Path Y */
} crnt_path_t;

/*
 * ServerNet PCI Adapter Registers Structure
 */
typedef struct spa_regs {
	int snet_rev;
	int snet_cin;
	int snet_node_id;
	int snet_config;
	int snet_int_cause_mask;
	int snet_link_ctl;
	int snet_activity_status;
} spa_regs_t;

/*
 * SPA Auto-Configuration Information
 */
typedef struct spa_acinfo {
	cm_num_t	ipl;
	cm_num_t	irq;
	cm_num_t	itype;
	cm_range_t	ioaddr;
	cm_range_t	memaddr;
	cm_num_t	dmac;
} spa_acinfo_t;

void ship_switch_detect(path_stat_t *);
int ship_card_check(void);
