#ifndef _UTIL_SYSMACROS_F_H	/* wrapper symbol for kernel use */
#define _UTIL_SYSMACROS_F_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/i386/uts/util/sysmacros_f.h /main/uw7_nj/1"
#ident	"$Header: /src/nsc_uw7/usr/src/i386_nsc/uts/util/sysmacros_f.h,v 1.6 2000/03/06 05:58:29 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

#ifndef NOT_NSC
#ifndef _KERNEL_HEADERS
#include <sys/nsc_defines.h>
#endif
#endif /* !NOT_NSC */

/*
 * Various family-specific system macros.
 */

#if !defined(_DDI)

/*
 * WARNING: The device number macros defined here should not be used by device
 * drivers or user software.  Device drivers should use the device functions
 * defined in the DDI/DKI interface (see also ddi.h).  Application software
 * should make use of the library routines available in makedev(3).  A set of
 * new device macros are provided to operate on the expanded device number
 * format supported in SVR4.  Macro versions of the DDI device functions are
 * provided for use by kernel proper routines only.
 */

#define O_BITSMAJOR     7       /* # of SVR3 major device bits */
#define O_BITSMINOR     8       /* # of SVR3 minor device bits */
#define O_MAXMAJ        0x7f    /* SVR3 max major value */
#define O_MAXMIN        0xff    /* SVR3 max major value */

#ifdef NSC_CSPECFS
/*
 * Since SVR4.2 really uses only 8 bits for the major and 18
 * for the minor, we'll take the remaining 6 for the node number
 * NSC_XXX:
 *	Unlike tlp5, uniware has 13-bit major numbers (0-8192).
 *	Until we find another scheme, we'll steal back our 6 bits
 *	for the node number.
 * NSC_XXXXXX:
 *	Unixware allocates a slot in the cdev table for each
 *	driver it sees. When staticly linking devices, it appears
 *	to use the major that corresponds to the index of the
 *	driver as it was found during the build. Thereforem, we need to 
 *	allow for major numbers up to 512 at this moment; and fix
 *	the problem correctly later on.
 */
#define L_BITSMAJOR     9       /* Current # of major device bits */
#define L_BITSMINOR     18      /* Current # of minor device bits */
#define L_BITSNODE	5	/* Bits left over for the node */
#define L_MAXMAJ        0x3fff  /* Max major number (w/node encoded) */
#define L_MAXMIN        0x3ffff /* Current maximum minor number */
#define L_MAXNODE	0x1f	/* Maximum node number */
#define L_NODEMASK	((u_long)L_MAXNODE << (L_BITSMAJOR+L_BITSMINOR))
#define L_BASEDEVMASK	(~L_NODEMASK)
#else /* !NSC_CSPECFS */
#define L_BITSMAJOR     14      /* Current # of major device bits */
#define L_BITSMINOR     18      /* Current # of minor device bits */
#define L_MAXMAJ	0x1fff	/* Current maximum major number;
				 * the high bit is reserved
				 */
#define L_MAXMIN        0x3ffff /* Current maximum minor number */
#endif /* NSC_CSPECFS */

#ifdef NSC
#if NSC_MAX_NODE_VALUE > L_MAXNODE
	#error "NSC_MAX_NODE_VALUE is too large to fit into fsid_t"
#endif
#endif /* NSC */

/*
 * Get internal major and minor device
 * components from expanded device number.
 */
#ifdef NSC_CSPECFS
#define _GETMAJOR(dev)	(major_t)(((dev_t)(dev) >> L_BITSMINOR) & \
				 (L_MAXMAJ >> L_BITSNODE))
#define getmajor(dev)	_GETMAJOR(dev)
#define _GETMINOR(dev)	(minor_t)((dev_t)(dev) & L_MAXMIN)
#define getminor(dev)	_GETMINOR(dev)
#define _GETNODE(dev)	(clusternode_t)(((dev_t)(dev) >> \
				(L_BITSMINOR + L_BITSMAJOR)) & L_MAXNODE)
#define getnode(dev)	_GETNODE(dev)
#define _SETNODE(dev, node) \
	(((dev_t)(dev) \
	  & ~((unsigned)L_MAXNODE << (L_BITSMINOR + L_BITSMAJOR))) \
	 | ((clusternode_t)(node) << (L_BITSMINOR + L_BITSMAJOR)))
#define setnode(dev, node)	_SETNODE(dev, node)
#else /* !NSC_CSPECFS */
#define _GETMAJOR(dev)	(major_t)(((dev_t)(dev) >> L_BITSMINOR) & L_MAXMAJ)
#define getmajor(dev)	_GETMAJOR(dev)
#define _GETMINOR(dev)	(minor_t)((dev_t)(dev) & L_MAXMIN)
#define getminor(dev)	_GETMINOR(dev)
#endif	/* NSC_CSPECFS */

/*
 * Get external major and minor device 
 * components from expanded device number.
 * For the i386 family, external == internal.
 * These interfaces return NODEV if values are out of range.
 */
#define _GETEMAJOR(dev)	(major_t)(((dev_t)(dev) >> L_BITSMINOR) > L_MAXMAJ ? \
				  NODEV : _GETMAJOR(dev))
#define getemajor(dev)	_GETEMAJOR(dev)
#define _GETEMINOR(dev)	_GETMINOR(dev)
#define geteminor(dev)	_GETEMINOR(dev)

/*
 * Macros for working with old device numbers (o_dev_t).
 */
#define o_getemajor(x)	(major_t)((((o_dev_t)(x) >> O_BITSMINOR) > O_MAXMAJ) ? \
			 NODEV : \
			 (((o_dev_t)(x) >> O_BITSMINOR) & O_MAXMAJ))
#define o_geteminor(x)	(minor_t)((o_dev_t)(x) & O_MAXMIN)

#endif /* !_DDI */

/*
 * Alignment of basic integral data types.
 */

	/*
	**	for i386
	*/
#define	SALIGN(p)		(char *)(((int)p+1) & ~1)
#define	IALIGN(p)		(char *)(((int)p+3) & ~3)
#define	LALIGN(p)		(char *)(((int)p+3) & ~3)

	/*
	**	some others (for ref)
	#ifdef	pdp11
	#define	SALIGN(p)		(char *)(((int)p+1) & ~1)
	#define	IALIGN(p)		(char *)(((int)p+1) & ~1)
	#define LALIGN(p)		(char *)(((int)p+1) & ~3)
	#endif
	#ifdef vax
	#define	SALIGN(p)		(char *)(((int)p+1) & ~1)
	#define	IALIGN(p)		(char *)(((int)p+3) & ~3)
	#define	LALIGN(p)		(char *)(((int)p+3) & ~3)
	#endif
	#ifdef	u3b2
	#define	SALIGN(p)		(char *)(((int)p+1) & ~1)
	#define	IALIGN(p)		(char *)(((int)p+3) & ~3)
	#define	LALIGN(p)		(char *)(((int)p+3) & ~3)
	#endif
	*/

/*
 * Relationship between pages and disk blocks.
 */
#define NDPP		8		/* Number of disk blocks per page */
#define DPPSHFT		3		/* Shift for disk blocks per page. */

#if defined(__cplusplus)
	}
#endif

#endif /* _UTIL_SYSMACROS_F_H */
