#ifndef _IO_UIO_H	/* wrapper symbol for kernel use */
#define _IO_UIO_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/io/uio.h /main/uw7_nj/1"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/io/uio.h,v 1.5 2000/03/06 05:52:52 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *	Copyright (c) 1982, 1986, 1988
 *	The Regents of the University of California
 *	All Rights Reserved.
 *	Portions of this document are derived from
 *	software developed by the University of
 *	California, Berkeley, and its contributors.
 */

/*
 * I/O parameter information.  A uio structure describes the I/O which
 * is to be performed by an operation.  Typically the data movement will
 * be performed by a routine such as uiomove(), which updates the uio
 * structure to reflect what was done.
 */

#ifdef _KERNEL_HEADERS

#include <util/types.h>	/* REQUIRED */

#else

#include <sys/types.h>	/* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

/* types changed for XPG4 V2 */

typedef struct iovec {
#if defined(_KERNEL) 	/* Preserve kernel source compatability */
	caddr_t	iov_base;
	int	iov_len;
#else
	void	*iov_base;	/* was caddr_t */
	size_t	iov_len;	/* was int */
#endif
} iovec_t;

#if defined(_KERNEL) || defined(_KMEMUSER)

typedef struct uio {
	iovec_t	*uio_iov;	/* pointer to array of iovecs */
	int	uio_iovcnt;	/* number of iovecs */
	off32_t	uio_off32;	/* file offset */
	short	uio_segflg;	/* address space (kernel or user) */
	short	uio_fmode;	/* file mode flags */
	off32_t	uio_limit;	/* u-limit (maximum byte offset) */
	int	uio_resid;	/* residual count */
	off64_t	uio_off64;
} uio_t;

#if !defined(_DDI)
#if defined(_FSKI) && _FSKI < 2
#define uio_offset	uio_off32
#else
#define uio_offset	uio_off64
#endif
#endif

/*
 * I/O direction.
 */
typedef enum uio_rw { UIO_READ, UIO_WRITE } uio_rw_t;

/*
 * Segment flag values.
 */
#ifdef NSC_ICS
typedef enum uio_seg { 
	UIO_USERSPACE, 
	UIO_SYSSPACE, 
	UIO_USERISPACE,
	UIO_NSCICSSPACE } uio_seg_t;
#else  /* !NSC_ICS */
typedef enum uio_seg { UIO_USERSPACE, UIO_SYSSPACE, UIO_USERISPACE } uio_seg_t;
#endif /* !NSC_ICS */

#endif /* _KERNEL || _KMEMUSER */

#ifdef _KERNEL

extern int uiomove_catch(void *kernbuf, long n, uio_rw_t rw, uio_t *uiop,
			 uint_t catch_flags);
extern int ureadc(int val, uio_t *uiop);
extern int uwritec(uio_t *uiop);
extern int uiomvuio(uio_t *ruiop, uio_t *wuiop);
extern void uioskip(uio_t *uiop, long n);
extern void uioupdate(uio_t *uiop, long n);
struct buf;
struct bcb;
extern struct buf *uiobuf(struct buf *, uio_t *);
extern int uiophysio(void (*strat)(struct buf *), struct buf *bp, dev_t dev, int rw, uio_t *uio);
extern int uiophysio2(void (*func)(struct buf *), struct buf *bp, const struct bcb *bcbp);

#if !defined(_DDI)

#define _UIOMOVE(kernbuf, n, rw, uiop) \
		uiomove_catch(kernbuf, n, rw, uiop, CATCH_KERNEL_FAULTS)
#define uiomove(kernbuf, n, rw, uiop)	_UIOMOVE(kernbuf, n, rw, uiop)

#endif /* !_DDI */


#else /* ! _KERNEL  */

extern ssize_t readv(int, const struct iovec *, int);
extern ssize_t writev(int, const struct iovec *, int);

#endif /* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif /* _IO_UIO_H */
