#ifndef _FS_VFS_H	/* wrapper symbol for kernel use */
#define _FS_VFS_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/fs/vfs.h /main/uw7_nj/3"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/fs/vfs.h,v 1.7 2000/03/06 05:51:12 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 * Data associated with mounted file systems.
 */

#ifdef _KERNEL_HEADERS

#include <util/types.h>		/* REQUIRED */
#include <util/ksynch.h>	/* REQUIRED */
#include <proc/cred.h>		/* REQUIRED */
#include <fs/statvfs.h>		/* REQUIRED */

#ifdef NSC_CFS
#include <util/debug.h>
#include "fs_hier.h"		/* needed for VFS_LOCK */
#endif /* NSC_CFS */

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/types.h>		/* REQUIRED */
#include <sys/ksynch.h>		/* REQUIRED */
#include <sys/cred.h>		/* REQUIRED */
#include <sys/statvfs.h>	/* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

#ifdef NSC_CFS
/* Stop annoying compiler warnings, fast!!! */
struct cms_mntinfo_t;
struct pathname;
#endif /* NSC_CFS */

/*
 * File system identifier. Should be unique (at least per machine).
 */
typedef struct {
	long val[2];			/* file system id type */
} fsid_t;

/*
 * File identifier.  Should be unique per filesystem on a single
 * machine.  This is typically called by a stateless file server
 * in order to generate "file handles".
 */
#define	MAXFIDSZ	16
#define	freefid(fidp) \
	kmem_free((fidp), sizeof (struct fid) - MAXFIDSZ + (fidp)->fid_len)

typedef struct fid {
	ushort_t	fid_len;		/* length of data in bytes */
	char		fid_data[MAXFIDSZ];	/* data (variable length) */
} fid_t;

#if defined(_KERNEL) || defined(_KMEMUSER)

typedef struct _vfsops_v1 _vfsops_v1_t;
typedef struct _vfsops_v2 _vfsops_v2_t;

/*
 * Structure per mounted file system.  Each mounted file system has
 * an array of operations and an instance record.  The file systems
 * are kept on a singly linked list headed by "rootvfs" and terminated
 * by NULL.
 */
typedef struct vfs {
	struct vfs	*vfs_next;		/* next VFS in VFS list */
	_vfsops_v2_t	*vfs_op;		/* operations on VFS */
	void		*vfs_op_real;		/* shadow for vfs_op */
	struct vnode	*vfs_vnodecovered;	/* vnode mounted on */
	ulong_t		vfs_flag;		/* flags */
	ulong_t		vfs_bsize;		/* native block size */
	int		vfs_fstype;		/* file system type index */
	fsid_t		vfs_fsid;		/* file system id */
	void *		vfs_data;		/* private data */
	dev_t		vfs_dev;		/* device of mounted VFS */
	ulong_t		vfs_bcount;		/* I/O count (accounting) */
	lock_t		vfs_mutex;		/* spin lock */
	ulong_t		vfs_count;		/* number of references */
	level_t		vfs_macceiling;		/* MAC ceiling of a mounted file sys */
	level_t		vfs_macfloor;		/* MAC floor of a mounted file system */
        pid_t           vfs_lockpid;            /* SSI - pid with vfs lock */
        k_lwpid_t       vfs_locklwpid;          /* SSI - LWPID with vfs lock */
#ifdef NSC_CFS
	struct vfs	*vfs_hash;		/* VFS hash list. */
#endif /* NSC_CFS */
} vfs_t;

/*
 * Filesystem type switch table.
 */
typedef struct vfssw {
	char		*vsw_name;	/* type name string */
	int		(*vsw_init)();	/* init routine */
	_vfsops_v2_t	*vsw_vfsops;	/* filesystem operations vector */
	void		*vsw_vfsops_real;
	long		vsw_flag;	/* flags */
	struct module	*vsw_modp;	/* pointer to the struct module */
					/* if this is a loadable module */
					/* and is loaded		*/
} vfssw_t;

#endif /* _KERNEL || _KMEMUSER */

/*
 * VFS flags.
 */
#define VFS_RDONLY	0x01		/* read-only vfs */
#define VFS_NOSUID	0x02		/* setuid disallowed */
#define VFS_REMOUNT	0x04		/* modify mount options only */
#define VFS_NOTRUNC	0x08		/* does not truncate long file names */
#define VFS_UNLINKABLE	0x10		/* unlink(2) can be applied to root */
#define VFS_BADBLOCK	0x20		/* disk based fs has bad block */
/*
 * SSI Clustering
 */
#define VFS_CFSONTOP    0x40            /* CFS is stacked on top of this vfs */
#define VFS_MNTLOCK     0x80            /* (un)mount in progress */
#define VFS_MNTWAIT     0x100           /* waiting for (un)mount to finish */
#ifdef NSC_CFS
#define	VFS_PFSFLAGS	0x1B		/* PFS VFS flags kept in CFS VFS as well. */
#define	VFS_MOUNTING	0x200		/* VFS is being mounted. */
#define	VFS_DISTRIB	0x400		/* VFS is distributed. */
#endif /* NSC_CFS */


/*
 * Argument structure for mount(2).
 */
struct mounta {
	char	*spec;
	char	*dir;
	int	flags;
	char	*fstype;
	char	*dataptr;
	int	datalen;
	int	*bkupptr;	/* SSI - Valid only when MS_CFSBKUP set. */
	int	bkuplen;	/* SSI - Valid only when MS_CFSBKUP set. */
};

/*
 * Reasons for calling the vfs_mountroot() operation.
 */

typedef enum whymountroot {
	ROOT_INIT, ROOT_REMOUNT, ROOT_UNMOUNT, 
	ROOT_SSI_FAILOVER, ROOT_SSI_REMOUNT		/* SSI */
} whymountroot_t;

#ifdef NSC_CFS
/*
 * VFS_STATVFSX flags.
 */

/* don't count fragments in free space calculations. */
#define STATVFSX_NOFRAG 0x01

#endif /* NSC_CFS */

/*
 * VFS_SYNC flags.
 */
#define SYNC_ATTR	0x01		/* sync attributes only */
#define SYNC_CLOSE	0x02		/* close open file */


#ifdef _KERNEL

/*
 * Operations supported on virtual file system.
 */
struct _vfsops_v1 {
	int	(*vfs_mount)(struct vfs *, struct vnode *,
			      struct mounta *, struct cred *);
	int	(*vfs_unmount)(struct vfs *, struct cred *);
	int	(*vfs_root)(struct vfs *, struct vnode **);
	int	(*vfs_statvfs)(struct vfs *, statvfs32_t *);
	int	(*vfs_sync)(struct vfs *, int, struct cred *);
	int	(*vfs_vget)(struct vfs *, struct vnode **, struct fid *);
	int	(*vfs_mountroot)(struct vfs *, enum whymountroot);
	int	(*vfs_setceiling)(struct vfs *, lid_t);
#ifdef NSC_CFS
	int	(*vfs_statvfsx)(struct vfs *, statvfs32_t *, int);
#endif /* NSC_CFS */
};

struct _vfsops_v2 {
	int	(*vfs_mount)(struct vfs *, struct vnode *,
			      struct mounta *, struct cred *);
	int	(*vfs_unmount)(struct vfs *, struct cred *);
	int	(*vfs_root)(struct vfs *, struct vnode **);
	int	(*vfs_statvfs)(struct vfs *, statvfs64_t *);
	int	(*vfs_sync)(struct vfs *, int, struct cred *);
	int	(*vfs_vget)(struct vfs *, struct vnode **, struct fid *);
	int	(*vfs_mountroot)(struct vfs *, enum whymountroot);
	int	(*vfs_setceiling)(struct vfs *, lid_t);
#ifdef NSC_CFS
	int	(*vfs_statvfsx)(struct vfs *, statvfs64_t *, int);
#else /* NSC_CFS */
	/*
	 * SSI - placeholders for SSI vfs ops
	 */
	int	(*vfs_statvfsx)(struct vfs *, struct statvfs *, int); /* SSI */
	int	(*vfs_cfs_get_fsid)(struct vfs *, const char *, const char *,
				    struct vfs *, void *,
				    fsid_t *fsp);	/* SSI */
#endif /* NSC_CFS */
};

#define VFS_MOUNT(vfsp, mvp, uap, cr) \
	(*(vfsp)->vfs_op->vfs_mount)(vfsp, mvp, uap, cr)
#define VFS_UNMOUNT(vfsp, cr)	(*(vfsp)->vfs_op->vfs_unmount)(vfsp, cr)
#define VFS_ROOT(vfsp, vpp)	(*(vfsp)->vfs_op->vfs_root)(vfsp, vpp)
#define	VFS_STATVFS(vfsp, sp)	(*(vfsp)->vfs_op->vfs_statvfs)(vfsp, sp)
#define VFS_SYNC(vfsp, flag, cr) \
		(*(vfsp)->vfs_op->vfs_sync)(vfsp, flag, cr)
#define VFS_VGET(vfsp, vpp, fidp) \
		(*(vfsp)->vfs_op->vfs_vget)(vfsp, vpp, fidp)
#define VFS_MOUNTROOT(vfsp, init) \
		 (*(vfsp)->vfs_op->vfs_mountroot)(vfsp, init)
#define VFS_SWAPVP(vfsp, vpp, nm) \
		(*(vfsp)->vfs_op->vfs_swapvp)(vfsp, vpp, nm)
#define VFS_SETCEILING(vfsp, level) \
		(*(vfsp)->vfs_op->vfs_setceiling)(vfsp, level)
#ifdef NSC_CFS
#define VFS_STATVFSX(vfsp, sp, fl) \
		(*(vfsp)->vfs_op->vfs_statvfsx)(vfsp, sp, fl)
#endif /* NSC_CFS */


/*
 * Public operations.
 */
extern void	vfs_mountroot();	/* mount the root */
extern void	vfs_add(struct vnode *, struct vfs *, int); /* add new vfs to vfslist */
extern void	vfs_remove(struct vfs *);	/* remove a vfs from mounted vfs list */
extern vfs_t 	*getvfs(fsid_t *);	/* return vfs given fsid */
extern vfs_t 	*vfs_devsearch(dev_t);	/* find vfs given device */
extern vfssw_t 	*vfs_getvfssw(char *);	/* find vfssw ptr given fstype name */
extern ulong_t	vf_to_stf(ulong);	/* map VFS flags to statfs flags */
extern int	dounmount(struct vfs *, cred_t *cr);	/* unmount a vfs */
#ifdef NSC_CFS
extern int	vfs_mntlock(struct vfs *);	/* Lock vfs for mount/umount */
extern int	vfs_mntlockx(struct vfs *, int); /* Close, but not the same. */
extern void	vfs_mntunlock(struct vfs *);	/* Unlock vfs for mount/umount */
extern int	vfs_check_mntlock(vfs_t *vfsp, 
				  int vfslist_locked,
				  int mounting_ok,
				  int hold_vfs); /* mntlock juggling. */
#endif /* NSC_CFS */

#define VFS_INIT(vfsp, op, op_real, data)	{		\
	(vfsp)->vfs_next = (struct vfs *)0;			\
	(vfsp)->vfs_op = (op);					\
	(vfsp)->vfs_op_real = (op_real);			\
	(vfsp)->vfs_flag = 0;					\
	(vfsp)->vfs_data = (data);				\
	LOCK_INIT(&((vfsp)->vfs_mutex), FS_VFSPHIER,		\
		  FS_VFSPPL, &vfs_mutex_lkinfo, KM_SLEEP);	\
}

#define VFS_LOCK(vfsp)		LOCK(&(vfsp)->vfs_mutex, FS_VFSPPL);
#define VFS_UNLOCK(vfsp, pl)	UNLOCK(&(vfsp)->vfs_mutex, pl);

#ifdef NSC_CFS

#define	VFS_HOLD(vfsp, held)	{				\
	pl_t	ipl;						\
	ipl = VFS_LOCK(vfsp);					\
	if ((vfsp->vfs_flag & VFS_MNTLOCK) &&			\
	     !((vfsp->vfs_lockpid == NSC_MY_PID) &&		\
	     (vfsp->vfs_locklwpid == NSC_MY_LWPID))) {		\
		(held) = FALSE;					\
	} else {						\
		(held) = TRUE;					\
		(vfsp)->vfs_count++;				\
	}							\
	VFS_UNLOCK(vfsp, ipl);					\
}

#define	VFS_RELE(vfsp)	{					\
	pl_t	ipl;						\
	ipl = VFS_LOCK(vfsp);					\
	ASSERT((vfsp)->vfs_count != 0);				\
	(vfsp)->vfs_count--;					\
	VFS_UNLOCK(vfsp, ipl);					\
}

#define	VFS_MNTLOCK_ASSERT(_vfsp) \
do { \
	ASSERT((_vfsp)->vfs_flag & VFS_MNTLOCK); \
	ASSERT((_vfsp)->vfs_lockpid == NSC_MY_PID); \
	ASSERT((_vfsp)->vfs_locklwpid == NSC_MY_LWPID); \
} \
while (0)

#else  /* !NSC_CFS */

#define	VFS_HOLD(vfsp)	{					\
	pl_t	ipl;						\
	ipl = VFS_LOCK(vfsp);					\
	(vfsp)->vfs_count++;					\
	VFS_UNLOCK(vfsp, ipl);					\
}

#define	VFS_RELE(vfsp)	{					\
	pl_t	ipl;						\
	ipl = VFS_LOCK(vfsp);					\
	(vfsp)->vfs_count--;					\
	VFS_UNLOCK(vfsp, ipl);					\
}

#endif /* !NSC_CFS */

/*
 * Globals.
 */
extern struct vfs *rootvfs;		/* ptr to root vfs structure */
extern struct vfssw vfssw[];		/* table of filesystem types */
extern char rootfstype[];		/* name of root fstype */
extern int nfstype;			/* # of elements in vfssw array */
extern sleep_t vfslist_lock;		/* Vfs list lock */
extern lkinfo_t vfs_mutex_lkinfo;	/* Per-Vfs lock info */

#ifdef NSC
/* Filesystem types. */
#endif /* NSC */

#ifdef NSC_CFS
extern int cfs_fstype;
extern int memfsfstype;
#endif /* NSC_CFS */

#ifdef NSC_CPROCFS
extern	int	cprocfstype;
#endif /* NSC_CPROCFS */

#ifdef NSC_CSPECFS
extern int cspecfs_fstype;
#endif /* NSC_CSPECFS */

#ifdef NSC_FDFS
extern int fdfs_fstype;
#endif /* NSC_FDFS */

#ifdef NSC_NAMEFS
extern int namefs_fstype;
#endif /* NSC_NAMEFS */

#endif /* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif /* _FS_VFS_H */
