#ident "@(#)$Header: /src/nsc_uw7/usr/src/nsc/uts/proc/vproc.h,v 3.33 2000/05/09 22:42:28 dzafman Exp $"

/*
 * Copyright (c) 1991, 1993, 1994 Locus Computing Corporation
 *        All Rights Reserved
 */

#ifndef _NSC_GENERIC_SYS_VPROC_H
#define _NSC_GENERIC_SYS_VPROC_H

#ifdef NSC_SVR42
#ifdef _KERNEL_HEADERS
#include <util/types.h>
#include <proc/signal.h>
#include <svc/systm.h>
#include <nsc/nsc.h>
#include <nsc/nsc_synch.h>
#include <fs/vnode.h>
#include <proc/procset.h>
#include <proc/priocntl.h>
#include <util/dl.h>
#elif defined(_KERNEL) || defined(_KMEMUSER)
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/systm.h>
#include <sys/nsc.h>
#include <sys/nsc_synch.h>
#include <sys/vnode.h>
#include <sys/procset.h>
#include <sys/dl.h>
#else
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/systm.h>
#include <sys/nsc.h>
#include <sys/nsc_synch.h>
#include <sys/vnode.h>
#include <sys/procset.h>
#include <sys/dl.h>
#endif /* _KERNEL_HEADERS */

#else  /* !NSC_SVR42 */

#include <sys/types.h>
#include <sys/resource.h>
#ifdef OSF1
#include <sys/tty.h>
#endif
#ifdef NSC_SVR4
#include <sys/systm.h>
#include <nsc/nsc.h>
#endif
#include <sys/vnode.h>
#include <sys/procset.h>
#endif /* !NSC_SVR42 */

#ifndef ESUCCESS
#define ESUCCESS 0
#endif

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

/*
 * declarations for generated data types.
 */
extern struct load_data_t;


/*
 * Vproc structure.
 */
struct	vproc {
	long		  vp_magic;	/* magic number id for vproc */
#define			  VP_MAGIC	0x63727076
	pid_t		  vp_pid;	/* vproc process identifier */
	int		  vp_ref_cnt;	/* vproc usage count */
	caddr_t  	  vp_data;	/* implementation dependent data */
	SPIN_LOCK_T  	  vp_ref_cnt_lock; /* lock for reference counting */
	struct pid_hash {
		struct vproc	*hashfwd;
		struct vproc	*hashbwd;
	} vp_pid_hash; 
#ifdef	VPROC_DEBUG
	unsigned long	  vp_index;	/* vproc index number */
	struct vproc	  *vp_next;	/* static link to next vproc */
#endif	
};
#define	vp_hashfwd	vp_pid_hash.hashfwd
#define	vp_hashbwd	vp_pid_hash.hashbwd

/*
 * The vproc table pointers.
 */
#ifndef NSC_SVR4
extern struct vproc *vproc;		/* the vproc process table, itself */
extern struct vproc *vprocNVPROC;	/* pointer past end of table */
extern struct vproc *vprocfree;		/* pointer to free vproc list */
#endif
extern int nvproc;			/* number of vprocs */

/*
 * Hash table for vproc table 
 */
#define VPROC_NHASH 256			/* Minimum number entries in table */
extern int vprochsz;			/* Actual size of the hash table */
extern struct vproc **vproc_hash;	/* hash table itself */
#define VPROCPIDHASH(pid) ((pid)&(vprochsz-1)) /* hash a pid to hash table */

extern LOCK_T vproc_list_lock;		/* Lock for free list and hash table */

extern struct vproc *vprocptr(pid_t pid, int needlock);
extern struct vproc *(*locate_vproc_pid_routine)(pid_t pid, char *str,
						 int needlock);

/*
 * locate/create a vproc given the pid of a process.
 * validate whether a process id is valid or not.
 *
 * The NOLOCK versions of the macros should only be used during
 * system startup initialization when it may not be possible to
 * acquire sleep locks (vprocptr() and locate_vproc_pid() both
 * acquire the VPROC_LIST_LOCK() sleep lock).  During startup
 * there shouldn't be any other contexts in contention so
 * skipping the locks should be okay.
 *
 * The NOFAIL and ORIGIN version of LOCATE_VPROC_PID() is intended
 * for use by VPROC nodedown code and should not be used for any
 * other purposes.  The NOFAIL version creates a new vproc even
 * at the pid's (surrogate) origin.  The ORIGIN assumes that it
 * is being called on the pid's origin node and avoids acquiring
 * the origin list lock recusively (via VPROC_IS_AT_ORIGIN()) in
 * some nodedown code.
 */
#define VPROCPTR(pid)		(vprocptr(pid, TRUE))
#define VPROCPTR_NOLOCK(pid)	(vprocptr(pid, FALSE))

/* LOCATE_VPROC_PID() option flags */
#define LVP_NOLOCK	0x01
#define LVP_NOFAIL	0x02
#define LVP_ORIGIN	0x04

#define	LOCATE_VPROC_PID(pid, str) \
			(*locate_vproc_pid_routine)(pid, str, 0)
#define	LOCATE_VPROC_PID_NOLOCK(pid, str) \
			(*locate_vproc_pid_routine)(pid, str, LVP_NOLOCK)
#define LOCATE_VPROC_PID_NOFAIL(pid, str) \
			(*locate_vproc_pid_routine)(pid, str, LVP_NOFAIL);
#define LOCATE_VPROC_PID_ORIGIN(pid, str) \
			(*locate_vproc_pid_routine)(pid, str, LVP_ORIGIN);

/*
 * Macros which define the existence rules for a vproc.
 */
#ifdef VPROC_DEBUG
#define	DECREMENT_VP_REFCNT(z, str) 		\
	(vproc_db4(z, "RELEASE", str, -1), 	\
	vproc_release(z))
#define	INCREMENT_VP_REFCNT(z, str)		\
	(vproc_db4(z, "HOLD", str, +1), 	\
	vproc_hold(z))
#else
#define	DECREMENT_VP_REFCNT(z, str)	vproc_release(z)
#define	INCREMENT_VP_REFCNT(z, str)	vproc_hold(z)
#endif /* VPROC_DEBUG */

#define	VPROC_HOLD(v, str)	INCREMENT_VP_REFCNT((v), (str))
#define	VPROC_RELE(v, str)	DECREMENT_VP_REFCNT((v), (str))


/*
 * Extern delcarations for lockdata variables.
 */
EXTERN_LOCKDATA_T(vproc_list_lkdata);
EXTERN_LOCKDATA_T(vp_ref_cnt_lkdata);


/*
 * Lock macros used by the virtual process code.
 */
#define VPROC_LIST_LOCK_INIT() do {					\
		INIT_LOCK(&vproc_list_lock, vproc_list_lkdata);		\
		} while(0)
#define VPROC_LIST_LOCK() do {						\
		LOCK_LOCK(&vproc_list_lock);				\
		} while(0)
#define VPROC_LIST_UNLOCK() do {					\
		UNLOCK_LOCK(&vproc_list_lock);				\
		} while(0)

#define VPROC_REFCNT_LOCK_INIT(v) do {					\
		INIT_SPIN_LOCK(&(v)->vp_ref_cnt_lock,			\
			       vp_ref_cnt_lkdata);			\
		} while(0)
#define VPROC_REFCNT_LOCK(v) do {					\
		LOCK_SPIN_LOCK(&(v)->vp_ref_cnt_lock);			\
		} while(0)
#define VPROC_REFCNT_UNLOCK(v) do {					\
		UNLOCK_SPIN_LOCK(&(v)->vp_ref_cnt_lock);		\
		} while(0)
#define VPROC_REFCNT_LOCK_DEINIT(v) do {				\
		DEINIT_SPIN_LOCK(&(v)->vp_ref_cnt_lock);		\
		} while(0)


typedef struct procinfo {

	int 	pi_code;			/* code specific to signal */
	int	pi_errno;
	uid_t	pi_ruid;			/* real uid of sender */
	uid_t	pi_uid;				/* uid of sender */
	pid_t	pi_pid;				/* process id of sender */
	pid_t	pi_sid;				/* session id of sender */

	struct {
		struct {			/*  SIGCLD	*/
			clock_t _utime;
			int	_status;
			clock_t _stime;
		} _cld;			

		struct {	/* SIGSEGV, SIGBUS, SIGILL and SIGFPE	*/
			caddr_t	_addr;		/* faulting address	*/
		} _fault;

		struct {			/* SIGPOLL, SIGXFSZ	*/
			long	_band;
		} _file;
		clusternode_t _sigarg;		/* SIGMIGRATE */

	} _data;

} procinfo_t;

#define pi_status	_data._cld._status
#define pi_stime	_data._cld._stime
#define pi_utime	_data._cld._utime
#define pi_addr		_data._fault._addr
#define pi_band		_data._file._band
#define pi_sigarg	_data._sigarg


#ifdef OSF1
typedef struct rusage pusage;
typedef struct rusage_dev pusage_dev;
#else  /* !OSF1 */
#ifndef NSC_SVR42
struct pu_timeval {
	long   pu_sec;			/* seconds used by process */
	long   pu_nsec;			/* nano seconds used by process */
};
#endif /* NSC_SVR42 */

struct	pusage {
#ifdef NSC_SVR42
	/*
	 * NSC_XXX: SVR4.2 uses a double long to represent the usage
	 *	      times in ticks.  A routine is provided to convert
	 *	      from ticks to timestruc, but not back.  Also the
	 *	      conversion routine is time consuming.  Need to
	 *	      rethink common usage time data structure.  Maybe
	 *	      have a union of the two types of data representation
	 *	      with a a discriminating flag.  The destination node
	 *	      would be responsible for conversion if it receives
	 *	      the data in a format different than it requires.
	 *	      This would be efficient for common case of homogeneous
	 *	      cluster.
	 */
	dl_t	pu_utime;		/* user time used */
	dl_t	pu_stime;		/* system time used */
	char	pu_cpu;			/* cpu usage for scheduling */
#else  /* !NSC_SVR42 */
#ifdef OSFTEMP
	struct timeval pu_utime;	/* user time used */
	struct timeval pu_stime;	/* system time used */
#else
	struct pu_timeval pu_utime;	/* user time used */
	struct pu_timeval pu_stime;	/* system time used */
	char	pu_cpu;			/* cpu usage for scheduling */
#endif
#endif /* NSC_SVR42 */
	long	pu_maxrss;
#define pu_first	pu_ixrss
	long	pu_ixrss;		/* integral shared memory size */
	long	pu_idrss;		/* integral unshared data " */
	long	pu_isrss;		/* integral unshared stack " */
	long	pu_minflt;		/* page reclaims */
	long	pu_majflt;		/* page faults */
	long	pu_nswap;		/* swaps */
	long	pu_inblock;		/* block input operations */
	long	pu_oublock;		/* block output operations */
	long	pu_msgsnd;		/* messages sent */
	long	pu_msgrcv;		/* messages received */
	long	pu_nsignals;		/* signals received */
	long	pu_nvcsw;		/* voluntary context switches */
	long	pu_nivcsw;		/* involuntary " */
#define pu_last		pu_nivcsw
};

struct pusage_dev {
	struct pusage pu_pusage;
	dev_t	      pu_dev;
	int	pu_wcode;		/* wait code of process */
	int	pu_signo;		/* signal number */
};
#endif /* !OSF1 */


#ifdef NSC_SVR4
/*
 * The following is for supporting SVR4 procset operations
 */

typedef struct vproc_clparams {
	long	id;
	long	params[(32 / sizeof(long))];
} vproc_clparams_t;

/*
 * Distributed procset parameters.  Provides a kind of "function pointer"
 * support for remote operations.  This structure is built in the base,
 * and decoded by pproc_procset_apply() and pproc_procset_resolve().
 *
 * NOTE:  When we backport this to SVR4.0, we'll need to further
 * resolve the parameters returned by proccmp() and change the cmp_args
 * structure here to match.  This will also mean modifying the SVR4.2
 * code accordingly.  I'm pretty sure this will just involve filling
 * the parms field properly as is done on SVR4.2 and setting "gotit", but
 * this needs more investigation.
 */
typedef struct	vproc_procset_params {
	int	platform;
	struct {
		int	function_id;
		union {
			struct {
				vproc_clparams_t parms;
				int		error;
			} set_args;
			struct {
				long		cid;
				int		cnt;
				int		gotit;
				vproc_clparams_t parms;
			} cmp_args;
			procset_t		lwp_ps;
			struct {
				int		sig;
				int		checkperm;
#ifndef NSC_UW7
				int		perm;
#endif
				clusternode_t	sigarg; /* SIGMIGRATE */
			} sig_args;
#ifdef NSC_UW
			struct ageparms age_args;
#endif
		} args;
	} function[2];
} vproc_procset_params_t;

#define VPROC_PROCSET_SET	0
#define VPROC_PROCSET_CMP	1
#define VPROC_PROCSET_ALLLWPS	2
#define VPROC_PROCSET_LWPS	3
#define VPROC_PROCSET_SIG	4
#define VPROC_PROCSET_SETAGE	5
#define VPROC_PROCSET_GETAGE	6
#define VPROC_PROCSET_GETTIME	7

/*
 * Flags for the "init is special" flag variable.
 */
#define VPROC_PROCSET_INIT_SPECIAL_FLAG		0x01
#define VPROC_PROCSET_HIT_INIT_FLAG		0x02
#define VPROC_PROCSET_HIT_NON_INIT_FLAG		0x04
#define VPROC_PROCSET_SKIP_CHECK_FLAG		0x08

#endif /* NSC_SVR4 */

/*
 * The following prototypes are for vproc operations functions.
 */

#define	IN
#define OUT
#define INOUT

extern int vpop_fork(
		IN struct vproc *v,	/* vproc being forked */
		OUT pid_t *pid,	/* return value (child pid) */
		IN caddr_t param1,	
		IN unsigned int param2, /* size of thread state */
		IN int flags,		/* fork flag arguments */
		IN clusternode_t node);	/* destination node */
#ifdef NSC_SVR4
#ifdef NSC_SVR40
extern int vpop_newproc(
		IN struct vproc *v,	/* vproc of parent of new proc */
		IN int cond,		/* condition flags */
		OUT pid_t *pid,		/* pid of newly created process */
		OUT int *perror,	/* error return */
		OUT int *is_child);	/* 0 => parent process */
					/* 1 => child process */
#endif
#ifdef NSC_SVR42
extern int vpop_spawn_proc(
		IN struct vproc *v,	/* vproc of parent of new proc */
		IN int cond,		/* condition flags */
		OUT pid_t *pid,		/* pid of new process */
		IN void (*func)(),	/* function pointer */
		IN void *arg);		/* function argument */
#endif
extern int vpop_select_node(
		IN struct vproc *v,	/* vproc of proc selecting node */
		IN char *fname,		/* file name */
		OUT clusternode_t *node); /* selected node */
extern int vpop_execve(
		IN struct vproc *v,	/* vproc being execed */
		IN char *fname,		/* file name */
		IN char **argv,		/* arguments */
		IN char **envp,		/* environment */
		OUT rval_t *rvp,	/* return values */
		IN clusternode_t node);	/* destination node */
extern int vpop_migrate(
		IN struct vproc *v,	/* vproc being migrated */
		IN clusternode_t node,	/* destination node */
		IN int is_sigmigrate);	/* 1 => generated by SIGMIGRATE */
#endif /* NSC_SVR4 */
extern int vpop_exit(
		IN struct vproc *v,	/* vproc that is exiting */
		IN int exitcode,	/* exit code */
		IN int why);		/* why code */
#ifdef OSF1
extern int vpop_free(
		IN struct vproc *v);	/* vproc just freed */
#endif
extern int vpop_wait(
		IN struct vproc *pp,	/* vproc of parent used to scan */
					/* parent-child-sibling list */
		IN pid_t pid,		/* specifies who we wait for */
		IN int options,		/* options to vary function */
		OUT int *wstat,		/* return status value */
					/* unused by SVR4 */
		OUT struct pusage_dev *pusage, /* resource usage statistics */
		OUT pid_t *ret_val);	/* return value */
					/* unused by SVR4 */
#ifdef OSF1
extern int vpop_proc_nice(
		IN struct vproc *v,	/* vproc being queried */
		INOUT int *nice,	/* nice value */
		IN procinfo_t *pinfo,	/* calling process information */
		IN int flag);		/* VPROC_SET or VPROC_GET */
extern int vpop_pgrp_nice(
		IN struct vproc *g,	/* vproc of group being queried */
		INOUT int *nice,	/* nice value */
		IN procinfo_t *pinfo,	/* calling process information */
		IN int flag);		/* VPROC_SET or VPROC_GET */
#endif /* OSF1 */
#ifdef NSC_SVR42
extern int vpop_postwait_post(
		IN struct vproc *v,	/* vproc of process to be posted to */
		OUT int *not_waiting);	/* whether the process was waiting */
extern int vpop_siglwp(
		IN struct vproc *v,	/* vproc of lwp being signaled */
		IN k_lwpid_t lwpid,     /* id of lwp being signaled */
		IN int sig,             /* signal to post */
		IN procinfo_t *pinfo,   /* calling process information */
		IN int flag);           /* affects signaling action */
#endif /* NSC_SVR42 */
extern int vpop_sigproc(
		IN struct vproc *v,	/* destination vproc */
		IN int sig,		/* signal to send */
		IN procinfo_t *pinfo, 	/* sending process's state */
		IN int flag);		/* signaling flags */
extern int vpop_sigpgrp(
		IN struct vproc *g,	/* destination pgrp leader vproc */
		IN int sig,		/* signal to send */
		IN procinfo_t *pinfo, 	/* sending process's state */
		IN int flag);		/* signaling flags */
extern int vpop_report_state(
		IN struct vproc *v,	/* vproc having state modified */
		IN int sig,		/* signal to send */
		IN procinfo_t *pinfo, 	/* sending process's state */
		IN int state);		/* selects zombie/stop/unstop */
extern int vpop_setsid(
		IN struct vproc *v);	/* vproc creating new session */
extern int vpop_setpgid(
		IN struct vproc *v,	/* vproc of process to assign */
		IN struct vproc *g,	/* vproc of group to be assigned to */
		IN pid_t pid,		/* caller's pid */
		IN pid_t sid);		/* caller's sid */
extern int vpop_getattr(
		IN struct vproc *v,	/* process being queried */
		OUT pid_t *ppid,	/* returns vp_ppid field */
		OUT pid_t *pgrp,	/* returns vp_pgrp field */
		OUT pid_t *sid,		/* returns vp_sid field */
		OUT int *job_control,	/* returns if job control applies */
		OUT int *has_sctty); /* returns if session tty control */
extern int vpop_setpinit(
		IN struct vproc *v);	/* process to be kernelized */
#ifdef VPROC_ND
extern int vpop_reclaim_child(
		IN struct vproc *v,	/* process to reclaim */
		IN pid_t ppid,		/* pid of new parent */
		IN time_t start);	/* p_start of child process */
#endif
#ifndef NSC_SVR42
extern int vpop_ptrace(
		IN struct vproc *vd,	/* debugger process */
		IN struct vproc *ve,	/* traced child process */
		IN int req,		/* trace request code */
		IN caddr_t addr,		/* memory address */
		IN int data,		/* data to write */
		OUT int *retval);	/* returned data */
#endif /* !NSC_SVR42 */
extern int vpop_terminal_sigpgrp(
		IN struct vproc *v,	/* vproc whose pgrp being singalled */
#ifdef NSC_SVR42
		IN k_lwpid_t lwpid,	/* lwp of signalling process */
#endif
		IN int sig,		/* signal to send */
		IN procinfo_t *pinfo, 	/* sending process's state */
		IN int flag);		/* flags */
extern int vpop_sigaction(
		IN struct vproc *v,	/* process processing signal */
		IN int signo,		/* signal being processed */
		IN void (*func)(),	/* handler for sig */
#ifdef NSC_SVR42
		IN k_sigset_t mask,
#else
		IN u_int mask,
#endif
		IN u_int flags,
		IN int is_ssig);
#ifdef OSF1
extern int vpop_ctty_getattr(
		IN struct vproc *v,	/* process being queried */
		IN struct vproc *s,	/* vproc of session leader */
		OUT pid_t *t_pgid,	/* foreground pgrp */
		OUT dev_t *dev,		/* controlling device */
		OUT struct tty **ttyp);	/* output param, controlling tty */
extern int vpop_get_task_port(
		IN struct vproc *v,	/* process being queried */
		IN procinfo_t *pinfo, 	/* sending process's state */
		OUT mach_port_t *portp, /* task port of process */
		IN flags);		/* calling flags */
extern int vpop_set_ctty(
		IN struct vproc *v,	/* process setting ctty */
		IN struct vproc *s,	/* vproc of session leader */
		IN struct tty *ttyp,	/* controlling tty */
		IN int flag);		/* VPROC_SET or VPROC_CLEAR */
#else  /* !OSF1 */
extern int vpop_setctty(
		IN struct vproc *v,	/* process which is getting  ctty */
		IN dev_t cttydev,	/* ctty device */
		IN clusternode_t cttynode,  /* device node for ctty */
		IN int flag);		/* set of VCTTY_ flags which 
					 * influence what is set or cleared */
#endif /* !OSF */
#ifdef TETRA
extern int vpop_bsession(
		IN struct vproc *v,	/* process which is getting  bsession */
		IN u_long bsession,	/* background session value */
		IN int flag);		/* set of  VCTTY_ flags which 
					 * influence what is set or cleared */
#endif
#ifndef OSF1
extern int vpop_is_alive(
		IN struct vproc *v,	/* process being queried */
		OUT clusternode_t *execnode);	/* execution node */
#endif



/*
 * The following macros are used to invoke the vproc operations:
 */
#ifdef OSF1
#define VPOP_FORK(v, _pid, _param1, _param2, _flags) \
	_VPOP_(vpop_fork, v, (v, _pid, _param1, _param2, _flags), \
		"(0x%x, &pid=0x%x, param1=%d, param2=%d, flags=0x%x)")
#else
#define VPOP_FORK(v, _pid, _param1, _param2, _flags, _node) \
	_VPOP_(vpop_fork, v, (v, _pid, _param1, _param2, _flags, _node), \
		"(0x%x, &pid=0x%x, param1=%d, param2=%d, flags=0x%x, node=%d)")
#endif
#ifdef NSC_SVR4
#ifdef NSC_SVR40
#define VPOP_NEWPROC(v, _cond, _pid, _perror, _is_child) \
	_VPOP_(vpop_newproc, v, (v, _cond, _pid, _perror, _is_child), \
		"(0x%x, cond=0x%x, &pid=0x%x, &perror=0x%x, is_child=%d)")
#endif /* NSC_SVR40 */
#ifdef NSC_SVR42
#define VPOP_SPAWN_PROC(v, _cond, _pid, _func, _arg) \
	_VPOP_(vpop_spawn_proc, v, (v, _cond, _pid, _func, _arg), \
		"(0x%x, cond=0x%x, &pid=0x%x, func=0x%x, arg=0x%x)")
#endif /* NSC_SVR42 */
#define VPOP_SELECT_NODE(v, _fname, _node) \
	_VPOP_(vpop_select_node, v, (v, _fname, _node), \
		"(0x%x, fname=0x%x, &node=0x%x)")
#define VPOP_EXECVE(v, _fname, _argv, _envp, _rvp, _node) \
	_VPOP_(vpop_execve, v, (v, _fname, _argv, _envp, _rvp, _node), \
		"(0x%x, fname=0x%x, argv=0x%x, envp=0x%x, &rvp=0x%x, node=%d)")
#define VPOP_MIGRATE(v, _node, _is_sigmigrate) \
	_VPOP_(vpop_migrate, v, (v, _node, _is_sigmigrate), \
		"(0x%x, node=%d, is_sigmigrate=%d)")
#endif /* NSC_SVR4 */
#define VPOP_EXIT(v, _exitcode, _why) \
	_VPOP_(vpop_exit, v, (v, _exitcode, _why), \
		"(0x%x, exitcode=%d, why=0x%x)")
#ifdef OSF1
#define VPOP_FREE(v) \
	_VPOP_(vpop_free, v, (v), \
		"(0x%x)")
#endif
#define VPOP_WAIT(pp, _pid, _options, _wstat, _ru_loc, _retval) \
	_VPOP_(vpop_wait, pp, (pp, _pid, _options, _wstat, _ru_loc, _retval), \
		"(0x%x, pid=%d, options=0x%x, &wstat=0x%x, &ru_loc=0x%x, &retval=0x%x)")
#ifndef NSC_SVR4
#define VPOP_PROC_NICE(v, _nice, _pinfo, _flag) \
	_VPOP_(vpop_proc_nice, v, (v, _nice, _pinfo, _flag), \
		"(0x%x, &nice=0x%x, &pinfo=0x%x, flag=0x%x)")
#define VPOP_PGRP_NICE(g, _nice, _pinfo, _flag) \
	_VPOP_(vpop_pgrp_nice, g, (g, _nice, _pinfo, _flag), \
		"(0x%x, &nice=0x%x, pinfo=0x%x, flag=0x%x)")
#endif /* !NSC_SVR4 */
#ifdef NSC_SVR42
#define VPOP_POSTWAIT_POST(v, _not_waiting) \
	_VPOP_(vpop_postwait_post, v, (v, _not_waiting), \
		"(0x%x, &not_waiting=0x%x)")
#define VPOP_SIGLWP(v, _lwpid, _sig, _pinfo, _flag) \
	_VPOP_(vpop_siglwp, v, (v, _lwpid, _sig, _pinfo, _flag), \
		"(0x%x, lwpid=%d, sig=%d, pinfo=0x%x, flag=0x%x)")
#endif /* NSC_SVR42 */
#define VPOP_SIGPROC(v, _sig,  _pinfo,  _flag) \
	_VPOP_(vpop_sigproc, v, (v, _sig,  _pinfo,  _flag), \
		"(0x%x, sig=%d,  pinfo=0x%x, flag=0x%x)")
#define VPOP_SIGPGRP(g, _sig, _pinfo,  _flag) \
	_VPOP_(vpop_sigpgrp, g, (g, _sig, _pinfo,  _flag), \
		"(0x%x, sig=%d, pinfo=0x%x, flag=0x%x)")
#define	VPOP_REPORT_STATE(v, _sig, _pinfo, _state) \
	_VPOP_(vpop_report_state, v, (v, _sig, _pinfo, _state), \
		"(0x%x, sig=%d, pinfo=0x%x, state=0x%x)")
#define VPOP_SETSID(v) \
	_VPOP_(vpop_setsid, v, (v), \
		"(0x%x)")
#define VPOP_SETPGID(v, _g, _pid, _sid) \
	_VPOP_(vpop_setpgid, v, (v, _g, _pid, _sid), \
		"(0x%x, g=0x%x, pid=%d, sid=%d)")
#define VPOP_GETATTR(v, _ppid, _pgid, _sid, _job_control, _has_sctty) \
	_VPOP_(vpop_getattr, v, (v, _ppid, _pgid, _sid, _job_control, _has_sctty), \
		"(0x%x, &ppid=0x%x, &pgid=0x%x, &sid=0x%x, &job_control=0x%x, &has_sctty=0x%x)")
#define VPOP_SETPINIT(v) \
	_VPOP_(vpop_setpinit, v, (v), \
		"(0x%x)")
#ifdef VPROC_ND
#define VPOP_RECLAIM_CHILD(v, _ppid, _start) \
	_VPOP_(vpop_reclaim_child, v, (v, _ppid, _start), \
		"(0x%x, ppid=%d, start=%x)")
#endif
#define VPOP_PTRACE(vd, _ve, _req, _addr, _data, _retval) \
	_VPOP_(vpop_ptrace, vd, (vd, _ve, _req, _addr, _data, _retval), \
		"(0x%x, ve=0x%x, req=%d, addr=0x%x, data=0x%x, retval=0x%x)")
#ifdef NSC_SVR42
#define VPOP_TERMINAL_SIGPGRP(v, _lwpid, _sig, _pinfo, _flag) \
	_VPOP_(vpop_terminal_sigpgrp, v, (v, _lwpid, _sig, _pinfo, _flag), \
		"(0x%x, lwpid=0x%x, sig=%d, pinfo=0x%x, flag=0x%x)")
#else  /* !NSC_SVR42 */
#define VPOP_TERMINAL_SIGPGRP(v, _sig, _pinfo, _flag) \
	_VPOP_(vpop_terminal_sigpgrp, v, (v, _sig, _pinfo, _flag), \
		"(0x%x, sig=%d, pinfo=0x%x, flag=0x%x)")
#endif /* !NSC_SVR42 */
#define	VPOP_SIGACTION(v, _signo, _disp, _mask, _flags, _is_ssig) \
	_VPOP_(vpop_sigaction, v, (v, _signo, _disp, _mask, _flags, _is_ssig), \
		"(0x%x, signo=%d disp=0x%x mask=0x%x flags=0x%x, is_ssig=%d)")
#ifdef OSF1
#define VPOP_CTTY_GETATTR(v, _s, _t_pgid, _dev, _ttyp) \
	_VPOP_(vpop_ctty_getattr, v, (v, _s, _t_pgid, _dev, _ttyp), \
		"(0x%x, s=0x%x, &t_pgid=0x%x, &dev=0x%x, &ttyp=0x%x)")
#define	VPOP_GET_TASK_PORT(v, _pinfo, _portp, _flags) \
	_VPOP_(vpop_get_task_port, v, (v, _pinfo, _portp, _flags), \
		"(0x%x, &pinfo = 0x%x, &port=0x%x, flags=0x%x)")
#define VPOP_SETCTTY(v, _s, _ttyp, _flag) \
	_VPOP_(vpop_set_ctty, v, (v, _s, _ttyp, _flag), \
		"(0x%x, s=0x%x, &ttyp=0x%x, &flag=0x%x)")
#else
#define VPOP_SETCTTY(v, _cttydev, _cttynode,  _flag) \
	_VPOP_(vpop_setctty, v, (v, _cttydev, _cttynode,  _flag), \
		"(0x%x, cttydev=0x%x, cttynode=0x%x,  &flag=0x%x)")
#ifdef TETRA
#define VPOP_BSESSION(v, _bsession,  _flag) \
	_VPOP_(vpop_bsession, v, (v, _bsession,  _flag), \
		"(0x%x, &bsession=0x%x,  &flag=0x%x)")
#endif
#define VPOP_IS_ALIVE(v, _execnode) \
	_VPOP_(vpop_is_alive, v, (v, _execnode), \
		"(0x%x, &execnode=0x%x)")
#endif

/*
 * The following are the flag values for some vproc ops. The following
 * rules apply to these values:
 *  VPOP_SIG*		can use VPROC_HAS_PRIV, VSIG_CANT_BLOCK,
 *			VSIG_CHILD_STOP, VSIG_IFTRACED, VSIG_INTERNAL
 *			VSIG_NOORPHAN VSIG_PGSIG, VSIG_QUEUE_INFO
 *  VPOP_*_NICE		can use VPROC_SET, VPROC_GET, VPROC_HAS_PRIV (ignored
 *			if VPROC_GET is also used)
 *  VPOP_TERMINAL_PGRP  VSIG_NOORPHAN  and VSIG_CHECKGROUP
 *  VPOP_FORK		VPROC_ISVFORK, VPROC_ISFORKALL
 *
 *  Note that the flag values from 0x100000 to 0x40000000 are reserved for
 *  use by the underlying implementation and should not be defined here
 */

#define VPROC_ISVFORK		0x000040 /* fork call is of type vfork */
#define VPROC_ISFORKALL		0x000080 /* dup all threads during fork */
#define	VPROC_SET		0x000040 /* set specified flag */
#define	VPROC_GET		0x000080 /* retrieve flag */
#define	VPROC_CLEAR		0x000100 /* clear specified flag */
#define	VPROC_HAS_PRIV		0x000200 /* don't check priviledges to send */
#define	VSIG_CANT_BLOCK		0x001000 /* process cannot block */
#define	VSIG_CHILD_STOP		0x002000 /* don't deliver if parent ignoring */
#define	VSIG_IFTRACED		0x004000 /* untrace and deliver if traced */
#define VSIG_NOORPHAN		0x008000 /* return ENOTTY if process/pgrp is
					  * a member or a orphan pgrp (and 
					  * don't send signal) */
#define VSIG_CHECKGROUP		0x010000 /* used with VPOP_TERMINAL_SIGPGRP; 
					  *check if the group is null; if so 
					  * deliver to proc; else to pgrp; */
#define VSIG_QUEUE_INFO		0x020000 /* flag to signalling indicating that
					  * information about the signal may
					  * wish to be queued */
#define VSIG_PGSIG		0x040000 /* flag to signalling to indicate
					  * that the signal was sent to a
					  * process group */
#define VSIG_INTERNAL		0x080000 /* signal was generated from within
					  * the kernel */

/*
 * The following are the flag values for the VPOP_SETCTTY and 
 * VPOP_BSESSION vproc ops.
 */

#define VCTTY_SETCTTY		0x000040 /* set controlling tty */
#define VCTTY_CLEARCTTY		0x000080 /* clear controlling tty */
#define VCTTY_INACTIVECTTY	0x000100 /* set controlling tty to inactive */
#define VCTTY_SETFGPGRP		0x010000 /* set foreground pgrp */
#define VCTTY_CLEARFGPGRP	0x020000 /* clear foreground pgrp */
#ifdef TETRA
#define VCTTY_SETBSESS		0x040000 /* set background session */
#define VCTTY_CLEARBSESS	0x080000 /* clear background session */
#endif

#define VPROC_RESERVED          0x7FF00000  /* reserved for the implementation*/

/*
 * These values are returned by the job_control parameter in VPOP_GETATTR
 */
#define	VPROC_NOT_PGRPLEADER	-1	/* vproc is not a pgrp leader */
#define	VPROC_ORPHANED		0	/* vproc is orphaned pgrp leader */
#define	VPROC_JOB_CONTROL	1	/* vproc is pgrp with job control */

/*
 * These are set state values accepted by SET_STATE.
 */
#define	VPROC_ZOMBIE		1	/* vproc est mort */
#define	VPROC_STOP		2	/* vproc has stopped */
#define	VPROC_UNSTOP		3	/* vproc has resumed */
#define VPROC_LEFT_PGRP		4	/* vproc has resigned */
#define VPROC_TRAP		5	/* vproc stopped for tracing */
#ifdef NSC_SVR4
#define VPROC_TRACE_RUN		6	/* /proc runs a ptrace()'d child */
#define VPROC_RUN		7	/* vproc enters run state */
#endif
#define VPROC_GHOST		8	/* vproc was killed by nodedown */

/*
 * These are option values recognized by WAIT and REAP.
 */
#define VPROC_WNOHANG		0x01	/* wait will not sleep */
#define VPROC_WNOWAIT		0x02	/* no actual reap - just get stat */
#define VPROC_WSTOPPED		0x04	/* wait for stopped by signals procs */
#define VPROC_WUNSTOPPED	0x08	/* wait for continued procs */
#define VPROC_WTRAPPED		0x10	/* wait for stopped by tracing procs */
#define VPROC_WZOMBIE		0x20	/* wait for zombie procs */

/*
 * These are why code values accepted by VPOP_EXIT.
 */
#define VPROC_EXITED		0x01    /* process has exited */
#define VPROC_KILLED		0x02    /* process was killed */
#define VPROC_DUMPED		0x03    /* process has coredumped */


/*
 * These values are request codes to VPOP_PTRACE.
 * Currently, they map directly to the underlying ptrace codes.
 */
#define VPROC_PT_TRACE_ME	0	/* child declares it's being traced */
#define VPROC_PT_READ_I		1	/* read word in child's I space */
#define VPROC_PT_READ_D		2	/* read word in child's D space */
#define VPROC_PT_READ_U		3	/* read word in child's user struct */
#define VPROC_PT_WRITE_I	4	/* write word in child's I space */
#define VPROC_PT_WRITE_D	5	/* write word in child's D space */
#define VPROC_PT_WRITE_U	6	/* write word in child's user struct */
#define VPROC_PT_CONTINUE	7	/* continue the child */
#define VPROC_PT_KILL		8	/* kill the child process */
#define VPROC_PT_STEP		9	/* single step the child */

/*
 * To debug vproc ops or not, that is the question...
 */
#ifdef	VPROC_DEBUG
#ifdef	__STDC__
#define _VPOP_(op, v, args, argfmt) \
	(vproc_db1(v,#op,argfmt), vproc_db2 args, \
	 vproc_db3(v, op args))
#else	/* __STDC__ */
#define _VPOP_(op, v, args, argfmt) \
	(vproc_db1(v,"op",argfmt), vproc_db2 args, \
	 vproc_db3(v, op args))
#endif	/* __STDC__ */
#else	/* VPROC_DEBUG */
#define _VPOP_(op, v, args, argfmt)	(op args)
#endif	/* VPROC_DEBUG */

/*
 * The following declarations represent the interface from the VPROC
 * layer to the base physical code.
 */

extern struct proc;

extern int pproc_exit(			/* process exit */
		struct proc *p,		/* process structure */
		int rv, 		/* exit code */
		int why);		/* why code */
extern int pproc_reap(			/* reap zombie process */
		struct proc *p,		/* process structure */
		int options,		/* options (from wait system call */
		int *wstat,		/* returned wait status */
		struct pusage_dev *pusage); /* returned usage stats */
extern int pproc_add_pusage(		/* accumulate child usage stats */
		struct proc *p,		/* process structure */
		struct pusage_dev *pusage);  /* stats */
extern int pproc_fork(			/* fork a process */
		struct proc *pp,	/* parent process structure */
		pid_t pid,		/* pid to use for child */
		struct proc **pchild,	/* returns child process structure */
#ifdef OSF1
		thread_state_t	new_state, /* thread state */
		unsigned int	new_state_count, /* thread state count */
#else
		caddr_t	param1,		 /* thread state */
		unsigned int	param2,  /* thread state count */
#endif
		int flags);		/* fork flags (isvfork) */
#ifdef NSC_SVR4
#ifdef NSC_SVR40
extern int pproc_newproc(		/* create a process */
		int cond,		/* condition flags */
		pid_t pid,		/* pid to use for child */
		struct proc **childproc,/* ptr to child's proc struct */
		int *perror);		/* error return */
#endif /* NSC_SVR40 */
#ifdef NSC_SVR42
extern int pproc_spawn_proc(		/* spawn a process */
		int cond,		/* condition flags */
		pid_t pid,		/* pid to use for child */
		struct proc **childproc,/* ptr to child's proc struct */
		void (*func)(void *),	/* ptr to function to be executed */
		void *arg);		/* ptr to function arguments */
#endif /* NSC_SVR42 */
extern int pproc_execve(		/* exec a process */
		char *fname,		/* filename */
		char **argp,		/* arguments */
		char **envp,		/* environment */
		rval_t *rvp,		/* return values */
		int isrexec);		/* 1 => performing remote exec */
extern void pproc_end_of_syscall(
		int scall);		/* syscall number */
#endif
extern int pproc_get_nice(		/* get nice value for process */
		struct proc *p,		/* process structure */
		int *n);		/* returns nice velue */
extern int pproc_set_nice(		/* set nice value for process */
		struct proc *chgp,	/* process structure to be set */
		int n,			/* nice value to set */
		procinfo_t *pinfo,	/* calling process info  */
		int flags);		/* privilege information */
extern int pproc_chksigmask(		/* check signal masks of process */
		struct proc *p,		/* process structure */
#ifdef NSC_SVR42
		k_lwpid_t lwpid,	/* id of lwp */
#endif
		int sig,		/* signal number to check */
		int *ismasked,		/* returns whether signal is masked */
		int *isignored);	/* returns whether signal is ignored */
#ifdef NSC_SVR42
extern int pproc_postwait_post(
		pid_t pid,		/* pid of process to be posted to */
		int *not_waiting);	/* whether the process was waiting */
extern int pproc_lwpsignal(
		struct proc *p,		/* process containing thread */
		k_lwpid_t lwpid,	/* thread id */
		int sig,		/* signal to be delivered */
		procinfo_t *pinfo,	/* calling process info  */
		int flags,		/* flags affecting signal delivery */
		cred_t *cr);		/* cred of calling process */
#endif /* NSC_SVR42 */
extern int pproc_psignal(		/* post signal to process */
		struct proc *p,		/* sendee's process structure */
		int sig,		/* signal to be delivered */
		procinfo_t *pinfo,	/* calling process info  */
		int flags,		/* flags affecting signal delivery */
		cred_t *cr);		/* cred of calling process */
extern int pproc_getattr(		/* get process attributes */
		struct proc *p,		/* process structure */
		pid_t *pid,		/* returns pid */
		pid_t *ppid,		/* returns pid of parent */
		pid_t *pgid,		/* returns process group id  */
		pid_t *sid,		/* returns session id */
		int *has_sctty,		/* returns if has controlling tty */
		int *has_execd);	/* returns if process has called exec */

extern int pproc_setattr(		/* set process attributes */
		struct proc *p,		/* process structure */
		struct vproc *v,	/* corresponding vproc structure */
		pid_t *pid,		/* (ptr to) pid */
		pid_t *ppid,		/* (ptr to) pid of parent */
		pid_t *pgid,		/* (ptr to) process group id  */
		pid_t *sid,		/* (ptr to) session id */
		int *has_sctty, 	/* (ptr to) if has controlling tty */
		int setrun);		/* set process to run state */
extern int pproc_ptrace(		/* perform trace request on process */
		struct proc *p,		/* process structure */
		int req,		/* request code */
		caddr_t addr,		/* memory address */
		int data,		/* data to write */
		int *retval);		/* returned data */
extern int pproc_signal_received(	/* learn if this process has any sigs */
		struct proc *p,		/* process to learn from */
		int flags);		/* flag values */
#ifdef OSF1
extern int pproc_release_cttyv(		/* release vnode for controlling tty */
		struct tty *tp);	/* tty struct of controlling terminal */
extern int pproc_get_task_port(		/* get task_port for process */
		struct proc *p,		/* process structure */
		procinfo_t *pinfo,	/* calling process info  */
		mach_port_t *portp,	/* returns port of corresponding task */
		int flags);		/* priviledge */
#else  /* !OSF1 */
extern int pproc_setctty(
		struct proc *p,		/* process structure */
		dev_t cttydev,		/* ctty device number */
		clusternode_t cttynode,	/* ctty node number */
		int flag);		/* flags to determine what action
					 * to do */
extern int pproc_getctty(
		struct proc *p,		/* process structure */
		dev_t  *cttydev,	/* controlling terminal device */
		clusternode_t *cttynode);  /* controlling terminal node */
extern int pproc_pinfo(
		struct proc *p,		/* process structure */
		procinfo_t *pinfo);	/* process information */
extern int pproc_pidgen(
		int flags,		/* flag to determine what type of
					 * pid to generate */
		pid_t *newpid);		/* generated pid */
extern void pproc_setsigact(
		struct proc *p,		/* process structure */
		int signo,		/* signal being applied */
		void (*func)(),		/* disposition of signal */
		k_sigset_t mask,	/* implementation dependent mask */
		int flags);		/* implementation dependent flags */
extern void pproc_remove_process(
		void);
extern void pproc_cleanup_process(
		void);
#ifdef TETRA
extern int pproc_bsession(
		struct proc *p,		/* process structure */
		ulong bsession,		/* bsession value */
		int flag);		/* flags to determine what action
					 * to do */
#endif
#endif /* !OSF */

/*
 * The following prototypes are for virtual process system operations
 *	(operations on the virtual process system as a whole, rather than
 *	individual processes).
 */

#ifdef NSC_SVR42
extern int vpsop_siglwp_pid(
		IN pid_t pid,		/* pid of process to signal */
		IN k_lwpid_t lwpid,	/* lwpid of thread to signal */
		IN int sig,		/* signal to post */
		IN procinfo_t *pinfo,	/* calling process information */
		IN int flag);		/* affects signaling action */
extern int vpsop_sigproc_pid(
		IN pid_t pid,		/* pid of process to signal */
		IN int sig,		/* signal to post */
		IN procinfo_t *pinfo,	/* calling process information */
		IN int flag);		/* affects signaling action */
extern int vpsop_sigpgrp_pid(
		IN pid_t pgid,		/* pgid of process group to signal */
		IN int sig,		/* signal to post */
		IN procinfo_t *pinfo,	/* calling process information */
		IN int flag);		/* affects signaling action */
#endif /* NSC_SVR42 */
#ifdef NSC_SVR4
extern int vpsop_procset_search(
		IN struct procset *psp,	/* Set of processes to which to apply */
					/* function. */
		IN int init_is_special,	/* process init specially */
		INOUT vproc_procset_params_t *fptr); /* "Function pointer" of */
					/* function to apply. */
extern int vpsop_procset_apply(
		IN struct procset *psp,	/* Set of processes to which to apply */
					/* function. */
		INOUT vproc_procset_params_t *fptr); /* "Function pointer" of */
				 	/* function to apply. */
#endif /* NSC_SVR4 */
#ifdef OSF1
extern int vpsop_sigprocset(
		IN struct procset *ps,	/* set of processes to be signalled */
		IN int signo,		/* signal to be sent */
		IN procinfo_t *pinfo,	/* calling process information */
		IN int flags);		/* priviledge flags */
extern int vpsop_procset_nice(
		IN struct procset *ps,	/* set of processes changed/queried */
		INOUT int *nice,	/* nice value */
		IN procinfo_t *pinfo,	/* calling process information */
		IN int flag);		/* VPROC_SET or VPROC_GET and priviledge*/
extern int vpsop_sethostid(
		IN long new_hostid);	/* hostid to be set */
extern int vpsop_sethostname(
		IN char *new_hostname,	/* hostname to be set */
		IN unsigned int new_hostnamelen); /* hostname length */
extern int vpsop_setdomainname(
		IN char *new_domainname, /* domainname to be set */
		IN unsigned int new_domainnamelen); /* domainname length */
extern int vpsop_reboot(
		IN int flags,		/* indicates whether panicking */
		IN int options);	/* options applying to reboot */
extern int vpsop_pid_from_task(
		IN task_t task,	/* task port to which pid refers to */
		OUT pid_t *pid,		/* pid of process using task */
		OUT char *comm,		/* command name for pid */
		INOUT unsigned int *commlen); /* length of command name */
extern int vpsop_table(
		IN int id,		/* id of action to take */
		IN int index,		/* index into table */
		IN caddr_t addr,	/* address in user space */
		IN int nel,		/* number of elements in "addr" */
		IN u_int lel,		/* length of each element in "addr" */
		OUT int *retval);	/* returned value */
#else  /* !OSF1 */
extern int vpsop_shutdown(
		IN int flag1,		/* reboot flag #1 */
		IN int flag2);		/* reboot flag #1 */
extern int vpsop_setlimit(
		IN int users,		/* Licensed number of users */
		IN int procs,		/* Licensed number of processors */
		IN int nodes);		/* Licensed number of nodes */
#endif /* !OSF1 */

/*
 * The following macros are used to invoke the virtual process system 
 * operations:
 */
#ifdef NSC_SVR42
#define VPSOP_SIGLWP_PID(_pid, _lwpid, _sig, _pinfo, _flag) \
	_VPSOP_(vpsop_siglwp_pid, (_pid, _lwpid, _sig, _pinfo, _flag), \
		"(pid=%d, lwpid=%d, sig=%d, &pinfo=0x%x, flag=0x%x)")
#define VPSOP_SIGPROC_PID(_pid, _sig, _pinfo, _flag) \
	_VPSOP_(vpsop_sigproc_pid, (_pid, _sig, _pinfo, _flag), \
		"(pid=%d, sig=%d, &pinfo=0x%x, flag=0x%x)")
#define VPSOP_SIGPGRP_PID(_pgid, _sig, _pinfo, _flag) \
	_VPSOP_(vpsop_sigpgrp_pid, (_pgid, _sig, _pinfo, _flag), \
		"(pgid=%d, sig=%d, &pinfo=0x%x, flag=0x%x)")
#endif /* NSC_SVR42 */
#ifdef NSC_SVR4
#define VPSOP_PROCSET_SEARCH(_psp, _init_special, _funcptr) \
	_VPSOP_(vpsop_procset_search, (_psp, _init_special, _funcptr), \
		"(&psp=0x%x, init_special=%d, funcptr=0x%x)")
#define VPSOP_PROCSET_APPLY(_psp, _funcptr) \
	_VPSOP_(vpsop_procset_apply, (_psp, _funcptr), \
		"(&psp=0x%x, funcptr=0x%x)")
#endif /* NSC_SVR4 */
#ifdef OSF1
#define VPSOP_SIGPROCSET(_ps, _signo, _pinfo, _flags) \
	_VPSOP_(vpsop_sigprocset, (_ps, _signo, _pinfo, _flags), \
		"(&ps=0x%x, signo=%d, pinfo=0x%x, flags=0x%x)")
#define VPSOP_PROCSET_NICE(_ps, _nice, _pinfo, _flags) \
	_VPSOP_(vpsop_procset_nice, (_ps, _nice, _pinfo, _flags), \
		"(&ps=0x%x, &nice=0x%x, pinfo=0x%x, flags=%d)")
#define VPSOP_SETHOSTID(_new_hostid) \
	_VPSOP_(vpsop_sethostid, (_new_hostid), \
		"(new_hostid=%d)")
#define VPSOP_SETHOSTNAME(_new_hostname, _new_hostnamelen) \
	_VPSOP_(vpsop_sethostname, (_new_hostname, _new_hostnamelen), \
		"(new_hostname=%s, new_hostnamelen=%d)")
#define VPSOP_SETDOMAINNAME(_new_domainname, _new_domainnamelen) \
	_VPSOP_(vpsop_setdomainname, (_new_domainname, _new_domainnamelen), \
		"(new_domainname=%s, new_domainnamelen=%d)")
#define VPSOP_REBOOT(_flag, _options) \
	_VPSOP_(vpsop_reboot, (_flag, _options), \
		"(flag=0x%x, options=0x%x)")
#define VPSOP_PID_FROM_TASK(_task, _pid, _comm, _commlen) \
	_VPSOP_(vpsop_pid_from_task, (_task, _pid, _comm, _commlen), \
		"(&task=0x%x, pid=0x%x, &comm=0x%x, &commlen=0x%x)")
#define VPSOP_TABLE(_id, _index, _addr, _nel, _lel, _retval) \
	_VPSOP_(vpsop_table, (_id, _index, _addr, _nel, _lel, _retval), \
		"(id=%d, index=%d, addr=0x%x, nel=%d, lel=%d, &retval=0x%x)")
#else  /* !OSF1 */
#define VPSOP_SHUTDOWN(_flag1, _flag2) \
	_VPSOP_(vpsop_shutdown, (_flag1, _flag2), \
		"(flag1=0x%x, flag2=0x%x)")
#define VPSOP_SETLIMIT(_users, _procs, _nodes) \
	_VPSOP_(vpsop_setlimit, (_users, _procs, _nodes), \
		"(users=%d, procs=%d, nodes=%d)")
#endif /* !OSF1 */

/*
 * To debug vproc system ops or not, that is the question...
 */
#ifdef	VPROC_DEBUG
#ifdef	__STDC__
#define _VPSOP_(op, args, argfmt) \
	(vps_db1(#op,argfmt), vps_db2 args, \
	 vps_db3(op args))
#else	/* __STDC__ */
#define _VPSOP_(op, args, argfmt) \
	(vps_db1("op",argfmt), vps_db2 args, \
	 vps_db3(op args))
#endif	/* __STDC__ */
#else	/* VPROC_DEBUG */
#define _VPSOP_(op, args, argfmt) \
	op args
#endif	/* VPROC_DEBUG */

/*
 * The following declarations represent the interface from the VPROC
 * system layer to the base physical code.
 */

extern int pps_procset_get_nice(
		struct procset *ps,	/* set of processes changed/queried */
		int *n);		/* nice value returned */
extern int pps_procset_set_nice(
		struct procset *ps,	/* set of processes changed/queried */
		int n,			/* nice value to set */
		procinfo_t *pinfo,	/* calling process info  */
		int flags);		/* priviledge flag */
#ifdef OSF1
extern int pps_sigprocset(
		struct procset *ps,	/* set of processes to be signalled */
		int signo,		/* signal to be sent */
		procinfo_t *pinfo,	/* calling process info  */
		int flags);		/* priviledge flag */
extern int pps_sethostid(
		long new_hostid);	/* hostid to be set */
extern int pps_sethostname(
		char *new_hostname,	/* hostname to be set */
		unsigned int new_hostnamelen); /* hostname length */
extern int pps_setdomainname(
		char *new_domainname,	/* domainname to be set */
		unsigned int new_domainnamelen); /* domainname length */
extern int pps_bootsync(
		int flags,		/* indicates whether panicking */
		int options);		/* options applying to reboot */
extern int pps_boot(
		int flags,		/* indicates whether panicking */
		int options);		/* options applying to reboot */
extern int pps_pid_from_task(
		task_t task,		/* task port to which pid refers to */
		pid_t *pid,		/* pid of process using task */
		char *comm,		/* command name for pid */
		unsigned int *commlen);	/* length of command name */
extern int pps_table(
		void *uap,		/* arguments to system call */
		int *retval);		/* return value */
#else
extern int pps_release_ctty(
		dev_t cttydev,		/* device to cleared as ctty */
		pid_t sid,		/* session id which is clearing ctty */
		pid_t *pgid);		/* foreground pgrp */
extern int pps_release_ctty_fgpgrp(
		dev_t cttydev,		/* controlling terminal device */
		pid_t fgpgrp);		/* foreground pgrp */
extern int pps_shutdown();
extern int pps_unmount_all();
extern int pps_reboot(
		int flag1,		/* reboot flag #1 */
		int flag2);		/* reboot flag #2 */
extern void pps_setlimit(
		int users,		/* Licensed number of users */
		int procs,		/* Licensed number of processors */
		int nodes);		/* Licensed number of nodes */
#endif
#ifdef NSC_SVR4
extern int pps_updateloads(
		clusternode_t from,	/* node sending the load */
		struct load_data_t *ld_data);	/* load average data */
#endif


#ifdef NSC_SVR4
#ifdef VPROC_DEBUG
#define VPROC_CTTY_TRACE(list) 			\
	do {					\
        extern int cttytrace;   		\
	extern void (*cttydebug_func)();	\
		if(cttytrace) 			\
			(*cttydebug_func) list;	\
	} while(0)
#else
#define VPROC_CTTY_TRACE(list)
#endif
#endif

#ifdef NSC_SVR4
#ifndef WAIT_ANY
#define WAIT_ANY -1
#endif
#endif

#endif  /* !_NSC_GENERIC_SYS_VPROC_H */
