/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

#ifndef _MEM_PAGE_H	/* wrapper symbol for kernel use */
#define _MEM_PAGE_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/mem/page.h /main/uw7_nj/3"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/mem/page.h,v 1.6 2000/03/06 05:53:28 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *	Copyright (c) 1982, 1986, 1988
 *	The Regents of the University of California
 *	All Rights Reserved.
 *	Portions of this document are derived from
 *	software developed by the University of
 *	California, Berkeley, and its contributors.
 */

/*
 * +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * 		PROPRIETARY NOTICE (Combined)
 * 
 * This source code is unpublished proprietary information
 * constituting, or derived under license from AT&T's UNIX(r) System V.
 * In addition, portions of such source code were derived from Berkeley
 * 4.3 BSD under license from the Regents of the University of
 * California.
 * 
 * 
 * 
 * 		Copyright Notice 
 * 
 * Notice of copyright on this source code product does not indicate 
 * publication.
 * 
 * 	(c) 1986,1987,1988,1989  Sun Microsystems, Inc
 * 	(c) 1983,1984,1985,1986,1987,1988,1989  AT&T.
 * 	          All rights reserved.
 *  
 */

#ifdef _KERNEL_HEADERS

#include <mem/anon.h>	/* REQUIRED */
#include <mem/mem_hier.h> /* REQUIRED */
#include <mem/vm_hat.h> /* REQUIRED */
#include <mem/vmparam.h> /* REQUIRED */
#include <svc/systm.h>	/* REQUIRED */
#include <util/ksynch.h> /* REQUIRED */
#include <util/param.h> /* REQUIRED */
#include <util/types.h> /* REQUIRED */

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <sys/ksynch.h> /* REQUIRED */
#include <sys/param.h> /* REQUIRED */
#include <sys/systm.h>	/* REQUIRED */
#include <sys/types.h> /* REQUIRED */
#include <sys/vmparam.h> /* REQUIRED */
#include <vm/anon.h>	/* REQUIRED */
#include <vm/vm_hat.h> /* REQUIRED */
#include <vm/mem_hier.h> /* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */

#if defined _KERNEL || defined _KMEMUSER

/*
 * struct page describes a physical page frame state and contents.
 * -----------
 *
 * Overview:
 *
 * Each physical page has a page structure, which is used to maintain
 * these pages as a cache.  A page can be found via a per-vnode AVL tree
 * indexed by offset.  If a page has a [vp, offset] identity, then it
 * is entered on a doubly linked circular list off the vnode (v_pages) using
 * the vpnext/vpprev pointers. This list is sorted by offset, with the
 * sort being achieved via the AVL tree index. If the p_free bit is on,
 * then the page is also on a doubly linked circular free list using the
 * p_next/p_prev pointers. If the p_pageout bit is on, then the page is
 * currently being written to backing store. If the p_invalid bit is on, then
 * the page is being filled in with valid data (usually by reading from
 * backing store). In the case of such I/O, the p_next/p_prev pointers are
 * used to link the pages together for a consecutive I/O request (and p_free
 * must be off). The p_next/p_prev pointers can be used for other lists (only
 * if p_free is off and there is no I/O in progress on the page).
 *
 * Detail:
 *
 * Each struct page has a pseudo reader/writer sleeplock associated with it.
 * A standard reader/writer sleeplock is not used here because the conditions
 * for lock aquisition involve page-specific state information in addition to
 * the standard number of current readers and writers.
 *
 * p_activecnt/p_invalid indicate the presence of reader/writer locks for
 * the page:
 *
 *	p_activecnt  p_invalid
 *	-----------  ---------
 *	     0		0	No locks
 *	     0		1	INVALID LOCK STATE ("can't happen")
 *	    >0		0	Reader (shared) lock(s) held.
 *	     1		1	Writer (exclusive) lock held.
 *	    >1		1	INVALID LOCK STATE ("can't happen")
 *
 * A reader lock is acquired when the caller needs to ensure that the
 * page state, identity (vnode, offset), and data are valid and remain so.
 *
 * A writer lock is acquired when the callers wishes to change the
 * page state, identity, or data contents.
 *
 * When a reader/writer lock cannot be immediately acquired, the caller
 * waits on the p_wait synchronization variable via PAGE_WAIT(pp).
 *
 * All waiters on p_wait are awoken via PAGE_BROADCAST(pp) whenever:
 *	- p_pageout is cleared
 *	- p_invalid is cleared (drop writer lock or downgrade to reader lock)
 *	- p_activecnt is decremented to zero
 *
 * If the p_retain bit is on, then the page is being retained for segmap.
 * Note that replicas are never retained. When a page is retained,
 * p_mapcount is incremented by one (as if the page had an extra
 * translation).
 *
 * Note that this procedure does not grant page reader/writer locks in
 * any particular order; all waiters race to get the lock.  Thus it is
 * possible for lock contenders (especially writers) to starve.
 *
 * Page reader/writer locks are held to guarantee kernel correctness of
 * operation.  (I.e., the lock holder would panic or Do Bad Things if the
 * lock were subverted.)
 *
 *
 * Fields in struct page are mutexed by a collection of locks including
 * the per vnode spin lock VN_PGLOCK(), VM_PAGEFREELOCK() and 
 * PAGE_VPLOCK() spinlocks, and the page reader/writer lock.  (Note that
 * vm_pagefreelock and VN_PGlock should always be manipulated via their 
 * appropriate macros, never directly. This is because the priority levels
 * associated with these locks are cached since the locking and the unlocking
 * agents can be in different functions. See, e.g., VM_PAGEFREELOCK() and
 * VN_PGLOCK()).
 *
 * - All fields are mutexed by P_VPGLOCK except as described below.
 *
 * - p_next/p_prev are mutexed by the VM_PAGEFREELOCK if the page is on the
 *   free list. However, if the page is on a vnode list, VN_PGLOCK only is
 *   required.
 *
 * - p_nio is used exclusively by the current logical I/O operation in
 *   progess on the page.  On file systems where the block size is
 *   smaller than the page size, there may be multiple concurrent 
 *   physical I/O operations using the page to accomplish the single
 *   logical I/O operation. p_nio is mutexed by the VN_PGLOCK.
 *
 *   A file system is not required to set p_nio. If p_nio is not set
 *   (i.e. if it is left at 0) then the pvn functions interpret this
 *   to mean one I/O needs to be initiated (equivalent to p_nio == 1).
 *   If p_nio is set, it will normally be by the VOP_GETPAGE and doputpage
 *   functions, or subroutines thereof. Holding the page locked in
 *   PI or PO state is sufficient to read p_nio (provided that no I/Os have
 *   yet been initiated). However, the VN_PGLOCK is is still required to
 *   modify p_nio (since p_nio shares a long with the page flags). Since a
 *   VOP_GETPAGE and/or doputpage functions is not permitted to directly
 *   access the VN_PGLOCK, these functions acquire the necessary lock
 *   by calling page_uselock().
 *
 *   When p_nio is decremented by pvn_done(), the PI/PO state can no longer
 *   play any role in mutexing p_nio (even for just reading), since multiple
 *   physical I/O operations could be simultaneously completing on the page.
 *
 * - Modification of the page identity fields requires holding both
 *   VN_PGLOCK() and PAGE_VPLOCK. VN_PGLOCK or PAGE_VPLOCK also needs to be
 *   held to read the following fields:
 *		p_vnode, p_offset
 *
 * - Modification of the p_free field requires holding both
 *   VM_PAGEFREELOCK() and VN_PGLOCK(pp) while holding a page writer lock.
 *   Thus holding either VM_PGLOCK() or VM_PAGEFREELOCK() is sufficient
 *   to read the p_free bit.
 *
 * - p_pgprv_data is controlled by the "owner of the page identity".  That is,
 *   if the page has vnode identity, p_pgprv_data is controlled by that vnode's
 *   filesystem (including anonfs if it's an anon page); otherwise, if the
 *   page is in use exclusively by one agent (e.g. by HAT as a page table),
 *   p_pgprv_data is controlled by that agent.  Any necessary mutexing within
 *   use by the controlling agent is the responsibility of that agent.
 *
 *   However, the page layer shares responsibility with anon for managing
 *   anon pages. Thus, the page layer examines fields:
 *		P_IDCACHE(pp), ANON_PGPRV(pp)->a_ap
 *   These fields are mutexed by the pseudo ANON_T_LOCK (see anon.h for
 *   more info).
 *
 * - p_hat is mutexed by the hat layer as it sees fit.
 */

#define PGPRVSZ	4	/* # of words of use-private data */

struct pageheader {	/* separate so marker pages don't need whole page_t */
	struct page	*pgh_vpnext;	/* next page in vnode list */
	struct page	*pgh_vpprev;	/* prev page in vnode list */
	struct vnode 	*pgh_vnode;	/* logical vnode this page is from */
	off64_t		 pgh_offset;	/* offset into vnode for this page */
};
#define p_vpnext	p_pghdr.pgh_vpnext
#define p_vpprev	p_pghdr.pgh_vpprev
#define p_vnode		p_pghdr.pgh_vnode
#define p_offset	p_pghdr.pgh_offset

typedef struct page {

	/*
	 * Page Identity Information:
	 *	Mutexed by VN_PGLOCK().
	 *      The p_vnode and p_offset fields require p_idlock.
	 */

	struct pageheader p_pghdr;	/* vnode identity info */
	struct page	*p_vpleft;	/* AVL tree left pointer: was p_hash */

	/* Mutexed by appropriate page list lock: (VN_PGLOCK or VM_FREELOCK) */	

	struct page	*p_next;	/* next page in free/intrans list */
	struct page	*p_prev;	/* prev page in free/intrans list
					   (max value is # fileblocks/page) */

	/* Mutexed by VN_PGLOCK (except p_free; see above): */

	ulong_t		 p_activecnt;	/* number of reader/writer locks held
					      (p_invalid implies writer lock) */

	clock_t		 p_timestamp;	/* time page was found to be dirty */

	sv_t		 p_wait;	/* to wait for page state change */

	uchar_t		 p_nio;		/* number of outstanding I/O reqs */

			 /*
			  * The p_type fields is only used if restricted
			  * DMA support is compiled into the kernel and
			  * rdma_mode != RDMA_DISABLED.
			  */
	uint_t		 p_type: 2,	/* STD_PAGE, DMA_PAGE, or EDMA_PAGE */
			 /*
			  * End of restricted DMA support.
			  */

			 p_retain: 1,	/* retained by segmap: was p_physdma */
			 p_pageout: 1,	/* data for page is intransit out */
			 p_invalid: 1,	/* data invalid; e.g. being paged in */
			 p_mod: 1,	/* software copy of modified bit */
			 p_ioerr: 1,	/* I/O error when p_nio > 1 */
			 p_free: 1,	/* page is on a clean/dirty free list */
                         p_realuser: 1, /* page is M_REAL_USER reserved */
                         p_replica: 1,  /* page is a replica */
			 p_balance: 2,	/* subtrees are equal depth */
			 p_may_write: 1;/* VOP_GETPAGE has granted write */
					/* permission */
	ulong_t		 p_mapcount;	/* vixible xlat count: was p_mapaddr */
	struct page	 *p_vpright;	/* AVL tree right pointer */

	/* constant for the life of the page structurte */
	uint_t		 p_pfn;		/* cached pfn, to speed page_pptonum */

	/* Mutexed by VN_PGLOCK: memory reservations */
	ushort_t	 p_rcount;	/* M_REAL resvs */
	ushort_t	 p_scount;	/* M_SWAP resvs */

	struct page      *p_set; 

	union phat	 p_hat;		/* HAT-specific structure.
					   This structure is for those fields
					   which must be valid for all pages
					   and can't use the page-private data.
					   One field, in particular, must be
					   part of this structure: mappings
					   (referred to as p_mapping),
					   whose exact semantics are hat-
					   specific, but which must always,
					   generically, be non-zero if the page
					   has any translations mapped to it,
					   and zero otherwise. */

	/*
	 * p_pgprv_data mutexing controlled by "owner of page identity".
	 * If not exclusively owned, p_pgprv_data is not used.
	 */

	ulong_t	 	p_pgprv_data[PGPRVSZ];	/* use-private data */

	/* The space for following two fields was occupied by p_lid */
	/* in the standard kernel (i.e. CC_PARTIAL not defined) */
	lock_t		p_idlock;	/* mutex for <p_vnode, p_offset> */
#ifdef NSC
	char		p_blkrsrv;	/* CFS page block reservations */
	char		p_distate;	/* CFS page data integrity state */
	char		p_filler[1];	/* space for future use */
#else  /* !NSC */
	char		p_filler[3];	/* space for future use */
#endif /* !NSC */
#ifdef CC_PARTIAL
	lid_t		p_lid;
#endif /* CC_PARTIAL */

	/* constant data */
	cgnum_t		p_cg;

	/* Mutexed by VN_PGLOCK (debug only) */
#ifdef DEBUG
	int		p_tree_height;
#endif 
} page_t;

#define PAGE_IS_LEAF(pp) (pp->p_vpleft == NULL && pp->p_vpright == NULL)
#define PAGE_IS_ROOT(pp) (pp->p_vpleft == NULL && pp->p_vpright == NULL)
#define PAGE_IS_MARKER(pp, vp) ((pp)->p_vnode != (vp))

#define IS_VALID_LEAF(pp) (PAGE_IS_LEAF(pp) && \
				(pp)->p_balance == BALANCED && \
				(pp)->p_tree_height == 0)
#define IS_VALID_ROOT(pp) (PAGE_IS_ROOT(pp) && (pp)->p_vnode->v_pages == (pp))

#ifdef DEBUG
#define PAGE_SET_HEIGHT(pp, ex)		((pp)->p_tree_height = (ex))
#define PAGE_BUMP_HEIGHT(pp, ex)	((pp)->p_tree_height += (ex))
#define VNODE_BUMP_COUNT(vp, chg)	((vp)->v_page_count += (chg))
#else
#define PAGE_SET_HEIGHT(pp, ex)		((void)0)
#define PAGE_BUMP_HEIGHT(pp, ex)	((void)0)
#define VNODE_BUMP_COUNT(vp, chg)	((void)0)
#endif

/*
 * The PAGE_STACK_SIZE is the maximal size of a stack needed to
 * traverse the AVL tree of the biggest possible file. In order to
 * make this code ready for 64 bit processors, we use a big number!
 */
#define PAGE_STACK_SIZE			52

typedef enum { LEFT, BALANCED, RIGHT } balance_t;

typedef enum {
	/*
	 * This case occurs when the decent through the tree hasn't
	 * traversed an unbalanced node. No rotations will is necessary.
	 * But the entire path, from the root to the parent of the new leaf,
	 * will become unbalanced.
	 */
	I_NONE,

	/*
	 * These cases occur when the decent through the tree has
	 * traversed an unbalanced node, but an addition seeks to
	 * restore the balance. Therefore, no ratations are necessary.
	 */
	I_LR,		/* balance point leaning LEFT, adding RIGHT */
	I_RL,		/* balance point leaning RIGHT, adding LEFT */

	/*
	 * The following are intermediates states during tree traversal.
	 * However, leaf insertion cannot begin in these states since
	 * the target node is occupied.
	 */
	I_LL,		/* balance LEFT, adding LEFT */
	I_RR,		/* balance RIGHT, adding RIGHT */

	/*
	 * In the following cases, the insertion parent is in fact a leaf.
	 * The following cases require a single "rotation" of the tree
	 * (in the opposite direction to the lean). These cases are
	 * differentated from the next two cases as an optimization.
	 */
	I_LLL,		/* balance LEFT, adding LEFT, LEFT */
	I_RRR,		/* balance RIGHT, adding RIGHT, RIGHT */

	/*
	 * The following cases require a single "rotation" of the tree
	 * (in the opposite direction to the lean).
	 */
	I_LLLX,		/* balance LEFT, adding LEFT, LEFT, MORE */
	I_RRRX,		/* balance RIGHT, adding RIGHT, RIGHT, MORE */

	/*
	 * In the following cases the insertion parent is in fact a leaf.
	 * The tree will be balanced by exchaning the new child
	 * with the node at the balance point. These cases are
	 * differentated from the four cases below as an
	 * optimization.
	 */
	I_LLR,		/* balance LEFT, adding LEFT, RIGHT */
	I_RRL,		/* balance LEFT, adding LEFT, LEFT */

	/*
	 * The following "dreaded" cases require the execution of
	 * a comlicated "double rotation" balancing act in order to
	 * compensate for two opposing leans. Even worse, there are
	 * four of these cases to render in the code.
	 */
	I_LLRL,		/* balance LEFT, adding LEFT, RIGHT, LEFT */
	I_LLRR,		/* balance LEFT, adding LEFT, RIGHT, RIGHT */
	I_RRLR,		/* balance RIGHT, adding RIGHT, LEFT, RIGHT */
	I_RRLL		/* balance RIGHT, adding RIGHT, LEFT, LEFT */
} adding_balance_t;


#endif /* _KERNEL || _KMEMUSER */

/*
 * VN_PGLOCK(), VM_PAGEFREELOCK() Lock Acquisition Order:
 * --------------------------------------------------
 *
 * When both these locks need to be concurrently held, VN_PGLOCK() must be
 * acquired first, then the VM_PAGEFREELOCK().
 *
 * The motivation for choosing this order is as follows:
 *
 * There are different parts of the VM system which would like
 * different orders.  (E.g., page allocation wants to acquire
 * VM_PAGEFREELOCK first and then VN_PGLOCK for the allocated
 * page; however, page deallocation wants the other order.
 *
 *
 * Callers may NOT concurrently hold VN_PGLOCKs for multiple vnodes.
 *
 * The motivation for this is simple deadlock avoidance.
 * This rule is assumed by VN_PGLOCK() when it caches the current
 * ipl in l.vpagelockpl which is only unique per-processor.
 *
 * 1)  page_lookup_or_create() would like to hold the VN_PGLOCK
 *     first and if it couldn't find a page with the right identity, 
 *     get the VM_PAGEFREELOCK to call page_get_l().
 * 2)  But page_get_l() itself would like to acquire the VM_PAGEFREELOCK
 *     first and THEN the VN_PGLOCK ; this is because if it found a
 *     page on the free list which still has vnode identity, it needs to
 *     remove the page from the AVL tree vp list and this would require
 *     the VN_PGLOCK lock.
 * 3)  page_unfree_l() also needs to acquire the VN_PGLOCK
 *     while holding VM_PAGEFREELOCK if it races with page_get_l(); if
 *     page_get_l() gets there first, then it needs to destroy remove the
 *     the page from the AVL and vp list, so that the caller can assign
 *     a new identity.
 * 4)  hat_map() pre-loads translations for pages associated with a
 *     vnode and if it finds the page is on the free list, it reclaims the
 *     page from the free list. Thus it would like to acquire the VN_PGLOCK
 *     first and then the VM_PAGEFREELOCK.
 *
 * We decided to require that the VN_PGLOCK be acquired BEFORE the 
 * VM_PAGEFREELOCK since cases 2 & 3 are infrequent and we optimize for
 * high runners (1 & 4).
 */

/*
 * Acquiring/releasing VM_PAGEFREELOCK() and VN_PGLOCK out of order:
 * ------------------------------------------------------------
 *
 * In cases where it is necessary to acquire VM_PAGEFREELOCK() before
 * (try locking) VN_PGLOCK:
 *
 *	The ipl value associated with the VM_PAGEFREELOCK() is
 *	VM_PAGE_IPL and this could be potentially different from
 * 	the ipl value associated with the VN_PGLOCK().
 *	Because of this, the VM_PAGEFREELOCK & P_VPGLOCK
 *	routines cannot be used because they hardcode the ipl values and
 *	assume the lock order will guarantee that the ipl level will not be
 *	inappropriately weakened. In this case, explicit calls to TRYLOCK()
 *	and UNLOCK() must be used.  (See below.)
 *
 * In cases where it is necessary to release these locks out of order
 * with respect to their acquisition order:
 *
 *	Because the ipl values active at lock acquisition time are opaquely
 *	cached by the VM_PAGEFREELOCK() & P_VPGLOCK() routines, the
 *	caller of these routines must release the locks in the reverse
 *	order they were acquired to restore the correct ipl levels.
 *
 *	When the caller cannot follow this order, and the caller will
 *	release both locks, the caller can explicitly call UNLOCK()
 *	passing the appropriate ipl value.
 *
 *	When the caller cannot follow this order, and the caller will
 *	NOT release both locks, the caller explicitly calls UNLOCK()
 *	for the lock it is releasing (passing the appropriate ipl level)
 *	and then updates the opaquely cached ipl value for the other
 *	lock so that a subsequent VM_PAGEFREEUNLOCK()/PAGE_VPGUNLOCK(pp)
 *	will restore the correct ipl level.
 *
 *
 * EXAMPLES:
 *
 * Lock stronger lock; remember initial ipl.
 * Trylock weaker (out of order) lock; ignore returned ipl value.
 * 
 * 	ipl = LOCK(&VM_PAGEFREELOCK, VM_PAGE_IPL);
 * 	if (VN_PGTRYLOCK(&pp->p_vnode) == INVPL) {
 * 		goto failed;	/ * trylock failed * /
 * 	}
 * 
 * 	...perform operation needing locking...
 * 
 * then either drop in acquisition order:
 * 
 * 	/ * retain ipl of stronger lock: * /
 * 	VN_PGUNLOCK(&pp->p_vnode, VM_PAGE_IPL);
 * 	UNLOCK(&vm_pagefreelock, ipl);
 * 
 * or drop out of acquisition order:
 * 
 * 	/ * downgrade to ipl of weaker lock: * /
 * 	UNLOCK(&vm_pagefreelock, VM_PAGE_IPL);
 * 	UNLOCK(&pp->p_vnode->VN_PGlock, ipl);
 * 
 * or delayed drop out of acquisition order:
 * 
 * 	/ * downgrade to ipl of weaker lock: * /
 * 	UNLOCK(&vm_pagefreelock, VM_PAGE_IPL);
 * 	/ * cache initial ipl for later PAGE_VPGUNLOCK(pp): * /
 * 	l.vpglockpl = ipl;
 */

#ifdef _KERNEL

/*
 * VM_PAGEFREELOCK() is a global spinlock which protects:
 *
 *	page free lists:
 *		mem_freemem, page_freelist, page_cachelist, page_dirtyalist, 
 *		page_dirtyflist, p_next, p_prev, p_free
 *
 * Note that manipulating most p_* fields also requires holding P_VPGLOCK
 * and possibly also a page reader/writer lock on the effected page.
 * See the discussion of struct page.
 *
 * vm_pagefreelock should always be manipulated via VM_PAGEFREELOCK() and
 * VM_PAGEFREEUNLOCK().
 */

extern lock_t	vm_pagefreelock;    /* mutex page free lists and ID lists */
extern pl_t	vm_pagefreelockpl;  /* pl for current vm_pagefreelock holder */

/*
 * VM_PAGEFREELOCK()
 * VM_PAGEFREEUNLOCK()
 *
 *	Spinlock (unlock) the global page free lists.
 *
 * Calling/Exit State:
 *
 *	VM_PAGEFREELOCK() caches the current ipl level for later use
 *	by VM_PAGEFREEUNLOCK().  The ipl value is cached in the global
 *	variable vm_pagefreelockpl. In order for this to work, the caller
 *	must release this in the reverse order they were acquired.
 */
#if (!VM_INTR_IPL_IS_PLMIN)

#define VM_PAGEFREELOCK()	(vm_pagefreelockpl = LOCK(&vm_pagefreelock, VM_PAGE_IPL))
#define VM_PAGEFREEUNLOCK()	UNLOCK(&vm_pagefreelock, vm_pagefreelockpl)

#else 	/* (VM_PAGE_IPL = PLMIN) */

#define VM_PAGEFREELOCK()	(vm_pagefreelockpl = LOCK_PLMIN(&vm_pagefreelock))
#define VM_PAGEFREEUNLOCK()	UNLOCK_PLMIN(&vm_pagefreelock, vm_pagefreelockpl)

#endif /* (!VM_INTR_IPL_IS_PLMIN) */

/*
 * VM_PAGEFREELOCK_OWNED()
 *
 *	Returns true if the calling context holds vm_pagefreelock,else false.
 *	(DEBUG-only)
 */

#ifdef DEBUG
#define VM_PAGEFREELOCK_OWNED()	LOCK_OWNED(&vm_pagefreelock)
#endif /* DEBUG */
 
/*
 * VM_PAGEFREELOCK_LOCKED()
 *
 *	Returns true if anyone holds vm_pagefreelock else false.
 *	(DEBUG-only)
 */

#ifdef DEBUG
#define VM_PAGEFREELOCK_LOCKED()	IS_LOCKED(&vm_pagefreelock)
#endif /* DEBUG */

/*
 * vm_hashlock and being maintained for agents external to VM (i.e.
 * for the Veritas Volume Manager).
 */
extern lock_t	vm_hashlock;    /* OBSOLETE: was mutex page hash list */
extern pl_t	vm_hashlockpl;  /* OBSOLETE: was pl for vm_hashlock holder */

/*
 * VM_HASHLOCK()
 * VM_HASHUNLOCK()
 * VM_TRYHASHLOCK()
 *
 *	Spinlock (unlock) the global hash lists.
 *
 * Calling/Exit State:
 *
 *	VM_HASHLOCK() caches the current ipl level for later use by
 *	VM_HASHUNLOCK(). The ipl value is cached in the global variable
 *	vm_hashlockpl. In order for this to work correctly, the caller
 *	must release the spin locks in the reverse order of that in
 *	which  the locks are acquired. 
 */
#if (!VM_INTR_IPL_IS_PLMIN)

#define VM_HASHLOCK()		\
	(vm_hashlockpl = LOCK(&vm_hashlock, VM_PAGE_IPL))
#define VM_HASHUNLOCK()		\
	UNLOCK(&vm_hashlock, vm_hashlockpl)
#define VM_TRYHASHLOCK()	\
	TRYLOCK(&vm_hashlock, VM_PAGE_IPL)

#else /* (!VM_INTR_IPL_IS_PLMIN) */

#define VM_HASHLOCK()		\
	(vm_hashlockpl = LOCK_PLMIN(&vm_hashlock))
#define VM_HASHUNLOCK()		\
	UNLOCK_PLMIN(&vm_hashlock, vm_hashlockpl)
#define VM_TRYHASHLOCK()	\
	TRYLOCK_PLMIN(&vm_hashlock)

#endif	/* (!VM_INTR_IPL_IS_PLMIN) */

/*
 * VM_HASHLOCK_OWNED()
 *
 *	Returns true if the calling context holds vm_hashlock() else false.
 *	(DEBUG-only)
 */

#ifdef DEBUG
#define VM_HASHLOCK_OWNED()	LOCK_OWNED(&vm_hashlock)
#endif /* DEBUG */

/*
 * VM_HASHLOCK_LOCKED()
 *
 *	Returns true if anyone holds vm_hashlock() else false.
 *	(DEBUG-only)
 */

#ifdef DEBUG
#define VM_HASHLOCK_LOCKED()	IS_LOCKED(&vm_hashlock)
#endif /* DEBUG */

#ifdef DEBUG
#define ASSERT_PP_IN_RANGE(pp) {\
		ASSERT(((vaddr_t) (pp) >= calloc_base) \
		       && ((vaddr_t) (pp) < calloc_end)); \
		if (PAE_ENABLED()) \
			ASSERT(kvtol2ptep64((pp)->p_pfn) != 0); \
		else \
			ASSERT(kvtol2ptep((pp)->p_pfn) != 0); \
		ASSERT((PFN_TO_SLICE((pp)->p_pfn) < pslice_table_entries) \
		       && (pslice_table[PFN_TO_SLICE((pp)->p_pfn)].ps_type \
			   == PS_PAGES) \
		       && (page_numtopp((pp)->p_pfn) == (pp))); \
			       }		
#else
#define ASSERT_PP_IN_RANGE(pp)	((void)0)
#endif				    

#define	VALID_PP(pp)	((pp) >= pages && ((pp) < epages) \
			 && (!PAGE_IS_DUMMY(pp)))
	
/*
 * Interfaces for locking/unlocking the VN_PGLOCK. 
 * 	For VM modules, _MEM_INTERNAL_ is defined, and the macros access 
 * the data structures directly.  For non-VM modules, _MEM_INTERNAL_ is 
 * not defined, and procedure calls are used to acquire or release the
 * VN_PGLOCK.     
 *	For early releases of Unixware 2.0X, the spin lock was defined
 * as a per page structure spin lock, and was called the page uselock.
 * The interfaces for acquiring or releasing the (per page 
 * structure) spin lock were the functions page_uselock and page_useunlock.
 * For forward compatibility of the early 2.0X file systems, these
 * functional interfaces are retained and perform the equivalent work
 * of the macros- VN_PGLOCK/VN_PGUNLOCK.
 */
#ifdef _MEM_INTERNAL_
/*
 * PAGE_VPGLOCK(pp)
 * PAGE_VPGUNLOCK(pp)
 *	Spinlock (unlock) the vnode identity of the page. 
 *
 * Calling/Exit State:
 *	The caller must guarantee an existant vnode for the page.
 */
#define PAGE_VPGLOCK(pp) 	\
	(ASSERT((pp)->p_vnode != NULL), VN_PGLOCK((pp)->p_vnode))

#define PAGE_VPGUNLOCK(pp)	\
	(ASSERT((pp)->p_vnode != NULL), VN_PGUNLOCK((pp)->p_vnode))

#define	PAGE_USELOCK(pp)	PAGE_VPGLOCK(pp)


#define	PAGE_USEUNLOCK(pp)	PAGE_VPGUNLOCK(pp)


#define	PAGE_VPGUNLOCK_PL(pp, pl)	\
	(ASSERT((pp)->p_vnode != NULL), VN_PGUNLOCK_PL((pp)->p_vnode, (pl)))

#define	PAGE_USEUNLOCK_PL(pp, pl)	PAGE_VPGUNLOCK_PL(pp, pl)


#define PAGE_TRYVPGLOCK(pp)	\
	(ASSERT((pp)->p_vnode != NULL), VN_PGTRYLOCK((pp)->p_vnode))

#define	PAGE_TRYUSELOCK(pp)		PAGE_TRYVPGLOCK(pp)

/*
 * PAGE_VPGLOCK_OWNED(pp)
 *	Wrapper for VN_PGLOCK_OWNED()
 */
#ifdef DEBUG
#define PAGE_VPGLOCK_OWNED(pp)	\
	(ASSERT((pp)->p_vnode != NULL), VN_PGLOCK_OWNED((pp)->p_vnode))

#define	PAGE_USELOCK_OWNED(pp)	PAGE_VPGLOCK_OWNED(pp)
#endif /* DEBUG */

/*
 * PAGE_VPGLOCK_LOCKED(pp)
 *	Wrapper for VN_PGLOCK_LOCKED()
 */
#ifdef DEBUG
#define PAGE_VPGLOCK_LOCKED(pp)	\
	(ASSERT((pp)->p_vnode != NULL), VN_PGLOCK_LOCKED((pp)->p_vnode))

#define	PAGE_USELOCK_LOCKED(pp)	PAGE_VPGLOCK_LOCKED(pp)
#endif /* DEBUG */

#else /* _MEM_INTERNAL_ */

extern void page_uselock(page_t *);
extern void page_useunlock(page_t *);
extern pl_t page_tryuselock(page_t *);

#define	page_vpglock	page_uselock
#define page_vpgunlock	page_useunlock
#define	page_tryvpglock	page_tryuselock

/*
 * Function call aliasing for PAGE_*USE*LOCK and PAGE_*VPG*LOCK
 * callers outside the mem module.
 *
 * Note: PAGE_VPGUNLOCK_PL callers are all internal
 * to the mem module. So it does not require aliasing.
 */
#define PAGE_USELOCK(pp)	page_uselock(pp)
#define PAGE_USEUNLOCK(pp)	page_useunlock(pp)
#define PAGE_TRYUSELOCK(pp)	page_tryuselock(pp)

#define PAGE_VPGLOCK(pp)	page_vpglock(pp)
#define PAGE_VPGUNLOCK(pp)	page_vpgunlock(pp)
#define PAGE_TRYVPGLOCK(pp)	page_tryvpglock(pp)

#ifdef DEBUG

extern boolean_t page_uselock_owned(page_t *);
extern boolean_t page_uselock_locked(page_t *);

#define	page_vpglock_owned	page_uselock_owned
#define	page_vpglock_locked	page_uselock_locked

#define PAGE_USELOCK_OWNED(pp)	page_uselock_owned(pp)
#define PAGE_USELOCK_LOCKED(pp)	page_uselock_locked(pp)

#define PAGE_VPGLOCK_OWNED(pp)	page_vpglock_owned(pp)
#define PAGE_VPGLOCK_LOCKED(pp)	page_vpglock_locked(pp)

#endif /* DEBUG */

#endif /* _MEM_INTERNAL_ */

/*
 * PAGE_WAIT(pp)
 *
 *	If bclnlist is not empty, then call cleanup hoping that the page
 *	you are waiting for changes state. If bclnlist is empty then
 *	Wait on page synchronization variable for page state to change.
 *
 * Calling/Exit State:
 *
 *	Caller must currently hold PAGE_VPGLOCK(pp).
 *	It is dropped but reacquired before returning.
 */

#define P_WAIT_PRI	(PRIMEM-1)	/* disp for p_wait SV_WAIT */

#define WAITERS_ON_PAGE(pp) SV_BLKD(&(pp)->p_wait)

#define PAGE_WAIT(vp, pp) \
	( \
		ASSERT(l.vpglockpl == PLBASE), \
		ASSERT(!VN_IS_RELEASED(vp)), \
		ASSERT(VN_PGLOCK_OWNED(vp)), \
		ASSERT(KS_HOLD1LOCK()), \
		((bclnlist) ? \
			( \
				VN_PGUNLOCK(vp), \
				cleanup() \
			) \
		      : SV_WAIT(&(pp)->p_wait, P_WAIT_PRI, &(vp)->v_pglock) \
		), \
		ASSERT(!VN_IS_RELEASED(vp)), \
		ASSERT(KS_HOLD0LOCKS()), \
		ASSERT(getpl() == PLBASE), \
		VN_PGLOCK(vp) \
	)

/*
 * boolean_t
 * PAGE_WAIT_EXISTS(pp)
 *
 *	Wait for the page as in PAGE_WAIT above.
 *
 * Calling/Exit State:
 *
 *	Return B_TRUE if the page still exists and B_FALSE otherwise.
 *
 *	Caller must currently hold PAGE_VPGLOCK(pp).
 *	It is dropped but reacquired before returning.
 *
 * Remarks:
 *	This macro avoids acquiring the PAGE_IDLOCK(pp) by assuming
 *	ATOMIC access to pointers. This works because page_destroy_identity()
 *	clears pp->p_vnode while holding VN_PGLOCK(pp). Similarly,
 *	page_create_identity() sets pp->p_vnode while holding VN_PGLOCK(pp).
 *	Therefore, if the page has some other identity, we will observe
 *	pp->p_vnode as NULL or some other value. But, never as pp->p_vnode.
 *	Once pp->p_vnode is verified, it is then safe to verify pp->p_offset.
 */

#define PAGE_WAIT_EXISTS(vp, pp, off) \
	( \
		ASSERT(l.vpglockpl == PLBASE), \
		ASSERT(VN_PGLOCK_OWNED(vp)), \
		ASSERT(KS_HOLD1LOCK()), \
		((bclnlist) ? \
			( \
				VN_PGUNLOCK(vp), \
				cleanup() \
			) \
		      : SV_WAIT(&(pp)->p_wait, P_WAIT_PRI, &(vp)->v_pglock) \
		), \
		ASSERT(KS_HOLD0LOCKS()), \
		ASSERT(getpl() == PLBASE), \
		VN_PGLOCK(vp), \
		PAGE_HAS_IDENTITY(pp, vp, off) \
	)

/*
 * PAGE_BROADCAST(pp)
 *
 *	Wakeup everyone blocked on a page synchronization variable.
 *
 * Calling/Exit State:
 *
 *	Caller must currently hold PAGE_VPGLOCK(pp).
 *
 * Description:
 *
 *	We only do this if there is someone currently blocked on the
 *	synchronization variable.  We use SV_BLKD() to check if there
 *	are any waiters as a performance optimization;  this is safe to
 *	do since any new blockers must hold PAGE_VPGLOCK(pp) and we already
 *	hold it here. We have a PAGE_BROADCAST_U macro that gets called
 *	from basyncdone(). There is a handshaking effort between
 *	PAGE_BROADCAST_U and PAGE_WAIT in calling cleanup.
 */

/* Unconditional page_broadcast : Don't check if anybody blocking */

#define PAGE_BROADCAST_U(pp)	SV_BROADCAST(&(pp)->p_wait, 0)

#define PAGE_BROADCAST(pp) \
	{ \
		if (SV_BLKD(&(pp)->p_wait)) \
			SV_BROADCAST(&(pp)->p_wait, 0); \
	}

/*
 * PAGE_IS_RDLOCKED(pp)
 * PAGE_IS_WRLOCKED(pp)
 * PAGE_IS_LOCKED(pp)
 *
 *	Returns true if the page is read locked, write locked, or
 *	locked in either mode.
 *
 * Calling/Exit State:
 *
 *	The returned result is stale unless the caller holds
 *	a page reader/writer lock or VN_PGLOCK().
 */

#define PAGE_IS_RDLOCKED(pp)	((pp)->p_activecnt > 0 && !(pp)->p_invalid)
#define PAGE_IS_WRLOCKED(pp)	((pp)->p_invalid)
#define PAGE_IS_LOCKED(pp)	page_is_locked((pp))
#define THIS_PAGE_IS_LOCKED(pp)	((pp)->p_activecnt > 0)

boolean_t page_is_locked(page_t *);

/*
 * PAGE_IN_USE(pp)
 *
 *	Returns true if the physical page, pp, is currently in use.
 *
 * Calling/Exit State:
 *
 *	Caller must currently hold VN_PGLOCK().
 *
 *	A struct page is in use if it has any reader/writer locks or
 *	any active translations.  A page should not be page_free()'d
 *	while it is in use.  Whenever a reader/writer lock is released
 *	(p_activecnt decremented) or a translation removed, the page
 *	should be freed via page_free() iff !PAGE_IN_USE(pp).
 *
 *	The complementary statement is that before a reader/writer lock is
 *	acquired (p_activecnt incremented) or a translation established, the
 *	page must have been first reclaimed from the free list if it is free.
 *	Usually this is done by routines which find pages such as page_lookup().
 */

#define	THIS_PAGE_IN_USE(pp)	((pp)->p_activecnt || (pp)->p_mapcount)
#define PAGE_IN_USE(pp)		page_in_use(pp)
#define PAGE_IS_REPLICA(pp)	((pp)->p_replica)
#define	PAGE_IS_FREE(pp)	page_is_free(pp)

boolean_t	page_in_use(page_t *);
boolean_t	page_is_free(page_t *);



/*
 * PAGE_HAS_IDENTITY(pp, vp, off)
 *
 *	Returns true if the physical page, pp, has the <vnode, offset> identity.
 *
 * Calling/Exit State:
 *
 *	The returned result is stale unless the caller holds one or more of
 *	a page reader/writer lock, or VN_PGLOCK()
 */

#define PAGE_HAS_IDENTITY(pp, vp, off) \
	    (((vp) == (pp)->p_vnode) && ((off) == (pp)->p_offset))

/*
 * PAGE_RECLAIM(page_t *pp)
 *
 *	Attempt to reclaim the given page from the free list, if it is free.
 *
 * Calling/Exit State:
 *
 *	See page_reclaim() for further description.
 */

#define PAGE_RECLAIM(pp)	(!pp->p_free || page_reclaim(pp))

/*
 * IS_PAGE_AGED(page_t *pp, clock_t stamp, clock_t interval)
 *
 *	Check if page has aged more than the given age interval.
 *
 * Calling/Exit State:
 *
 *	No locks need to be held since this is only an approximation.
 */

#define IS_PAGE_AGED(pp, stamp, interval) \
	(TICKS_BETWEEN((pp)->p_timestamp, (stamp)) > (interval))

/*
 * void
 * PAGE_SETMOD(page_t *pp)
 *	Set the modify flag in a page and in the associated vnode.
 *
 * Calling/Exit State:
 *	The VN_PGLOCK is held by the caller.
 *
 * Description:
 *	This function assumes that all writable translations installed
 *	into visible mappings in a user address space are for pages with
 *	vnode identity. Actually, all pages installed into any visible
 *	mapping have vnode identity at this time.
 */
#define PAGE_SETMOD(pp) {				\
	vnode_t *vp = (pp)->p_vnode;			\
							\
	ASSERT(VN_PGLOCK_OWNED(vp));			\
	ASSERT(vp != NULL);				\
							\
	(pp)->p_mod = 1;				\
	VN_SETMOD(vp);					\
}

/*
 * Page ID lock operations.
 */
#define PAGE_IDLOCK(pp)		LOCK(&(pp)->p_idlock, VM_PAGE_IPL)
#define PAGE_TRY_IDLOCK(pp)	\
		((void) TRYLOCK(&(pp)->p_idlock, VM_PAGE_IPL))
#define PAGE_IDUNLOCK(pp, ipl)	UNLOCK(&(pp)->p_idlock, ipl)


/*
 * uint_t pageroundup(uint_t size)
 * uint_t pagerounddown(uint_t size)
 *
 *	Round a byte count or address up or down to a page boundary.
 */
#define pageroundup(size)	(((size) + PAGESIZE - 1) & PAGEMASK)
#define pagerounddown(size)	((size) & PAGEMASK)

/*
 * page_get() request flags.
 *
 * SLEEP (0) and NOSLEEP (1) are defined in <param.h>.
 *
 * The P_NODMA flag is obsolete. Like 0, it signifies no DMA requirements.
 * However, it still exists for binary compatibility.
 */
#define P_DMA			0x0002	/* must have DMAable pages */
#define P_NODMA			0x0004	/* pages have no DMA requirements */
#define P_EDMA			0x0008  /* must have EDMAable pages */
#define P_DMA_MASK	(P_DMA|P_EDMA)

#define P_MAY_WRITE		0x0010	/* require/set p_may_fault */
#define P_SETMOD		0x0020
#define P_RETAIN		0x0040	/* retain the page longer */
#define P_DONTNEED		0x0080	/* don't need the page at all */

/* 
 * The following flags are used by page_unfree_l() to indicate if the
 * caller intends to create the page with this identity. If not, then
 * this routine does not drop any previously held locks in the failure
 * case in the race with page_get_l().
 */
#define P_LOOKUP	0x0040
#define P_LOOKUP_OR_CREATE	0x0080
/* 
 * following flag is to inform freemem_resv routines that reservation
 * should be done only on the cg passed in 
 */
#define P_REQ_CG        0x0100

#if defined(NSC_DSWAP) || defined(NSC_CSPECFS) || defined(NSC_CFS)
/*
 * Indicates the caller is requesting a page that will be used for a
 * stacking filesystem, requires additional memory to be paged out.
 */
#define P_STACKINGFS		0x0200
#endif /* NSC_DSWAP || NSC_CSPECFS || NSC_CFS */
#endif	/* _KERNEL */

#if defined _KERNEL || defined _KMEMUSER

/*
 * Page types:
 *
 *	If restricted DMA support is compiled into the kernel, pages will
 *	be divided into several types. In the following discussion, the
 *	meaning of the word ``DMAable'' is family specific. However, the
 *	family will generally use ``DMAable'' to mean that all devices
 *	performing DMA can access the page, whereas ``non-DMAable'' means
 *	that some devices performing DMA cannot access the memory. This
 *	division of memory will work conveniently for a family with at most
 *	two classes of I/O devices: ``old'' devices with a DMA restriction
 *	(where all old devices have the same restriction), and ``new''
 *	devices with no restriction.
 *
 *	STD_PAGE
 *
 *		Pages which are not necessarily DMAable.
 *
 *	DMA_PAGE
 *
 *		Pages which are DMAable to minimum constraints (usually
 *		24-bits) and are allocated to clients which require
 *		 DMAability. The only way a VM client can allocate
 *		memory from these pages is through kmem_alloc() family
 *		functions.
 *
 *	EDMA_PAGE
 *
 *		Pages which are DMAable to 32-bits. EDMA pages are only
 *		used on systems with physical memory above 4GB.
 *
 *	NO_PAGE
 *
 *		Under no circumstances do pages of type NO_PAGE exist. Type
 *		NO_PAGE exists as part of an optimization for the
 *		freemem_resv() family of functions. It provides a
 *		``secondary page type'' which can never be allocated from.
 *
 * Memory is divided into these page types according to the rdma_mode in
 * effect:
 *
 *	RDMA_DISABLED, RDMA_MICRO:
 *
 *		All pages become STD_PAGE. In this case STD_PAGE pages
 *		are in fact DMAable (since all of memory is DMAable).
 *
 * 	RDMA_SMALL:
 *
 *		Pages are divided into two pools: a DMA_PAGE pool for
 *		24-bit (MAXDMAPAGE) DMAable memory, and a STD_PAGE pool
 *		for everything else.
 *
 *	RDMA_LARGE:
 *
 *		Pages are divided into three pools: a DMA_PAGE pool for
 *		24-bit (MAXDMAPAGE) DMAable memory; a EDMA_PAGE pool for
 *		32-bit DMAable memory, and a STD_PAGE pool for everything
 *		else.
 */

#define STD_PAGE	0
#define DMA_PAGE	1
#define EDMA_PAGE	2
#define NO_PAGE		3
#define NPAGETYPE	4
extern int page_ntype_used;

typedef struct  page_slice {
          uchar_t    ps_type;      /* empty, idf or pages */
          cgnum_t  ps_cg;          /* CPU Group */
          union {
                  /* pointer to first page structure */
                  page_t  *psi_pages;
                  /* Bitmap to differentiate IDF pages from holes */
                  uint_t  *psi_exclude;
          } ps_info;
} page_slice_t;

#define ps_pages 	ps_info.psi_pages
#define ps_exclude 	ps_info.psi_exclude

/*
 * page_slice mstypes
 */
#define PS_EMPTY         0               /* no memory present */
#define PS_PAGES         1               /* page structs present */
#define PS_PURE_NONPAGED 2               /* identity free - no holes */
#define PS_NONPAGED	 3		 /* identity free with a bitmap for
					    holes */

/*
 * special pfn values for the page structures
 */
#define PAGE_PFN_EMPTY  0xffffffff      /* no memory here */
#define PAGE_PFN_IDF    0xfffffffe      /* identity free */
#define PAGE_IS_DUMMY(pp) (((pp)->p_pfn == PAGE_PFN_EMPTY) \
			   || ((pp)->p_pfn == PAGE_PFN_IDF))

/* Upper bound on the size of the page slice table */
#define MAX_SLICE_ENTRIES 2048
#define MAX_SLICE_BITS 11

/* memory here */
extern page_slice_t *pslice_table;

extern uint_t pslice_table_entries;        /* entries in the table */
extern uint_t pslice_pages;              /* pages in a slice */

extern uint_t pslice_page_shift;          /* log_base2(slice_pages) */
extern uint_t pslice_page_mask;           /* The corresponding mask */

#define PFN_TO_SLICE(pfn) ((pfn) >> pslice_page_shift)
#define PFN_TO_SLICER(pfn) (((pfn) + (pslice_pages - 1)) >> pslice_page_shift)
#define SLICE_TO_PFN(slice) ((slice) << pslice_page_shift)
#define SLICE_PAGE_NUM(pfn) ((pfn) & ~pslice_page_mask)

/* Bitmap related macros -- highly sensitive to the width of an integer! */
#define MAPWORD(slicep, bit_position) \
	((slicep)->ps_exclude + ((bit_position) >> 5))
#define BITSET(bitmap, bit_position) \
	((bitmap) |= (1 << ((bit_position) & 0x1f)))
#define BITTEST(bitmap, bit_position) \
	((bitmap) & (1 << ((bit_position) & 0x1f)))

#endif /* _KERNEL || _KMEMUSER */

#ifdef _KERNEL

extern page_t **page_cs_table;		/* contiguous section table */
#ifdef DEBUG
extern int page_cs_table_size;		/* # entries in page_cs_table */
#endif

/*
 * uint_t
 * page_pptonum(const page_t *pp)
 *
 *	Translate page_t to hardware physical page frame number.
 *
 * Calling/Exit State:
 *
 *	Returns the page frame number corresponding to "pp".
 */
#define page_pptonum(pp) ((pp)->p_pfn)

/*
 * cgnum_t
 * page_home(const page_t *pp)
 *
 *	Translate page_t to the cpu-group number.
 *
 * Calling/Exit State:
 *
 *	Returns the cpu-group corresponding to "pp".
 */
#define page_home(pp) ((pp)->p_cg)

/* page structure address for 1st page pool page (of a type) in the system */
extern page_t	*pages;		

/* one past last page pool page */
extern page_t	*epages;

extern vaddr_t calloc_base, calloc_end; /* For ASSERT_PP_IN_RANGE */

extern int mem_freemem[NPAGETYPE];	/* # free pages per type */
extern int page_dirtylists_size[NPAGETYPE];
					/*
					 * # dirty but not-in-use pages (both
					 * file and non-file) of each type.
					 */
extern int maxfreemem[NPAGETYPE];	/* max # free pages per type */

/*
 * Paging control thresholds.
 */
extern int mem_lotsfree[NPAGETYPE];
extern int mem_desfree[NPAGETYPE];
extern int mem_minfree[NPAGETYPE];

#define lotsfree	(mem_lotsfree[STD_PAGE])
#define desfree		(mem_desfree[STD_PAGE])
#define minfree		(mem_minfree[STD_PAGE])

/*
 * Macros for checking mem_freemem and maxfreemem against threshholds of
 * various types.
 */
#define cg_freemem(cgno) (*CGVAR(&cg.mem_freemem[STD_PAGE], int, (cgno)))
#define cg_page_dl_size(cgno) (*CGVAR(&cg.page_dirtylists_size[STD_PAGE],    \
                                      int, (cgno)))

#define page_dl_size	(page_dirtylists_size[STD_PAGE])
#define page_fl_size	(page_freelist_size[STD_PAGE])
#define page_type(pp)	((pp)->p_type)

#ifdef NOT_CCNUMA
#define freemem          	mem_freemem[STD_PAGE]
#define min_mem_notinuse()	(freemem + MIN(page_dl_size, nswappgfree))
#else	/* CCNUMA */
#define freemem          	(total_freemem(STD_PAGE))
#define avail_freemem		(total_avail_freemem())
#define min_mem_notinuse()	(total_mem_notinuse())
#endif	/* CCNUMA */

#define cg_min_mem_notinuse(cgno) (cg_freemem(cgno) + \
			MIN(cg_page_dl_size(cgno), (nswappgfree / Ncg)))
#define IS_VALID_REPLICA(pp) ((pp)->p_set != pp &&\
		              (pp)->p_invalid == 0 &&\
		              (pp)->p_activecnt == 0 &&\
		     	      (pp)->p_mod == 0 &&\
		     	      (pp)->p_retain == 0 &&\
		     	      (pp)->p_replica == 1)

/*
 * Variables controlling locking of physical memory.
 */
extern uint_t pages_pp_maximum;		/* tuning: lock + claim <= max */
extern uint_t pages_dkma_maximum;

#ifdef DEBUG
#define EXTRA_PAGE_STATS 1
#endif

#if defined(EXTRA_PAGE_STATS) || defined(lint)

/*
 * The statistics in struct page_tcnt are only adjusted
 * by routines which hold either the VM_PAGEFREELOCK() or the
 * VN_PGLOCK().
 *
 * The following are protected by VM_PAGEFREELOCK():
 * 	pc_free* stats, pc_get* stats, pc_reclaim* stats.
 *
 * The following are protected by VN_PGLOCK():
 *	pc_exists* stats 
 */

struct page_tcnt {
uint_t	pc_free;		/* pages freed */
uint_t	pc_free_dontneed;	/* pages freed with dontneed */
uint_t	pc_free_free;		/* pages freed into free list */
uint_t	pc_free_cache_clean;	/* pages freed into clean cache list */
uint_t	pc_add_retain;		/* pages placed onto the retain list */
uint_t	pc_free_cache_dirty;	/* pages freed into dirty cache list */
uint_t	pc_free_anon_cachelist;	/* pages freed into anon cache list */
uint_t	pc_get;			/* get's (successful + failed) */
uint_t	pc_get_fail_NOSLEEP;	/* get's failed due to NOSLEEP */
uint_t	pc_get_fail_TRYLOCK;	/* vestigeal */
uint_t	pc_get_cache;		/* pages gotten from cache list */
uint_t	pc_get_retain;		/* pages gotten from retain list */
uint_t	pc_get_anon_cache;	/* pages gotten from anon cache list */
uint_t	pc_get_free;		/* pages gotten from free list */
uint_t	pc_getaligned;		/* get_aligned's (successful + failed) */
uint_t	pc_getaligned_fail_NOSLEEP; /* get_aligned's failed due to NOSLEEP */
uint_t	pc_getaligned_unavail;	/* get_aligned's not immediately available */
uint_t	pc_reclaim;		/* reclaim's during & not during pageout */
uint_t	pc_reclaim_pgout;	/* reclaim's during pageout */
uint_t	pc_reclaim_lost;	/* attempted reclaim's lost due to race */
uint_t	pc_exist_hit;		/* exist's that find page */
uint_t	pc_exist_miss;		/* exist's that don't find page */
#define	PC_GET_CNT	10
uint_t	pc_get_npages[PC_GET_CNT+1]; /* histogram of get's num pages */
};

extern struct page_tcnt pagecnt;

#define	BUMPPGCOUNT(x)	++(x)

#else	/* defined(EXTRA_PAGE_STATS) || defined(lint) */

#define	BUMPPGCOUNT(x) 

#endif	/* defined(EXTRA_PAGE_STATS) || defined(lint) */


/*
 * Page frame operations.
 */

struct buf;
struct anon;
uint_t  total_freemem(uint_t);
uint_t  total_avail_freemem(void);
uint_t  total_mem_notinuse(void);
void	init_mem_globals();
page_t	*page_init_chunk(page_t *, uint_t, paddr64_t, page_t **, boolean_t);
void	page_init(void);
void	page_cs_init(void);
void 	page_slice_idf_init(void);
#ifdef DEBUG
void    page_slice_verify(void);
#endif
page_t *page_get(uint_t, uint_t, cgnum_t);
page_t *page_get_anon(cgnum_t);
page_t *page_get_aligned(uint_t, paddr_t, paddr_t, paddr_t, uint_t, cgnum_t);
void	page_free(page_t *, uint_t);
void	page_free_l(page_t *, uint_t);
page_t *page_exists(const struct vnode *, off64_t);
page_t *page_find_first(const struct vnode *, off64_t);
page_t *page_find_smallest(const struct vnode *);
page_t *page_exists_vpglock(struct vnode *, off64_t);
void	page_downgrade_lock(page_t *);
void	page_unlock(page_t *);
void	page_list_unlock(page_t *);
void	page_anon_unlock(struct anon *, uint_t);
void	page_add(page_t **, page_t *);
void	page_sub(page_t **, page_t *);
void	page_sortadd(page_t **, page_t *);
void	page_setmod(page_t *);
void	page_setmod_iowait(page_t *);
void	page_setmod_unlock_l(page_t *, uint_t);
int	page_reclaim(page_t *);
int	page_reclaim_l(page_t *, u_int flags);
void	page_abort(page_t *);
void	page_abort_l(page_t *);
void	page_assign_identity(page_t *, vnode_t *, off64_t);
page_t *page_numtopp(uint_t pfn);
page_t * page_numtopp_safe(uint_t pfn);
#ifdef CCNUMA
cgnum_t pfntoCG(uint_t pfn);
#endif
boolean_t mainstore_memory(paddr64_t addr);
page_t *page_numtopp_idx_lowres(uint_t pfn, uint_t idx, uint_t idx_res);
page_t *page_phystopp(paddr64_t);
paddr64_t page_pptophys(const page_t *);
void	pagezero(page_t *, ulong_t, ulong_t);
void	ppcopy(page_t *, page_t *);
void	ppcopyrange(page_t *, page_t *, uint_t, uint_t);
void	pzero(ppid_t);
void	pcopy(ppid_t from_ppid, ppid_t to_ppid);
void	pageout_init(void);
int	page_anonpageout(int, clock_t, clock_t, page_t **);
void	page_swapreclaim(int);
boolean_t page_cache_query(struct vnode *, off64_t);
boolean_t page_anon_decref(anon_t *);
int	page_maxfreemem(cgnum_t);
page_t  *page_find_closest(struct seg *, page_t *, uint_t);
cgnum_t	page_place(struct seg *, vnode_t *);
page_t *page_lookup_range(int, vnode_t *, off64_t, uint_t);
void	page_unlock_range(page_t *, int, uint_t);
page_t *page_lookup_or_create_anon(struct seg *, struct anon *);
#ifndef UNIPROC
vnode_t *page_compound_lock(page_t *, boolean_t *);
#endif /* UNIPROC */
void page_add_retain(page_t *);
void page_del_retain(page_t *);
int page_age_retain(clock_t, ulong_t);

#ifdef _FSKI

void	page_find_iowait(struct vnode *, off_t);
void	page_find_unload(struct vnode *, off_t);
void	page_find_zero(struct vnode *, off_t);
off_t	page_getoffset(page_t *pp);
page_t *page_lazy_create(struct seg *, struct vnode *, off_t);
page_t *page_lookup_or_create(struct seg *, struct vnode *, off_t, uint_t);

#else

void	page_find_iowait(struct vnode *, off64_t);
void	page_find_unload(struct vnode *, off64_t);
void	page_find_zero(vnode_t *, off64_t);
off64_t	page_getoffset(page_t *pp);
page_t *page_lazy_create(struct seg *, struct vnode *, off64_t);
page_t *page_lookup_or_create(struct seg *, struct vnode *, off64_t, uint_t);

#endif	/* _FSKI */

#ifdef CCNUMA
void	page_abort_replica(page_t *);
page_t  *page_find_closest(struct seg *, page_t *, uint_t);
page_t	*page_find_closest_l(struct seg *, page_t *, uint_t);
#endif

/*
 * HAT callback interfaces.
 */

/*
 * Loading a translation into the kernel address space.
 *
 * Increment p_mapcount.
 */
#define PAGE_KAS_LOAD(pp) {               \
	ASSERT(VN_PGLOCK_OWNED((pp)->p_vnode)); \
	ASSERT(!IS_ANONVP((pp)->p_vnode));      \
	++(pp)->p_mapcount;                     \
}

/*
 * Unloading a translation from the kernel address space.
 *
 * Decrement p_mapcount.
 *
 * This macro must be invoked by the hat layer before
 * the page is freed.
 */
#define PAGE_KAS_UNLOAD(pp) {             \
	ASSERT(VN_PGLOCK_OWNED((pp)->p_vnode)); \
	ASSERT(!IS_ANONVP((pp)->p_vnode));      \
	--(pp)->p_mapcount;                     \
}

/*
 * Loading a translation into a user address space.
 *
 * Increment P_MAPADDR(pp).
 *
 * If the page is anonymous and this is the first
 * translation being loaded for it, then set
 * P_ASLIST(pp) and P_MAPADDR(pp).
 */
#define PAGE_UAS_LOAD(as, addr, pp) {           \
	ASSERT(VN_PGLOCK_OWNED((pp)->p_vnode)); \
	if (++(pp)->p_mapcount == 1 &&		\
	    IS_ANONVP((pp)->p_vnode)) {         \
		ASSERT(P_MAPADDR(pp) == 0);	\
		ASSERT(P_ASLIST(pp) == NULL);   \
		P_MAPADDR(pp) = (addr);		\
		P_ASLIST(pp) = (as)->a_anchor;  \
	}                                       \
	ASSERT(!PAGE_IS_FREE((pp)));		\
}

/*
 * Unloading a translation from a user address space.
 *
 * Decrement p_mapcount.
 *
 * This macro must be invoked by the hat layer before
 * the page is freed.
 */
#define PAGE_UAS_UNLOAD(as, pp) {                 \
	ASSERT(VN_PGLOCK_OWNED((pp)->p_vnode));         \
	ASSERT((pp)->p_mapcount != 0);                  \
	if (--(pp)->p_mapcount == 0 &&               	\
	    IS_ANONVP((pp)->p_vnode)) {			\
		P_MAPADDR(pp) = NULL;			\
		P_ASLIST(pp) = NULL;			\
	}						\
}

/*
 * Anon support
 */

/*
 * Is an anonymous page in core.
 *
 * Information is stale upon return.
 */
#define page_cache_query_anon(ap) \
	(((ap)->an_page) != NULL)

#endif	/* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif	/* _MEM_PAGE_H */
