#ifndef _MEM_PVN_H	/* wrapper symbol for kernel use */
#define _MEM_PVN_H	/* subject to change without notice */

#ident	"@(#)unixsrc:usr/src/common/uts/mem/pvn.h /main/uw7_nj/2"
#ident	"$Header: /src/nsc_uw7/usr/src/common_nsc/uts/mem/pvn.h,v 1.6 2000/03/06 05:53:29 zabby Exp $"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *	Copyright (c) 1982, 1986, 1988
 *	The Regents of the University of California
 *	All Rights Reserved.
 *	Portions of this document are derived from
 *	software developed by the University of
 *	California, Berkeley, and its contributors.
 */

/*
 * +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * 		PROPRIETARY NOTICE (Combined)
 * 
 * This source code is unpublished proprietary information
 * constituting, or derived under license from AT&T's UNIX(r) System V.
 * In addition, portions of such source code were derived from Berkeley
 * 4.3 BSD under license from the Regents of the University of
 * California.
 * 
 * 
 * 
 * 		Copyright Notice 
 * 
 * Notice of copyright on this source code product does not indicate 
 * publication.
 * 
 * 	(c) 1986,1987,1988,1989  Sun Microsystems, Inc
 * 	(c) 1983,1984,1985,1986,1987,1988,1989  AT&T.
 * 	          All rights reserved.
 *  
 */
/*
 * VM - paged vnode.
 *
 * The VM system manages memory as a cache of paged vnodes.
 * This file describes the interfaces to common subroutines
 * used to help implement the VM/file system routines.
 */

#ifdef _KERNEL_HEADERS

#include <mem/page.h> /* REQUIRED */
#include <mem/mem_hier.h> /* REQUIRED */
#include <mem/seg.h> /* REQUIRED */
#include <mem/memresv.h> /* REQUIRED */
#include <util/list.h> /* REQUIRED */
#include <util/types.h> /* REQUIRED */

#elif defined(_KERNEL) || defined(_KMEMUSER)

#include <vm/page.h> /* REQUIRED */
#include <vm/seg.h> /* REQUIRED */
#include <vm/mem_hier.h> /* REQUIRED */
#include <vm/memresv.h> /* REQUIRED */
#include <sys/list.h> /* REQUIRED */
#include <sys/types.h> /* REQUIRED */
#ifndef NOT_NSC
#include <sys/nsc_defines.h>
#endif /* !NOT_NSC */

#endif /* _KERNEL_HEADERS */


#if defined(_KERNEL) || defined(_KMEMUSER)

/*
 * Vnode/Page Mapping structure for segment chaining.
 *
 *	All fields are mutexed by the VN_PAGELOCK of the
 *	associated vnode.
 */
struct pvn_mapping {
	list_t			pvm_list;
	struct seg		*pvm_seg;
};

#define PVN_LIST_TO_MAPPING(listp)	((struct pvn_mapping *)(listp))

#endif /* _KERNEL || _KMEMUSER */

#if defined(_KERNEL)

struct buf;
struct cred;
struct vnode;
struct anon;

void	pvn_init(void);
void	pvn_fail(page_t *plist, int flags);
#ifdef NSC_CFS
void    pvn_done_pages(page_t *plist, int flags);
#endif
void	pvn_done(struct buf *bp);
void	pvn_unload(vnode_t *vp);
#ifdef NSC_CFS
void    pvn_unload_range(vnode_t *vp, off64_t off, uint_t len, int flag);
#endif
int	pvn_memresv(page_t *pp, mresvtyp_t mtype, uint_t flag);
void	pvn_memunresv(struct vnode *vp, off64_t offset, mresvtyp_t mtype);
void	pvn_memunresv_pp(page_t *, mresvtyp_t);
void	pvn_memunresv_pp_l(page_t *, mresvtyp_t mtype);
void	pvn_memunresv_anon(struct anon *, mresvtyp_t);
void	pvn_cache_memresv(page_t *pp, mresvtyp_t mtype);
boolean_t pvn_memresv_query(struct vnode *vp, off64_t offset);
boolean_t pvn_memresv_query_anon(struct anon *);
boolean_t pvn_memresv_swap(uint_t swresv, void (*scan_func)(), void *arg,
			   struct vnode *vp);
void	pvn_map_vnode(struct pvn_mapping *pvmp, vnode_t *vp, int type);
#ifdef NSC_CFS
int	pvn_walk_range(int (*func)(), vnode_t *vp, off64_t off, uint_t len);
#endif
void	pvn_unmap_vnode(struct pvn_mapping *pvmp, vnode_t *vp);
void	pvn_pagexlat(page_t *pp, void (*hat_func)());
void	pvn_migrate_range(vnode_t *, off64_t, size_t, place_t);

#ifdef _FSKI

void	pvn_abort_range(struct vnode *vp, off_t off, uint_t len);
boolean_t pvn_abort_range_nosleep(struct vnode *vp, off_t off, uint_t len);
int     pvn_getdirty_range(int (*func)(), vnode_t *vp, off_t roff,
	    uint_t rlen, off_t doff, uint_t dlen, off_t filesize, 
	    int flags, cred_t *cr);
int	pvn_getpages(int (*getapage)(), struct vnode *vp, off_t off,
	    uint_t len, uint_t *protp, page_t *pl[], uint_t plsz,
	    struct seg *seg, vaddr_t addr, enum seg_rw rw,
	    struct cred *cred);
page_t *pvn_kluster(struct vnode *vp, off_t off, struct seg *seg,
	    vaddr_t addr, off_t *offp, uint_t *lenp, off_t vp_off,
	    uint_t vp_len, page_t *pp);
int	pvn_trunczero(struct vnode *vp, off_t off, uint_t zbytes);

#else

void	pvn_abort_range(struct vnode *vp, off64_t off, uint_t len);
boolean_t pvn_abort_range_nosleep(struct vnode *vp, off64_t off, uint_t len);
int     pvn_getdirty_range(int (*func)(), vnode_t *vp, off64_t roff,
	    uint_t rlen, off64_t doff, uint_t dlen, off64_t filesize, 
	    int flags, cred_t *cr);
int	pvn_getpages(int (*getapage)(), struct vnode *vp, off64_t off,
	    uint_t len, uint_t *protp, page_t *pl[], uint_t plsz,
	    struct seg *seg, vaddr_t addr, enum seg_rw rw,
	    struct cred *cred);
page_t *pvn_kluster(struct vnode *vp, off64_t off, struct seg *seg,
	    vaddr_t addr, off64_t *offp, uint_t *lenp, off64_t vp_off,
	    uint_t vp_len, page_t *pp);
int	pvn_trunczero(struct vnode *vp, off64_t off, uint_t zbytes);

#endif	/* _FSKI */

#define PVN_NICE_VAL 35

/*
 * When requesting pages from the VOP_GETPAGE routines, the segment drivers
 * allocate space to return PVN_KLUSTER_NUM pages, which map PVN_KLUSTER_SZ
 * worth of bytes. These numbers are chosen to be the minimum of
 * the max's given in terms of bytes and pages. In addition, PVN_KLUSTER_SZ
 * cannot exceed MAXBIOSIZE.
 */
#define PVN_MAX_KLUSTER_SZ	0x10000		/* kluster size limit */
#define PVN_MAX_KLUSTER_NUM	0x10		/* kluster page limit */

#if PVN_MAX_KLUSTER_SZ > PVN_MAX_KLUSTER_NUM * PAGESIZE
#define PVN_KN	PVN_MAX_KLUSTER_NUM
#else
#define PVN_KN	((PVN_MAX_KLUSTER_SZ + PAGEOFFSET) >> PAGESHIFT)
#endif

#if PVN_KN < (MAXBIOSIZE >> PAGESHIFT)
#define PVN_KLUSTER_NUM		PVN_KN
#else
#define PVN_KLUSTER_NUM		btop(MAXBIOSIZE)
#endif

#define PVN_KLUSTER_SZ		ptob(PVN_KLUSTER_NUM)

/*
 * MAX_PUSHES is the maximum number of pages pushed in a single io by
 * pageout.  It must not exceed MAXBIOSIZE.
 */
#define MAX_PUSHES		btop(MAXBIOSIZE)

#if (!VM_INTR_IPL_IS_PLMIN)

#define PVN_MEMRESV_LOCK()	LOCK(&pvn_memresv_lock, VM_PVNMEMRESV_IPL)
#define PVN_MEMRESV_UNLOCK(ipl)	UNLOCK(&pvn_memresv_lock, (ipl))

#else  /* (VM_INTR_IPL_IS_PLMIN) */

#define PVN_MEMRESV_LOCK()      LOCK_PLMIN(&pvn_memresv_lock)
#define PVN_MEMRESV_UNLOCK(ipl) UNLOCK_PLMIN(&pvn_memresv_lock, (ipl))

#endif /* VM_INTR_IPL_IS_PLMIN */

/*
 * void
 * PAGE_UNLOAD_L(page_t *pp)
 *	Unload the translations for the specified page.
 *
 * Calling/Exit State:
 *	The caller holds the VN_PGLOCK for pp->p_vnode.
 */
#define PAGE_UNLOAD_L(pp) (                     \
	ASSERT(VN_PGLOCK_OWNED((pp)->p_vnode)), \
	(((pp)->p_retain) ? page_del_retain(pp) : (void)0), \
	pvn_pagexlat((pp), l.hatops->hat_pageunload)      \
)

/*
 * void
 * PAGE_GETMOD_L(page_t *pp)
 *	Forward the translation dirty bits of the specified page back into the
 *	pp->p_mod (and also back into the vnode). The translations are not
 *	affected.
 *
 * Calling/Exit State:
 *	The caller holds the VN_PGLOCK for pp->p_vnode.
 */
#define PAGE_GETMOD_L(pp) (                       \
	ASSERT(VN_PGLOCK_OWNED((pp)->p_vnode)),   \
	pvn_pagexlat((pp), l.hatops->hat_pagegetmod)        \
)

/*
 * void
 * PAGE_SYNCMOD_L(page_t *pp)
 *	Sync the translation dirty bits of the specified page back into the
 *	pp->p_mod (and also back into the vnode). The dirty bits in the
 *	translations are cleared.
 *
 * Calling/Exit State:
 *	The caller holds the VN_PGLOCK for pp->p_vnode.
 */
#define PAGE_SYNCMOD_L(pp) (                    \
	ASSERT(VN_PGLOCK_OWNED((pp)->p_vnode)), \
	pvn_pagexlat((pp), l.hatops->hat_pagesyncmod)     \
)

/*
 * boolean_t
 * PAGE_IS_UNLOADING(void (*hat_func)())
 *	Is an unload in progress.
 *
 * Calling/Exit State:
 *	None.
 */
#define PAGE_IS_UNLOADING(hat_func)	(hat_func == l.hatops->hat_pageunload)

#endif /* _KERNEL */

#if defined(__cplusplus)
	}
#endif

#endif /* _MEM_PVN_H */
