/* @(#)unixsrc:usr/src/common/lib/libvxvm/common/libvxvm/voldisk.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/lib/libvxvm/common/libvxvm/voldisk.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VOLDISK_H
#define _VOLDISK_H

/*
 * general structure defining disk attributes
 */

struct disk_attrs {
	char		 dka_da_name[NAME_SZ];	/* disk device name */
	char		 dka_dg_name[NAME_SZ];	/* disk group for disk */
	char		 dka_diskid[VOL_UUID_SZ]; /* unique ID for disk */
	char		 dka_hostid[VOL_UUID_SZ]; /* controlling host */
	char		 dka_dgid[VOL_UUID_SZ];	/* disk group for disk */
	ushort_t	 dka_version_major;	/* major format version */
	ushort_t	 dka_version_minor;	/* minor format version */
	ulong_t		 dka_flags;		/* flags affecting device */
	ushort_t	 dka_pub_id;		/* public partition id */
	ushort_t	 dka_priv_id;		/* private partition id */
	voff_t		 dka_pub_offset;	/* offset to public region */
	voff_t		 dka_pub_len;		/* length of public partition */
	voff_t		 dka_priv_offset;	/* offset to private region */
	voff_t		 dka_priv_len;		/* len of private partition */
	time_t		 dka_update_time; 	/* time of last update */
	volseqno_t	 dka_update_seqno;	/* seqno of last update */
	size_t		 dka_atomic_iosize;	/* setor size of disk */
	voff_t		 dka_primary_header;	/* offset to primary header */
	voff_t		 dka_alternate_header;	/* offset to alternate header */
	size_t		 dka_nconfig;		/* number of config copies */
	size_t		 dka_nlogs;		/* number of log copies */
	size_t		 dka_config_len;	/* size of config copies */
	size_t		 dka_log_len;		/* size of log copies */
};

/* bits defined in dka_flags */
#define DKA_FLAG_PRIVATE	0x0001	/* disk has a private region */
#define DKA_FLAG_VOLATILE	0x0002	/* disk contents are volatile */
#define DKA_FLAG_INVALID	0x0004	/* disk header is invalid */
#define DKA_FLAG_ALIAS		0x0008	/* header implies device aliasing */
#define DKA_FLAG_INVALTOC	0x0010	/* disk TOC blocks are invalid */
#define DKA_FLAG_AUTOIMPORT	0x0800	/* disk should be auto-imported */
#define DKA_FLAG_SHARED		0x1000	/* disk is shared */
#define DKA_FLAG_ALTUSED	0x2000	/* primary header is bad, alt is ok */

/*
 * disk_contents - complete information about a disk
 */

struct disk_contents {
	struct disk_attrs dkc_attrs;		/* general attributes */
	size_t		 dkc_ntoc;		/* number of toc sectors */
	voff_t		*dkc_primary_toc_list;	/* primary toc locations */
	voff_t		*dkc_alternate_toc_list; /* alternate toc locations */
	size_t		 dkc_nregion;		/* number of defined regions */
	struct disk_region **dkc_regions;	/* region definitions */
	size_t		 dkc_spec_size;		/* size of spec_attrs buffer */
	char		*dkc_spec_attrs;	/* buffer of extended attrs */
};

#define DKR_TYPE_LEN	8
#define DKR_TYPE_SZ	(DKR_TYPE_LEN + 1)

struct disk_region {
	char		 dkr_type[DKR_TYPE_SZ];	/* type of region */
	ushort_t	 dkr_flags;		/* flags for disk region */
	ushort_t	 dkr_info_size;		/* size of region info */
	voff_t		 dkr_offset;		/* location of region */
	voff_t		 dkr_len;		/* length of region */
	char		 dkr_info[1/*info_size*/]; /* region info */
};
/* values for dkr_flags */
#define DKR_FLAG_PUBLIC		0x0001		/* part of public partition */
#define DKR_FLAG_NEW		0x0002		/* flagged for creation */
#define DKR_FLAG_DELETE		0x0004		/* flagged for deletion */
#define DKR_FLAG_DISABLED	0x0008		/* disabled config/log copy */

/*
 * common disk operations supported by disk types defined in vold.
 * These operations are used in conjunction with the disk_op() and
 * disk_query() library functions.
 *
 * Operation numbers of 1000 or greater are reserved for use by
 * OEM-defined disk operations.
 */

#define DISK_OP_NOOP		0		/* do nothing */
#define DISK_OP_ONLINE		1		/* enable use of the disk */
#define DISK_OP_OFFLINE		2		/* disable use of the disk */
#define DISK_OP_SET		3		/* set disk attributes */
#define DISK_OP_ADDREGION	4		/* add to private region */
#define DISK_OP_RMREGION	5		/* remove from private region */
#define DISK_OP_CLEARIMPORT	6		/* remove any import state */
#define DISK_OP_FLUSH		7		/* flush all disk information */
#define DISK_OP_GETCONTENTS	8		/* get all disk information */
#define DISK_OP_CHECK		9		/* validate disk usability */
#define DISK_OP_MAX		9		/* last standard op */

#if defined(__STDC__)
extern int   disk_init(const char *, const char *, const char *, size_t);
extern int   disk_define(const char *, const char *, const char *, size_t);
extern int   disk_get_attrs(const char *, struct disk_attrs *);
extern int   disk_op(const char *, int, const void *, size_t);
extern VOID *disk_query(const char *, int, const void *, size_t);
extern int   disk_online(const char *);
extern int   disk_offline(const char *);
extern int   disk_reonline_all(void);
extern int   disk_set(const char *, const char *, size_t);
extern int   disk_addregion(const char *, const struct disk_region *);
extern int   disk_rmregion(const char *, const struct disk_region *);
extern int   disk_clearimport(const char *);
extern int   disk_check(const char *);
extern struct disk_contents *disk_get_contents(const char *);
#else	/* not __STDC__ */
extern int   disk_init();
extern int   disk_define();
extern int   disk_get_attrs();
extern int   disk_op();
extern VOID *disk_query();
extern int   disk_online();
extern int   disk_offline();
extern int   disk_reonline_all();
extern int   disk_set();
extern int   disk_addregion();
extern int   disk_rmregion();
extern int   disk_clearimport();
extern int   disk_check();
extern struct disk_contents *disk_get_contents();
#endif	/* __STDC__ */

#endif	/* _VOLDISK_H */
