/* @(#)unixsrc:usr/src/common/uts/io/vxvm/unixware/voldefs.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/unixware/voldefs.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLDEFS_H
#define _VXVM_VOLDEFS_H

/*
 * voldefs.h - base definitions for VxVM
 *
 * This header file contains fundamental types and defines used
 * throughout the volume manager.
 */

/*
  * First define some lint needed things.
  */

#if defined(lint)

extern int    vol_global_zero;
#define ZERO  vol_global_zero
#if defined(STATIC)
#undef STATIC
#endif /* STATIC */
#define STATIC        static
#else

#define ZERO  0

#endif /* lint */


typedef long voff_t;			/* device offsets and lengths */
typedef unsigned long uvoff_t;		/* unsigned version */
typedef unsigned volflag_t;		/* standard-length flag field */

/*
 * VOFF_T_MAX	- maximum value for a voff_t (which is signed)
 * VOFF_T_MIN	- minimum value for a voff_t
 * UVOFF_T_MAX	- maximum value for a uvoff_t
 *
 * These definitions rely on twos-complement arithmetic and presume
 * that chars are eight bits.
 */

#define VOFF_T_MAX	\
	((voff_t)(((voff_t)1 << (sizeof(voff_t) * 8 - 2)) + \
		  (((voff_t)1 << (sizeof(voff_t) * 8 - 2)) - (voff_t)1)))
#define VOFF_T_MIN	((voff_t)(-VOFF_T_MAX - (voff_t)1))
#define UVOFF_T_MAX	\
	((uvoff_t)((uvoff_t)1 << (sizeof(uvoff_t) * 8 - 1)))

/*
 * VOFF_T_QUAD - is the voff_t type more than 32 bits
 *
 * Set VOFF_T_QUAD to 1 if voff_t is a 64 bit type
 */

#if !defined(VOFF_T_QUAD)
#define VOFF_T_QUAD	0
#endif

/*
 * VOL_BIG_ENDIAN - true if this machine is bigendian
 *
 * Set this to 1 for a machine with a big-endian byte order.
 * Since this is for UnixWare set it to 0.
 */
#define VOL_BIG_ENDIAN	0

/*
 * volseqno_t - 64-bit sequence number type
 * volrid_t   - 64-bit record/object identifier type
 *
 * This type is composed of two 32-bit unsigned integers which are
 * combined to form a 64-bit number.  A limited set of operations are
 * defined on this 64-bit number.
 */

struct volseqno {
#if VOL_BIG_ENDIAN
	unsigned long seqno_hi;
	unsigned long seqno_lo;
#else
	unsigned long seqno_lo;
	unsigned long seqno_hi;
#endif
};
typedef struct volseqno volseqno_t;	/* configuration sequence numbers */
typedef struct volseqno volrid_t;	/* record/object identifiers */

/*
 * VOLSEQNO_EQ	    - test for equality of two volseqno_t values
 * VOLSEQNO_LT, VOLSEQNO_GT, VOLSEQNO_GE, VOLSEQNO_LE - magnitute comparisons
 * VOLSEQNO_EQ_ZERO - test for a zero volseqno_t value
 * VOLSEQNO_RESET   - clear a volseqno_t value to zero
 * VOLSEQNO_SET	    - set a volseqno_t value from two 32-bit values
 * VOLSEQNO_HI	    - return the high 32-bits of a volseqno_t value
 * VOLSEQNO_LO	    - return the low 32-bits of a volseqno_t value
 * VOLSEQNO_ZERO    - a zero suitable for a initializing a volseqno_t value
 */

#define VOLSEQNO_EQ(seq1, seq2)	\
	((seq1).seqno_lo == (seq2).seqno_lo &&		\
	 (seq1).seqno_hi == (seq2).seqno_hi)
#define VOLSEQNO_LT(seq1, seq2)	\
	((seq1).seqno_hi < (seq2).seqno_hi ||		\
	 ((seq1).seqno_hi == (seq2).seqno_hi && 	\
	  (seq1).seqno_lo < (seq2).seqno_lo))
#define VOLSEQNO_GT(seq1, seq2)	\
	((seq1).seqno_hi > (seq2).seqno_hi ||		\
	 ((seq1).seqno_hi == (seq2).seqno_hi && 	\
	  (seq1).seqno_lo > (seq2).seqno_lo))
#define VOLSEQNO_GE(seq1, seq2)	(! VOLSEQNO_LT((seq1), (seq2)))
#define VOLSEQNO_LE(seq1, seq2)	(! VOLSEQNO_GT((seq1), (seq2)))
#define VOLSEQNO_INC(seq)	\
	((seq).seqno_lo == 0xffffffffu?			\
	 ((seq).seqno_lo = 0, (seq).seqno_hi++):	\
	 (seq).seqno_lo++)
#define VOLSEQNO_DEC(seq)	\
	((seq).seqno_lo == 0x0 ?				\
	 (seq).seqno_hi--, (seq).seqno_lo = 0xffffffffu :	\
	 (seq).seqno_lo--)
#define VOLSEQNO_EQ_ZERO(seq)	((seq).seqno_hi == 0 && (seq).seqno_lo == 0)
#define VOLSEQNO_RESET(seq)	((seq).seqno_hi = 0, (seq).seqno_lo = 0)
#define VOLSEQNO_SET(seq, hi, lo)	\
	((seq).seqno_hi = (hi),		\
	 (seq).seqno_lo = (lo))
#define VOLSEQNO_HI(seq)	((seq).seqno_hi)
#define VOLSEQNO_LO(seq)	((seq).seqno_lo)
#define VOLSEQNO_CONST_ZERO	{ 0, 0 }

/*
 * VOLRID_EQ, VOLRID_EQ_ZERO, VOLRID_RESET, VOLRID_SET, VOLRID_HI, VOLRID_LO
 *
 * Equivalent macros for handling a volrid_t value.
 */

#define VOLRID_EQ(rid1, rid2)	(VOLSEQNO_EQ((rid1), (rid2)))
#define VOLRID_EQ_ZERO(rid)	(VOLSEQNO_EQ_ZERO((rid)))
#define VOLRID_RESET(rid)	(VOLSEQNO_RESET((rid)))
#define VOLRID_SET(rid, hi, lo)	(VOLSEQNO_SET((rid), (hi), (lo)))
#define VOLRID_HI(rid)		(VOLSEQNO_HI(rid))
#define VOLRID_LO(rid)		(VOLSEQNO_LO(rid))

/*
 * VOL_BLOCKSIZE  - size of volume manager blocks (must match system
 *		    sector size
 * VOL_BLOCKSHIFT - shift coount matching VOL_BLOCKSIZE
 * VOL_BTOD(b)	  - convert bytes to volume manager disk blocks
 * VOL_BTODR(b)	  - convert bytes to volume manager disk blocks (round up)
 * VOL_DTOB(d)	  - convert volume manager disk blocks to bytes
 */
#if !defined(VOL_BLOCKSIZE)
#define VOL_BLOCKSIZE	NBPSCTR
#endif	/* !VOL_BLOCKSIZE */
#if !defined(VOL_BLOCKSHIFT)
#define VOL_BLOCKSHIFT	SCTRSHFT
#endif	/* !VOL_BLOCKSHIFT */
#define VOL_BTOD(b)	((b) >> VOL_BLOCKSHIFT)
#define VOL_BTODR(b)	(((b) + (VOL_BLOCKSIZE - 1)) >> VOL_BLOCKSHIFT)
#define VOL_DTOB(d)	((d) << VOL_BLOCKSHIFT)

/*
 * voleventno_t - integer suitable for event numbers used for identifying
 * trace events.
 */
typedef unsigned long	voleventno_t;

/*
 * VOLRID_HASH - return a suitable modulo for hashing on a record ID
 *
 * This doesn't need to be a perfect modulo, using the modulo of the
 * low 32 bits is sufficient.
 */

#define VOLRID_HASH(rid, mod)	(VOLRID_LO(rid) % (unsigned long)(mod))

/*
 * maximum alignment requirements for this machine
 *
 * VOL_ALIGN_SIZE	- size of largest C data type used in volume manager
 * VOL_ALIGN(p)		- return the pointer rounded up to an alignment boundary
 */

#define VOL_ALIGN_SIZE	(sizeof(vol_align_t))
#define VOL_ALIGN_MASK	(~(VOL_ALIGN_SIZE - 1))
typedef union {
	voff_t		_vol_align_v;
	long		_vol_align_l;
	unsigned long	_vol_align_ul;
	int		_vol_align_i;
	unsigned	_vol_align_u;
} vol_align_t;
#define VOL_ALIGN(p)	\
	((VOID *)(((ulong_t)(p) + VOL_ALIGN_SIZE - 1) & VOL_ALIGN_MASK))
#define VOL_ALIGN_NUM(i)	\
	((i + VOL_ALIGN_SIZE - 1) & VOL_ALIGN_MASK)

/*
 * qsort comparator interface - many are non-posix (i.e., the
 * comparator function doesn't take CONST VOID * arguments). Just
 * typedef one here for use in utilities.
 */
typedef const void *  qsortarg_t;

/*
 * twalk/tsearch/etc. interface - similar to qsort.
 */
typedef const void *  twalkarg_t;

/*
 * global constants
 */

#define NAME_LEN	31		/* bytes in a name */
#define NAME_SZ		(NAME_LEN + 1)	/* bytes in a name field */
#define UTIL_LEN	14		/* chars in a utility defined field */
#define UTIL_SZ		(UTIL_LEN + 1)	/* bytes per utility defined field */
#define UTIL_NUM	3		/* number of utility defined fields */
#define PNAME_SEPARATOR	'/'		/* path name separator */
#define COMMENT_LEN	40		/* chars in a comment field */
#define COMMENT_SZ	(COMMENT_LEN+1)	/* bytes in a comment field */
#define STATE_LEN	14		/* chars per utility state field */
#define STATE_SZ	(STATE_LEN + 1)	/* bytes per utility state field */
#define FSTYPE_LEN	14		/* max length of fstype string */
#define	FSTYPE_SZ	(FSTYPE_LEN+1)	/* size of fstype field */
#define DK_PATH_LEN	79		/* chars in a disk /dev pathname */
#define DK_PATH_SZ	(DK_PATH_LEN+1)	/* bytes per disk /dev pathname */
#define SD_PATH_LEN	DK_PATH_LEN	/* chars in a subdisk /dev pathname */
#define SD_PATH_SZ	(SD_PATH_LEN+1)	/* bytes per subdisk /dev pathname */
#define V_STOPTS_LEN	32		/* chars of volume start options */
#define V_STOPTS_SZ	(V_STOPTS_LEN+1) /* bytes per volume start args */
#define VOL_UUID_LEN	64		/* chars in a unique identifier */
#define VOL_UUID_SZ	(VOL_UUID_LEN+1) /* bytes in unique identifier */
#define DA_TYPE_LEN	14		/* disk device type len */
#define DA_TYPE_SZ	(DA_TYPE_LEN+1)	/* bytes in disk device type */
#define DA_INFO_LEN	128		/* disk device type-specific info len */
#define DA_INFO_SZ	(DA_INFO_LEN+1)	/* bytes in disk device info */
#define VOL_PURGE	0		/* Purge the records for vol_unload */
#define VOL_DONT_PURGE	1		/* Keep the record after KERNEL_RESET */

/*
 * VOLIOC - ioctl number space for volume manager ioctls
 *
 * All volume manager ioctl numbers should be or'd with this value.
 */

#define	VOLIOC	(('V' << 24) | ('O' << 16) | ('L' << 8))
#define VOL_IOCTL_CODE(code)	(VOLIOC | code)

/*
 * Possible valid range of VxVM ioctls:
 */

#define VOL_MIN_IOCTL		(VOLIOC | 0)
#define VOL_MAX_IOCTL		(VOLIOC | 255)

#define VOL_VALID_IOCTL(cmd)	\
	(VOL_MIN_IOCTL <= (cmd) && (cmd) <= VOL_MAX_IOCTL)

/*
 * Volspec minor device numbers
 */
#define	MAXVSPEC	4			/* four special devices */
#define VOLCONFIG	0			/* vold interface */
#define VOLIOTRACE	1			/* volevent interface */
#define VOLIODAEMON	2			/* voliod interface */
#define VOLINFO		3			/* volinfo interface */

/*
 * Defines for identifying the "complete" slice for disks.
 */
#define VOL_FULL_SLICE		'0'
#define VOL_FULL_SLICE_NO	0 
#define VOL_FULL_SLICE_STR	"s0"
#define VOL_PRIV_OFFSET		0
#define VOL_PUB_OFFSET		0
#define VOL_GHOST_LEN		0

/*
 * Definitions for dev_t abstraction
 */
#define VOLNODEV	NODEV
typedef dev_t		voldevno_t;

/*
 * interlocks - define max and number of skiplevels.
 */
#define	VOL_ILOCK_NUM_SKIPLEVEL	8
#define	VOL_ILOCK_MAX_SKIPLEVEL 7

/*
 * Hotsparing definitions
 *
 * Hotsparing attempts to find a "closest" disk as a hotspare. To do
 * this, it assigns a numerical value to each disk based on it's
 * controller, target and disk, with controller being most significant
 * and disk being least significant. The number is therefore determined by:
 *
 * (((((controller * VOL_HS_CNTRL_MULT) + busid) * VOL_HS_BUS_MULT) +
 *					target) * VOL_HS_TARG_MULT) + disk
 *
 * In order for this to work, each of the _MULT defines should be larger
 * than the maximum number of the next less significant objects, i.e.
 * VOL_CNTRL_MULT should be larger than the maximum number of targets
 * per controller.
 */

#define	VOL_HS_TARG_MULT	100
#define	VOL_HS_BUS_MULT		100
#define	VOL_HS_CNTRL_MULT	100

/*
 * CONST, VOID, VOLATILE - const, void (for void *), and volatile in __STDC__
 *
 * When using an ANSI C compiler, these will be set appropriately,
 * otherwise, CONST and VOLATILE will be nul macros, and VOID will be
 * a char (suitable for use as a pointer type).
 *
 * VOID is used only in the context of pointers where ANSI C defines
 * the generic (void *) pointer type.
 */

#if defined(__STDC__)

#if !defined(CONST)
#define CONST const
#endif	/* CONST */
#if !defined(VOID)
#define VOID void
#endif	/* VOID */
#if !defined(VOLATILE)
#define VOLATILE volatile
#endif	/* VOLATILE */

#else	/* not __STDC__ */

#if !defined(CONST)
#define CONST
#endif	/* CONST */
#if !defined(VOID)
#define VOID char
#endif	/* VOID */
#if !defined(VOLATILE)
#define VOLATILE
#endif	/* VOLATILE */

#endif	/* __STDC__ */

/*
 * define the ANSI C offsetof if it is not defined already
 */

#ifndef offsetof
#define offsetof(t, e)		((char *)(&((t *)0)->e) - (char *)(t *)0)
#endif	/* offsetof */

#define VOL_MAX(a,b)    ((a) > (b)? (a): (b))
#define VOL_MIN(a,b)    ((a) < (b)? (a): (b))

/*
 * UnixWare specific data structures, macros, and ioctls for info.
 */
typedef struct  {
        void    *obj;                   /* Save the obj pointer here */
        daddr_t v_blkno;                /* Save the block offset here */
} vol_priv_t;

#define SET_KTHREAD_NAME(string)	(u.u_lwpp->l_name = string)
#define VOL_SYS_INFO_IOCTLS

/*
 * Some user only macros. 
 */
#ifndef	_KERNEL

#define VOL_DELAY(_period) (poll((struct pollfd *)NULL, 0, (_period)))

#endif	/* _KERNEL */

#endif	/* _VXVM_VOLDEFS_H */
