/* @(#)unixsrc:usr/src/common/uts/io/vxvm/common/voldobject.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/common/voldobject.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLDOBJECT_H
#define _VXVM_VOLDOBJECT_H

/*
 * voldobject.h - VxVM 1.2 vold/kernel configuration object interface structures
 */

/*
 * object type definitions:
 *
 * Each object type is defined by an interface structure, optional
 * association information, and an assert structure.
 *
 * The interface structure defines the object itself.  The base
 * interface structure is provided by vold when an object is created
 * or changed.  The base interface structure is returned when the
 * object is fetched from the kernel.
 *
 * The association information structure, if defined for an object
 * type, defines the information that is stored with a child object
 * that is associated with a parent object of the indicated type.  For
 * example, the plex type defines an association information structure
 * that is stored with with all objects (e.g., subdisks and
 * subvolumes) that are associated with the plex.  The association
 * information is stored in the kernel volobject.obj_assoc_info
 * pointer of the child object.  It is uninterpreted by the child
 * object.
 *
 * NOTE:  All association information structures must begin with a
 *	  size_t value that defines the size of the information
 *	  structure.
 *
 * Each object type defines an assert structure that is loaded when an
 * object is locked within a transaction.  This is used to ensure that
 * information obtained for a utility by vold (when vold first started
 * processing a transaction) remains unchanged until the transaction
 * commits.  Only interesting kernel-modifiable values are asserted.
 * If one of the asserted values changes, the kernel should abort the
 * transaction.
 */

/*
 * all objects have status and control flag fields that contain a set
 * common flags and a set of type-specific flags.  The lower 16 bits
 * are reserved for common flags.
 *
 * The common flags are:
 */

/* flag values for object control flags */
#define VOLK_KFLAG_ENABLED	0x00000001	/* object is enabled */
#define VOLK_KFLAG_READ		0x00000002	/* object is readable */
#define VOLK_KFLAG_WRITE	0x00000004	/* object is writable */
#define VOLK_KFLAG_NO_RESOURCES	0x00000008	/* object has no resources */
#define VOLK_KFLAG_DETACH	0x00000010	/* object itself is detached */
#define VOLK_KFLAG_PRESERVE	0x00000020	/* keep after RESET_KERNEL */

/* flag values for object status flags */
#define VOLK_SFLAG_BUSY		0x00000001	/* object is in active use */
#define VOLK_SFLAG_DIRTY	0x00000002	/* object has been written */
#define VOLK_SFLAG_DETACH	0x00000004	/* object detached itself */

/*
 * Parent objects can store assertable flags in a child object's
 * obj_parent_sflag field.  The lower 4 bits of this field are reserved
 * for generic kernel-defined flags, the next 4 bits are reserved for common
 * flags (by convention).  All other bits are object type specific.
 *
 * The common parent status flags are:
 */

#define VOLK_PSFLAG_DETACH	0x00000010	/* child detached from parent */

/*
 * volkbase - base structure used for standard objects
 *
 * Standard objects begin with a standard base structure.  This
 * structure contains:
 *
 *  kbase_lasterr_tid	- config tid in effect for the last
 *			  non-corrected error on the object.
 *  kbase_kflag		- control flags set by vold to control the kernel.
 *  kbase_sflag		- status flags set by the kernel to inform vold.
 */

struct volkbase {
	volseqno_t	 kbase_lasterr_tid;	/* 0x00 last error config tid */
	volflag_t	 kbase_kflag;		/* 0x08 control flags */
	volflag_t	 kbase_sflag;		/* 0x0c status flags */
};						/* 0x10 */

/*
 * volkbase_assert - base assert for standard objects
 *
 * Standard objects have assert information that begins with this
 * structure.
 */

struct volkbase_assert {
	volseqno_t	 ka_lasterr_tid;	/* 0x00 last error config tid */
	volflag_t	 ka_sflag;		/* 0x08 status flags */
};						/* 0x0c */

/*
 * SUBDISK OBJECTS
 *
 * A subdisk object defines a subrange of a disk.  A subdisk is
 * primarily defined by the disk, an offset with the disk's public
 * region, and a length.
 */

struct volksd {
	struct volkbase	 ksd_base;		/* 0x00 base information */
	volrid_t	 ksd_da_rid;		/* 0x10 disk */
	voff_t		 ksd_offset;		/* 0x18 disk offset */
	voff_t		 ksd_len;		/* 0x1c subdisk length */
};						/* 0x20 */
#define ksd_lasterr_tid	 ksd_base.kbase_lasterr_tid
#define ksd_kflag	 ksd_base.kbase_kflag
#define ksd_sflag	 ksd_base.kbase_sflag


/*
 * SUBVOLUME OBJECTS
 *
 * A subvolume object defines a subrange of another object.  A
 * simple volume is primary defined by the child object, the offset
 * within the child object's address space, and a length.
 */

struct volksv {
	struct volkbase	 ksv_base;		/* 0x00 base information */
	volrid_t	 ksv_child;		/* 0x18 underlying object */
	voff_t		 ksv_offset;		/* 0x20 object offset */
	voff_t		 ksv_len;		/* 0x24 subdisk length */
};						/* 0x28 */
#define ksv_lasterr_tid	 ksv_base.kbase_lasterr_tid
#define ksv_kflag	 ksv_base.kbase_kflag
#define ksv_sflag	 ksv_base.kbase_sflag


/*
 * PLEX OBJECTS
 *
 * Plex objects define only striped objects.  Concatenated objects are
 * a special case of striped objects, with a stripe width that matches
 * the object length, and with one stripe column that contains all
 * associated objects.
 *
 * Each child object associated with a plex defines the stripe column
 * and stripe column offset of the child object within the plex.
 */

struct volkplex {
	struct volkbase	 kpl_base;		/* 0x00 base information */
	size_t		 kpl_columns;		/* 0x18 stripe columns */
	voff_t		 kpl_width;		/* 0x1c stripe width */
	voff_t		 kpl_len;		/* 0x20 plex length to report */
};						/* 0x24 */
#define kpl_lasterr_tid	 kpl_base.kbase_lasterr_tid
#define kpl_kflag	 kpl_base.kbase_kflag
#define kpl_sflag	 kpl_base.kbase_sflag

struct volkplex_asinfo {
	size_t		kplas_size;		/* 0x00 asinfo size */
	size_t		kplas_column;		/* 0x04 stripe column */
	voff_t		kplas_offset;		/* 0x08 stripe column offset */
};						/* 0x0c */


/*
 * VOLUME OBJECTS
 *
 * A volume object contains some number of associated child objects
 * that are maintained with the same contents, providing redundancy.
 * The association information for a child of a volume object defines
 * flags that affect the volume's treatment of that child.
 */

struct volkvol {
	struct volkbase	 kv_base;		/* 0x00 base information */
	volrid_t	 kv_pref_mirror_rid;	/* 0x18 preferred mirror */
	voff_t		 kv_len;		/* 0x20 volume length */
	voff_t		 kv_rwback_offset;	/* 0x24 read recovery offset */
	u_long		 kv_recover_seqno;	/* 0x28 appl recovery seqno */
};						/* 0x34 */
#define kv_lasterr_tid	 kv_base.kbase_lasterr_tid
#define kv_kflag	 kv_base.kbase_kflag
#define kv_sflag	 kv_base.kbase_sflag

struct volkvol_asinfo {
	size_t		 kvas_size;		/* 0x00 asinfo size */
	volflag_t	 kvas_flags;		/* 0x04 association flags */
	volrid_t	 kvas_detsd_rid;	/* 0x08 detach sd rid */
};						/* 0x10 */

struct volkvol_assert {
	struct volkbase_assert	kvasrt_base;	/* 0x00 base assert info */
	ulong_t			kvasrt_seqno;	/* 0x0c recovery seqno */
};						/* 0x14 */
#define kvasrt_lasterr_tid	kvasrt_base.ka_lasterr_tid
#define kvasrt_sflag		kvasrt_base.ka_sflag

/* volume-specific flag values for vokvol.kv_kflag */
#define VOLKV_KFLAG_RDPOLMASK	0x000f0000	/* read policy mask */
#define VOLKV_KFLAG_EXCEPTMASK	0x00f00000	/* exception policies */
#define VOLKV_KFLAG_WRITEBACK	0x01000000	/* writeback on read error */
#define VOLKV_KFLAG_RWBACK	0x02000000	/* recover mirrors on read */
#define VOLKV_KFLAG_WRITECOPY	0x04000000	/* stabilize write buffers */
#define VOLKV_KFLAG_NO_LOGDIRTY	0x08000000	/* Don't do dirty logging */
#define	VOLKV_KFLAG_APP_RECOVER 0x10000000	/* needs app recovery */

/* volume-specific flag values for volkvol.kv_sflag */
#define VOLKV_SFLAG_RWBACK	0x00010000	/* still do recover on read */
#define VOLKV_SFLAG_APP_DIRTY	0x00020000	/* needs app recovery */
#define VOLKV_SFLAG_APP_RECOVER	0x00040000	/* dirtied in APP_RECOV mode */

/* convenience macros */
#define VOLKV_KFLAG_RDPOL(kflag)	((kflag) & VOLKV_KFLAG_RDPOLMASK)
#define VOLKV_KFLAG_SETRDPOL(kflag, rdpol)	\
	((kflag) &= ~VOLKV_KFLAG_RDPOLMASK,		\
	 (kflag) |= (rdpol))
#define VOLKV_KFLAG_EXCEPT(kflag)	((kflag) & VOLKV_KFLAG_EXCEPTMASK)
#define VOLKV_KFLAG_SETEXCEPT(kflag, except)	\
	((kflag) &= ~VOLKV_KFLAG_EXCEPTMASK,		\
	 (kflag) |= (except))

/* volume read policies */
#define VOLKV_RD_ROUND		0x00010000	/*   round-robin policy */
#define VOLKV_RD_PREFER		0x00020000	/*   preferred-plex policy */

/* exception policies for volume */
#define VOLKV_EXCEPT_DET_SPARSE	0x00100000	/* detach mirror, or all
						 * matching incomplete plexes
						 * if no complete plexes would
						 * remain */

/* flags values for volkvol_asinfo.kvas_flags */
#define VOLKV_ASFLAG_COMPLETE	0x00000001	/* consider mirror complete */
#define VOLKV_ASFLAG_NOERROR	0x00000002	/* ignore in error handling */
#define	VOLKV_ASFLAG_DETACH	0x00000004	/* child detached by parent */
#define VOLKV_ASFLAG_REGIONLOG	0x00000008	/* child as a DRL volume */

/*
 * RAID OBJECTS
 *
 * Raid objects define a RAID5-layout stripe object defined with a
 * specified number of columns, and a specified stripe element width.
 * A log object can be associated which will be used as a logging area
 * for data and parity.
 *
 * Each child object associated with a raid object defines the stripe
 * column and stripe column offset of the child object within the raid.
 * Currently, RAID objects just use the volkplex_asinfo structure.
 */

struct volkraid {
	struct volkbase	 kra_base;		/* 0x00 base information */
	volrid_t	 kra_log;		/* 0x10 log object */
	size_t		 kra_columns;		/* 0x18 raid columns */
	voff_t		 kra_width;		/* 0x1c element width */
	voff_t		 kra_len;		/* 0x20 raid object length */
	voff_t		 kra_recover_offset;	/* 0x24 offset of stale prty */
};						/* 0x28 */
#define kra_lasterr_tid	 kra_base.kbase_lasterr_tid
#define kra_kflag	 kra_base.kbase_kflag
#define kra_sflag	 kra_base.kbase_sflag

/* raid-specific flag values for volkraid.kra_kflag */
#define	VOLKRA_KFLAG_STALE	0x00010000	/* parity in RAID is stale */

/* raid-specific flag values for volkraid.kra_sflag */
#define VOLKRA_SFLAG_LOG_DETACH	0x00010000	/* log object detached */

struct volkraid_asinfo {
	size_t		kraas_size;		/* 0x00 asinfo size */
	int		kraas_flags;		/* 0x04 flags */
	size_t		kraas_column;		/* 0x08 stripe column */
	voff_t		kraas_offset;		/* 0x0c stripe column offset */
};						/* 0x10 */

/*
 * raid-specific flag values for volkraid_asinfo.kraas_flags.
 */
#define	VOLKRAAS_LOG	0x1	/* association is for a log object */
#define VOLKRAAS_DETACH	0x2	/* associated object is detached */
#define VOLKRAAS_WRONLY	0x4	/* associated object is a write-only object */

/* VOLKRA_LOG_MIN must match V_RAID_LOG_MIN in volintf.h */
#define VOLKRA_LOG_MIN	3	/* minimum log length, measured in stripes */

#endif	/* _VXVM_VOLDOBJECT_H */
