/* @(#)unixsrc:usr/src/common/uts/io/vxvm/common/voldrl.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/common/voldrl.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLDRL_H
#define _VXVM_VOLDRL_H

/*
 * voldrl.h - volume manager header for dirty region logging (DRL)
 */

/*
 * DRL magic number
 */
#define DRL_MAGIC 		('D' << 24 | 'R' << 16 | 'L' << 8 | '2')

/*
 * DRL header size in number of bytes
 */
#define VOLDRL_HEADER_SIZE	24

/*
 * Maximum DRL log subdisk size in number of blocks
 */
#define VOLDRL_LOGSD_MAX	64

/*
 * Macros for loading and storing DRL header fields
 *
 * The DRL header contains the following information:
 *	ulong_t		magic		"DRL magic number"
 *	ulong_t		clear		"clear flag"
 *	ulong_t		serial_lo	"low part of serial number"
 *	ulong_t		serial_hi	"high part of serial number"
 *	ulong_t		regionsz	"region size in blocks"
 *	ulong_t		volsz		"volume size"
 */

#define VOLDRL_LONG_STORE(logp, offset, l) \
	(((unsigned char *)(logp))[offset] = ((l) >> 24)&0xff,		\
	 ((unsigned char *)(logp))[offset+1] = ((l) >> 16)&0xff,	\
	 ((unsigned char *)(logp))[offset+2] = ((l) >> 8)&0xff,		\
	 ((unsigned char *)(logp))[offset+3] = (l)&0xff)

#define VOLDRL_LONG_LOAD(logp, offset, l) \
	((l) = (((ulong_t)((unsigned char *)(logp))[offset] << 24) |	\
		((ulong_t)((unsigned char *)(logp))[offset+1] << 16) |	\
		((ulong_t)((unsigned char *)(logp))[offset+2] << 8) |	\
		((ulong_t)((unsigned char *)(logp))[offset+3])))

#define VOLDRL_STORE_MAGIC(logp) \
		VOLDRL_LONG_STORE(logp, 0, DRL_MAGIC)
#define VOLDRL_STORE_CLEARFLAG(logp, flag) \
		VOLDRL_LONG_STORE(logp, VOL_ULONG_BYTES, flag)
#define VOLDRL_STORE_SERIAL_LO(logp, lo) \
		VOLDRL_LONG_STORE(logp, 2*VOL_ULONG_BYTES, lo)
#define VOLDRL_STORE_SERIAL_HI(logp, hi) \
		VOLDRL_LONG_STORE(logp, 3*VOL_ULONG_BYTES, hi)
#define VOLDRL_STORE_REGIONSZ(logp, sz) \
		VOLDRL_LONG_STORE(logp, 4*VOL_ULONG_BYTES, sz)
#define VOLDRL_STORE_VOLSZ(logp, sz) \
		VOLDRL_LONG_STORE(logp, 5*VOL_ULONG_BYTES, sz)

#define VOLDRL_LOAD_MAGIC(logp, magic) \
		VOLDRL_LONG_LOAD(logp, 0, magic)
#define VOLDRL_LOAD_CLEARFLAG(logp, flag) \
		VOLDRL_LONG_LOAD(logp, VOL_ULONG_BYTES, flag)
#define VOLDRL_LOAD_SERIAL_LO(logp, lo) \
		VOLDRL_LONG_LOAD(logp, 2*VOL_ULONG_BYTES, lo)
#define VOLDRL_LOAD_SERIAL_HI(logp, hi) \
		VOLDRL_LONG_LOAD(logp, 3*VOL_ULONG_BYTES, hi)
#define VOLDRL_LOAD_REGIONSZ(logp, sz) \
		VOLDRL_LONG_LOAD(logp, 4*VOL_ULONG_BYTES, sz)
#define VOLDRL_LOAD_VOLSZ(logp, sz) \
		VOLDRL_LONG_LOAD(logp, 5*VOL_ULONG_BYTES, sz)

#endif	/* _VXVM_VOLDRL_H */
