/* @(#)unixsrc:usr/src/common/uts/io/vxvm/common/volkern.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/common/volkern.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLKERN_H
#define _VXVM_VOLKERN_H

/*
 * volkern.h - miscellaneous VxVM kernel structure definitions
 */

#ifdef __STDC__
struct volopencount;
struct voldevice;
struct volobject;
struct voldisk;
struct voldg;
struct volklogcb;
struct volklogbuf;
union  volklogent;
struct volkdglog;
struct volklogcopy;
#endif	/* __STDC__ */

/*
 * structure to track opens through the various interfaces and of various
 * types.  The opn_mask field is a bitmap of or'd in open types, formed by
 * shifting 1 left by the open type number.  The opn_layer field keeps
 * track of the number of layered opens.
 *
 * Note: OTYP_LYR is set in opn_mask if opn_layer is non-zero.  This makes
 *	 it possible to check whether an entity is open just by checking
 *	 for a non-zero value for opn_mask.
 */
struct volopencount {
	volflag_t opn_mask;			/* 0x00 mask of open types */
	int       opn_layer;			/* 0x04 cnt of layered opens */
};						/* 0x08 */
#define VOLOPEN_MASK(otyp)	(1 << (otyp))
#define VOLOPEN_ISOPEN(opncnt)	((opncnt).opn_mask != 0)

#define V_OFLAGS	(FREAD|FWRITE)		/* flags passed to underlying
						   dirvers for open/close */

/*
 * voldevice - a structure mapping a device number to a configuration object
 *
 * The top-level open counts for volume device nodes are kept in this
 * structure.  On the first open, the open reference count is
 * incremented for the referenced objects, and all of its decendents.
 * On the last close, the same open reference counts are all decremented.
 *
 * Device mappings are kept in hash tables by device number.
 */
struct voldevice {
	struct voldevice    *vdev_next;		/* 0x00 next in hash bucket */
	volflag_t	     vdev_flags;	/* 0x04 virtual device flags */
	voldevno_t	     vdev_dev;		/* 0x08 virtual device number */
	struct volobject    *vdev_object;	/* 0x0c object for entry */
	struct volopencount  vdev_opencnt;	/* 0x10 track object opens */
	size_t		     vdev_tracerefcnt;	/* 0x18 # of I/O trace refs */
	ushort_t	     vdev_maxiosize;	/* 0x1c max I/O size (blocks) */
	ushort_t	     vdev_ncylinder;	/* 0x1e #cyls to report */
	ushort_t	     vdev_ntrack;	/* 0x20 tracks/cyl to report */
	ushort_t	     vdev_nsector;	/* 0x22 sects/track to report */

	ushort_t	     vdev_interleave;	/* 0x24 interleave to report */
	ushort_t	     vdev_rpm;		/* 0x26 rpm to report */
	ushort_t	     vdev_write_reinstruct;	/* 0x28 ?? to report */
	ushort_t	     vdev_read_reinstruct;	/* 0x2a ?? to report */

	struct vx_ioparameters vdev_ioparms;	/* 0x2c desired I/O sizes */
	struct voldevice_sys vdev_system;	/* 0x7c system specific info*/
};						/* 0x?? */

/* flag values for voldevice.vdev_flags */
#define VOLDEV_FLAG_CHANGE	0x00010000	/* pending change of device */
#define VOLDEV_FLAG_DELETE	0x00020000	/* pending removal of device */
#define VOLDEV_FLAG_IOTRACE	0x00040000	/* I/O tracing on device */
#define VOLDEV_FLAG_PRESERVE	0x00080000	/* keep after RESET_KERNEL */
#define VOLDEV_TRANS_FLAGS	\
	(VOLDEV_FLAG_CHANGE | VOLDEV_FLAG_DELETE | VOLDEV_FLAG_ASSERTMASK)

/*
 * voldevlist - hash table of virtual disk device mappings
 * voldevlistiter - structure for iterating through a virtual device hash table
 */
#define VOLDEVLIST_NBUCKETS	128		/* # of device hash buckets */

struct voldevlist {
	struct voldevice *buckets[VOLDEVLIST_NBUCKETS];
};

struct voldevlistiter {
	struct voldevice **bucket;		/* 0x00 next bucket */
	int		   nleft;		/* 0x04 remaining # buckets */
	struct voldevice  *next;		/* 0x08 next object to return */
};						/* 0x0c */

/*
 * volktrans_klocks - buffer of object locks for a transaction
 *
 * The list of all lock buffers issued by vold for a transaction are
 * kept in a list (using this structure) for use when the transaction
 * is committed.
 */
struct volktrans_klocks {
	struct volktrans_klocks	*ktl_next;	/* 0x00 next in list */
	struct volklock_buf	 klb;		/* 0x04 vold lock buffer */
};						/* 0x14 */

/*
 * This is the kernel's internal disk structure
 */
struct voldisk {
	struct volkdisk		 d_kd;		/* 0x000 external disk struct */
	struct voldisk		*d_next;	/* 0x0e4 next global disklist */
	struct voldg	 	*d_dg;		/* 0x0e8 assoc disk group */
	ulong_t			 d_flags;	/* 0x0ec disk flags */
	size_t			 d_opencnt;	/* 0x0f0 # of open subdisks */
	size_t			 d_refcnt;	/* 0x0f4 # of referenced sd's */
	size_t			 d_newrefcnt;	/* 0x0f8 # uncommitted refcnt */
	size_t			 d_tracerefcnt;	/* 0x0fc # of I/O trace refs */
	struct volklogcopy	*d_logtab;	/* 0x100 ptr to log area tbl */
	int			 d_nlogtab;	/* 0x104 # d_logtab entries */
	int			 d_curcpy;	/* 0x108 cur log copy ptr */
	size_t			 d_curloglen;	/* 0x10c cur klog write len */
	struct vol_rwstats	 d_stats;	/* 0x110 I/O statistics */
	volfastlock_t		 d_lock;	/* 0x114 fastlock for disk */
	struct voldisk_sys	 d_system;	/* 0x120 system specific info*/
};						/* 0x124 */

/* flags for voldisk.d_flags */
#define VOLD_FLAG_IOTRACE	0x00000001	/* disk I/O tracing enabled */

/* convenience macros for accessing d_kd member of voldisk */
#define dc_da_name	d_kd.kd_da_name
#define dc_dm_name	d_kd.kd_dm_name
#define dc_da_rid	d_kd.kd_da_rid
#define dc_dg_iid	d_kd.kd_dg_iid
#define dc_dm_rid	d_kd.kd_dm_rid
#define dc_kflag	d_kd.kd_kflag
#define dc_sflag	d_kd.kd_sflag
#define dc_vflag	d_kd.kd_vflag
#define dc_pub_dev	d_kd.kd_pub_dev
#define dc_priv_dev	d_kd.kd_priv_dev
#define dc_pub_len	d_kd.kd_pub_len
#define dc_priv_len	d_kd.kd_priv_len
#define dc_type		d_kd.kd_type
#define dc_info		d_kd.kd_info
#define dc_maxiosize	d_kd.kd_maxiosize

/*
 * This is the kernel's internal disk-group structure
 */
struct voldg {
	struct volkdg		 dg_kdg;	/* 0x00 external group struct */
	struct voldg		*dg_next;	/* 0x7c diskgroup list next */
	struct voldisk		*dg_disks;	/* 0x80 list of group's disks */
	struct volobjlist	*dg_objlist;	/* 0x84 current object list */
	struct volobjlist	*dg_newobjlist;	/* 0x88 uncommitted objects */
	struct volobjlist	*dg_oldobjlist;	/* 0x8c uncommitted objects */
	struct volkdglog	*dg_klog;	/* 0x90 klog cntl structure */
	int			 dg_nklog;	/* 0x94 # of klog sectors */
	unsigned char	       (*dg_klogblk)[NBPSCTR];	/* 0x98 klog block */
	struct volsio		*dg_klogsio;	/* 0x9c klog sio for flushes */
	volfastlock_t		 dg_lock;	/* 0xa0 diskgroup lock */
	struct voldg_sys	 dg_system;	/* 0xac system-specific info */
};						/* 0x?? */

/* convenience macros for accessing dg_kdg member of voldg */
#define	dgc_uuid	dg_kdg.kdg_uuid
#define	dgc_iid		dg_kdg.kdg_iid
#define	dgc_name	dg_kdg.kdg_name
#define	dgc_kflag	dg_kdg.kdg_kflag
#define	dgc_vflag	dg_kdg.kdg_vflag
#define	dgc_config_tid	dg_kdg.kdg_config_tid

struct volkdglog {
	int			dgl_flags;	/* 0x000 status flag */
	int			dgl_nused;	/* 0x004 # of in-use entries */
};						/* 0x008 */

#define VOL_KLOG_BLKCLEAN	0x0		/* klog block is clean */
#define VOL_KLOG_BLKDIRTY	0x1		/* klog block is dirty */
#define VOL_KLOG_BLKFLUSHED	0x2		/* blk has been flushed */

/*
 * The following states are the values that are valid for the dl_state
 * element of the voldisklog structure that follows.
 */
enum volcpystate {
	VOL_KLOG_NOLOG,		/* no log copy allocated here */
	VOL_KLOG_VALID,		/* log is valid */
	VOL_KLOG_IOERR		/* log has had an I/O error */
};

/*
 * An array of the following structures is pointed to by each disk
 * structure.
 * The structure contains a description of each instance or copy of the
 * log on disk.
 */
struct volklogcopy {
	struct volklogarea	*vl_klogareap;	/* 0x00 ptr to log area table */
	int			vl_nklogarea;	/* 0x04 size of table */
	enum volcpystate	vl_state;	/* 0x08 state of the log */
};						/* 0x0c */

/*
 * Structure for maintaining open protocols for the volspec driver devices
 */
struct volspec {
	int			vs_flags;	/* 0x00 flags */
	volocsleep_t		vs_open_sleep;	/* 0x04 open/close sleeplock */
	struct volopencount	vs_opencnt;	/* 0x08 open count struct */
};

/*
 * Flags for voldiocb structure
 */
#define	VOL_VDIOCB_BUSY		0x0001		/* I/O is in progress */
#define VOL_VDIOCB_ERROR	0x0002		/* an error occurred */

/*
 * VOL_BPSYNC_NHASH - Define the size of the hash list of sync locks used
 * to provide synchronization points for kios sleeping on bp_mapin space.
 * The number does better as a prime number since the hashing function
 * generally takes place based on the address of the kio.  These have the
 * tendancy to be allocated on page boundaries making simple address
 * hashing more of a problem.
 */
#define VOL_BPSYNC_NHASH	101

/*
 * VOL_BPSYNC_HASH - Return the address of a volsync_t in the vol_bpsync_hash
 * hash list.
 */
#define VOL_BPSYNC_HASH(bp)	\
		(&vol_bpsync_hash[((unsigned int)(bp) % VOL_BPSYNC_NHASH)])

/*
 * Licensing macros
 *  VOL_KLIC_CHECK() -- check to see if a specific feature is licensed, and
 *	return non-zero if the feature is licensed.
 */
#ifdef LIC
#define VOL_KLIC_CHECK(lic)	vol_klicense_check((lic))
#else	/* not LIC */
#define VOL_KLIC_CHECK(lic)	1
#endif	/* LIC */

#endif	/* _VXVM_VOLKERN_H */
