/* @(#)unixsrc:usr/src/common/uts/io/vxvm/unixware/volkio.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/unixware/volkio.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLKIO_H
#define _VXVM_VOLKIO_H

/*
 * volkio.h - system-dependent interface to kernel-generated I/Os
 */

/*
 * volkio_t - type for system-generated I/O
 *
 * This is a system-dependent type that must be passed around to
 * indicate the original I/O passed into the volume manager.
 */
typedef struct buf volkio_t;

/* macros for internal use on volkio_t structures */
#ifdef UW_2_1
#define VOLKIO_SAVEOBJECT(kio, obj)     ((kio)->b_priv.un_ptr = (void *)(obj))
#define VOLKIO_RESTOREOBJECT(kio)       ((kio)->av_back =               \
                                        (buf_t *)((kio)->b_priv.un_ptr))
#endif /* UW_2_1 */
#define VOLKIO_SETOBJECT(kio, obj)	((kio)->av_back = (volkio_t *)(obj))
#define VOLKIO_GETOBJECT(kio)		((struct volobject *)(kio)->av_back)
#define VOLKIO_ISDONE(kio)		(((kio)->b_flags & B_DONE) != 0)
#define VOLKIO_DEVICE(kio)		((kio)->b_edev)
#define VOLKIO_OFFSET(kio)		((kio)->b_blkno)
#define VOLKIO_LEN(kio)			((kio)->b_bcount >> SCTRSHFT)
#define VOLKIO_SETLEN(kio, len)		((kio)->b_bcount = (len << SCTRSHFT))
#define VOLKIO_BUFSIZE(kio)		((kio)->b_bcount)
#define VOLKIO_ADDR(kio)		((kio)->b_un.b_addr)
#define VOLKIO_GETERROR(kio)		(geterror(kio))
#define VOLKIO_READ(kio)		(((kio)->b_flags & B_READ) != 0)
#define VOLKIO_WRITE(kio)		(!VOLKIO_READ(kio))
#define VOLKIO_SETPRIVATE(kio, priv)	((kio)->b_forw = (volkio_t *)(priv))
#define VOLKIO_GETPRIVATE(kio)		((VOID *)(kio)->b_forw)
#define VOLKIO_SETSTARTTIME(kio, time)	((kio)->b_start = time)
#define VOLKIO_GETSTARTTIME(kio)	((kio)->b_start)
#define VOLKIO_SETOPNO(kio, opno)       ((kio)->b_resid = (opno))
#define VOLKIO_GETOPNO(kio)             ((kio)->b_resid)
#define VOLKIO_COPYOPNO(skio, dkio)     ((dkio)->b_resid = (skio)->b_resid)
#define VOLKIO_CLRWAKEUP(kio)		(VOLKIO_SETOPNO((kio), 0))
#define VOLKIO_WAKEUP(kio)		(VOLKIO_SETOPNO((kio), 1))
#define VOLKIO_WOKEN(kio)		(VOLKIO_GETOPNO(kio) == 1)
#define VOLKIO_SETERROR(kio, err, resid) \
	(bioerror((kio), (err)), (kio)->b_resid = (resid))

/*
 * macros for defining buffers that do or do not use I/O memory lists
 *
 * VOLKIO_SYS_RESET	- reset a buffer (including system buffer) that
 *			  does not use I/O memory lists.
 * VOLKIO_SET_IOMEM	- set a new buffer to represent an I/O memory list
 *			  with a specified block offset.
 * VOLKIO_HAS_IOMEM	- return TRUE if kio is I/O memory-list based.
 * VOLKIO_IOMEM_LIST	- return the I/O memory list for a KIO.
 * VOLKIO_IOMEM_LIST	- return the I/O memory skip length (in bytes).
 */
#ifdef UW_2_1
#define b_chain b_priv2.un_ptr
#endif /* UW_2_1 */

#define VOLKIO_SYS_RESET(kio)		((kio)->b_chain = NULL)
#define VOLKIO_SET_IOMEM(kio, iomem, skip)				\
	((kio)->b_chain = (struct buf *)(iomem), 			\
	 (kio)->b_un.b_addr = (caddr_t)VOL_DTOB(skip))
#define VOLKIO_HAS_IOMEM(kio)		((kio)->b_chain != NULL)
#define VOLKIO_IOMEM_LIST(kio)		((struct voliomemlist *)(kio)->b_chain)
#define VOLKIO_IOMEM_SKIP(kio)		((size_t)(kio)->b_un.b_addr)

/*
 * macros to help with kio memory iteration.
 * VOLKIO_ITER_RESET	- reset an iterator structure such that it will not
 *			  be mistaken for an initialized structure later.
 * VOLKIO_ITER_TEST	- test if the supplied iterator function has been
 *			  initialized.
 */
#define VOLKIO_ITER_RESET(iter)		((iter)->kmi_kio = NULL)
#define VOLKIO_ITER_TEST(iter)		((iter)->kmi_kio != NULL)

/*
 * volkiomemiter - structure for iterating through backing memory of a KIO
 *
 * This structure is used by the volkiomem_*() functions (see kiosubr.c).
 */
struct volkiomemiter {
	caddr_t		 kmi_addr;	/* address of current segment */
	size_t		 kmi_size;	/* size of current segment */
	int		 kmi_type;	/* KIO-type for iteration purposes */
	volkio_t	*kmi_kio;	/* KIO being iterated through */
	union {				/* KIO-type specific fields: */
		struct {		/* I/O memory iterator */
			struct voliomemiter memiter;
		} mi;
		struct volkiomem_uvaddr_iter {
			struct as *as;		/* user's address space */
			caddr_t    kvaddr;	/* allocated kernel address */
			caddr_t    uvaddr;	/* current user address */
			size_t	   remain;	/* remainder in buf */
		} mua;
	} kmu;
};
/* values for kmi_type: */
#define VOLKIOMEM_TYPE_IOMEM	1	/* KIO-type is I/O memory list */
#define VOLKIOMEM_TYPE_KADDR	2	/* KIO-type is kernel addressable */
#define VOLKIOMEM_TYPE_UADDR	3	/* KIO-type is user addressable */

/* also, see VOLKIO_IODIR below */

/*
 * VOLKIO_MARKSTABLE - mark a buf as pointing to a stable memory region
 * VOLKIO_CLEANUP    - clear any marks prior to doneing a buffer
 * VOLKIO_ISSTABLE   - check whether a buf points to stable memory
 * VOLKIO_STABLE_MEMORY - return memory (in blocks) required to stabilize KIO
 */
#ifdef B_PRIVLG
#define VOLKIO_FLAG_STABLE      B_PRIVLG
#else   /* not B_PRIVLG */
#define VOLKIO_FLAG_STABLE	B_ERROR
#endif  /* B_PRIVLG */
#define VOLKIO_MARKSTABLE(kio)	((kio)->b_flags |= VOLKIO_FLAG_STABLE)
#define VOLKIO_CLEANUP(kio)	((kio)->b_flags &= ~VOLKIO_FLAG_STABLE)
#define VOLKIO_ISSTABLE(kio)	(((kio)->b_flags & VOLKIO_FLAG_STABLE) != 0)
#define VOLKIO_STABLE_MEMORY(kio, kiooffset, len)	(len)

/*
 * volkioq - structure to link together system I/O structures
 */
struct volkioq {
	volkio_t *kioq_head;
	volkio_t *kioq_tail;
	size_t	  kioq_count;
};

/*
 * VOLKIOQ_RESET - reset queue q to be empty
 * VOLKIOQ_SET   - set queue q to contain just kio
 */
#define VOLKIOQ_RESET(q)	\
	((q)->kioq_head = (q)->kioq_tail = NULL, (q)->kioq_count = 0)
#define VOLKIOQ_SET(q, kio)	\
	((kio)->av_forw = NULL,					\
	 (q)->kioq_head = (q)->kioq_tail = (kio),		\
	 (q)->kioq_count = 1)

/*
 * VOLKIOQ_MORE  - true if queue q has more entries
 * VOLKIOQ_COUNT - return the number of KIOs on the queue
 * VOLKIOQ_DEQ   - dequeue and return the first element on queue q
 */
#define VOLKIOQ_MORE(q)		((q)->kioq_head != NULL)
#define VOLKIOQ_COUNT(q)	((q)->kioq_count)
#define VOLKIOQ_DEQ(q, kio)	\
	((kio) = (q)->kioq_head,				\
	 --(q)->kioq_count,					\
	 ((q)->kioq_head = (kio)->av_forw)?			\
	 ((kio)->av_forw = NULL): ((q)->kioq_tail = NULL))

/*
 * VOLKIOQ_APPEND - append an kio at the end of I/O queue q
 * VOLKIOQ_INSERT - insert an kio at the start of I/O queue q
 * VOLKIOQ_APPENDQ - append a list of kios to the end of I/O queue q1
 */
#define VOLKIOQ_APPEND(q, kio)	\
	((kio)->av_forw = NULL,					\
	 (q)->kioq_count++,					\
	 (q)->kioq_head?					\
	 ((q)->kioq_tail = (q)->kioq_tail->av_forw = (kio)):	\
	 ((q)->kioq_head = (q)->kioq_tail = (kio)))

#define VOLKIOQ_INSERT(q, kio)	\
	((q)->kioq_count++,					\
	 ((q)->kioq_tail == NULL? (q)->kioq_tail = (kio): 0),	\
	 (kio)->av_forw = (q)->kioq_head,			\
	 (q)->kioq_head = (kio))

#define VOLKIOQ_APPENDQ(q1, q2)	\
	((q1)->kioq_count += (q2)->kioq_count,			\
	 (q2)->kioq_head?					\
	 (((q1)->kioq_head == NULL?				\
	   ((q1)->kioq_head = (q2)->kioq_head):		\
	   ((q1)->kioq_tail->av_forw = (q2)->kioq_head)),	\
	  ((q1)->kioq_tail = (q2)->kioq_tail)): 0)

#endif	/* _VXVM_VOLKIO_H */
