/* @(#)unixsrc:usr/src/common/uts/io/vxvm/common/volklog.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/common/volklog.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

/*
 * sys/volkern.h - volume manager kernel only header file
 */

#ifndef _SYS_VOLKLOG_H
#define _SYS_VOLKLOG_H

/*
 * The following structure defines the kernel image of the disk group log,
 * which is the list of log blocks for a disk group. The dgl_blk array
 * contains the sector-sized network independant form of the log block as it
 * appears on disk. The format of the block is as follows:
 *
 *		dgl_blk on intel contents:
 *		_________________________________________ __
 *		|	0x000 magic number		|  |
 *		|---------------------------------------|  |
 *		|	0x004 transaction ID		|   - VOL_KLOGENT_SIZE
 *		|---------------------------------------|  |
 *		|	0x00c sequence number		|  |
 *		|---------------------------------------|  |
 *		|	0x010 klog length		|  |
 *		|---------------------------------------|  |
 *		|	0x014 klog blk #		|  |
 *		|---------------------------------------| --
 *		|	0x018 log entry [0]		|
 *		|	0x030 log entry [1]		|
 *		|		.			|
 *		|		.			|
 *		|		.			|
 *		|	0x1e0 log entry [NLOGENT - 1]	|
 *		-----------------------------------------
 *
 */

#define VOL_KLOG_MAGIC	       (('K' << 24) | \
				('L' << 16) | \
				('O' << 8) | \
				('G'))

/* Following are the valid log entry types */
typedef enum vol_klogent_type {
	MIN_KLOG_TYPE_ENT,
	VOL_KLOG_DETACH,	/* object detach log entry */
	VOL_KLOG_DIRTY,		/* object open log entry */
	VOL_KLOG_COMMIT,	/* transaction commit log entry */
	VOL_KLOG_LOGDETACH,	/* volume BCL has failed - overrides VOLDIRTY */
	VOL_KLOG_APP_DIRTY,	/* application recovery seqno */
	MAX_KLOG_TYPE_ENT
} volklogent_t;

/*
 * The following defines the maximum entry size for the log entries based
 * on the largest (abort) record size. Generally this will be 21 bytes
 * padded to a 4-byte boundary for a total of 24 bytes.
*/
#define VOL_KLOGENT_SIZE	\
	(((1 + VOL_RID_BYTES + VOL_RID_BYTES +  VOL_ULONG_BYTES) + 4) & \
	~(4 - 1))

/*
 * NLOGENT is the number of entries for each log block based on the overhead
 * of the header rounded to the log entry size.
 */
#define NLOGENT	\
	((NBPSCTR - VOL_KLOGENT_SIZE) / VOL_KLOGENT_SIZE)


/*
 * This structure is returned by VOL_KLOG_DUMPENT ioctl.
 *
 * The following union forms the content of each entry in the disk log.
 * This union is used for the interface between the caller of the logging
 * code and the internal code. Once the entry has been passed into the logging
 * routine, then it is converted into the network independant format as it will
 * be stored on disk.
 */
union volklogent {
	char		le_logtype;		/* 0x00 type of log entry */
	struct {
		char		rle_logtype;	/* 0x00 type of log entry */
		volrid_t	rle_rid;	/* 0x04 object ID */
		volseqno_t	rle_config_id;	/* 0x0c config ID of op */
	}		le_record;		/* 0x14 */
	struct {
		char		cle_logtype;	/* 0x00 type of log entry */
		volseqno_t	cle_old_conf_id; /* 0x04 old configuration ID */
		volseqno_t	cle_new_conf_id; /* 0x0c new configuration ID */
	}		le_commit;		/* 0x14 */
	struct {
		char		sle_logtype;	/* 0x00 type of log entry */
		volrid_t	sle_rid;	/* 0x04 object ID */
		volseqno_t	sle_config_id;	/* 0x0c config ID */
		u_long		sle_sequence;	/* 0x14 sequence value */
	}		le_sequence;		/* 0x18 */
	char		le_filleur[24];		/* 0x00 size of each entry */
};

#define le_rid		le_record.rle_rid
#define le_config_id	le_record.rle_config_id
#define le_oconfig_id	le_commit.cle_old_conf_id
#define le_nconfig_id	le_commit.cle_new_conf_id

#endif	/* _SYS_VOLKLOG_H */
