/* @(#)unixsrc:usr/src/common/uts/io/vxvm/unixware/vollocks.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/unixware/vollocks.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

/*
 * vollocks - Definition of all VM locks.
 */

#ifndef _VXVM_VOLLOCKS_H
#define _VXVM_VOLLOCKS_H

#if defined(UNIPROC)
#undef MP
#endif

#ifdef SOLARIS
#include	<sys/ksynch.h>
#include	<sys/t_lock.h>
#include	<sys/thread.h>
#ifdef SOLARIS_2_5
#include	<sys/spl.h>
#else /* !SOLARIS_2_5 */
#include	<sys/psw.h>
#endif /* SOLARIS_2_5 */
#endif /* SOLARIS */

typedef struct volspin     volspin_t;
typedef struct volsleep    volsleep_t;
typedef struct volsync     volsync_t;
typedef struct volrwsleep  volrwsleep_t;
typedef	struct volrwlock   volrwlock_t;

#if defined(VOL_HAVE_FSPIN) && !defined(TED_LOCKS)
typedef fspin_t            volfastlock_t;
#else   /* not VOL_HAVE_FSPIN && !TED_LOCKS */
typedef struct volspin	   volfastlock_t;
#endif  /* VOL_HAVE_FSPIN && !TED_LOCKS */
 
/*
 * spinlock locking hierarchy
 *
 * if fast locks are being emulated by regular spinlocks, use a
 * large hierarchy number, with the same number for all fast locks
 * (we presume that only one fast lock is held at a time, and that
 * no other locks are obtained while holding a fast lock).
 *
 * For spinlocks that protect most synchronization variables, we
 * presume that only fast spinlocks could be held in addition to the
 * sync protect spinlock.
 */

#define VOL_HSPIN_SYNCLOCK	31	/* spinlock to protect synch wait */
#define VOL_FSPIN_FASTLOCK	32	/* fast spinlock */

/*
 * Generic lock macros and structures. The following structures contain both
 * the UP and MP definitions for their contents.  They are padded to the
 * same size for the reference Intel architecture so that structure offsets
 * for structures containing these locks can be consistent.
 */
 
/*
 * struct volspin is used to implement the generic spinlock.
 */
struct volspin {
#ifdef MP
#ifdef SOLARIS
	kmutex_t	sp_spin;		/* 0x00 */
	pl_t		sp_pri;			/* 0x08 */
#else /* SOLARIS */
        lock_t          *sp_spin;               /* 0x00 */
        lkinfo_t        *sp_lkinfop;            /* 0x04 */
        pl_t            sp_pri;                 /* 0x08 */
#endif /* SOLARIS */
#else /* ! MP */
        int             sp_pri;                 /* 0x00 */
        int             *sp_pad[2];             /* 0x04 */
#endif /* MP */
#if defined(TED_LOCKS)
	struct volted_pinfo	*sp_owner;	/* 0x0c pinfo of lock holder */
	int			sp_nheld;	/* 0x10 number of times held */
	struct volspin		*sp_next;	/* 0x14 next spinlock in list*/
	struct volspin		*sp_prev;	/* 0x18 prev spinlock in list*/
	/* XXX the following was sp_hier, which is a macro in sys/ksynch.h */
	int			sp_hierarchy;	/* 0x1c spinlock hierarchy
						        number. locked by
							increasing value */
	char			*sp_ofile;	/* 0x20 file name of owner */
	ulong_t			sp_oline;	/* 0x24 line # of owner */
	int			sp_magic;	/* 0x28 magic num for asserts
							about alloc status */
#endif /* TED_LOCKS */
};

/*
 * struct volrwlock is used to implement the generic rwlock.
 */
struct volrwlock {
	rwlock_t		*rw_lockp;	/* 0x00 */
	lkinfo_t		*rw_lkinfop;	/* 0x04 */
#if defined(TED_LOCKS)
	struct volted_pinfo	*sp_owner;	/* 0x0c pinfo of lock holder */
	int			sp_nheld;	/* 0x10 number of times held */
	struct volspin		*sp_next;	/* 0x14 next spinlock in list */
	struct volspin		*sp_prev;	/* 0x18 prev spinlock in list */
	int			sp_hierarchy;	/* 0x1c spinlock hierarchy
						        number. locked by
							increasing value */
	char			*sp_ofile;	/* 0x20 file name of owner */
	ulong_t			sp_oline;	/* 0x24 line # of owner */
	int			sp_magic;	/* 0x28 magic number for asserts
							about alloc status */
#endif /* TED_LOCKS */
};

/*
 * struct volsleep is used to implement the generic sleep lock.
 */
struct volsleep {
#if defined(MP)
#if defined(SOLARIS)
	kmutex_t	sl_sleep;		/* 0x00 sleep lock */
	pl_t		*sl_pri;		/* 0x08 saved priority level */
#else /* SOLARIS */
        sleep_t         *sl_sleep;              /* 0x00 sleep lock */
        lkinfo_t        *sl_lkinfop;            /* 0x04 */
        pl_t            *sl_pri;                /* 0x08 saved priority level */
#endif /* SOLARIS */
#else /* MP */
        ulong_t         sl_flags;               /* 0x00 sleep flags */
        int             sl_pri;                 /* 0x04 saved priority level */
        int             *sl_pad;                /* 0x08 */
#endif /* MP */
#if defined(TED_LOCKS)
	struct volted_pinfo	*sl_owner;	/* 0x0c pinfo of lock holder */
	int			sl_nheld;	/* 0x10 number of times held */
	struct volsleep		*sl_next;	/* 0x14 next lock in list */
	struct volsleep		*sl_prev;	/* 0x18 prev lock in list */
	struct volted_pinfo	*sl_want;	/* 0x1c procs wanting lock */
	int			sl_magic;	/* 0x20 magic number for asserts
							about alloc status */
#endif /* TED_LOCKS */
};

/*
 * flags used by the UP sleeplock macros
 */
#define _VOL_SWANT              0x1             /* the sleep lock is wanted */
#define _VOL_SLOCK              0x2             /* the sleep lock is held */

/*
 * struct volsync is used to implement the generic sync lock, equivalent
 * to traditional sleep/wakeup or SVR4 MP syncronization variables.
 */
struct volsync {
#if defined(MP)
#if defined(SOLARIS)
	kcondvar_t	sy_sync;	/* 0x00 sync variable 		*/
#else /* SOLARIS */
        sv_t            *sy_sync;               /* 0x00 sync variable */
        ulong_t         sy_flags;               /* 0x04 sync flags */
#endif /* SOLARIS */
#else /* MP */
        int             sy_sync;                /* 0x00 wait channel */
        ulong_t         sy_flags;               /* 0x04 sync flags */
#endif /* MP */
#if defined(TED_LOCKS)
	int		sy_wait;	/* 0x08 procs waiting on lock	*/
	struct volspin	*sy_spin;	/* 0x0c spinlock held by procs 
						sleeping on synclock	*/
	int		sy_magic;	/* 0x10 magic num for asserts
						about alloc status	*/
#endif /* TED_LOCKS */
};

#if defined(TED_LOCKS)
struct volted_pinfo {
	struct volted_pinfo	*hnext;		/* 0x00 - next in hash list */
	struct volted_pinfo	*hprev;		/* 0x04 - prev in hash list */
	u_int			tid;		/* 0x08 - id for thread */
	int			flags;		/* 0x0c - flags */
	int			refcnt;		/* 0x10 - reference count */
	struct volted_pinfo     *want_next; 	/* 0x14 */
	struct volted_pinfo	*want_prev; 	/* 0x18 */
	int			pri;		/* 0x1c */
	struct volspin		*spinp;		/* 0x20 */
	int			nspin;		/* 0x24 */
	struct volsleep		*sleepp;	/* 0x28 */
	int			nsleep;		/* 0x2c */
};                                              /* 0x30 */

#define VOLTED_PFLAG_BUSY	0x1	/* entry is busy */
#define VOLTED_PFLAG_ALLOC	0x2	/* entry has been allocated from kmem */

struct volted_phash {
	struct volted_pinfo	*first;
	struct volted_pinfo	*last;
};

#endif /* TED_LOCKS */

/*
 ****************************************************************************
 * Start of MP section for locks. All locks in this section are MP versions *
 * of the locks in the following section.                                   *
 ****************************************************************************
 */
#ifdef MP

/*
 * Spinlock macros.
 *
 * On the SVR4/MP platform, spinlocks are called basic locks.  The only
 * semantic for these locks is to exclude other execution contexts in a
 * sequence of non-reentrant code.  Spinlocks should be held for short
 * periods only since, on an MP, they may cause serialization of operation
 * through a single processor.  They may be obtained while holding other
 * locks, although should always be obtained in the same order.  They must
 * be freed in the reverse sequence from which they are obtained.
 */
#ifdef SOLARIS
/* On solaris, use MUTEX_DRIVER-type locks with a NULL priority arg; this
 * creates adaptive spinlocks, which the Solaris folks assure us is the
 * right thing to do.
 */

#define _VOL_ALLOCSPINLOCK(spin)					\
        mutex_init(&(spin)->sp_spin, "vxio_spin_lock", MUTEX_DRIVER,	\
		   NULL)

#define _VOL_FREESPINLOCK(spin) 	mutex_destroy(&(spin)->sp_spin)
#define _VOL_SPINLOCK(spin)		mutex_enter(&(spin)->sp_spin)
#define _VOL_UNSPINLOCK(spin)		mutex_exit(&(spin)->sp_spin)

#else /* ! SOLARIS */

#ifdef NSC_BUG_FIX

/*
 * Debug kernels panic due to lock hierarchy problems; make all spinlocks
 * 2; rw-spinlocks are defined to be 1. I also initialized the lk_name field
 * of the lkinfo structure so problems could be debugged more easily.
 */

#define _VOL_ALLOCSPINLOCK(spin)                                        \
        do {                                                            \
                (spin)->sp_lkinfop =                                    \
                        _VOLMEMZALLOC(sizeof(*(spin)->sp_lkinfop),      \
                                      VOLMEM_SLEEP);                    \
		(spin)->sp_lkinfop->lk_name = #spin;			\
                (spin)->sp_spin = LOCK_ALLOC(2, pldisk,			\
			(spin)->sp_lkinfop, KM_SLEEP);                  \
        } while (ZERO)

#else  /* !NSC_BUG_FIX */
 
#define _VOL_ALLOCSPINLOCK(spin)                                        \
        do {                                                            \
                (spin)->sp_lkinfop =                                    \
                        _VOLMEMZALLOC(sizeof(*(spin)->sp_lkinfop),      \
                                      VOLMEM_SLEEP);                    \
                (spin)->sp_spin = LOCK_ALLOC(1, pldisk, (spin)->sp_lkinfop,\
                        KM_SLEEP);                                      \
        } while (ZERO)

#endif /* !NSC_BUG_FIX */
 
#define _VOL_FREESPINLOCK(spin)                                         \
        do {                                                            \
                LOCK_DEALLOC((spin)->sp_spin);                          \
                _VOLMEMFREE((spin)->sp_lkinfop,                         \
                        sizeof(*(spin)->sp_lkinfop));                   \
        } while (ZERO)
 
#define _VOL_SPINLOCK(spin)                                             \
                ((spin)->sp_pri = LOCK((spin)->sp_spin, plhi))
 
#define _VOL_UNSPINLOCK(spin)   (UNLOCK((spin)->sp_spin, (spin)->sp_pri))
#endif /* SOLARIS */

/*
 * Sleeplock macros.
 *
 * A sleeplock is a mutual exclusion lock for a critical section of code
 * that will cause the calling process to sleep if the lock is already
 * held.  Sleeplocks cannot be acquired with a spinlock held, but can be
 * held with other sleeplocks held.
 */

#ifdef SOLARIS
/*
 * For Solaris, adaptive mutexes are used for sleep locks
 */
#define _VOL_ALLOCSLEEPLOCK(slp)					\
        mutex_init(&(slp)->sl_sleep, "vxio_spin_lock", MUTEX_DRIVER,	\
		   NULL)

#define _VOL_FREESLEEPLOCK(slp)		mutex_destroy(&(slp)->sl_sleep)
#define _VOL_SLEEPLOCK(slp)		mutex_enter(&(slp)->sl_sleep)
#define _VOL_UNSLEEPLOCK(slp)		mutex_exit(&(slp)->sl_sleep)

#define _VOL_TRYSLEEPLOCK(slp, stat)					\
        ((stat) = mutex_tryenter(&(slp)->sl_sleep))

#else /* ! SOLARIS */

#ifdef NSC_BUG_FIX
 
#define _VOL_ALLOCSLEEPLOCK(slp)                                        \
        do {                                                            \
                (slp)->sl_lkinfop =                                     \
                        _VOLMEMZALLOC(sizeof(*(slp)->sl_lkinfop),       \
                                      VOLMEM_SLEEP);                    \
		(slp)->sl_lkinfop->lk_name = #slp;			\
                (slp)->sl_sleep = SLEEP_ALLOC(1, (slp)->sl_lkinfop,     \
                        KM_SLEEP);                                      \
        } while (ZERO)

#else  /* !NSC_BUG_FIX */
 
#define _VOL_ALLOCSLEEPLOCK(slp)                                        \
        do {                                                            \
                (slp)->sl_lkinfop =                                     \
                        _VOLMEMZALLOC(sizeof(*(slp)->sl_lkinfop),       \
                                      VOLMEM_SLEEP);                    \
                (slp)->sl_sleep = SLEEP_ALLOC(1, (slp)->sl_lkinfop,     \
                        KM_SLEEP);                                      \
        } while (ZERO)
 
#endif /* !NSC_BUG_FIX */

#define _VOL_FREESLEEPLOCK(slp)                                         \
        do {                                                            \
                SLEEP_DEALLOC((slp)->sl_sleep);                         \
                _VOLMEMFREE((slp)->sl_lkinfop, sizeof(*(slp)->sl_lkinfop));\
        }while (ZERO)
 
#define _VOL_SLEEPLOCK(slp)     (SLEEP_LOCK((slp)->sl_sleep, pridisk))
 
#define _VOL_UNSLEEPLOCK(slp)   (SLEEP_UNLOCK((slp)->sl_sleep))
 
#define _VOL_TRYSLEEPLOCK(slp, stat)                                    \
                ((stat) = SLEEP_TRYLOCK((slp)->sl_sleep))
 
#endif /* SOLARIS */

/*
 * Synclock macros.
 *
 * On a SVR4/MP platform, synclocks are implemented as synchronization
 * variables.  They act in a similar way to the basic unix sleep-wakeup
 * mechanism except that in order to ensure that a wakeup is not missed,
 * they must be called with a spinlock held.  The spinlock is atomically
 * released when the caller goes to sleep.  A wakeup is thus guaranteed
 * not to occur between the acquiring of the spinlock and the start of
 * the sleep.  On exit from the sleep, the spinlock is re-acquired.
 */

#ifdef SOLARIS
#define _VOL_ALLOCSYNC(sync)						\
        (cv_init(&(sync)->sy_sync, "vxio_sync_lock", CV_DRIVER, NULL))

#define _VOL_FREESYNC(sync)						\
	(cv_destroy(&(sync)->sy_sync))

#define _VOL_SYNCWAIT(sync, spin)					\
        (cv_wait(&(sync)->sy_sync, &(spin)->sp_spin))

#define _VOL_SYNCWAITSIG(sync, spin, signalled)				\
	do {								\
		signalled = cv_wait_sig(&(sync)->sy_sync,		\
					&(spin)->sp_spin);		\
		signalled = !signalled;					\
	} while (ZERO)

#define _VOL_SYNCSIG(sync)	 	(cv_broadcast(&(sync)->sy_sync))
#define _VOL_SYNCSIG1(sync)	 	(cv_signal(&(sync)->sy_sync))

#else /* not SOLARIS */
 
#define _VOL_ALLOCSYNC(sync)    ((sync)->sy_sync = SV_ALLOC(KM_SLEEP))
 
#define _VOL_FREESYNC(sync)     (SV_DEALLOC((sync)->sy_sync))
 
#define _VOL_SYNCWAIT(sync, spin)                                       \
        do {                                                            \
                SV_WAIT((sync)->sy_sync, pridisk, (spin)->sp_spin);     \
                _VOL_SPINLOCK(spin);                                    \
        } while (ZERO)
 
#define _VOL_SYNCWAITSIG(sync, spin, signalled)                         \
        do {                                                            \
                signalled = SV_WAIT_SIG((sync)->sy_sync, prihi,         \
                        (spin)->sp_spin);                               \
                _VOL_SPINLOCK(spin);                                    \
                signalled = !signalled;                                 \
        } while (ZERO)
 
#define _VOL_SYNCSIG(sync)       SV_BROADCAST((sync)->sy_sync, 0)
 
#define _VOL_SYNCSIG1(sync)      SV_SIGNAL((sync)->sy_sync, 0)
 
#endif /* SOLARIS */

#ifdef SOLARIS
/*
 * Solaris doesn't need the SWAP_LOCK and SWAP_UNLOCK macros, because
 * we can use kernel threads for I/O daemons.
 */
#define _SWAP_LOCK()
#define _SWAP_UNLOCK()
#else   /* not SOLARIS */
 
#ifdef UW_2_1
#define _SWAP_LOCK()                                                    \
                do {                                                    \
                        proc_t  *cprocp = NULL;                         \
                                                                        \
                        (void)drv_getparm(UPROCP, (ulong_t *)&cprocp);  \
                        DONTSWAP(cprocp);                               \
                } while (ZERO)
 
#define _SWAP_UNLOCK()                                                  \
                do {                                                    \
                        proc_t  *cprocp = NULL;                         \
                                                                        \
                        (void)drv_getparm(UPROCP, (ulong_t *)&cprocp);  \
                        cprocp->p_flag |= ~(P_NOSWAP | SKIP_AGING_FLAGS);\
                } while (ZERO)
 
#else /* !UW_2_1 */

#define _SWAP_LOCK()                                                    \
                do {                                                    \
                        proc_t  *cprocp = NULL;                         \
                        pl_t    pri;                                    \
                                                                        \
                        (void)drv_getparm(UPROCP, (ulong_t *)&cprocp);  \
                        pri = splhi();                                  \
                        locked_or(cprocp->p_flag, SSYS);                \
                        splx(pri);                                      \
                } while (ZERO)
 
#define _SWAP_UNLOCK()                                                  \
                do {                                                    \
                        proc_t  *cprocp = NULL;                         \
                        pl_t    pri;                                    \
                                                                        \
                        (void)drv_getparm(UPROCP, (ulong_t *)&cprocp);  \
                        pri = splhi();                                  \
                        locked_and(cprocp->p_flag, ~SSYS);              \
                        splx(pri);                                      \
                } while (ZERO)
#endif /* UW_2_1 */
 
#endif  /* SOLARIS */

/*
 * if real fast spinlocks exist, define macros for them
 */
 
#ifdef VOL_HAVE_FSPIN
 
#define _VOL_ALLOCFASTLOCK(flock)       \
        do {                                    \
                FSPIN_INIT(&flock->fl_fspin);   \
        } while (ZERO)
#define _VOL_FREEFASTLOCK(flock)
#define _VOL_FASTLOCK(flock)            \
        do {                                    \
                FSPIN_LOCK(&flock->fl_fspin);   \
        } while (ZERO)
#define _VOL_UNFASTLOCK(flock)          \
        do {
                FSPIN_UNLOCK(&flock->fl_fspin);
        } while (ZERO)

#else /* !VOL_HAVE_FSPIN */

#define _VOL_ALLOCFASTLOCK(flock)	_VOL_ALLOCSPINLOCK(flock)
#define _VOL_FREEFASTLOCK(flock)	_VOL_FREESPINLOCK(flock)
#define _VOL_FASTLOCK(flock)		_VOL_SPINLOCK(flock)
#define _VOL_UNFASTLOCK(flock)		_VOL_UNSPINLOCK(flock)

#endif  /* VOL_HAVE_FSPIN */

#else /* NOT MP */
/*
 *******************************************************************************
 * Start of UP section for locks. All locks in this section are UP versions of *
 * the locks in the prior section.                                             *
 *******************************************************************************
 */

/*
 * Spinlock macros.
 *
 * On a UP platform, spinlocks reduce to interrupt locks.  The only
 * semantic for these locks is to exclude other execution contexts in a
 * sequence of non-reentrant code.  Spinlocks should be held for short
 * periods only since they prevent interrupts from occuring and can lead
 * to poor real-time response characteristics. They may be obtained while
 * holding other locks, although should always be obtained in the same
 * order.  They must be freed in the reverse sequence from which they are
 * obtained so that the stored priorities can be restored correctly.
 */
#define _VOL_ALLOCSPINLOCK(spin)
#define _VOL_FREESPINLOCK(spin)
#define _VOL_SPINLOCK(spin)             ((spin)->sp_pri = splhi())
#define _VOL_UNSPINLOCK(spin)           (splx((spin)->sp_pri))
 
/*
 * Sleeplock macros.
 *
 * A sleeplock is a mutual exclusion lock for a critical section of code
 * that will cause the calling process to sleep if the lock is already
 * held.  Sleeplocks cannot be acquired with a spinlock held, but can be
 * held with other sleeplocks held.
 */
#define _VOL_ALLOCSLEEPLOCK(slp)        ((slp)->sl_flags = 0)
#define _VOL_FREESLEEPLOCK(slp)
/* FROM SCO
 * PZERO -> PRIMED since ddi-7, PRIMED is now defined in param.h and
 * definition of PZERO has been moved to ddi.h.
 * However, global int primed should be used here to resolve the
 * catch-22: param.h and ddi.h are mutually exclusive to each other.
 */
#define _VOL_SLEEPLOCK(slp)                                             \
        do {                                                            \
                int     x;                                              \
                                                                        \
                x = splhi();                                            \
                for (;;) {                                              \
                        if (((slp)->sl_flags & (_VOL_SLOCK | _VOL_SWANT)) == \
                            0) {                                        \
                                (slp)->sl_flags |= _VOL_SLOCK;          \
                                break;                                  \
                        } else {                                        \
                                (slp)->sl_flags |= _VOL_SWANT;          \
                                (void)sleep((caddr_t)(slp), primed);     \
                        }                                               \
                }                                                       \
                splx(x);                                                \
        } while (ZERO)
#define _VOL_UNSLEEPLOCK(slp)                                           \
        do {                                                            \
                int     x;                                              \
                                                                        \
                x = splhi();                                            \
                (slp)->sl_flags &= ~_VOL_SLOCK;                         \
                if ((slp)->sl_flags & _VOL_SWANT) {                     \
                        (slp)->sl_flags &= ~_VOL_SWANT;                 \
                        (void)wakeup((caddr_t)(slp));                   \
                }                                                       \
                splx(x);                                                \
        } while (ZERO)
#define _VOL_TRYSLEEPLOCK(slp, stat)                                    \
        do {                                                            \
                int     x;                                              \
                                                                        \
                x = splhi();                                            \
                if (((slp)->sl_flags & (_VOL_SLOCK | _VOL_SWANT)) == 0) {\
                        (slp)->sl_flags |= _VOL_SLOCK;                  \
                        (stat) = 1;                                     \
                } else {                                                \
                        (stat) = 0;                                     \
                }                                                       \
                splx(x);                                                \
        } while (ZERO)
 
/*
 * Synclock macros.
 *
 * On a UP platform, synclocks are implemented as basic unix sleep-wakeup
 * calls except that in order to ensure that a wakeup is not missed,
 * they must be called with a spinlock held.  The spinlock is atomically
 * released when the caller goes to sleep.  A wakeup is thus guaranteed
 * not to occur between the acquiring of the spinlock and the start of
 * the sleep.  On exit from the sleep, the spinlock is re-acquired.
 */
#define _VOL_ALLOCSYNC(sync)    ((sync)->sy_flags = 0)
#define _VOL_FREESYNC(sync)
/* FROM SCO
 * PZERO -> PRIMED since ddi-7, PRIMED is now defined in param.h and
 * definition of PZERO has been moved to ddi.h.
 * However, global int primed should be used here to resolve the
 * catch-22: param.h and ddi.h are mutually exclusive to each other.
 */
#define _VOL_SYNCWAIT(sync, s)                                          \
        do {                                                            \
                int     savepri = (s)->sp_pri;                          \
                                                                        \
                (void)sleep((caddr_t)&((sync)->sy_sync), primed);        \
                (void)splhi();                                          \
                (s)->sp_pri = savepri;                                  \
        } while (ZERO)
#define _VOL_SYNCWAITSIG(sync, s, signalled)                            \
        do {                                                            \
                int     savepri = (s)->sp_pri;                          \
                                                                        \
                signalled = sleep((caddr_t)&((sync)->sy_sync), PSLEP | PCATCH);\
                (void)splhi();                                          \
                (s)->sp_pri = savepri;                                  \
        } while (ZERO)
#define _VOL_SYNCSIG(sync)      ((void)wakeup((caddr_t)&((sync)->sy_sync)))
#define _VOL_SYNCSIG1(sync)     _VOL_SYNCSIG(sync)

/*
 * Other locking macros.
 */

#ifdef SOLARIS
 
/*
 * Solaris doesn't need the SWAP_LOCK and SWAP_UNLOCK macros, because
 * we can use kernel threads for I/O daemons.
 */
 
#else /* SOLARIS */
 
#define _SWAP_LOCK()                                                    \
                do {                                                    \
                        proc_t  *cprocp = NULL;                         \
                        int     pri;                                    \
                        (void)drv_getparm(UPROCP, (ulong_t *)&cprocp);  \
                        pri = splhi();                                  \
                        cprocp->p_swlocks++;                            \
                        cprocp->p_flag |= (SSWLOCKS | SSYS);            \
                        splx(pri);                                      \
                } while (ZERO)
#define _SWAP_UNLOCK()                                                  \
                do {                                                    \
                        proc_t  *cprocp = NULL;                         \
                        int     pri;                                    \
                        (void)drv_getparm(UPROCP, (ulong_t *)&cprocp);  \
                        pri = splhi();                                  \
                        if (--cprocp->p_swlocks == 0) {                 \
                                cprocp->p_flag &= ~SSWLOCKS;            \
                        }                                               \
                        cprocp->p_flag &= ~SSYS;                        \
                        splx(pri);                                      \
                } while (ZERO)
 
#endif /* ! SOLARIS */
 
 
/*
 * on a UP, always use the above spin macros for "fast" locks
 */
 
#define _VOL_ALLOCFASTLOCK(flock)       _VOL_ALLOCSPINLOCK(flock)
#define _VOL_FREEFASTLOCK(flock)        _VOL_FREESPINLOCK(flock)
#define _VOL_FASTLOCK(flock)            _VOL_SPINLOCK(flock)
#define _VOL_UNFASTLOCK(flock)          _VOL_UNSPINLOCK(flock)
 
#endif /* MP */


/*
 * Start of section for TED definitions used for lock debugging.
 */
#if defined(TED_LOCKS)

#ifdef __STDC__
extern int  _ted_setpri(int);
extern void _ted_testpri(int);
extern void _vol_allocspinlock(ulong_t, char *, struct volspin *, int);
extern void _vol_freespinlock(ulong_t, char *, struct volspin *);
extern void _vol_spinlock(struct volspin *, char *, ulong_t);
extern void _vol_unspinlock(struct volspin *);
extern void _vol_allocsleeplock(ulong_t, char *, struct volsleep *);
extern void _vol_freesleeplock(ulong_t, char *, struct volsleep *);
extern void _vol_sleeplock(struct volsleep *);
extern void _vol_unsleeplock(struct volsleep *);
extern void _vol_trysleeplock(struct volsleep *, int *);
extern void _vol_allocsync(ulong_t, char *, struct volsync *);
extern void _vol_freesync(ulong_t, char *, struct volsync *);
extern void _vol_syncwait(struct volsync *, struct volspin *);
extern void _vol_syncwaitsig(struct volsync *, struct volspin *, int *);
extern void _vol_syncsig(struct volsync *);
extern void _vol_syncsig1(struct volsync *);
#else	/* not __STDC__ */
extern int  _ted_setpri();
extern void _ted_testpri();
extern void _vol_allocspinlock();
extern void _vol_freespinlock();
extern void _vol_spinlock();
extern void _vol_unspinlock();
extern void _vol_allocsleeplock();
extern void _vol_freesleeplock();
extern void _vol_sleeplock();
extern void _vol_unsleeplock();
extern void _vol_trysleeplock();
extern void _vol_allocsync();
extern void _vol_freesync();
extern void _vol_syncwait();
extern void _vol_syncwaitsig();
extern void _vol_syncsig();
extern void _vol_syncsig1();
#endif	/* __STDC__ */

#define VOL_ALLOCSPINLOCK(sp, hier)	_vol_allocspinlock(__LINE__, __FILE__,\
							   (sp), (hier))
#define VOL_FREESPINLOCK(sp)		_vol_freespinlock(__LINE__, __FILE__, \
							  (sp))
#define VOL_SPINLOCK(sp)		_vol_spinlock(sp, __FILE__, __LINE__)
#define VOL_UNSPINLOCK(sp)		_vol_unspinlock(sp)
#define VOL_ALLOCSLEEPLOCK(slp)		_vol_allocsleeplock(__LINE__,	\
							    __FILE__, (slp))
#define VOL_FREESLEEPLOCK(slp)		_vol_freesleeplock(__LINE__, __FILE__,\
							   (slp))
#define VOL_SLEEPLOCK(slp)		_vol_sleeplock((slp))
#define VOL_UNSLEEPLOCK(slp)		_vol_unsleeplock((slp))
#define VOL_TRYSLEEPLOCK(slp, ret)	_vol_trysleeplock((slp), &(ret))
#define VOL_ALLOCSYNC(sync)		_vol_allocsync(__LINE__, __FILE__, \
						       (sync))
#define VOL_FREESYNC(sync)		_vol_freesync(__LINE__, __FILE__, \
						      (sync))
#define VOL_SYNCWAIT(sync, sp)		_vol_syncwait((sync), (sp))
#define VOL_SYNCWAITSIG(sync, sp, ret)	_vol_syncwaitsig((sync), (sp), &(ret))
#define VOL_SYNCSIG(sync)		_vol_syncsig(sync)
#define VOL_SYNCSIG1(sync)		_vol_syncsig1(sync)
#define VOL_CONTEXT_ENTER()		_vol_context_enter(0)
#define VOL_CONTEXT_EXIT()		_vol_context_exit()
#define VOL_CALLOUT_CONTEXT_ENTER()	_vol_context_enter(1)
#define VOL_CALLOUT_CONTEXT_EXIT()	_vol_context_exit()

#else /* not TED_LOCKS */

#define VOL_ALLOCSPINLOCK(spin, hier)	_VOL_ALLOCSPINLOCK(spin)
#define VOL_FREESPINLOCK(spin)		_VOL_FREESPINLOCK(spin)
#define VOL_SPINLOCK(spin)		_VOL_SPINLOCK(spin)
#define VOL_UNSPINLOCK(spin)		_VOL_UNSPINLOCK(spin)
#define VOL_ALLOCSLEEPLOCK(slp)		_VOL_ALLOCSLEEPLOCK(slp)
#define VOL_FREESLEEPLOCK(slp)		_VOL_FREESLEEPLOCK(slp)
#define VOL_SLEEPLOCK(slp)		_VOL_SLEEPLOCK(slp)
#define VOL_UNSLEEPLOCK(slp)		_VOL_UNSLEEPLOCK(slp)
#define VOL_TRYSLEEPLOCK(slp, ret)	_VOL_TRYSLEEPLOCK(slp, ret)
#define VOL_ALLOCSYNC(sync)		_VOL_ALLOCSYNC(sync)
#define VOL_FREESYNC(sync)		_VOL_FREESYNC(sync)
#define VOL_SYNCWAIT(sync, sp)		_VOL_SYNCWAIT(sync, sp)
#define VOL_SYNCWAITSIG(sync, sp, ret)	_VOL_SYNCWAITSIG(sync, sp, ret)
#define VOL_SYNCSIG(sync)		_VOL_SYNCSIG(sync)
#define VOL_SYNCSIG1(sync)		_VOL_SYNCSIG1(sync)
#define SYS_SWAP_LOCK()			_SWAP_LOCK()
#define SYS_SWAP_UNLOCK()		_SWAP_UNLOCK()
#define VOL_CONTEXT_ENTER()
#define VOL_CONTEXT_EXIT()
#define VOL_CALLOUT_CONTEXT_ENTER()
#define VOL_CALLOUT_CONTEXT_EXIT()

#endif /* TED_LOCKS */

/*
 * NOTE: For TED, always use spinlocks rather than fastlocks
 */
#if defined(VOL_HAVE_FSPIN) && !defined(TED_ACTIVE)
 
#define VOL_ALLOCFASTLOCK(flock)        _VOL_ALLOCFASTLOCK(flock)
#define VOL_FREEFASTLOCK(flock)         _VOL_FREEFASTLOCK(flock)
#define VOL_FASTLOCK(flock)             _VOL_FASTLOCK(flock)
#define VOL_UNFASTLOCK(flock)           _VOL_UNFASTLOCK(flock)
 
#else   /* not (VOL_HAVE_FSPIN && !TED_ACTIVE) */

#define VOL_ALLOCFASTLOCK(flock)	VOL_ALLOCSPINLOCK((flock), \
							  VOL_FSPIN_FASTLOCK)
#define VOL_FREEFASTLOCK(flock)		VOL_FREESPINLOCK((flock))
#define VOL_FASTLOCK(flock)		VOL_SPINLOCK((flock))
#define VOL_UNFASTLOCK(flock)		VOL_UNSPINLOCK((flock))

#endif  /* VOL_HAVE_FSPIN && !TED_ACTIVE */

/*
 * read/write (i.e., shared/exclusive) sleep locks are defined in terms of
 * other locks.  Note: writers are given priority over readers.
 */

struct volrwsleep {
	struct volsync	rws_wsync;		/* notify lock for writers */
	struct volsync	rws_rsync;		/* notify lock for readers */
	struct volspin	rws_spin;		/* protect synch locks */
	int		rws_wlocked;		/* non-zero if write locked */
	int		rws_readers;		/* count of read locks */
	int		rws_writers;		/* count of writers */
};

/*
 * volopenclose_sleep - sleep locks for device open and close
 *
 * A special lock that gives the device close threads higer priority than the
 * device open threads.  It is the argument type for functions volopenenter(),
 * volcloseenter(), volopencloseexit().
 */

struct volopenclose_sleep {
	int		ocs_closers;		/* number of waiting closers */
	int		ocs_locked;		/* non-zero if locked */
	struct volsync	ocs_sync;		/* open close sync lock */
	struct volspin 	ocs_spin;		/* protect sync lock */
};

typedef struct volopenclose_sleep volocsleep_t;

#define volopenexit(ocs)	volopencloseexit(ocs)
#define volcloseexit(ocs)	volopencloseexit(ocs)
#define volopencloseenter(ocs)	volopenenter(ocs)

#ifdef __STDC__
extern void vol_allocrwsleep(struct volrwsleep *);
extern void vol_freerwsleep(struct volrwsleep *);
extern void vol_rwsleep_wrlock(struct volrwsleep *);
extern int  vol_rwsleep_try_wrlock(struct volrwsleep *);
extern void vol_rwsleep_unwrlock(struct volrwsleep *);
extern void vol_rwsleep_rdlock(struct volrwsleep *);
extern int  vol_rwsleep_try_rdlock(struct volrwsleep *);
extern void vol_rwsleep_unrdlock(struct volrwsleep *);
extern void vol_allococsleep(struct volopenclose_sleep *);
extern void vol_freeocsleep(struct volopenclose_sleep *);
#else	/* not __STDC__ */
extern void vol_allocrwsleep();
extern void vol_freerwsleep();
extern void vol_rwsleep_wrlock();
extern int  vol_rwsleep_try_wrlock();
extern void vol_rwsleep_unwrlock();
extern void vol_rwsleep_rdlock();
extern int  vol_rwsleep_try_rdlock();
extern void vol_rwsleep_unrdlock();
extern void vol_allococsleep();
extern void vol_freeocsleep();
#endif	/* __STDC__ */

#define VOL_ALLOCRWSLEEP(l)		vol_allocrwsleep(l)
#define VOL_FREERWSLEEP(l)		vol_freerwsleep(l)
#define VOL_RWSLEEP_WRLOCK(l)		vol_rwsleep_wrlock(l)
#define VOL_RWSLEEP_TRY_WRLOCK(l)	vol_rwsleep_try_wrlock(l)
#define VOL_RWSLEEP_UNWRLOCK(l)		vol_rwsleep_unwrlock(l)
#define VOL_RWSLEEP_RDLOCK(l)		vol_rwsleep_rdlock(l)
#define VOL_RWSLEEP_TRY_RDLOCK(l)	vol_rwsleep_try_rdlock(l)
#define VOL_RWSLEEP_UNRDLOCK(l)		vol_rwsleep_unrdlock(l)

/*
 * The following definitions (along with volrwlock above) are a result
 * of IR 12826. There is a corr. IR 15299 for hpvm. Though this is
 * currently applicable for UnixWare 2.1, the gemini implications would
 * be the same.
 */

#ifdef NSC_BUG_FIX

#define VOL_ALLOCRWLOCK(rwlp)						\
	do {								\
		(rwlp)->rw_lkinfop = 					\
			_VOLMEMZALLOC(sizeof(*(rwlp)->rw_lkinfop),	\
				      VOLMEM_SLEEP);			\
		(rwlp)->rw_lkinfop->lk_name = #rwlp;			\
		(rwlp)->rw_lockp = RW_ALLOC(1, pldisk,			\
					 (rwlp)->rw_lkinfop, KM_SLEEP);	\
	} while (ZERO)

#else  /* !NSC_BUG_FIX */

#define VOL_ALLOCRWLOCK(rwlp)						\
	do {								\
		(rwlp)->rw_lkinfop = 					\
			_VOLMEMZALLOC(sizeof(*(rwlp)->rw_lkinfop),	\
				      VOLMEM_SLEEP);			\
		(rwlp)->rw_lockp = RW_ALLOC(1, pldisk,			\
					 (rwlp)->rw_lkinfop, KM_SLEEP);	\
	} while (ZERO)

#endif /* !NSC_BUG_FIX */

#define VOL_FREERWLOCK(rwlp)						\
	do {								\
		RW_DEALLOC((rwlp)->rw_lockp);				\
		_VOLMEMFREE((rwlp)->rw_lkinfop,				\
			sizeof(*(rwlp)->rw_lkinfop));			\
	} while (ZERO)

#define VOL_RW_RDLOCK(rwlp, old_pl)					\
		(old_pl = RW_RDLOCK((rwlp)->rw_lockp, plhi))

#define VOL_RW_TRYRDLOCK(rwlp, old_pl)					\
		(old_pl = RW_TRYRDLOCK((rwlp)->rw_lockp, plhi))

#define VOL_RW_UNRDLOCK(rwlp, old_pl)					\
		(RW_UNLOCK((rwlp)->rw_lockp, old_pl))

#define VOL_RW_WRLOCK(rwlp, old_pl)					\
		(old_pl = RW_WRLOCK((rwlp)->rw_lockp, plhi))

#define VOL_RW_TRYWRLOCK(rwlp, old_pl)					\
		(old_pl = RW_TRYWRLOCK((rwlp)->rw_lockp, plhi))

#define VOL_RW_UNWRLOCK(rwlp, old_pl)					\
		(RW_UNLOCK((rwlp)->rw_lockp, old_pl))

#endif /* _VXVM_VOLLOCKS_H */
