/* @(#)unixsrc:usr/src/common/uts/io/vxvm/common/volsio.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/common/volsio.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLSIO_H
#define _VXVM_VOLSIO_H

/*
 * volsio.h - structure definitions for I/O stages
 */

#ifdef __STDC__
struct volsio;
struct volsioops;
struct volsioq;
struct volobject;
#endif	/* __STDC__ */

/*
 * volsio - base I/O stage control structure
 *
 * Each stage in an I/O that needs a control structure generates one or
 * more structures that begin with a volsio structure.  volsio
 * structures can be queued onto a start list or onto the done list for
 * an I/O.
 */
#ifndef _VOLSIO_OP_STACK
#define _VOLSIO_OP_STACK	3
#endif	/* _VOLSIO_OP_STACK */
struct volsio {
#ifdef VOLSIO_DEBUG
	struct volsio        *_sio_list_next;	/* 0x00 next in global list */
	struct volsio        *_sio_list_prev;	/* 0x04 prev in global list */
	CONST char	     *_sio_reg_file;	/* 0x08 registration file */
	int		      _sio_reg_line;	/* 0x0c registration line */
	struct {
		CONST char   *_sio_op_file;	/* +0x00 file of operation */
		int	      _sio_op_line;	/* +0x04 lineno of operation */
		CONST char   *_sio_op;		/* +0x08 operation */
		void	     *_sio_op_arg;	/* +0x0c arg to operation */
	} _sio_op_stack[_VOLSIO_OP_STACK];	/* 0x10 */
#endif /* VOLSIO_DEBUG */			/* 0x40 + */
	struct volsioops     *sio_ops;		/* 0x00 operations vector */
	struct volsio	     *sio_next;		/* 0x04 next in daemon chain */
	struct volsio	     *sio_parent;	/* 0x08 parent I/O */
	struct volsio	     *sio_forw;		/* 0x0c forward sibling link */
	struct volsio	     *sio_back;		/* 0x10 backward sibling link */
	struct volobject     *sio_object;	/* 0x14 object for stage */
	VOID		     *sio_private;	/* 0x18 parent private ptr */
	volkio_t	     *sio_kio;		/* 0x1c base I/O definition */
	volflag_t	      sio_flags;	/* 0x20 flag field */
	size_t		      sio_buf_offset;	/* 0x24 offset within I/O */
	voff_t		      sio_offset; 	/* 0x28 offset within object */
	voff_t		      sio_len;		/* 0x2c length of this I/O */
	int		      sio_errno;	/* 0x30 error, if any */
	struct volobject     *sio_error_object;	/* 0x34 object that sig'd err */
	voff_t		      sio_error_offset;	/* 0x38 offset in sig'ing obj */
	clock_t		      sio_start_time;	/* 0x3c clock time of iogen */
	voleventno_t	      sio_eventno;	/* 0x40 assigned event number */
};						/* 0x44/0x84 */

/*
 * The last 16 bits of the sio_flags is reserved for basic sio operations.
 */
#define VOLSIO_FLAG_MASK	   0x0000ffff	

/* flag values for volsio.sio_flags */
#define VOLSIO_FLAG_IODIR	   0x00000001	/* I/O direction */
#define VOLSIO_FLAG_WRITE	   0x00000000	/* write operation */
#define VOLSIO_FLAG_READ	   0x00000001	/* read operation */
#define VOLSIO_FLAG_GOT_RESOURCES  0x00000002	/* resources locked */
#define VOLSIO_FLAG_QUEUE_DONE	   0x00000004	/* queue iodone to daemon */
#define VOLSIO_FLAG_WAIT_KDONE	   0x00000008	/* wait before done'ing kio */
#define VOLSIO_FLAG_KDONE	   0x00000010	/* kio can be done'd */
#define VOLSIO_FLAG_WAIT_SIODONE   0x00000020	/* wait before done'ing sio */
#define VOLSIO_FLAG_SIODONE	   0x00000040	/* sio can be done'd */
#define VOLSIO_FLAG_NO_IOD	   0x00000080	/* no regular I/O daemons */
#define VOLSIO_FLAG_RWOP	   0x00000100	/* accum r/w stats for object */
#define VOLSIO_FLAG_IOTRACE	   0x00000200	/* make r/w I/O trace for I/O */
#define VOLSIO_FLAG_STABILIZED	   0x00000400	/* this SIO was stabilized */
#define VOLSIO_FLAG_SIGNALLED	   0x00000800	/* signalled while sleeping */
#define VOLSIO_FLAG_LOGDONE	   0x00001000   /* dirty logging is done */
#define VOLSIO_FLAG_NOREPORT	   0x00002000	/* don't report I/O errors */
#define VOLSDSIO_FLAG_DISKTRACE	   0x00004000	/* disk I/O trace is on */

/* convenience macros */
#define VOLSIO_IODIR(sio)	((sio)->sio_flags & VOLSIO_FLAG_IODIR)
#define VOLSIO_READ(sio)	(VOLSIO_IODIR((sio)) == VOLSIO_FLAG_READ)
#define VOLSIO_WRITE(sio)	(VOLSIO_IODIR((sio)) == VOLSIO_FLAG_WRITE)
#define VOLSIO_NEED_RESOURCES(sio)	\
	(! ((sio)->sio_flags & VOLSIO_FLAG_GOT_RESOURCES))
#define	VOLSIO_STABILIZED(sio) ((sio)->sio_flags & VOLSIO_FLAG_STABILIZED)

/*
 * VOLSIO_SET_ERROR - signal a non-derived error on an SIO
 * VOLSIO_COPY_ERROR - copy error information from a child SIO
 */
#define VOLSIO_SET_ERROR(sio, err)	\
	((sio)->sio_errno = (err),				\
	 (sio)->sio_error_object = (sio)->sio_object,		\
	 (sio)->sio_error_offset = (sio)->sio_offset)

#define VOLSIO_COPY_ERROR(sio, child)	\
	((sio)->sio_errno = (child)->sio_errno,			\
	 (sio)->sio_error_object = (child)->sio_error_object,	\
	 (sio)->sio_error_offset = (child)->sio_error_offset)

#define VOLSIO_RESET_ERROR(sio)	\
	((sio)->sio_errno = 0,					\
	 (sio)->sio_error_object = (struct volobject *)NULL,	\
	 (sio)->sio_error_offset = 0)
/*
 * VOLSIO_STATS_SETUP - perform standard SIO setup for statistics
 */
#define VOLSIO_STATS_SETUP(sio)	(VOL_GET_CLOCK((sio)->sio_start_time))

/*
 * VOLKIO_IODIR - return an I/O dir from a system I/O
 */
#define VOLKIO_IODIR(kio)	\
	(VOLKIO_WRITE(kio)? VOLSIO_FLAG_WRITE: VOLSIO_FLAG_READ)

/*
 * volsioops - operations vector for an I/O stage
 *
 * sop_type:	this is a string that can be used when debugging to
 *		print the type of the SIO.
 *
 * sop_start:	called after the I/O is constructed to construct any new
 *		I/Os for this stage, or to start physical I/Os.  If
 *		'sleepflag' is VOLMEM_NOSLEEP, then sop_start should
 *		try to avoid operations that can sleep.  Append any
 *		newly constructed I/Os onto the startq, and append any
 *		I/O completions to process to the doneq.  If a sleep
 *		is avoided by aborting the operation, return -1;
 *		otherwise, return 0.
 *
 * sop_childdone:
 *		called when a child I/O completes.  The child is
 *		identified by 'child'.  If the child I/O had an error,
 *		sio_errno will be set for the child.  It is up to the
 *		child I/O's processing to ensure that sop_childdone is
 *		called on the parent I/O.  sop_childdone can queue a
 *		done function by appending an sio to 'doneq', or a start
 *		function by appending to 'startq'.
 *
 * sop_done:
 *		called when a physical I/O completes.  If an error was
 *		encountered, the error will be stored in sio_errno.
 *		Return -1 to queue the done for later processing; return
 *		0 otherwise.
 *
 * sop_callback:
 *		operates as a client call back.  When a server is requested
 *		to provide some operation or privelege, it queues up the
 *		requesting client's sio, and when the operation can occur,
 *		calls the client's sio callback function.  The callback
 *		function can perform actions and queue additional sios on
 *		either a supplied start or done Q.
 *
 * sop_free:	called to free the volsio and any structures allocated
 *		from it.  This can be called before the I/O is
 *		started, in which case the I/O is effectively aborted.
 *		The function that generated the volsio structure is
 *		expected to ensure that sop_free is called.
 *
 * If sleepflag is VOLMEM_NOSLEEP, the done function may be called
 * from interrupt context, so sleeps are not allowed.
 * VOLIOS_FLAG_QUEUE_DONE can be set on the I/O to ensure that
 * sop_done is called from daemon context.
 *
 * New I/Os can be generated by appending new I/O stage structures to
 * 'startq'.  New I/O completions can be appended to 'doneq'.
 */
#define VOLSIOOP_TYPE_SZ	16

struct volsioops {
	char sop_type[VOLSIOOP_TYPE_SZ];
#ifdef __STDC__
	int  (*sop_start)(struct volsio *sio, struct volsioq *startq,
			  struct volsioq *doneq, int sleepflag);
	void (*sop_childdone)(struct volsio *sio, struct volsio *child,
			      struct volsioq *startq, struct volsioq *doneq);
	int  (*sop_done)(struct volsio *sio, struct volsioq *startq,
			 struct volsioq *doneq, int sleepflag);
	int  (*sop_callback)(struct volsio *sio_in, int service, VOID *private,
			     struct volsioq *startq, struct volsioq *doneq,
			     int sleepflag);
	void (*sop_free)(struct volsio *sio);
#else	/* not __STDC__ */
	int  (*sop_start)();
	void (*sop_childdone)();
	int  (*sop_done)();
	int  (*sop_callback)();
	void (*sop_free)();
#endif	/* __STDC__ */
};

#define VOLSIOTYPE(sio)		((sio)->sio_ops->sop_type)

/*
 * Callback service IDs. - These values identify a type of callback
 * operation.  Numbers 1-1000 are reserved for core Volume Manager code.
 */
#define VOL_CB_KLOGSTART	1	/* start a klogging op. */
#define	VOL_CB_KLOGDONE		2	/* klog flush completed successfully */
#define	VOL_CB_KLOGERR		3	/* flush failed */

/*
 * volsio_stable - structure for SIO stabilization
 *
 * Objects that need to be able to stabilize I/O memory must use SIOs
 * with the volsio_stable structure appended directly to the volsio
 * structure (this is incompatibility with SIO synchronization).  They
 * can then use volsio_stabilize() and volsio_unstabilize()) to handle
 * stabilization of the I/O memory to be used by the SIO.
 */
struct volsio_stable {
	volkio_t	   *siost_saved_kio;
	size_t		   siost_saved_buf_offset;
};

/*
 * volsio_sync - structure for SIO synchronization
 *
 * Functions that wish to synchronize with SIOs must use SIOs with the
 * volsio_sync structure appended to the volsio structure.  They can
 * then use volsiowait() to process and wait for completion of the SIO.
 */
struct volsio_sync {
	volspin_t	    sios_spin;
	volsync_t	    sios_sync;
};

/*
 * volsyncsio - the base for an synchronizable SIO
 */
struct volsyncsio {
	struct volsio	    sio;
	struct volsio_sync  sync;
};

/*
 * volstablesio - the base for a stabilizable SIO
 *
 * The volsio_stable structure is defined in vollocal.h.
 */
struct volstablesio {
	struct volsio	     sio;
	struct volsio_stable stable;
};

/*
 * volsioq - structure for maintaining a queue of I/O stages
 */
struct volsioq {
	struct volsio	   *sioq_head;
	struct volsio	   *sioq_tail;
	size_t		    sioq_count;
};

#ifdef VXVM_DEBUG
struct volsiolinkq {
	struct volsio	*head;			/* 0x00 */
	struct volsio	*tail;			/* 0x04 */
	u_int		count;			/* 0x08 */
};						/* 0x0c */
#endif /* VXVM_DEBUG */

/*
 * for backward compatibility with older debug macros
 */
#define VOLSIOQ_RESETSETLINK(q)
#define VOLSIOQ_LINK(q, sio)
#define VOLSIOQ_UNLINK(q, sio)
#define VOLSIOQ_LINKED(sio)	(0)

/*
 * _VOLSIOQ_RESET - reset queue q to be empty
 * _VOLSIOQ_SET   - set queue q to contain just sio
 */
#define _VOLSIOQ_RESET(q)	\
	((q)->sioq_head = (q)->sioq_tail = NULL, (q)->sioq_count = 0)
#define _VOLSIOQ_SET(q, sio)	\
	((sio)->sio_next = NULL,				\
	 (q)->sioq_head = (q)->sioq_tail = (sio),		\
	 (q)->sioq_count = 1)

/*
 * _VOLSIOQ_MORE - true if queue q has more entries
 * _VOLSIOQ_DEQ  - dequeue and return the first element on queue q
 */
#define _VOLSIOQ_MORE(q)	((q)->sioq_head != NULL)
#define _VOLSIOQ_COUNT(q)	((q)->sioq_count)
#define _VOLSIOQ_DEQ(q, sio)	\
	((sio) = (q)->sioq_head,				\
	 --(q)->sioq_count,					\
	 ((q)->sioq_head = (sio)->sio_next)?			\
	 ((sio)->sio_next = NULL): ((q)->sioq_tail = NULL))

/*
 * _VOLSIOQ_APPEND - append an sio at the end of I/O queue q
 * _VOLSIOQ_INSERT - insert an sio at the beginning of I/O queue q
 */
#define _VOLSIOQ_APPEND(q, sio)	\
	((sio)->sio_next = NULL,				\
	 (q)->sioq_count++,					\
	 (q)->sioq_head?					\
	 ((q)->sioq_tail = (q)->sioq_tail->sio_next = (sio)):	\
	 ((q)->sioq_head = (q)->sioq_tail = (sio)))

#define _VOLSIOQ_INSERT(q, sio)	\
	((q)->sioq_count++,					\
	 ((q)->sioq_tail == NULL? (q)->sioq_tail = (sio): 0),	\
	 (sio)->sio_next = (q)->sioq_head,			\
	 (q)->sioq_head = (sio))

#define _VOLSIOQ_ITER(q, sio)	\
	((sio)? ((sio) = (sio)->sio_next): ((sio) = (q)->sioq_head))

/*
 * _VOLSIOQ_APPENDQ - append queue q2 at the end of I/O queue q1
 * _VOLSIOQ_INSERTQ - insert queue q2 at the beginning of I/O queue q1
 */
#define _VOLSIOQ_APPENDQ(q1, q2)	\
	((q1)->sioq_count += (q2)->sioq_count,			\
	 (q2)->sioq_head?					\
	 (((q1)->sioq_head == NULL?				\
	   ((q1)->sioq_head = (q2)->sioq_head):			\
	   ((q1)->sioq_tail->sio_next = (q2)->sioq_head)),	\
	  ((q1)->sioq_tail = (q2)->sioq_tail)): 0)

#define _VOLSIOQ_INSERTQ(q1, q2)	\
	((q1)->sioq_count += (q2)->sioq_count,			\
	 (q2)->sioq_head?					\
	 (((q1)->sioq_head == NULL ?				\
	   ((q1)->sioq_tail = (q2)->sioq_tail):			\
	   ((q2)->sioq_tail->sio_next = (q1)->sioq_head)),	\
	  ((q1)->sioq_head = (q2)->sioq_head)): 0)

#ifdef VOLSIO_DEBUG

#define VOLSIO_PUSH_OP(sio, op, oparg) \
	_volsio_push_op((sio), __FILE__, __LINE__, (op), (caddr_t)(oparg))
#define _VOLSIO_REGISTER(sio, file, line) \
	do {							\
		(sio)->_sio_list_prev = NULL;			\
		(sio)->_sio_reg_file = (file);			\
		(sio)->_sio_reg_line = (line);			\
		VOL_FASTLOCK(&volsio_list_lock);		\
		(sio)->_sio_list_next = volsio_list;		\
		if (volsio_list) {				\
			volsio_list->_sio_list_prev = (sio);	\
		}						\
		volsio_list = (sio);				\
		volsio_count++;					\
		VOL_UNFASTLOCK(&volsio_list_lock);		\
	} while (0)
#define VOLSIO_REGISTER(sio) _VOLSIO_REGISTER((sio), __FILE__, __LINE__)
#define VOLSIO_UNREGISTER(sio) \
	do {							\
		VOL_FASTLOCK(&volsio_list_lock);		\
		if ((sio)->_sio_list_next) {			\
			(sio)->_sio_list_next->_sio_list_prev =	\
				(sio)->_sio_list_prev;		\
		}						\
		if ((sio)->_sio_list_prev) {			\
			(sio)->_sio_list_prev->_sio_list_next =	\
				(sio)->_sio_list_next;		\
		} else {					\
			volsio_list = (sio)->_sio_list_next;	\
		}						\
		--volsio_count;					\
		VOL_UNFASTLOCK(&volsio_list_lock);		\
	} while (0)
#define VOLSIO_ALLOC(size, flag) \
	_volsio_mem_alloc((size), (flag), __FILE__, __LINE__)
#define VOLSIO_FREE(sio, size)	_volsio_mem_free((struct volsio *)(sio), (size))

#define VOLSIOQ_RESET(q)	_VOLSIOQ_RESET(q)
#define VOLSIOQ_SET(q, sio) \
	(VOLSIO_PUSH_OP((sio), "SIOQ_SET", (q)),		\
	 _VOLSIOQ_SET((q), (sio)))
#define VOLSIOQ_MORE(q)		_VOLSIOQ_MORE(q)
#define VOLSIOQ_COUNT(q)	_VOLSIOQ_COUNT(q)
#define VOLSIOQ_DEQ(q, sio) \
	(_VOLSIOQ_DEQ((q), (sio)),				\
	 VOLSIO_PUSH_OP((sio), "SIOQ_DEQ", (q)))
#define VOLSIOQ_APPEND(q, sio) \
	(VOLSIO_PUSH_OP((sio), "SIOQ_APPEND", (q)),		\
	 _VOLSIOQ_APPEND((q), (sio)))
#define VOLSIOQ_INSERT(q, sio) \
	(VOLSIO_PUSH_OP((sio), "SIOQ_INSERT", (q)),		\
	 _VOLSIOQ_INSERT((q), (sio)))
#define VOLSIOQ_ITER(q, sio)	_VOLSIOQ_ITER(q, (sio))
#define VOLSIOQ_APPENDQ(q1, q2)					\
	do {							\
		struct volsio	*_sp = NULL;			\
		while (_VOLSIOQ_ITER((q2), _sp)) {		\
			VOLSIO_PUSH_OP(_sp, "APPENDQ", (q1));	\
		}						\
		_VOLSIOQ_APPENDQ((q1), (q2));			\
	} while (0)
#define VOLSIOQ_INSERTQ(q1, q2)					\
	do {							\
		struct volsio	*_sp = NULL;			\
		while (_VOLSIOQ_ITER((q2), _sp)) {		\
			VOLSIO_PUSH_OP(_sp, "INSERTQ", (q1));	\
		}						\
		_VOLSIOQ_INSERTQ((q1), (q2));			\
	} while (0)

#else	/* not VXVM_VOLSIOQ_DEBUG */

#define VOLSIO_PUSH_OP(sio, op, oparg)
#define VOLSIO_REGISTER(sio)
#define VOLSIO_UNREGISTER(sio)
#define VOLSIO_ALLOC(size,flag)	volmemzalloc((size), (flag))
#define VOLSIO_FREE(sio, size)	volmemfree((caddr_t)(sio), (size))

#define VOLSIOQ_RESET(q)	_VOLSIOQ_RESET(q)
#define VOLSIOQ_SET(q, sio)	_VOLSIOQ_SET(q, sio)
#define VOLSIOQ_MORE(q)		_VOLSIOQ_MORE(q)
#define VOLSIOQ_COUNT(q)	_VOLSIOQ_COUNT(q)
#define VOLSIOQ_DEQ(q, sio)	_VOLSIOQ_DEQ(q, sio)
#define VOLSIOQ_APPEND(q, sio)	_VOLSIOQ_APPEND(q, sio)
#define VOLSIOQ_INSERT(q, sio)	_VOLSIOQ_INSERT(q, sio)
#define VOLSIOQ_ITER(q, sio)	_VOLSIOQ_ITER(q, sio)
#define VOLSIOQ_APPENDQ(q1, q2)	_VOLSIOQ_APPENDQ(q1, q2)
#define VOLSIOQ_INSERTQ(q1, q2)	_VOLSIOQ_INSERTQ(q1, q2)

#endif	/* VXVM_VOLSIOQ_DEBUG */

#endif	/* _VXVM_VOLSIO_H */
