/* @(#)unixsrc:usr/src/common/uts/io/vxvm/common/volstats.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/common/volstats.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLSTATS_H
#define _VXVM_VOLSTATS_H

/*
 * include voldioctl.h to get the object type #defines
 */
#ifdef _KERNEL_HEADERS
#include <voldioctl.h>
#else
#include <vxvm/voldioctl.h>
#endif

/*
 * vol_io_stats - interface structure for returning statistics for a
 *	single object.
 * vol_io_stats_list - list of object statistics to retrieve.
 *
 * ioctls: VOL_GET_IO_STATS, VOL_CLEAR_IO_STATS
 *
 * The stats ioctls take a vol_io_stats_list ioctl, so that a single
 * ioctl can operate on any reasonable number of objects.
 */
struct vol_io_stats {
	volrid_t	vs_dg_iid;	/* 0x00	disk group rid */
	volrid_t	vs_rid;		/* 0x08	rid of object */
	int		vs_object_type;	/* 0x10	object type */
	size_t		vs_stats_size;	/* 0x14	size of stats buffer */
	struct vol_stats *vs_stats;	/* 0x18	stats buffer */
};					/* 0x1c */

struct vol_io_stats_list {
	size_t			vsl_nobject;	/* 0x00 # of objects */
	struct vol_io_stats	*vsl_stats;	/* 0x04	list of objects */
};						/* 0x08 */

/*
 * vol_stats - basic statistics structure
 *
 * Most statistics are formed as a concatenation of the vol_stats
 * structure.
 */
struct vol_stats {
	size_t		vs_op;		/* 0x00 count of I/O operations */
	size_t		vs_fail_op;	/* 0x04 count of failed I/Os */
	size_t		vs_blocks;	/* 0x08 total blocks of I/O */
	time_t		vs_time;	/* 0x0c accumulated I/O times */
};					/* 0x10 */

/*
 * Although the interfact does not absolutely require this, all objects
 * provided with VxVM 2.0 present statistics in a form that can be treated
 * as an array of vol_stats structures.  It is suggested that this same
 * convention be use for future objects, or at least that statistics that
 * do not follow the vol_stats array convention be added AFTER all
 * stats that can be put into vol_stats structures.
 *
 * To make it easier for callers, each object defines a stats structure
 * defining all stats returned for the object type, and also defines each
 * individual type of stat as an index into a vol_stats structure.
 */

/*
 * vol_rwstats - statistics for reads and writes
 *
 * This is the basic structure for reads and writes for virtual disks,
 * kernel I/O objects, and for kernel disks.
 */
struct vol_rwstats {
	struct vol_stats  vrws_read;	/* 0x00 read statistics */
	struct vol_stats  vrws_write;	/* 0x10 write statistics */
};					/* 0x20 */
/* indices for basic read/write stats: */
#define VOL_READ_STATS		0	/* read stats in volstats array */
#define VOL_WRITE_STATS		1	/* write stats in volstats array */

/*
 * Object-type specific stats:
 *
 * NOTE: All existing object types begin with the same fields as the
 * vol_rwstats structure, so that an application that just wants read
 * and write stats can supply enough space to hold vol_rwstats without
 * having to know what additional stats a particular object can
 * provide.
 */

/*
 * vol_mirrorvol_stats - statistics for a mirrorvol object
 *
 * This is the stats structure for a mirrorvol object, which includes
 * the base read/write stats, plus some additional statistics.
 */
struct vol_mirrorvol_stats {
	struct vol_stats  vmvs_read;	/* 0x00 read statistics */
	struct vol_stats  vmvs_write;	/* 0x10 write statistics */
	struct vol_stats  vmvs_vread;	/* 0x20 verify-read statistics */
	struct vol_stats  vmvs_vwrite;	/* 0x30 verify-write statistics */
	struct vol_stats  vmvs_atcpy;	/* 0x40 atomic-copy statistics */
	struct vol_stats  vmvs_cread;	/* 0x50 corrected-read statistics */
	struct vol_stats  vmvs_rwback;	/* 0x60 read-writeback recovery stats */
};
					/* 0x70 */
/* 
 * indices for extended mirrorvol stats
 */
#define VOL_MV_VREAD_STATS	2	/* verify-read in volstats array */
#define VOL_MV_VWRITE_STATS	3	/* verify-write in volstats array */
#define VOL_MV_ATCPY_STATS	4	/* atomic-copy in volstats array */
#define VOL_MV_CREAD_STATS	5	/* corrected-read in volstats array */
#define VOL_MV_RWBACK_STATS	6	/* read-writeback in volstats array */

/*
 * vol_raid_stats - stats for a raid object
 *
 * This is the stats structure for a raid object, which includes
 * the base read/write stats, plus some additional statistics.
 */
struct vol_raid_stats {
	struct vol_stats  vrs_read;	/* 0x00 read statistics - RAID */
	struct vol_stats  vrs_write;	/* 0x10 write statistics - RAID */
	struct vol_stats  vrs_fsw;	/* 0x20 full-stripe writes - STRIPE */
	struct vol_stats  vrs_rmw;	/* 0x30 read-modify-writes - STRIPE */
	struct vol_stats  vrs_rsw;	/* 0x40 reconstruct writes - STRIPE */
	struct vol_stats  vrs_rsr;	/* 0x50 reconstruct reads - REGION */
	struct vol_stats  vrs_zero;	/* 0x60 zero ioctls */
	struct vol_stats  vrs_resync;	/* 0x70 parity resync ioctls */
	struct vol_stats  vrs_recnstr;	/* 0x80 reconstruct ioctls */
	struct vol_stats  vrs_verify;   /* 0x90 verify ioctls */
};					/* 0xa0 */

/* 
 * indices for extended mirrorvol stats
 */
#define VOL_RAID_FSW_STATS	2	/* full-stripe write */
#define VOL_RAID_RMW_STATS	3	/* read-modify-write */
#define VOL_RAID_RSW_STATS	4	/* reconstruct-write */
#define VOL_RAID_RSR_STATS	5	/* reconstruct-read */
#define VOL_RAID_ZERO_STATS	6	/* zero ioctl */
#define VOL_RAID_RESYNC_STATS	7	/* resync ioctl */
#define VOL_RAID_RECNSTR_STATS	8	/* reconstruct ioctl */
#define VOL_RAID_VERIFY_STATS	9	/* verify ioctl */

#endif	/* _VXVM_VOLSTATS_H */
