/* @(#)unixsrc:usr/src/common/uts/io/vxvm/common/voltrace.h /main/uw7_nj/1 %G% %U% - %Q% */
#ident	"@(#)unixsrc:usr/src/common/uts/io/vxvm/common/voltrace.h /main/uw7_nj/1"

/*
 * Copyright(C)1996 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef _VXVM_VOLTRACE_H
#define _VXVM_VOLTRACE_H

/*
 * voltrace.h - volume manager header for kernel tracing
 */

/*
 * include voldioctl.h to get the object type #defines
 */
#ifdef _KERNEL_HEADERS
#include <voldioctl.h>
#else
#include <vxvm/voldioctl.h>
#endif

/*
 * Fundamental entry types for a trace record
 */
#define VOLIOT_TYPE_DEVREAD	0	/* virtual device read operation */
#define VOLIOT_TYPE_DEVWRITE	1	/* virtual device write operation */
#define VOLIOT_TYPE_DISKREAD	2	/* physical disk read operation */
#define VOLIOT_TYPE_DISKWRITE	3	/* physical disk write operation */
#define VOLIOT_TYPE_OBJREAD	4	/* read to a kernel I/O object */
#define VOLIOT_TYPE_OBJWRITE	5	/* write to a kernel I/O object */
#define VOLIOT_TYPE_OBJEXT	6	/* extended operation to a kernel */
					/* I/O object. */
#define VOLIOT_TYPE_MASK	0x07	/* Mask for fundamental entry types */

/*
 * The following bits may be or'd into the fundamental entry type:
 *
 *	VOLIOT_TYPE_ERROR	This is an error trace event, not a regular
 *				I/O trace event.
 *
 *	VOLIOT_TYPE_COMPLETE	This is an I/O completion record.
 *
 *	VOLIOT_ASYNC		The original I/O request was for an
 *				asynchronous I/O.
 */
#define VOLIOT_TYPE_ERROR	0x10
#define VOLIOT_TYPE_COMPLETE	0x20
#define VOLIOT_ASYNC		0x40

/*
 * Types of trace events
 */
#define VOLIOT_IO		1
#define VOLIOT_ERROR		2
#define VOLIOT_COMPLETE		3

/*
 * Types for extended trace records for special operations. These are
 * essentially subtypes passed as part of the trace record buffer to
 * indicate to the utilities what specific type of extended trace record
 * it is.
 */
#define	VOLIOT_EXT_ACOPY	0x01	/* atomic copy event - mirrorvols */
#define	VOLIOT_EXT_VREAD	0x02	/* verify read event - mirrorvols */
#define VOLIOT_EXT_VWRITE	0x03	/* verify write event - mirrorvols */
#define	VOLIOT_EXT_RRWRBK	0x04	/* reread-writeback - mirrorvols */
#define VOLIOT_EXT_R5_ZERO	0x05	/* raid5 zero ioctl event */
#define VOLIOT_EXT_R5_RESYNC	0x06	/* raid5 resync ioctl event */
#define VOLIOT_EXT_R5_RECOVER	0x07	/* raid5 recover ioctl event */
#define VOLIOT_EXT_R5_VERIFY	0x08	/* raid5 verify ioctl event */
#define VOLIOT_EXT_R5_RSR	0x09	/* raid5 reconstruct read event */
#define VOLIOT_EXT_R5_FSW	0x0a	/* raid5 full stripe write event */
#define VOLIOT_EXT_R5_RSW	0x0b	/* raid5 reconstruct write event */
#define VOLIOT_EXT_R5_RMW	0x0c	/* raid5 read modify write event */

/*
 * Union for trace object want sets
 */
typedef union voliot_want_set {
	int	ws_type;		/* tracing set type */
	struct ws_obj {
		int		wso_type;	/* tracing set type */
		volrid_t	wso_rid;	/* record ID for object */
		volrid_t	wso_dg_iid;	/* import ID of disk group */
	} ws_obj;
	struct ws_objset {
		int		wsos_type;	/* tracing set type */
		int 		wsos_obj_type;	/* disk or kernel obj type */
		volrid_t	wsos_dg_iid;	/* import ID of disk group */
	} ws_objset;
	struct ws_dev {
		int		wsd_type;	/* tracing set type */
		voldevno_t	wsd_device;	/* device number */
	} ws_dev;
	struct ws_devset {
		int		wsds_type;	/* tracing set type */
		volrid_t	wsds_dg_iid;	/* import ID of disk group */
	} ws_devset;
} voliot_want_set_t;

/*
 * Structure for trace want list
 */
typedef struct voliot_want_list {
	size_t			wl_nsets;	/* number of object sets */
	voliot_want_set_t	**wl_sets;	/* object sets */
} voliot_want_list_t;

/*
 * Possible values the ws_type field in the voliot_want_set structure.
 */
#define VOLIOT_WANT_OBJECT	1	/* disk or kernel I/O object */
#define VOLIOT_WANT_OBJSET	2	/* disk or kernel I/O object set */
#define VOLIOT_WANT_DEVICE	3	/* virtual disk device */
#define VOLIOT_WANT_DEVSET	4	/* virtual disk device set */

#endif	/* _VXVM_VOLTRACE_H */
