/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

#ident	"@(#)foomib.c	1.2"
/*
 * Copyright 1991, 1992 Unpublished Work of Novell, Inc.
 * All Rights Reserved.
 *
 * This work is an unpublished work and contains confidential,
 * proprietary and trade secret information of Novell, Inc. Access
 * to this work is restricted to (I) Novell employees who have a
 * need to know to perform tasks within the scope of their
 * assignments and (II) entities other than Novell who have
 * entered into appropriate agreements.
 *
 * No part of this work may be used, practiced, performed,
 * copied, distributed, revised, modified, translated, abridged,
 * condensed, expanded, collected, compiled, linked, recast,
 * transformed or adapted without the prior written consent
 * of Novell.  Any use or exploitation of this work without
 * authorization could subject the perpetrator to criminal and
 * civil liability.
 */

/****************************************************************************
** Source file:  foomib.c
**
** Description:  A sample Network Management Daemon process running on UnixWare.
**               It interfaces the SNMP Agent running on UnixWare with
**               SMUX protocol.
**
** Contained functions:
**                      init_foo()
**                      sync_foo()
**                      get_single_instance()
**                      set_single_instance()
**                      get_serialLineTable()
**                      set_serialLineTable()
**
** Author:   Rick Bell
**
** Date Created:  September 12, 1995
**
** COPYRIGHT STATEMENT: (C) COPYRIGHT 1993 by Novell, Inc.
**                      Property of Novell, Inc.  All Rights Reserved.
**
****************************************************************************/

/* including system include files */
#include <stdio.h>

#include "snmp.h"
#include "objects.h"

#include "foosmux.h"

static char *productName;
static int  boardStatus;
static int  new_boardStatus;
static struct in_addr exampleIpAddr;
static struct in_addr new_exampleIpAddr;
static int  newflag_exampleIpAddr;
static OID  exampleObjectID;
static OID  new_exampleObjectID;
static int  numberLines;

extern myname;
struct sltable {
               int   serialLineNumber;
               int   serialLineBaudRate;
               int   new_serialLineBaudRate;
               char  serialLineTermLocation[LOCATIONSIZE];
               char  new_serialLineTermLocation[LOCATIONSIZE];
               int   newflag_serialLineTermLocation;
               } serialLineTable[MAXTABLESIZE]; 

/* Forward References */
static int get_single_instance(OI oi, register struct type_SNMP_VarBind *v, 
								int offset);
static int set_single_instance(OI oi, register struct type_SNMP_VarBind *v, 
								int offset);
static int get_serialLineTable(OI oi, register struct type_SNMP_VarBind *v, 
								int offset);
static int set_serialLineTable(OI oi, register struct type_SNMP_VarBind *v, 
								int offset);
void init_foo()
   {
   register OT ot;
   int   i = 0;

   productName = "Foo Serial I/O Board";
   boardStatus = ENABLED;
   new_boardStatus = 0;
   exampleIpAddr.S_un.S_un_b.s_b1 = 123; /* IP Address: 123.45.67.89 */
   exampleIpAddr.S_un.S_un_b.s_b2 = 45; /* IP Address: 123.45.67.89 */
   exampleIpAddr.S_un.S_un_b.s_b3 = 67; /* IP Address: 123.45.67.89 */
   exampleIpAddr.S_un.S_un_b.s_b4 = 89; /* IP Address: 123.45.67.89 */
   newflag_exampleIpAddr = 0;
   exampleObjectID = (OID) make_obj_id_from_dot(DEFAULT_EXAMPLEOBJECTID);
   new_exampleObjectID = NULLOID;
   numberLines = MAXTABLESIZE;

   for(i = 0; i < numberLines; i++) 
      {
      serialLineTable[i].serialLineNumber = i + 1;
      serialLineTable[i].serialLineBaudRate = 3;
   	(void) sprintf(serialLineTable[i].serialLineTermLocation, 
		"This is the Term Location for Serial Line No: %d.\n", i);
      }

   if(ot = text2obj("productName")) 
      {
      ot->ot_getfnx = get_single_instance;
      ot->ot_info = (caddr_t) PRODUCTNAME;
      }

   if(ot = text2obj("boardStatus")) 
      {
      ot->ot_getfnx = get_single_instance;
      ot->ot_setfnx = set_single_instance;
      ot->ot_info = (caddr_t) BOARDSTATUS;
      }

   if(ot = text2obj("exampleIpAddr")) 
      {
      ot->ot_getfnx = get_single_instance;
      ot->ot_setfnx = set_single_instance;
      ot->ot_info = (caddr_t) EXAMPLEIPADDR;
      }

   if(ot = text2obj("exampleObjectID")) 
      {
      ot->ot_getfnx = get_single_instance;
      ot->ot_setfnx = set_single_instance;
      ot->ot_info = (caddr_t) EXAMPLEOBJECTID;
      }

   if(ot = text2obj("numberLines")) 
      {
      ot->ot_getfnx = get_single_instance;
      ot->ot_info = (caddr_t) NUMBERLINES;
      }

   if(ot = text2obj("serialLineNumber")) 
      {
      ot->ot_getfnx = get_serialLineTable;
      ot->ot_info = (caddr_t) SERIALLINENUMBER;
      }

   if(ot = text2obj("serialLineBaudRate")) 
      {
      ot->ot_getfnx = get_serialLineTable;
      ot->ot_setfnx = set_serialLineTable;
      ot->ot_info = (caddr_t) SERIALLINEBAUDRATE;
      }

   if(ot = text2obj("serialLineTermLocation")) 
      {
      ot->ot_getfnx = get_serialLineTable;
      ot->ot_setfnx = set_serialLineTable;
      ot->ot_info = (caddr_t) SERIALLINETERMLOCATION;
      }
   }

void sync_foo(int cor)
   {
   int   i = 0;

   if(new_boardStatus)
      boardStatus = new_boardStatus;

   if(boardStatus == DISABLED)
	{
	OID             mibvar_oid_ptr[2];
	VarBindList    *trap_varbindlist;

	mibvar_oid_ptr[0] = text2oid("boardStatus.0");
	trap_varbindlist = make_varbind(mibvar_oid_ptr[0], INTEGER_TYPE,
  				0, (long) boardStatus, NULL, (OID) 0);

	mibvar_oid_ptr[1] = text2oid("serialLineNumber.2");
	trap_varbindlist->next = make_varbind(mibvar_oid_ptr[1], INTEGER_TYPE, 
  					0, (long) 2, NULL, (OID) 0);

	if(smux_trap(trap_enterpriseSpecific, LINE_DEATH_ERROR_TRAP,
  		(struct type_SNMP_VarBindList *) trap_varbindlist) == NOTOK)
        	LIB_ERROR2("smux_trap: %s [%s]", smux_error(smux_errno), smux_info);
	}

   if(newflag_exampleIpAddr)
      exampleIpAddr = new_exampleIpAddr;

   if(new_exampleObjectID != NULLOID)
      exampleObjectID = oid_cpy(new_exampleObjectID);

   for(i = 0; i < numberLines; i++) 
      {
      if(serialLineTable[i].new_serialLineBaudRate) 
         {
         serialLineTable[i].serialLineBaudRate
                              = serialLineTable[i].new_serialLineBaudRate;
         serialLineTable[i].new_serialLineBaudRate = 0;
         }

      if(serialLineTable[i].newflag_serialLineTermLocation) 
         {
         (void) strcpy(serialLineTable[i].serialLineTermLocation,
                        serialLineTable[i].new_serialLineTermLocation);
         serialLineTable[i].newflag_serialLineTermLocation = 0;
         }
      }

   new_boardStatus = 0;
   newflag_exampleIpAddr = 0;
   new_exampleObjectID = NULLOID;

   for(i = 0; i < numberLines; i++) 
      {
      serialLineTable[i].new_serialLineBaudRate = 0;
      serialLineTable[i].newflag_serialLineTermLocation = 0;
      }
   }

static int get_single_instance(OI oi, register struct type_SNMP_VarBind *v, 
								int offset)
   {
            int   ifvar;
   register OID   oid = oi->oi_name;
   register OT    ot = oi->oi_type;
            OID   new;

   ifvar = (int) ot->ot_info;

   switch(offset) 
      {
      case SMUX__PDUs_get__request:
         if(oid->oid_nelem != ot->ot_name->oid_nelem + 1
               || oid->oid_elements[oid->oid_nelem - 1] != 0) 
            {
            return error__status_noSuchName;
            }
      break;

      case SMUX__PDUs_get__next__request:
         if(oid->oid_nelem == ot->ot_name->oid_nelem)
            {

            if((new = oid_extend(oid, 1)) == NULLOID)
               return NOTOK;
            new->oid_elements[new->oid_nelem - 1] = 0;

            if(v->name)
               free_SNMP_ObjectName(v->name);

            v->name = new;
            }
         else
            return NOTOK;
      break;

      default:
         printf("%s: This is a General Error Failure. The offset is %d.\n", myname, offset);
            return error__status_badValue;
/*
      return error__status_genErr;
*/
      }

   switch(ifvar) 
      {
      case PRODUCTNAME:
      return o_string(oi, v, productName, strlen(productName));

      case BOARDSTATUS:
      return o_integer(oi, v, boardStatus);

      case EXAMPLEIPADDR:
      return o_ipaddr(oi, v, (caddr_t) &exampleIpAddr);

      case EXAMPLEOBJECTID:
      return o_specific(oi, v, (caddr_t) exampleObjectID);

      case NUMBERLINES:
      return o_integer(oi, v, numberLines);

      default:
      return error__status_noSuchName;
      }
   }

static int set_single_instance(OI oi, register struct type_SNMP_VarBind *v, 
								int offset)
   {
            int   ifvar;
   register OID   oid = oi->oi_name;
   register OT    ot = oi->oi_type;
            OS    os = ot->ot_syntax;
            ObjectSyntax *value;

            int   i = 0;

   ifvar = (int) ot->ot_info;

   if(oid->oid_nelem != ot->ot_name->oid_nelem + 1)
      return error__status_noSuchName;

   if(os == NULLOS)
            return error__status_badValue;
/*
      return error__status_genErr;
*/

   switch(ifvar) 
      {
      case BOARDSTATUS:
         if((*os->os_decode)(&value, v->value) == NOTOK)
            return error__status_badValue;

         i = *((int *) value);
         (*os->os_free) (value);

         if((i != 1) && (i != 2))
            return error__status_badValue;

         new_boardStatus = i;

      return error__status_noError;

      case EXAMPLEIPADDR:
         if((*os->os_decode) (&value, v->value) == NOTOK)
            return error__status_badValue;

         new_exampleIpAddr = ((struct sockaddr_in *) value)->sin_addr;
         newflag_exampleIpAddr = 1;
         (*os->os_free) (value);

      return error__status_noError;

      case EXAMPLEOBJECTID:
         if((*os->os_decode) (&value, v->value) == NOTOK)
            return error__status_badValue;

         if((OID) value == NULLOID)
            new_exampleObjectID = NULLOID;
         else
            new_exampleObjectID = oid_cpy((OID) value);

         (*os->os_free) (value);

      return error__status_noError;

      default:
      return error__status_noSuchName;
      }
   }

static int get_serialLineTable(OI oi, register struct type_SNMP_VarBind *v, 
								int offset)
	{
            int   ifvar;
   register OID   oid = oi->oi_name;
   register OT    ot = oi->oi_type;
            OID   new;

            int   index = 0;
            int   i = 0;
            int   ifnum = 0;


   ifvar = (int) ot->ot_info;

   switch(offset) 
      {
      case SMUX__PDUs_get__request:
         if(oid->oid_nelem != ot->ot_name->oid_nelem + 1)
            return error__status_noSuchName;

      index = oid->oid_elements[oid->oid_nelem - 1];

      if(index >= numberLines)
            return error__status_noSuchName;

      break;

      case SMUX__PDUs_get__next__request:
         if(oid->oid_nelem > ot->ot_name->oid_nelem)
            {
            i = ot->ot_name->oid_nelem;
            oid->oid_elements[i]++;

            if(oid->oid_elements[i] >= numberLines)
               return NOTOK;
            }

         if(oid->oid_nelem == ot->ot_name->oid_nelem)
            {
            index = 0;

            if((new = oid_extend(oid, 1)) == NULLOID)
               return NOTOK;

            new->oid_elements[new->oid_nelem - 1] = index;

            if(v->name)
               free_SNMP_ObjectName(v->name);

            v->name = new;

            oid = new;  /* For try_again  */
            }
         else
            index = oid->oid_elements[ot->ot_name->oid_nelem];
      break;

      default:
            return error__status_badValue;
/*
      return error__status_genErr;
*/
      }

	ifnum = index;

	switch(ifvar) 
		{
		case SERIALLINENUMBER:
    	return o_integer(oi, v, ifnum);

		case SERIALLINEBAUDRATE:
    	return o_integer(oi, v, serialLineTable[ifnum].serialLineBaudRate);

		case SERIALLINETERMLOCATION:
    	return o_string(oi, v, serialLineTable[ifnum].serialLineTermLocation,
         				strlen(serialLineTable[ifnum].serialLineTermLocation));

		default:
    	return error__status_noSuchName;
	}
}

static int set_serialLineTable(OI oi, register struct type_SNMP_VarBind *v, 
								int offset)
   {
            int   ifvar;
   register OID   oid = oi->oi_name;
   register OT    ot = oi->oi_type;
            OS    os = ot->ot_syntax;
            ObjectSyntax *value;
            OctetString *cp;

            int   index = 0;
            int   i = 0;
            int   ifnum = 0;

   ifvar = (int) ot->ot_info;

   if(oid->oid_nelem != ot->ot_name->oid_nelem + 1)
      return error__status_noSuchName;

   index = oid->oid_elements[oid->oid_nelem - 1];

   if(index >= numberLines)
            return error__status_noSuchName;

   if(os == NULLOS)
            return error__status_badValue;
/*
      return error__status_genErr;
*/

    ifnum = index;

   switch(ifvar) 
      {
      case SERIALLINEBAUDRATE:
         if((*os->os_decode) (&value, v->value) == NOTOK)
            return error__status_badValue;

         i = *((int *) value);
         (*os->os_free) (value);

         if((i != BAUD1200) && (i != BAUD2400) && (i != BAUD9600)
               && (i != BAUD19800))
            return error__status_badValue;

         serialLineTable[ifnum].new_serialLineBaudRate = i;

      return error__status_noError;

      case SERIALLINETERMLOCATION:
         if((*os->os_decode) (&value, v->value) == NOTOK)
            return error__status_badValue;

         cp = ((OctetString *) value);
         serialLineTable[ifnum].newflag_serialLineTermLocation = 1;

         if(cp == NULL) 
            {
            serialLineTable[ifnum].new_serialLineTermLocation[0] = '\240';
            } 
         else 
            {
            if(cp->length <= LOCATIONSIZE)
               (void) strncpy(serialLineTable[ifnum].new_serialLineTermLocation,
		                        (char *) cp->octet_ptr, cp->length);
            else 
               {
               LIB_ERROR("\\\\(WARNING: serialLineTermLocation string too long\\\\");
               (void)strncpy(serialLineTable[ifnum].new_serialLineTermLocation,
                             (char *) cp->octet_ptr, LOCATIONSIZE - 1);
               }

        /* Silently ignore DisplayStrings that are too long. */
            if(strlen((char *) cp->octet_ptr) >= LOCATIONSIZE)
               serialLineTable[i].new_serialLineTermLocation[LOCATIONSIZE - 1] = '\240';
            }

         (*os->os_free) (value);

      return error__status_noError;

      default:
      return error__status_noSuchName;
      }
   }
